% Main GOAT application file.
% Copyright (C) Ori Adam, Dec. 2011.
%
% see also GOAT_AutoDownload, GOAT_MyDefaults, GOAT_Environment, GOAT_Defaults, GOAT_MyDefaults

function GOAT(GI)

warning off all
disp ''
disp ''
disp 'Launching GOAT . . .'
disp ''

% cd to GOAT/Code directory 
GOAT_Root = fileparts(which('GOAT.m')) ; 
if ~strcmp(GOAT_Root,cd)
    disp(['Changing directory to: ',GOAT_Root]) ; 
    cd(GOAT_Root) ; 
end


try
    if GI.AutoPilot == 1
        G.AutoPilot.Mode = 'On' ;
    else
        G.AutoPilot.Mode = 'Off' ;
    end
catch
    G.AutoPilot.Mode = 'Off' ;
end
G.EnvironmentFlag = 0 ; % GUI mode
G = GOAT_Defaults(G)  ; % Initial Defauls values
G = GUI_Initiate(G)   ; % Initiate User Interface
if G.AutoPilot.Flag
    set(G.h_Analyze,'string','Auto-Pilot','BackgroundColor',G.GUI.ActiveColor) ;
end
G.DataSelected = 1   ; % Data Selection Idx
GOAT_plotUpdate(G)        ; % Update Display
drawnow              ;
GUIcon = 1 ;
while GUIcon   % Main While Loop
    G.Flags.breakFlag = 0 ; % when 1, kills compile data process
    G.continueFlag    = 0 ; % Whether to continue during main While Loop
    if G.AutoPilot.Flag   && G.AutoPilot.ActionNumber <= G.AutoPilot.NumberOfActions
        Actions = G.AutoPilot.Actions(~strcmpi(G.AutoPilot.Actions(:,G.AutoPilot.ActionNumber),'None'),G.AutoPilot.ActionNumber) ;
        for j = 1: length(Actions)
            disp(['GOAT Auto Code ',int2str(G.AutoPilot.ActionNumber),': ',cell2mat(Actions(j))]) ;
            eval(cell2mat(Actions(j))) ;
        end
    else
        try
            set(0,'CurrentFigure',G.GOAT_Fig) ;
            set(gcf,'userdata',0) ;
            set(G.h_Analyze,'Enable','on')  ; drawnow ; % re-enable GO button
            uiwait(gcf);
            try
                set(0,'CurrentFigure',G.GOAT_Fig) ;
            catch % Figure was deleted
                GUIcon = 0 ;
                continue ;
            end
            G.select = get(gcf,'userdata');
        catch
            GUIcon = 0 ;
            continue ;
        end
        try
            switch G.select
                case 0
                    continue ;
                case 1 % Select Data Folder
                    if G.Settings.KeyPressFlag
                        uicontrol(G.h_DataSelectionT) ; % Place focus on data dropdown menu
                    end
                    DataSelected0 = G.DataSelected ;
                    DataFolder0   = G.DataFolder   ;
                    G.DataSelected = get(G.h_DataSelection,'Value') ;
                    if G.DataSelected == length(G.DataFolders) ;
                        try
                            G.DataFolder = uigetdir('..') ;
                            try
                                if G.DataFolder == 0  % cancelled uigetdir
                                    G.DataFolder   = DataFolder0 ;
                                    G.DataSelected = DataSelected0 ;
                                    set(G.h_DataSelection,'Value',DataSelected0) ;
                                    continue ;
                                end
                            end
                        catch % error during uigetdir
                            G.DataFolder   = DataFolder0 ;
                            G.DataSelected = DataSelected0 ;
                            set(G.h_DataSelection,'Value',DataSelected0) ;
                            continue ;
                        end
                        
                        try
                            % First check that this DataFolder is valid by checking whether it contains valid Calendars
                            [DataFolderPath,DataFolderName] = fileparts(G.DataFolder) ;
                            [DataPathPath,DataPathName] = fileparts(DataFolderPath) ;
                            if sum(strcmpi(DataFolderName,{'Daily','Monthly','List','Julian','Daily365','Daily360', ...
                                    'DailyX1','DailyX2','DailyX4','DailyX6','DailyX8','DailyX12','DailyX24', ...
                                    'Daily365X1','Daily365X2','Daily365X4','Daily365X6','Daily365X8','Daily365X12','Daily365X24', ...
                                    'Daily360X1','Daily360X2','Daily360X4','Daily360X6','Daily360X8','Daily360X12','Daily360X24', ...
                                    }))
                                G.DataFolder = DataFolderPath ;
                                set(G.h_message,'string',['A Calendar folder was selected. DataFolder was changed to ',G.DataFolder]) ; drawnow ; pause(2) ; % Update Message
                            elseif sum(strcmpi(DataPathName,{'Daily','Monthly','List','Julian','Daily365','Daily360', ...
                                    'DailyX1','DailyX2','DailyX4','DailyX6','DailyX8','DailyX12','DailyX24', ...
                                    'Daily365X1','Daily365X2','Daily365X4','Daily365X6','Daily365X8','Daily365X12','Daily365X24', ...
                                    'Daily360X1','Daily360X2','Daily360X4','Daily360X6','Daily360X8','Daily360X12','Daily360X24', ...
                                    }))
                                G.DataFolder = DataPathPath ;
                                set(G.h_message,'string',['A Field folder was selected. DataFolder was changed to ',G.DataFolder]) ; drawnow ; pause(2) ; % Update Message
                            end
                            [G,Flag] = FindDatasets(G) ;
%                             else % check whether the selected folder contains valid calendar folders
%                                 DataFolderDir = dir(G.DataFolder) ;
%                                 DataFolderDir = DataFolderDir([DataFolderDir(:).isdir],1) ;
%                                 if ~(sum(strncmpi({DataFolderDir.name},'Daily',5))  || ...
%                                         sum(strncmpi({DataFolderDir.name},'Julian',6)) || ...
%                                         sum(ismember({DataFolderDir.name},{'Monthly','List','monthly','list'})))
%                                     [G,Flag] = FindDatasets(G) ;
%                                 end
%                             end
                            if ~Flag
                                set(G.h_message,'string',{[G.DataFolder, '    does not contain valid datasets'] ; ['DataFolder was reverted to-->     ',DataFolder0]}) ; % Update Message
                                G.DataFolder   = DataFolder0 ;
                                G.DataSelected = DataSelected0 ;
                                set(G.h_DataSelection,'Value',DataSelected0) ;
                                continue ;
                            end
                            
                        catch
                            % Nevermind. Get error message.
                        end
                    else
                        G.DataFolder = fullfile(cell2mat(G.DataFolderAddress(G.DataSelected)),cell2mat(G.DataFolders(G.DataSelected)),'') ;
                    end
                    
                    set(G.h_DataSelection,'Tooltipstring',G.DataFolder) ;
                    G.TimeFieldIdx = 1 ;
                    G = GOAT_Initialize_Dataset(G) ;
                    if strcmp(G.TimeField,'Monthly') ;
                        G.Limits.Default.DataClimSmooth = 1 ;
                    end
                    if exist(fullfile(G.DataFolder,G.TimeField,'info.mat'),'file')
                        try
                            load(fullfile(G.DataFolder,G.TimeField,'info.mat')) ;
                            G.Limits.LinearInfo = infostr ;
                        catch
                            G.Limits.LinearInfo = '' ;
                        end
                    else
                        G.Limits.LinearInfo = '' ;
                    end
                    set(G.h_info,'TooltipString',G.Limits.LinearInfo) ;
                    set(G.h_Field,'string',G.DataFieldsString,'value',1) ;
                    set(G.h_TempRes,'string',G.TempResStr,'value',G.TimeFieldIdx);
                    G = FilterColor(G) ;
                    try
                        set(G.h_Field,'TooltipString',['[ ',G.DataFieldUnits,' ]   ',G.FieldDescription]) ;
                    end
                    G = GOAT_setProjectionStr(G) ;
                    set(G.h_Projection,'string',G.GUI.ProjectionStr) ;
                    continue ;
                case 11 % GO Button
                    if G.ProjectionType == 10
                        G = GOAT_LINK_Install(G) ;
                        continue ;
                    elseif G.Scripts.ScriptsIdx > 1
                        ToolBoxFunc = cell2mat(G.Scripts.Scripts(G.Scripts.ScriptsIdx-1)) ;
                        try
                            eval(['G = GOAT_Script_',ToolBoxFunc,'(G) ;']) ;
                        catch
                            try
                                eval(['G = ',ToolBoxFunc,'(G) ; ']) ;
                            catch
                                try
                                    eval(['GOAT_Script_',ToolBoxFunc]) ;
                                catch
                                    try
                                        eval(ToolBoxFunc) ;
                                    catch
                                        disp(['Error while trying to run the script ',ToolBoxFunc]) ;
                                    end
                                end
                            end
                        end
                        continue ;
                    else
                        if G.Settings.HoldFlag && ~G.Settings.HoldDifFlag
                            try
                                G = GOAT_holdAnalyze(G) ;
                            catch
                                G = Analyze_Data(G) ;
                                disp 'Error while attempting to create hold plot.'
                            end
                        else
                            G = Analyze_Data(G) ;
                        end
                        if G.continueFlag ;  continue ;   end
                    end
                case 50  % Filters
                    G = GOAT_Filters(G) ;
                    G = FilterColor(G) ;
                    continue ;
                case 100 % Projection
                    CurrentProjection = G.ProjectionType ;
                    if ~G.ProjectionType == get(G.h_Projection,'Value')
                        G.Settings.HoldFlag = 0 ;
                        set(G.rhs_hold,'Value',0) ;
                    end
                    if G.Settings.KeyPressFlag
                        uicontrol(G.h_DataSelectionT) ; % Place focus on data dropdown menu
                    end
                    G.ProjectionType = get(G.h_Projection,'Value') ;
                    if any(G.ProjectionType == [8,9]) && G.Settings.SinLatFlag
                        G.Settings.SinLatFlag = 0 ;
                        set(G.rhs_SinLat,'value',G.Settings.SinLatFlag) ;
                    end
                    G = GOAT_FieldSettings(G)  ; % Customized Fiels Display Settings
                    
                    if ~((CurrentProjection == 2 && G.ProjectionType == 1) || (CurrentProjection == 1 && G.ProjectionType == 2))
                        continue ;
                    end
                case 200 % Field
                    if G.Settings.KeyPressFlag
                        uicontrol(G.h_DataSelectionT) ; % Place focus on data dropdown menu
                    end
                    G.DataField = cell2mat(G.DataFields(get(G.h_Field,'Value'))) ;
                    G = GOAT_GetDims(G)        ; % get presure, longitude, latitude and hours dimensions
                    G = GOAT_GetFieldSpan(G)   ; % get available data years and days
                    G = GOAT_NewFieldLimits(G) ; % Make sure new field matches existing limits and adjust if necessary
                    G = GOAT_FieldSettings(G)  ; % Customized Fiels Display Settings
                    try
                        set(G.h_Field,'TooltipString',['[ ',G.DataFieldUnits,' ]   ',G.FieldDescription]) ;
                    catch
                        set(G.h_Field,'TooltipString','') ;
                    end
                    G = FilterColor(G) ;
                    continue ;
                case 230 % Step Index
                    TimeStep = get(G.h_Timestep,'Value') ;
                    if TimeStep > 0 % Zero or negative numbers cannot be accepted
                        G.TimeStep = TimeStep ;
                    end
                    continue ;
                case {231,232} % Step back (231) or Forward (232)
                    if G.select == 231
                        incSign = -1 ;
                    else
                        incSign = 1  ;
                    end
                    try
                        if G.Limits.S2 - G.Limits.S1 > 100 && G.TimeStepN > 1
                            button = questdlg('The Operation you requested may take very long. Proceed Anyway?','Long Calculation Warning','Continue','Abort','Abort') ;
                            if strcmp(button,'Abort')
                                G.continueFlag = 1 ;
                                continue ;
                            end
                        end
                        for n = 1: G.TimeStepN
                            switch lower(G.TimeFieldVirtual)
                                case 'monthly'
                                    switch G.TimeStep
                                        case 1 ; inc = 1  ; % 1 month
                                        case 2 ; inc = 3  ; % 3 month
                                        case 3 ; inc = 6  ; % 6 month
                                        case 4 ; inc = 12 ; % year
                                    end
                                case 'list'
                                    inc = 1  ; % 1 step
                                otherwise
                                    [YYY,X] = GOAT_ParseTimeResolution(G.TimeFieldVirtual) ;
                                    switch G.TimeStep
                                        case 1 ; inc = 1  ; % 1 day
                                        case 2
                                            switch X
                                                case 1
                                                    inc = 3  ; % 3 day
                                                otherwise
                                                    inc = 2 ;
                                            end
                                        case 3
                                            switch X
                                                case 1
                                                    inc = 7  ; % week
                                                case 2
                                                    inc = 3*X  ; % 3 day
                                                case 4
                                                    inc = 1*X  ; % 1 day
                                                case 8
                                                    inc = 4    ; % 12 Hour
                                                case 12
                                                    inc = 3    ; % 6 Hour
                                                case 24
                                                    inc = 3    ; % 3 Hour
                                            end
                                        case 4
                                            switch X
                                                case 1
                                                    inc = 14 ; % 2 week
                                                case 2
                                                    inc = 7*X  ; % week
                                                case 4
                                                    inc = 3*X  ; % 3 day
                                                case 8
                                                    inc = 1*X  ; % 1 day
                                                case 12
                                                    inc = 4  ; % 8 Hour
                                                case 24
                                                    inc = 4  ; % 4 Hour
                                            end
                                        case 5
                                            switch X
                                                case 1
                                                    inc = 30 ; % 30 day
                                                case 2
                                                    inc = 14*X ; % 2 week
                                                case 4
                                                    inc = 7*X  ; % week
                                                case 8
                                                    inc = 3*X  ; % 3 day
                                                case 12
                                                    inc = 6  ; % 12 Hour
                                                case 24
                                                    inc = 6  ; % 6 Hour
                                            end
                                        case 6
                                            switch X
                                                case 1
                                                    inc = 90 ; % 90 day
                                                case 2
                                                    inc = 30*X ; % 30 day
                                                case 4
                                                    inc = 14*X ; % 2 week
                                                case 8
                                                    inc = 7*X  ; % week
                                                case 12
                                                    inc = 12  ; % Day
                                                case 24
                                                    inc = 8  ; % 8 Hour
                                            end
                                        case {7,8,9,10}              % year
                                            if X==1 || (G.TimeStep == 8 && X==2) ...
                                                    || (G.TimeStep == 9 && X==4)
                                                [year,month,day] = GOAT_serial2date(G.Addresses.Default,G.Limits.S1) ;
                                                inc = YYY*X ;
                                                if incSign
                                                    if month>2 && GOAT_leapyear(year+1) ;  inc = 366*X ; end
                                                    if month<3 && GOAT_leapyear(year)   ;  inc = 366*X ; end
                                                else
                                                    if month>2 && GOAT_leapyear(year)   ;  inc = 366*X ; end
                                                    if month<3 && GOAT_leapyear(year-1) ;  inc = 366*X ; end
                                                end
                                            elseif  G.TimeStep == 7 && X==2
                                                inc = 90*X ; % 90 day
                                            elseif  G.TimeStep == 7 && X==4
                                                inc = 30*X ; % 30 day
                                            elseif  G.TimeStep == 7 && X==8
                                                inc = 14*X ; % 14 day
                                            elseif  G.TimeStep == 7 && X==12
                                                inc = 6*X ; % 3 day
                                            elseif  G.TimeStep == 7 && X==24
                                                inc = 12 ; % 12 hour
                                            elseif  G.TimeStep == 8 && X==4
                                                inc = 90*X ; % 30 day
                                            elseif  G.TimeStep == 8 && X==8
                                                inc = 30*X ; % 14 day
                                            elseif  G.TimeStep == 8 && X==12
                                                inc = 7*X ; % Week
                                            elseif  G.TimeStep == 8 && X==24
                                                inc = 1*X ; % 1 day
                                            elseif  G.TimeStep == 9 && X==8
                                                inc = 90*X ; % 14 day
                                            elseif  G.TimeStep == 9 && X==12
                                                inc = 14*X ; % 2 Week
                                            elseif  G.TimeStep == 9 && X==24
                                                inc = 3*X ; % 3 day
                                            end
                                    end
                                    
                            end
                            if G.select == 231
                                if G.Limits.S1 - inc >= G.Limits.Smin
                                    G.Limits.S1 = G.Limits.S1 - inc ;
                                    G.Limits.S2 = G.Limits.S2 - inc ;
                                else
                                    G.StepGUIFlag = 0 ;
                                    continue ;
                                end
                            else
                                if G.Limits.S2 + inc <= G.Limits.Smax
                                    G.Limits.S1 = G.Limits.S1 + inc ;
                                    G.Limits.S2 = G.Limits.S2 + inc ;
                                else
                                    G.StepGUIFlag = 0 ;
                                    continue ;
                                end
                            end
                            G.StepGUIFlag = 1 ;
                            if G.Scripts.ScriptsIdx > 1
                                ToolBoxFunc = cell2mat(G.Scripts.Scripts(G.Scripts.ScriptsIdx-1)) ;
                                try
                                    eval(['G = GOAT_Script_',ToolBoxFunc,'(G) ;']) ;
                                catch
                                    try
                                        eval(['G = ',ToolBoxFunc,'(G) ; ']) ;
                                    catch
                                        try
                                            eval(['GOAT_Script_',ToolBoxFunc]) ;
                                        catch
                                            try
                                                eval(ToolBoxFunc) ;
                                            catch
                                                disp(['Error while trying to run the script ',ToolBoxFunc]) ;
                                            end
                                        end
                                    end
                                end
                            else
                                if G.Settings.HoldFlag
                                    G = GOAT_holdAnalyze(G) ;
                                else
                                    G = Analyze_Data(G) ;
                                end
                            end
                            if G.continueFlag ;  continue ;   end
                            if G.Settings.HoldFlag
                                G = GOAT_holdUpdate(G)     ; % Held Figure Update Display
                            else
                                GOAT_plotUpdate(G)        ; % Update Display
                            end
                            set(G.h_message,'string',G.message) ; % Update Message
                            drawnow ;
                            pause(G.Settings.TimeStepPause) ; % TimeStepPause in secs
                            continue ;
                        end
                        G.StepGUIFlag = 0 ;
                    catch
                        G.StepGUIFlag = 0 ;
                        set(G.h_message,'string','Error: Could Not Process Last Auto-TimeStep !') ; % Update Message
                        drawnow ;
                        continue ;
                    end
                    continue ;
                case 233 % N TimeStep Increments
                    G.TimeStepN = round(str2double(get(G.h_TimestepE,'String'))) ;
                    continue ;
                case 300 % Calendar
                    if G.Settings.KeyPressFlag
                        uicontrol(G.h_DataSelectionT) ; % Place focus on data dropdown menu
                    end
                    set(G.h_TempRes,'TooltipString','Select Calendar. Virtual Calendars are shown in ''teal''. keypress = ''t''') ;
                    G.TimeFieldIdx = get(G.h_TempRes,'value') ;
                    G = GOAT_Initialize_Dataset(G) ;
                    if strcmp(G.TimeField,'Monthly') ;
                        G.Limits.Default.DataClimSmooth = 1 ;
                    end
                    set(G.h_Field,'string',G.DataFieldsString,'value',1) ;
                    set(G.h_Filters,'Backgroundcolor',G.GUI.ButtonColor)
                    if exist(fullfile(G.DataFolder,G.TimeField,'info.mat'),'file')
                        try
                            load(fullfile(G.DataFolder,G.TimeField,'info.mat')) ;
                            G.Limits.LinearInfo = infostr ;
                        catch
                            G.Limits.LinearInfo = '' ;
                        end
                    else
                        G.Limits.LinearInfo = '' ;
                    end
                    set(G.h_info,'TooltipString',G.Limits.LinearInfo) ;
                    drawnow ;
                    continue ;
                case 400 % Scripts
                    G.Scripts.ScriptsIdx = get(G.h_ToolBoxes,'value') ;
                    if G.Scripts.ScriptsIdx == 1
                        set(G.h_ToolBoxes,'TooltipString','select a script and press ''GO''')
                    else
                        try
                            set(G.h_ToolBoxes,'Tooltipstring',help(G.Scripts.ScriptFileNames{G.Scripts.ScriptsIdx-1})) ;
                        catch
                            % nevermind. Could be there is no help text or script is not a function
                            set(G.h_ToolBoxes,'Tooltipstring','') ;
                        end
                    end
                    continue ;
                case 900 % Export GOAT structure G or Export Data to to Structure D
                    G = GOAT_Export(G)   ;
                    continue ;
                case 901
                    G = Settings(G)                                    ;
                case 999
                    GUIcon = 0                                         ; % Exit
                    continue ;
                case 1101 % ColorMap
                    G.Settings.ColorMapIdx = get(G.rhs_ColorMap,'Value') ;
                case 1102 % LineWidth
                    G.Settings.LineWidthIdx = get(G.rhs_LineWidth,'Value') ;
                    G.Settings.LineWidth = (G.Settings.LineWidthIdx-2)/2 ;
                case 1103
                    G.Settings.XgridFlag = get(G.rhs_Xgrid,'value') ;
                    if G.Settings.HoldFlag
                        G.Settings.XgridFlag = abs(G.Settings.XgridFlag - 1) ;
                        set(G.rhs_Xgrid,'value',G.Settings.XgridFlag) ;
                        continue ;
                    end
                case 1104
                    G.Settings.YgridFlag = get(G.rhs_Ygrid,'value') ;
                    if G.Settings.HoldFlag
                        G.Settings.YgridFlag = abs(G.Settings.YgridFlag - 1) ;
                        set(G.rhs_Ygrid,'value',G.Settings.YgridFlag) ;
                        continue ;
                    end
                case 1105
                    G.Settings.FixZLimitsFlag = get(G.rhs_FixZ,'value') ;
                    if G.Settings.HoldFlag
                        G.Settings.FixZLimitsFlag = abs(G.Settings.FixZLimitsFlag - 1) ;
                        set(G.rhs_FixZ,'value',G.Settings.FixZLimitsFlag) ;
                        continue ;
                    else
                        if G.Settings.FixZLimitsFlag && G.Settings.SymZLimitsFlag
                            AbsMax = max([abs(G.Settings.ZlimitMin),G.Settings.ZlimitMax]) ;
                            G.Settings.ZlimitMin = G.Settings.ZlimitMin - G.Settings.ZlimitBuffer*AbsMax ;
                            G.Settings.ZlimitMax = G.Settings.ZlimitMax + G.Settings.ZlimitBuffer*AbsMax ;
                        else
                            try
                                MinZ = min(G.Data(:));
                                MaxZ = max(G.Data(:));
                                AbsMax = max([abs(MinZ) , abs(MaxZ)]) ;
                                G.Settings.ZlimitMin = MinZ - G.Settings.ZlimitBuffer*AbsMax ;
                                G.Settings.ZlimitMax = MaxZ + G.Settings.ZlimitBuffer*AbsMax;
                            catch
                                set(G.h_message,'string','Error: Could not obtain Z limits!') ; %
                            end
                        end
                    end
                case 1106 % Coast LineWidth
                    G.Settings.CoastLineWidthIdx = get(G.rhs_CoastLineWidth,'Value') ;
                    G.Settings.CoastLineWidth = G.Settings.CoastLineWidthIdx/2-.5 ;
                case 1107 % Contouring
                    G.Settings.ContourIdx = get(G.rhs_Contour,'Value') ;
                case 1108 % Zlimit Buffer
                    G.Settings.ZlimitBuffer =  str2double(get(G.rhs_Zbuffer,'string'))/100 ;
                    if ~G.Settings.HoldFlag
                        if G.Settings.FixZLimitsFlag && G.Settings.SymZLimitsFlag
                            AbsMax = max([abs(G.Settings.ZlimitMin),G.Settings.ZlimitMax]) ;
                            G.Settings.ZlimitMin = G.Settings.ZlimitMin - G.Settings.ZlimitBuffer*AbsMax ;
                            G.Settings.ZlimitMax = G.Settings.ZlimitMax + G.Settings.ZlimitBuffer*AbsMax ;
                        elseif G.Settings.FixZLimitsFlag
                            try
                                MinZ = min(G.Data(:));
                                MaxZ = max(G.Data(:));
                                AbsMax = max([abs(MinZ) , abs(MaxZ)]) ;
                                G.Settings.ZlimitMin = MinZ - G.Settings.ZlimitBuffer*AbsMax ;
                                G.Settings.ZlimitMax = MaxZ + G.Settings.ZlimitBuffer*AbsMax;
                            catch
                                set(G.h_message,'string','Error: Could not obtain Z limits!') ; %
                            end
                        end
                    end
                case 1109 % LogLevles
                    G.Settings.LogLevelsFlag =  get(G.rhs_LogLevels,'value') ;
                    if G.Settings.HoldFlag
                        G.Settings.LogLevelsFlag = abs(G.Settings.LogLevelsFlag - 1) ;
                        set(G.rhs_LogLevels,'value',G.Settings.LogLevelsFlag) ;
                        continue
                    end
                case 1120 % SinLat
                    G.Settings.SinLatFlag =  get(G.rhs_SinLat,'value') ;
                    if G.Settings.HoldFlag
                        G.Settings.SinLatFlag = abs(G.Settings.SinLatFlag - 1) ;
                        set(G.rhs_SinLat,'value',G.Settings.SinLatFlag) ;
                        continue
                    end
                    if any(G.ProjectionType == [8,9])
                        G.Settings.SinLatFlag = abs(G.Settings.SinLatFlag - 1) ;
                        set(G.rhs_SinLat,'value',G.Settings.SinLatFlag) ;
                        continue
                    end
                case 1110 % Contour Step Level
                    try
                        ContourLevelStep = str2double(get(G.rhs_StepLevel,'string')) ;
                        MinZ = min(G.Data(:));
                        MaxZ = max(G.Data(:));
                        if isnan(ContourLevelStep)
                            G.Settings.ContourLevelStep = 0 ;
                            set(G.rhs_StepLevel,'string',num2str(G.Settings.ContourLevelStep,5))    ;
                        elseif  ContourLevelStep > (MaxZ - MinZ)/100 || ContourLevelStep == 0
                            G.Settings.ContourLevelStep = ContourLevelStep ;
                        else
                            set(G.rhs_StepLevel,'string',num2str(G.Settings.ContourLevelStep,5))    ;
                        end
                    catch
                        set(G.rhs_StepLevel,'string',num2str(G.Settings.ContourLevelStep,5))    ;
                    end
                case 1111 % Sym-Z
                    G.Settings.SymZLimitsFlag = get(G.rhs_SymZ,'value') ;
                    if G.Settings.HoldFlag
                        G.Settings.SymZLimitsFlag = abs(G.Settings.SymZLimitsFlag-1) ;
                        set(G.rhs_SymZ,'value',G.Settings.SymZLimitsFlag) ;
                        continue ;
                    else
                        try
                            MinZ = min(min(G.Data));
                            MaxZ = max(max(G.Data));
                            AbsMax = max([abs(MinZ) , abs(MaxZ)]) ;
                            G.Settings.ZlimitMin = - AbsMax ;
                            G.Settings.ZlimitMax =   AbsMax ;
                        catch
                            set(G.h_message,'string','Error: Could not obtain Z limits!') ; %
                        end
                    end
                case 1115 % Inv-cmap
                    G.Settings.ReverseColorMap = get(G.rhs_RevColMap,'Value') ;
                    if G.Settings.HoldFlag
                        G.Settings.ReverseColorMap = abs(G.Settings.ReverseColorMap - 1) ;
                        set(G.rhs_RevColMap,'value',G.Settings.ReverseColorMap) ;
                        continue ;
                    end
                case 1200 % Hold
                    G.Settings.HoldFlag = get(G.rhs_hold,'Value') ;
                    if ~isfield(G,'Data') && G.Settings.HoldFlag
                        switch G.Settings.ErrorSound
                            case 'goat'
                                soundsc(G.Error.Sound,G.Error.SoundFs) ;
                            case 'beep'
                                beep ;
                        end
                        set(G.h_message,'string','No Data to HOLD!') ; % Update Message
                        G.Settings.HoldFlag = 0 ;
                        set(G.rhs_hold,'Value',G.Settings.HoldFlag) ;
                        G.Settings.HoldDifFlag = 0 ;
                        set(G.rhs_holddif,'value',G.Settings.HoldDifFlag,'Visible','off')
                        drawnow ;
                        continue ;
                    else
                        if G.Settings.HoldFlag
                            set(G.rhs_holddif,'Visible','on')
                            G = HoldData(G) ;
                        else
                            G.Settings.HoldDifFlag = 0 ;
                            set(G.rhs_holddif,'value',G.Settings.HoldDifFlag,'Visible','off')
                        end
                        continue ;
                    end
                case 1300 % HOLD-DIF
                    G.Settings.HoldDifFlag = get(G.rhs_holddif,'Value') ;
                    %                     if any(G.ProjectionType == [5,6,7]) && G.Settings.HoldDifFlag % Not supported for Hovmoller Plots
                    %                        G.Settings.HoldDifFlag = 0 ;
                    %                        set(G.rhs_holddif,'Value',G.Settings.HoldDifFlag) ;
                    %                     end
                    continue ;
            end
        catch GOAT_Error
            
            G.message = 'Error: Operation could not be completed.' ; % Update Message
            try
                cprintf('err',['Error ',int2str(G.Error.ErrorNum),': ']) ;
                cprintf('text',[GOAT_Error.message,'\n']) ;
            catch
                disp(['Error ',int2str(G.Error.ErrorNum),': ',GOAT_Error.message]) ;
            end
            for e = 1: length(GOAT_Error.stack)
                disp(['Error in ''',GOAT_Error.stack(e).name,''', Line: ',int2str(GOAT_Error.stack(e).line)]) ;
            end
            G.Error.ErrorNum = G.Error.ErrorNum + 1 ;
            switch G.Settings.ErrorSound
                case 'goat'
                    soundsc(G.Error.Sound,G.Error.SoundFs) ;
                case 'beep'
                    beep ;
            end
            try
            set(G.h_Analyze,'Enable','on')  ; drawnow ; % Enable GO button in case some error occured after it was disabled
            end
        end
        try
            if G.Settings.HoldFlag && ~G.Settings.HoldDifFlag
                G = GOAT_holdUpdate(G)     ; % Held Figure Update Display
            else
                G = GOAT_plotUpdate(G)        ; % Update Display
            end
            set(G.h_message,'string',G.message) ; % Update Message
            drawnow ;
        catch
            %GUIcon = 0 ;
            continue ;
        end
    end
end

try
    close(G.GOAT_Fig) ;
catch
    %nevermind
end
disp 'GOAT Closed'
return
% XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
%  GOAT INTERNAL FUNCTIONS (Global Functions Are Placed in 'Utils' Folder)
% XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
% Initiate user Interface
function G = GUI_Initiate(G)
G = GOAT_setProjectionStr(G) ;
G.GOAT_Fig = figure('name',['GOAT: Geophysical Observation Analysis Tool. Ver: ',G.Consts.Version,'   Written By Ori Adam.'],'position',G.GUI.pos.Main) ;
set(G.GOAT_Fig,'Color',G.Display.FigBackgroudColor) ;

Line = .0 ;
G.h_message = uicontrol('Style', 'text','Units','normalized','pos',[0,G.GUI.Hgui - Line - G.GUI.HbuttonHeight*.8,1,G.GUI.HbuttonHeight*.8], 'string',G.message);
uicontrol(G.h_message) ;  set(G.h_message,'ForegroundColor',G.GUI.messageColor,'FontSize',G.Display.TextSize_M, 'TooltipString','Message and Alert Area','FontName',G.GUI.Fonts.MessageFont) ;

h_sitelink = uicontrol('Units','normalized','pos',[G.GUI.RHS_b_right-G.GUI.RHStextlength,G.GUI.Hedge,G.GUI.RHStextlength*1.5,G.GUI.HbuttonHeight*.8], 'string',{'<Html><FONT color="blue" face="Cambria"><u>GOAT-GEO.ORG</u></font></Html>'});
uicontrol(h_sitelink) ;  set(h_sitelink,'BackgroundColor',G.Display.FigBackgroudColor,'FontSize',G.Display.TextSize_S,'HorizontalAlignment','right','FontName',G.GUI.Fonts.uiHeaderFont, ...
    'fontweight','bold', 'callback','web(''http://www.goat-geo.org'',''-browser'')') ;
cObj = findjobj(h_sitelink) ;
cObj.Border = [] ;

% Analyze Button
Line = G.GUI.NumLines -0.05 ;
%Line = 1.3 ;
G.h_Analyze = uicontrol('Units','normalized','pos',[G.GUI.b_left,G.GUI.Hgui-G.GUI.Hedge-Line*G.GUI.Hparts,G.GUI.TextLength*1.33,G.GUI.HbuttonHeight*1.05], ...
    'string','GO',  'callback','set(gcbf,''userdata'',11); uiresume;');
uicontrol(G.h_Analyze) ;  set(G.h_Analyze,'BackgroundColor',[.5 .7 .9],'ForegroundColor',G.GUI.DropTextColor,'FontSize',G.Display.TextSize_L,'fontweight','bold', ...
    'TooltipString','Process Data According to selected Projection, Filters, or Scripts. Keypress = ''g''','FontName',G.GUI.Fonts.uiButtonFont ) ;

%Data Selection
Line = 1.3 ;
G.h_DataSelectionT = uicontrol('Style', 'text','Units','normalized','pos',[G.GUI.b_left,G.GUI.Hgui-G.GUI.Hedge-Line*G.GUI.Hparts,G.GUI.TextLength,G.GUI.HbuttonHeight], 'string','Dataset',  'callback',['uiresume;']);
uicontrol(G.h_DataSelectionT) ;  set(G.h_DataSelectionT,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',G.GUI.HeaderColor,'FontSize',G.Display.TextSize_M,'horizontalalignment','left','FontName',G.GUI.Fonts.uiHeaderFont) ;

Line = 2 ;
G.h_DataSelection = uicontrol('style','popup','Units','normalized','pos',[G.GUI.b_left,G.GUI.Hgui-G.GUI.Hedge-Line*G.GUI.Hparts,G.GUI.TextLength,G.GUI.HbuttonHeight], ...
    'string',G.DataFoldersString,  'callback','set(gcbf,''userdata'',1); uiresume;');
uicontrol(G.h_DataSelection) ;  set(G.h_DataSelection,'BackgroundColor',G.GUI.ButtonColor,'ForegroundColor',G.GUI.DropTextColor,'FontSize',G.Display.TextSize_M, ...
    'TooltipString','Select Data Source For Analysis. keypress = ''d''','FontName',G.GUI.Fonts.uiButtonFont ) ;
G.h_DataSelection_handle = gco ;

% Temporal Resolution
Line = 3.3 ;
G.h_TempResT = uicontrol('Style', 'text','Units','normalized','pos',[G.GUI.b_left,G.GUI.Hgui-G.GUI.Hedge-Line*G.GUI.Hparts,G.GUI.TextLength,G.GUI.HbuttonHeight], 'string','Calendar');
uicontrol(G.h_TempResT) ;  set(G.h_TempResT,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',G.GUI.HeaderColor,'FontSize',G.Display.TextSize_M,'horizontalalignment','left','FontName',G.GUI.Fonts.uiHeaderFont ) ;
Line = 4 ;
G.h_TempRes = uicontrol('style','popup','Units','normalized','pos',[G.GUI.b_left,G.GUI.Hgui-G.GUI.Hedge-Line*G.GUI.Hparts,G.GUI.TextLength,G.GUI.HbuttonHeight], ...
    'string',G.TempResStr,  'callback','set(gcbf,''userdata'',300); uiresume;');
uicontrol(G.h_TempRes) ;  set(G.h_TempRes,'BackgroundColor',G.GUI.ButtonColor,'ForegroundColor',G.GUI.DropTextColor,'FontSize',G.Display.TextSize_M, ...
    'TooltipString','Select available temporal resolutions. keypress = ''t''','FontName',G.GUI.Fonts.uiButtonFont) ;
G.h_TempRes_handle = gco ;

% Fields
Line = 5.3 ;
G.h_FieldT = uicontrol('Style', 'text','Units','normalized','pos',[G.GUI.b_left,G.GUI.Hgui-G.GUI.Hedge-Line*G.GUI.Hparts,G.GUI.TextLength*.8,G.GUI.HbuttonHeight], 'string','Field',  'callback',['uiresume;']);
uicontrol(G.h_FieldT) ;  set(G.h_FieldT,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',G.GUI.HeaderColor,'FontSize',G.Display.TextSize_M,'horizontalalignment','left' ,'FontName',G.GUI.Fonts.uiHeaderFont) ;
% info
G.h_info = uicontrol('Style', 'text','Units','normalized','pos',[G.GUI.b_left+G.GUI.TextLength*.8,G.GUI.Hgui-G.GUI.Hedge-Line*G.GUI.Hparts,G.GUI.TextLength*.2,G.GUI.HbuttonHeight], ...
    'string','[ i ]','TooltipString','Display info string from info.mat of TimeResolution, Field, or Linear folder',...
    'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',G.GUI.HeaderColor,'FontSize',G.Display.TextSize_M);
uicontrol(G.h_info) ;
Line = 6 ;
G.h_Field = uicontrol('style','popup','Units','normalized','pos',[G.GUI.b_left,G.GUI.Hgui-G.GUI.Hedge-Line*G.GUI.Hparts,G.GUI.TextLength,G.GUI.HbuttonHeight], ...
    'string',G.DataFieldsString,  'callback','set(gcbf,''userdata'',200); uiresume;');
uicontrol(G.h_Field) ;  set(G.h_Field,'BackgroundColor',G.GUI.ButtonColor,'ForegroundColor',G.GUI.DropTextColor,'FontSize',G.Display.TextSize_M, ...
    'TooltipString','Select Field for Analysis. keypress = ''b''','FontName',G.GUI.Fonts.uiButtonFont ) ;
G.h_Field_handle = gco ;

% Projection Type
Line = 7.3 ;
G.h_ProjectionT = uicontrol('Style', 'text','Units','normalized','pos',[G.GUI.b_left,G.GUI.Hgui-G.GUI.Hedge-Line*G.GUI.Hparts,G.GUI.TextLength,G.GUI.HbuttonHeight], 'string','Projection',  'callback',['uiresume;']);
uicontrol(G.h_ProjectionT) ;  set(G.h_ProjectionT,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',G.GUI.HeaderColor,'FontSize',G.Display.TextSize_M,'horizontalalignment','left','FontName',G.GUI.Fonts.uiHeaderFont) ;
Line = 8 ;
G.h_Projection = uicontrol('style','popup','Units','normalized','pos',[G.GUI.b_left,G.GUI.Hgui-G.GUI.Hedge-Line*G.GUI.Hparts,G.GUI.TextLength,G.GUI.HbuttonHeight], ...
    'string',G.GUI.ProjectionStr,  'callback','set(gcbf,''userdata'',100); uiresume;');
uicontrol(G.h_Projection) ;  set(G.h_Projection,'BackgroundColor',G.GUI.ButtonColor,'ForegroundColor',G.GUI.DropTextColor,'FontSize',G.Display.TextSize_M , ...
    'TooltipString','Select Projection Type. keypress = ''p''','FontName',G.GUI.Fonts.uiButtonFont ) ;
G.h_Projection_handle = gco ;

% Filters/ Weights
Line = 9 ;
G.h_Filters = uicontrol('Units','normalized','pos',[G.GUI.b_left,G.GUI.Hgui-G.GUI.Hedge-Line*G.GUI.Hparts,G.GUI.TextLength,G.GUI.HbuttonHeight], ...
    'string','FILTERS', 'callback','set(gcbf,''userdata'',50); uiresume;');
uicontrol(G.h_Filters) ;  set(G.h_Filters,'BackgroundColor',G.GUI.ButtonColor,'ForegroundColor',G.GUI.ButtonTextColor,'FontSize',G.Display.TextSize_M , ...
    'TooltipString','Add Filters and Weights. keypress = ''f''' ,'FontName',G.GUI.Fonts.uiButtonFont) ;

% ToolBoxes
Line = 10.3 ;
G.h_ToolBoxesT = uicontrol('Style', 'text','Units','normalized','pos',[G.GUI.b_left,G.GUI.Hgui-G.GUI.Hedge-Line*G.GUI.Hparts,G.GUI.TextLength,G.GUI.HbuttonHeight], 'string','Scripts');
uicontrol(G.h_ToolBoxesT) ;  set(G.h_ToolBoxesT,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',G.GUI.HeaderColor,'FontSize',G.Display.TextSize_M,'horizontalalignment','left' ,'FontName',G.GUI.Fonts.uiHeaderFont, ...
    'Tooltipstring','m-files located at the Scripts folder that execute when selected') ;
Line = 11 ;
G.h_ToolBoxes = uicontrol('style','popup','Units','normalized','pos',[G.GUI.b_left,G.GUI.Hgui-G.GUI.Hedge-Line*G.GUI.Hparts,G.GUI.TextLength,G.GUI.HbuttonHeight], ...
    'string',G.Scripts.ScriptsString,  'callback','set(gcbf,''userdata'',400); uiresume;');
uicontrol(G.h_ToolBoxes) ;  set(G.h_ToolBoxes,'BackgroundColor',G.GUI.ButtonColor,'ForegroundColor',G.GUI.DropTextColor,'FontSize',G.Display.TextSize_M, ...
    'TooltipString','select a script and press ''GO''','FontName',G.GUI.Fonts.uiButtonFont) ;

Line = 12.2 ;
G.h_TimeStepT = uicontrol('Style', 'text','Units','normalized','pos',[G.GUI.b_left,G.GUI.Hgui-G.GUI.Hedge-Line*G.GUI.Hparts,G.GUI.TextLength,G.GUI.HbuttonHeight], 'string','TimeStep',  'callback',['uiresume;']);
uicontrol(G.h_TimeStepT) ;  set(G.h_TimeStepT,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',G.GUI.HeaderColor,'FontSize',G.Display.TextSize_M, ...
    'TooltipString','Auto-Repeat last analysis forward or backwards in time.','FontName',G.GUI.Fonts.uiHeaderFont) ;
Line = 13 ;
im = imread(fullfile('Art','BackArrow.png')) ;
G.h_TimestepBack = uicontrol('Units','normalized','pos',[G.GUI.b_left,G.GUI.Hgui-G.GUI.Hedge-Line*G.GUI.Hparts,G.GUI.TextLength/4,G.GUI.HbuttonHeight],  'callback',['set(gcbf,''userdata'',231); uiresume;']);
uicontrol(G.h_TimestepBack) ;  set(G.h_TimestepBack,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',G.GUI.HeaderColor,'FontSize',G.Display.TextSize_M ...
    ,'TooltipString','Auto-Repeat last analysis backwards by designated N Times and Time Step. keypress = ''leftarrow''','FontName',G.GUI.Fonts.uiButtonFont) ;
set(G.h_TimestepBack,'CData',im) ;
G.h_TimestepE = uicontrol('Style', 'edit','Units','normalized','pos',[G.GUI.b_left+G.GUI.TextLength/4,G.GUI.Hgui-G.GUI.Hedge-Line*G.GUI.Hparts,G.GUI.TextLength/2,G.GUI.HbuttonHeight], 'string',int2str(G.TimeStepN),  'callback',['set(gcbf,''userdata'',233); uiresume;']);
uicontrol(G.h_TimestepE) ;  set(G.h_TimestepE,'BackgroundColor',[.95 .95 .95],'ForegroundColor',G.GUI.ButtonTextColor,'FontSize',G.Display.TextSize_M ...
    ,'TooltipString','Number of Times Auto step is repeated','FontName',G.GUI.Fonts.uiButtonFont) ;
im = imread(fullfile('Art','ForwardArrow.png')) ;
G.h_TimestepForward = uicontrol('Units','normalized','pos',[G.GUI.b_left+G.GUI.TextLength*3/4,G.GUI.Hgui-G.GUI.Hedge-Line*G.GUI.Hparts,G.GUI.TextLength/4,G.GUI.HbuttonHeight],  'callback',['set(gcbf,''userdata'',232); uiresume;']);
uicontrol(G.h_TimestepForward) ;  set(G.h_TimestepForward,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',G.GUI.HeaderColor,'FontSize',G.Display.TextSize_M,...
    'TooltipString','Auto-Repeat last analysis forward by designated N Times and Time Step. keypress = ''rightarrow''','FontName',G.GUI.Fonts.uiButtonFont) ;
set(G.h_TimestepForward,'CData',im) ;
Line = 14 ;
G.h_Timestep = uicontrol('style','popup','Units','normalized','pos',[G.GUI.b_left,G.GUI.Hgui-G.GUI.Hedge-Line*G.GUI.Hparts,G.GUI.TextLength,G.GUI.HbuttonHeight], ...
    'string',G.TimeStepStr,  'callback','set(gcbf,''userdata'',230); uiresume;');
uicontrol(G.h_Timestep) ;  set(G.h_Timestep,'BackgroundColor',G.GUI.ButtonColor,'ForegroundColor',G.GUI.DropTextColor,'FontSize',G.Display.TextSize_M, ...
    'TooltipString','Select Auto Time-Step' ,'FontName',G.GUI.Fonts.uiButtonFont) ;

Line = G.GUI.NumLines - 3;
G.h_Settings = uicontrol('Units','normalized','pos',[G.GUI.b_left,G.GUI.Hgui-G.GUI.Hedge-Line*G.GUI.Hparts,G.GUI.TextLength,G.GUI.HbuttonHeight], 'string','SETTINGS',  'callback','set(gcbf,''userdata'',901); uiresume;');
uicontrol(G.h_Settings) ;  set(G.h_Settings,'BackgroundColor',G.GUI.ButtonColor,'ForegroundColor',G.GUI.ButtonTextColor,'FontSize',G.Display.TextSize_M, ...
    'TooltipString','Visualization and general configuration options. keypress = ''s''','FontName',G.GUI.Fonts.uiButtonFont ) ;

Line = G.GUI.NumLines - 2;
G.h_Export = uicontrol('Units','normalized','pos',[G.GUI.b_left,G.GUI.Hgui-G.GUI.Hedge-Line*G.GUI.Hparts,G.GUI.TextLength,G.GUI.HbuttonHeight], 'string','EXPORT',  'callback','set(gcbf,''userdata'',900); uiresume;');
uicontrol(G.h_Export) ;  set(G.h_Export,'BackgroundColor',G.GUI.ButtonColor,'ForegroundColor',G.GUI.ButtonTextColor,'FontSize',G.Display.TextSize_M,'TooltipString','Export handles or Analyzed Data to .fig and .mat files. keypress = ''x''' ,'FontName',G.GUI.Fonts.uiButtonFont) ;

% RHS Toolbar - - - - - - - - - - - - - - - - -
Line = 1.3 ;
h_Text1 = uicontrol('Style', 'text','Units','normalized','pos',[G.GUI.RHS_b_right,G.GUI.Hgui-G.GUI.Hedge-Line*G.GUI.RHparts,G.GUI.RHStextlength,G.GUI.RHbuttonHeight], 'string','Plotting');
uicontrol(h_Text1) ;  set(h_Text1,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',G.GUI.HeaderColor,'FontSize',G.Display.TextSize_M,'HorizontalAlignment','left','FontName',G.GUI.Fonts.uiHeaderFont) ;
Line = 2.3 ;
h_StepLevelT = uicontrol('style','text','Units','normalized','pos',[G.GUI.RHS_b_right,G.GUI.Hgui-G.GUI.Hedge-Line*G.GUI.RHparts,G.GUI.RHStextlength,G.GUI.RHbuttonHeight], 'string','Contour Step');
uicontrol(h_StepLevelT) ;  set(h_StepLevelT,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'HorizontalAlignment','left', ...
    'TooltipString','Contour Step Level. 0 = Auto.','FontName',G.GUI.Fonts.uiHeaderFont) ;
Line = 3 ;
G.rhs_StepLevel = uicontrol('style','edit','Units','normalized','pos',[G.GUI.RHS_b_right,G.GUI.Hgui-G.GUI.Hedge-Line*G.GUI.RHparts,G.GUI.RHStextlength,G.GUI.RHbuttonHeight],...
    'string',num2str(G.Settings.ContourLevelStep),  'callback','set(gcbf,''userdata'',1110); uiresume;');
uicontrol(G.rhs_StepLevel) ;  set(G.rhs_StepLevel,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'HorizontalAlignment','left', ...
    'TooltipString','Contour Step Level. 0 = Auto.','FontName',G.GUI.Fonts.uiButtonFont) ;
% ColorMap
Line = 4.3 ;
h_ColorMapT = uicontrol('style','text','Units','normalized','pos',[G.GUI.RHS_b_right,G.GUI.Hgui-G.GUI.Hedge-Line*G.GUI.RHparts,G.GUI.RHStextlength,G.GUI.RHbuttonHeight], 'string','ColorMap');
uicontrol(h_ColorMapT) ;  set(h_ColorMapT,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'HorizontalAlignment','left','FontName',G.GUI.Fonts.uiHeaderFont) ;
Line = 5 ;
G.rhs_ColorMap = uicontrol('style','popup','Units','normalized','pos',[G.GUI.RHS_b_right,G.GUI.Hgui-G.GUI.Hedge-Line*G.GUI.RHparts,G.GUI.RHStextlength,G.GUI.RHbuttonHeight],...
    'string',G.Settings.ColorMapStr,  'callback','set(gcbf,''userdata'',1101); uiresume;');
uicontrol(G.rhs_ColorMap) ;  set(G.rhs_ColorMap,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'HorizontalAlignment','left','Value',G.Settings.ColorMapIdx,'FontName',G.GUI.Fonts.uiButtonFont) ;
G.h_rhs_ColorMap = gco ;
% Rev Color Map
Line = 6 ;
G.rhs_RevColMap = uicontrol('style','checkbox','Units','normalized','pos',[G.GUI.RHS_b_right,G.GUI.Hgui-G.GUI.Hedge-Line*G.GUI.RHparts,G.GUI.RHStextlength,G.GUI.RHbuttonHeight], 'string','Inv-Map',...
    'value',G.Settings.ReverseColorMap,'callback','set(gcbf,''userdata'',1115); uiresume;');
uicontrol(G.rhs_RevColMap) ;  set(G.rhs_RevColMap,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'HorizontalAlignment','left', ...
    'TooltipString','Reverse Color Map','FontName',G.GUI.Fonts.uiHeaderFont) ;
% Line Width
Line = 7.3 ;
h_LineWidthT = uicontrol('style','text','Units','normalized','pos',[G.GUI.RHS_b_right,G.GUI.Hgui-G.GUI.Hedge-Line*G.GUI.RHparts,G.GUI.RHStextlength,G.GUI.RHbuttonHeight], 'string','LineWidth');
uicontrol(h_LineWidthT) ;  set(h_LineWidthT,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'HorizontalAlignment','left','FontName',G.GUI.Fonts.uiHeaderFont) ;
Line = 8 ;
G.rhs_LineWidth = uicontrol('style','popup','Units','normalized','pos',[G.GUI.RHS_b_right,G.GUI.Hgui-G.GUI.Hedge-Line*G.GUI.RHparts,G.GUI.RHStextlength,G.GUI.RHbuttonHeight],...
    'string','Darken edgecolor |None | 0.5 | 1 | 1.5 | 2 | 2.5 | 3 | 3.5 | 4 | 4.5 | 5 | 5.5 | 6 | 6.5 | 7 | 7.5 | 8 | 8.5 | 9 | 9.5 | 10 ',  'callback','set(gcbf,''userdata'',1102); uiresume;');
uicontrol(G.rhs_LineWidth) ;  set(G.rhs_LineWidth,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'HorizontalAlignment','left','Value',G.Settings.LineWidthIdx,'FontName',G.GUI.Fonts.uiButtonFont) ;
G.h_rhs_LineWidth = gco ;
% Coast Line Width
Line = 9.3 ;
h_CoastLineWidthT = uicontrol('style','text','Units','normalized','pos',[G.GUI.RHS_b_right,G.GUI.Hgui-G.GUI.Hedge-Line*G.GUI.RHparts,G.GUI.RHStextlength,G.GUI.RHbuttonHeight], 'string','Coastline');
uicontrol(h_CoastLineWidthT) ;  set(h_CoastLineWidthT,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'HorizontalAlignment','left','FontName',G.GUI.Fonts.uiHeaderFont) ;
Line = 10  ;
G.rhs_CoastLineWidth = uicontrol('style','popup','Units','normalized','pos',[G.GUI.RHS_b_right,G.GUI.Hgui-G.GUI.Hedge-Line*G.GUI.RHparts,G.GUI.RHStextlength,G.GUI.RHbuttonHeight],...
    'string','none | 0.5 | 1 | 1.5 | 2 | 2.5 | 3 | 3.5 | 4 | 4.5 | 5 | 5.5 | 6 | 6.5 | 7 | 7.5 | 8 | 8.5 | 9 | 9.5 | 10 ',  'callback','set(gcbf,''userdata'',1106); uiresume;');
uicontrol(G.rhs_CoastLineWidth) ;  set(G.rhs_CoastLineWidth,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'HorizontalAlignment','left','Value',G.Settings.CoastLineWidthIdx,'FontName',G.GUI.Fonts.uiButtonFont) ;
G.h_rhs_CoastLineWidth = gco ;
Line = 11 ;
G.rhs_Xgrid = uicontrol('style','checkbox','Units','normalized','pos',[G.GUI.RHS_b_right,G.GUI.Hgui-G.GUI.Hedge-Line*G.GUI.RHparts,G.GUI.RHStextlength,G.GUI.RHbuttonHeight], 'string','X-Grid',...
    'value',G.Settings.XgridFlag,'callback','set(gcbf,''userdata'',1103); uiresume;');
uicontrol(G.rhs_Xgrid) ;  set(G.rhs_Xgrid,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'HorizontalAlignment','left', ...
    'TooltipString','Display X Grid lines','FontName',G.GUI.Fonts.uiHeaderFont) ;
Line = 12 ;
G.rhs_Ygrid = uicontrol('style','checkbox','Units','normalized','pos',[G.GUI.RHS_b_right,G.GUI.Hgui-G.GUI.Hedge-Line*G.GUI.RHparts,G.GUI.RHStextlength,G.GUI.RHbuttonHeight], 'string','Y-Grid',...
    'value',G.Settings.YgridFlag,'callback','set(gcbf,''userdata'',1104); uiresume;');
uicontrol(G.rhs_Ygrid) ;  set(G.rhs_Ygrid,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'HorizontalAlignment','left', ...
    'TooltipString','Display Y Grid lines','FontName',G.GUI.Fonts.uiHeaderFont) ;
% Symmetric Z limit
Line = 13 ;
G.rhs_SymZ = uicontrol('style','checkbox','Units','normalized','pos',[G.GUI.RHS_b_right,G.GUI.Hgui-G.GUI.Hedge-Line*G.GUI.RHparts,G.GUI.RHStextlength,G.GUI.RHbuttonHeight], 'string','Sym-Z',...
    'value',G.Settings.SymZLimitsFlag,'callback','set(gcbf,''userdata'',1111); uiresume;');
uicontrol(G.rhs_SymZ) ;  set(G.rhs_SymZ,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'HorizontalAlignment','left', ...
    'TooltipString','Make Z limits Symmetric with respect to 0 (goes well with Rose, CeJulien, or BlueWhiteRed colormaps).','FontName',G.GUI.Fonts.uiHeaderFont) ;
Line = 14 ;
G.rhs_FixZ = uicontrol('style','checkbox','Units','normalized','pos',[G.GUI.RHS_b_right,G.GUI.Hgui-G.GUI.Hedge-Line*G.GUI.RHparts,G.GUI.RHStextlength,G.GUI.RHbuttonHeight],'string','Fix-Z',...
    'value',G.Settings.FixZLimitsFlag,'callback','set(gcbf,''userdata'',1105); uiresume;');
uicontrol(G.rhs_FixZ) ;  set(G.rhs_FixZ,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'HorizontalAlignment','left', ...
    'TooltipString','Fix Z limits (Helpful during multiple auto stepping)','FontName',G.GUI.Fonts.uiHeaderFont) ;
Line = 15 ;
G.rhs_LogLevels = uicontrol('style','checkbox','Units','normalized','pos',[G.GUI.RHS_b_right,G.GUI.Hgui-G.GUI.Hedge-Line*G.GUI.RHparts,G.GUI.RHStextlength,G.GUI.RHbuttonHeight], 'string','Log-Y',...
    'value',G.Settings.LogLevelsFlag,  'callback','set(gcbf,''userdata'',1109); uiresume;');
uicontrol(G.rhs_LogLevels) ;  set(G.rhs_LogLevels,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'HorizontalAlignment','left', ...
    'TooltipString','logarithmic Y ordinate','FontName',G.GUI.Fonts.uiHeaderFont) ;
Line = 16 ;
G.rhs_SinLat = uicontrol('style','checkbox','Units','normalized','pos',[G.GUI.RHS_b_right,G.GUI.Hgui-G.GUI.Hedge-Line*G.GUI.RHparts,G.GUI.RHStextlength,G.GUI.RHbuttonHeight], 'string','Sin(lat)',...
    'value',G.Settings.SinLatFlag,  'callback','set(gcbf,''userdata'',1120); uiresume;');
uicontrol(G.rhs_SinLat) ;  set(G.rhs_SinLat,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'HorizontalAlignment','left', ...
    'TooltipString','Area weighted (sin(latitude)) ordinate','FontName',G.GUI.Fonts.uiHeaderFont) ;
Line = 17 ;
G.rhs_hold = uicontrol('style','checkbox','Units','normalized','pos',[G.GUI.RHS_b_right,G.GUI.Hgui-G.GUI.Hedge-Line*G.GUI.RHparts,G.GUI.RHStextlength,G.GUI.RHbuttonHeight],'string','HOLD',...
    'value',G.Settings.HoldFlag,'callback','set(gcbf,''userdata'',1200); uiresume;');
datatip = sprintf(['Hold current plot and data. ', ...
    '\nIf HOLD-DIFF is not selected,', ...
    '\nConsecutive plots will overlay with grey contours.']) ;
uicontrol(G.rhs_hold) ;  set(G.rhs_hold,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'HorizontalAlignment','left', ...
    'TooltipString',datatip,'FontName',G.GUI.Fonts.uiHeaderFont,'fontweight','bold') ;
Line = 18 ;
G.rhs_holddif = uicontrol('style','checkbox','Units','normalized','pos',[G.GUI.RHS_b_right+G.GUI.RHStextlength/8,G.GUI.Hgui-G.GUI.Hedge-Line*G.GUI.RHparts,G.GUI.RHStextlength*7/8,G.GUI.RHbuttonHeight],'string','DIFF',...
    'value',G.Settings.HoldDifFlag,'callback','set(gcbf,''userdata'',1300); uiresume;');
datatip = sprintf(['Show the difference between present analysis and held data.', ...
    '\nNote:   Not supported in Hovmoller projections']) ;
uicontrol(G.rhs_holddif) ;  set(G.rhs_holddif,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'HorizontalAlignment','left', ...
    'TooltipString',datatip,'FontName',G.GUI.Fonts.uiHeaderFont,'fontweight','bold') ;
if G.Settings.HoldFlag
    set(G.rhs_holddif,'Visible','on')
else
    G.Settings.HoldDifFlag = 0 ;
    set(G.rhs_holddif,'Visible','off')
end
% Line = G.GUI.NumLines ;
% G.h_Exit = uicontrol('Units','normalized','pos',[1-G.GUI.b_left-G.GUI.TextLength/3,G.GUI.Hgui-G.GUI.Hedge-Line*G.GUI.Hparts,G.GUI.TextLength/3,G.GUI.HbuttonHeight], 'string','Exit',  'callback','set(gcbf,''userdata'',999); uiresume;');
% uicontrol(G.h_Exit) ;  set(G.h_Exit,'BackgroundColor',G.GUI.ButtonColor,'ForegroundColor',[.5 .1 .1],'FontSize',G.Display.TextSize_M,'TooltipString','Exit' ,'FontName',G.GUI.Fonts.uiButtonFont) ;
if G.Settings.KeyPressFlag
    %set(G.GOAT_Fig,'KeyPressFcn',{@GOATKeyPress,G},'WindowKeyPressFcn',{@GOATKeyPress,G}) ;
    set(G.GOAT_Fig,'KeyPressFcn',{@GOATKeyPress,G},'WindowKeyPressFcn',{@GOATKeyPress,G},'KeyReleaseFcn',{@GOATKeyRelease,G},'WindowKeyReleaseFcn',{@GOATKeyRelease,G}) ;
end
uicontrol(G.h_DataSelectionT) ;
% Analyze Button
function G = Analyze_Data(G)
set(G.h_Analyze,'Enable','off')  ; drawnow ; % Disable GO button
if ~G.StepGUIFlag
    G = GOAT_Limits(G) ;
end
drawnow ;
if ~G.continueFlag
    G = GOAT_Compile_data_Series(G) ;
    if G.Settings.HoldDifFlag
        G = Analyze_Diff(G) ;
    end
    G.message = dateMessage(G) ;
    G.continueFlag = 0 ;
end
try
set(G.h_Analyze,'Enable','on')  ; drawnow ; % re-enable GO button
catch
    % nevermind. Possibly the GOAT GUI was closed after get m code
end
% Settings Button
function G = Settings(G)
pos        = G.GUI.pos.Settings ;
G.Settings_Fig = figure('name','GOAT Settings','position',pos) ;
NumLines = 10 ;
bl = 0.05              ; % Left Edge
bt = 0.05              ; % Top Edge
lh = (1-2*bt)/NumLines ; % Line Height
cl = (1-4*bl)/4        ; % column width (a total of 5 columns)
set(G.Settings_Fig,'Color',G.Display.FigBackgroudColor) ;
set(G.Settings_Fig,'MenuBar','none')
LinksStr = '' ;
for i = 1:length(G.Settings.functionLinks)
    LinksStr = [LinksStr,cell2mat(G.Settings.functionLinksTitle(i)),' |'] ;
end
try
    for i = 1:length(G.Settings.webLinks)
        LinksStr = [LinksStr,cell2mat(G.Settings.webLinksTitle(i)),' |'] ;
    end
catch
    % nevermind, No web links
end
LinksStr = LinksStr(1:end) ;
% Display Columm
Line = 1 ;
h_Text1 = uicontrol('Style', 'text','Units','normalized','pos',PosFunc(bl,bt,lh,cl,1,Line), 'string','Plotting');
uicontrol(h_Text1) ;  set(h_Text1,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',G.GUI.HeaderColor,'FontSize',G.Display.TextSize_L,'HorizontalAlignment','left','FontName',G.GUI.Fonts.uiHeaderFont) ;
% topography
Line = Line + 1 ;
h_topographyT = uicontrol('style','text','Units','normalized','pos',PosFunc(bl,bt,lh,cl,1,Line), 'string','Topography');
uicontrol(h_topographyT) ;  set(h_topographyT,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_M,'HorizontalAlignment','left','FontName',G.GUI.Fonts.uiHeaderFont) ;
h_topography = uicontrol('style','popup','Units','normalized','pos',PosFunc(bl,bt,lh,cl,2,Line),...
    'string','None | Atmosphere | Ocean | All ',  'callback','set(gcbf,''userdata'',171); uiresume;');
uicontrol(h_topography) ;  set(h_topography,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'HorizontalAlignment','left','Value',G.topo.TypeFlag,'FontName',G.GUI.Fonts.uiButtonFont) ;
Line = Line + 1 ;
h_topointT = uicontrol('style','text','Units','normalized','pos',PosFunc(bl,bt,lh,cl,1,Line), 'string','Topography interval (m)');
uicontrol(h_topointT) ;  set(h_topointT,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_M,'HorizontalAlignment','left','FontName',G.GUI.Fonts.uiHeaderFont) ;
h_topoint = uicontrol('style','edit','Units','normalized','pos',PosFunc(bl,bt,lh,cl,2,Line),...
    'string',num2str(G.topo.interval,4),  'callback','set(gcbf,''userdata'',172); uiresume;');
uicontrol(h_topoint) ;  set(h_topoint,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'HorizontalAlignment','left','FontName',G.GUI.Fonts.uiButtonFont) ;
Line = Line + 1 ;
h_topolineT = uicontrol('style','text','Units','normalized','pos',PosFunc(bl,bt,lh,cl,1,Line), 'string','Topography linewidth');
uicontrol(h_topolineT) ;  set(h_topolineT,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_M,'HorizontalAlignment','left','FontName',G.GUI.Fonts.uiHeaderFont) ;
h_topoline = uicontrol('style','popup','Units','normalized','pos',PosFunc(bl,bt,lh,cl,2,Line),...
    'string','0.5 | 1 | 1.5 | 2 | 2.5 | 3 | 3.5 | 4 | 4.5 | 5 | 5.5 | 6 | 6.5 | 7 | 7.5 | 8 | 8.5 | 9 | 9.5 | 10 ',  'callback','set(gcbf,''userdata'',173); uiresume;');
uicontrol(h_topoline) ;  set(h_topoline,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'HorizontalAlignment','left','Value',G.topo.TopotLineWidthIdx,'FontName',G.GUI.Fonts.uiButtonFont) ;
Line = Line + 1 ;
h_ZbufferT = uicontrol('style','text','Units','normalized','pos',PosFunc(bl,bt,lh,cl,1,Line), 'string','Fix-Z Buffer (%)');
uicontrol(h_ZbufferT) ;  set(h_ZbufferT,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_M,'HorizontalAlignment','left', ...
    'TooltipString','Z limit buffer in proportion to Abs(max(data))','FontName',G.GUI.Fonts.uiHeaderFont) ;
h_Zbuffer = uicontrol('style','edit','Units','normalized','pos',PosFunc(bl,bt,lh,cl,2,Line),...
    'string',num2str(G.Settings.ZlimitBuffer*100,3),  'callback','set(gcbf,''userdata'',108); uiresume;');
uicontrol(h_Zbuffer) ;  set(h_Zbuffer,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'HorizontalAlignment','left', ...
    'TooltipString','Z limit buffer in proportion to Abs(max(data))','FontName',G.GUI.Fonts.uiButtonFont) ;
% Smoothed Climatology
Line = 7 ;
h_smoothClimT = uicontrol('style','text','Units','normalized','pos',PosFunc(bl,bt,lh,cl,1,Line), 'string','Smooth Climatology');
uicontrol(h_smoothClimT) ;  set(h_smoothClimT,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_M,'HorizontalAlignment','left','FontName',G.GUI.Fonts.uiHeaderFont) ;
if strcmp(G.TimeField,'Monthly')
    smoothStr = 'None|3|5|7|9|11' ;
else
    smoothStr = 'None|3|5|7|9|11|13|15|17|19|21|23|25|27|29|31' ;
end
h_smoothClim = uicontrol('style','popup','Units','normalized','pos',PosFunc(bl,bt,lh,cl,2,Line),...
    'string',smoothStr,  'callback','set(gcbf,''userdata'',199); uiresume;','value',(G.Limits.Default.DataClimSmooth+1)/2);
uicontrol(h_smoothClim) ;  set(h_smoothClim,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'HorizontalAlignment','left','FontName',G.GUI.Fonts.uiButtonFont) ;
% Nan Mean Flag
Line = 8 ;
h_NanMean = uicontrol('style','checkbox','Units','normalized','pos',PosFunc(bl,bt,lh,cl,1,Line),'string','Use Nanmean',...
    'value',G.Settings.NanMeanFlag,'callback','set(gcbf,''userdata'',204); uiresume;');
uicontrol(h_NanMean) ;  set(h_NanMean,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_M,'HorizontalAlignment','left', ...
    'TooltipString','When ''On'', nanmean(x) is used to average data instead of mean(x)','FontName',G.GUI.Fonts.uiHeaderFont) ;
% RHS  General Column
Line = 2 ;
h_Text2 = uicontrol('Style', 'text','Units','normalized','pos',PosFunc(bl,bt,lh,cl,3,Line), 'string','General');
uicontrol(h_Text2) ;  set(h_Text2,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',G.GUI.HeaderColor,'FontSize',G.Display.TextSize_L,'HorizontalAlignment','left','FontName',G.GUI.Fonts.uiHeaderFont) ;
h_General = uicontrol('style','popup','Units','normalized','pos',[bl*3 + cl*3,1-bt-Line*lh + lh*.1,cl*1 ,lh*.9],...
    'string','GUIDE | Update GOAT | Edit Constants ','value',1,'callback','set(gcbf,''userdata'',300); uiresume;');
uicontrol(h_General) ;  set(h_General,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'HorizontalAlignment','left', ...
    'TooltipString','General actions','FontName',G.GUI.Fonts.uiButtonFont) ;

% Line = 2 ; % ,
% h_UpdateT = uicontrol('Units','normalized','pos',[bl*3 + cl*3,1-bt-Line*lh + lh*.1,cl*1 ,lh*.9], 'string','Update GOAT');
% uicontrol(h_UpdateT) ;  set(h_UpdateT,'BackgroundColor',[.85 .85 .85],'ForegroundColor',G.GUI.DropTextColor,'FontSize',G.Display.TextSize_XL,'HorizontalAlignment','left','fontweight','bold', ...
%     'TooltipString','Update to the latest version. Warning: this will close this session of GOAT.',  'callback','set(gcbf,''userdata'',400); uiresume;','FontName',G.GUI.Fonts.uiButtonFont) ;
% GOAT GUIDE
% Line = 1 ; % [bl*3 + cl*2,1-bt-Line*lh + lh*.1,cl*2 ,lh*.9]
% h_GuideT = uicontrol('Units','normalized','pos',[bl*3 + cl*3,1-bt-Line*lh + lh*.1,cl*1 ,lh*.9], 'string','GOAT GUIDE');
% uicontrol(h_GuideT) ;  set(h_GuideT,'BackgroundColor',G.GUI.colors.celeste,'ForegroundColor',G.GUI.DropTextColor,'FontSize',G.Display.TextSize_XL,'HorizontalAlignment','left','fontweight','bold', ...
%     'TooltipString','open GOAT_GUIDE.pdf',  'callback','set(gcbf,''userdata'',300); uiresume;','FontName',G.GUI.Fonts.uiButtonFont) ;
% Update Consts
% Line = 4 ; % [bl*3 + cl*3,1-bt-Line*lh + lh*.1,cl*1 ,lh*.9]
% h_ConstsT = uicontrol('Units','normalized','pos',[bl*3 + cl*3,1-bt-Line*lh + lh*.1,cl*1 ,lh*.9], 'string','Edit Constants');
% uicontrol(h_ConstsT) ;  set(h_ConstsT,'BackgroundColor',[.85 .85 .85],'ForegroundColor',G.GUI.DropTextColor,'FontSize',G.Display.TextSize_M,'HorizontalAlignment','left', ...
%     'TooltipString','Change Default Consts (g, Cp, etc...)',  'callback','set(gcbf,''userdata'',206); uiresume;','FontName',G.GUI.Fonts.uiButtonFont) ;
% % WaitbarCutoff
% Line = 3 ;
% h_WaitbarCutoffT = uicontrol('style','text','Units','normalized','pos',PosFunc(bl,bt,lh,cl,3,Line), 'string','Display Progress Cutoff');
% uicontrol(h_WaitbarCutoffT) ;  set(h_WaitbarCutoffT,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_M,'HorizontalAlignment','left', ...
%     'TooltipString','The Number of Data Loadings above which the Waitbar appears','FontName',G.GUI.Fonts.uiHeaderFont) ;
% h_WaitbarCutoff = uicontrol('style','edit','Units','normalized','pos',PosFunc(bl,bt,lh,cl,4,Line),...
%     'string',int2str(G.Settings.WaitbarCutoff),  'callback','set(gcbf,''userdata'',201); uiresume;');
% uicontrol(h_WaitbarCutoff) ;  set(h_WaitbarCutoff,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'HorizontalAlignment','left', ...
%     'TooltipString','The Number of Data Loadings above which the waitbar appears','FontName',G.GUI.Fonts.uiButtonFont) ;
% TimeStep Pause Interval
Line = 5 ;
h_TimeStepPauseT = uicontrol('style','text','Units','normalized','pos',PosFunc(bl,bt,lh,cl,3,Line), 'string','TimeStep Pause (sec)');
uicontrol(h_TimeStepPauseT) ;  set(h_TimeStepPauseT,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_M,'HorizontalAlignment','left', ...
    'TooltipString','Mininal Pause (sec) Between Auto TimeStep Frames','FontName',G.GUI.Fonts.uiHeaderFont) ;
h_TimeStepPause = uicontrol('style','edit','Units','normalized','pos',[bl*3 + cl*3,1-bt-Line*lh + lh*.1,cl*1 ,lh*.9],...
    'string',num2str(G.Settings.TimeStepPause,3),  'callback','set(gcbf,''userdata'',202); uiresume;');
uicontrol(h_TimeStepPause) ;  set(h_TimeStepPause,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'HorizontalAlignment','left', ...
    'TooltipString','Mininal Pause (sec) Between Auto TimeStep Frames','FontName',G.GUI.Fonts.uiButtonFont) ;

% P or Height Levels
Line = 6 ;
h_LevelsT = uicontrol('style','text','Units','normalized','pos',PosFunc(bl,bt,lh,cl,3,Line), 'string','Vertical Level Type');
uicontrol(h_LevelsT) ;  set(h_LevelsT,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_M,'HorizontalAlignment','left', ...
    'TooltipString','Vertical ordinate type','FontName',G.GUI.Fonts.uiHeaderFont) ;
h_Levels = uicontrol('style','popup','Units','normalized','pos',[bl*3 + cl*3,1-bt-Line*lh + lh*.1,cl*1 ,lh*.9],...
    'string','Pressure (mb) | Depth (m) | Height (m) | Potential Temperature (K) | Sigma | Geometric-Level (increasing with height) | Level (decreasing with height)','value',G.Settings.LevelsIdx,'callback','set(gcbf,''userdata'',205); uiresume;');
uicontrol(h_Levels) ;  set(h_Levels,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'HorizontalAlignment','left', ...
    'TooltipString','Vertical ordinate type','FontName',G.GUI.Fonts.uiButtonFont) ;
% Links
Line = 8 ;
h_LinksT = uicontrol('style','text','Units','normalized','pos',PosFunc(bl,bt,lh,cl,3,Line), 'string','Links');
uicontrol(h_LinksT) ;  set(h_LinksT,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_M,'HorizontalAlignment','left', ...
    'TooltipString','Useful web and info links','FontName',G.GUI.Fonts.uiHeaderFont) ;
h_Links = uicontrol('style','popup','Units','normalized','pos',[bl*3 + cl*3,1-bt-Line*lh + lh*.1,cl*1 ,lh*.9],...
    'string',LinksStr,'value',1,'callback','set(gcbf,''userdata'',210); uiresume;');
uicontrol(h_Links) ;  set(h_Links,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'HorizontalAlignment','left', ...
    'TooltipString','Useful web and info links','FontName',G.GUI.Fonts.uiButtonFont) ;
Line = NumLines ;
h_OK = uicontrol('Units','normalized','pos',[bl/2,1-bt-Line*lh,cl*4 + bl*3,lh*1.1], 'string','OK',  'callback','set(gcbf,''userdata'',999); uiresume;');
uicontrol(h_OK) ;  set(h_OK,'BackgroundColor',G.GUI.colors.celeste,'ForegroundColor',G.GUI.DropTextColor,'FontSize',16,'fontweight','bold','FontName',G.GUI.Fonts.uiButtonFont,'TooltipString','keypress = ''g''') ;

if G.Settings.KeyPressFlag
    set(G.Settings_Fig,'KeyPressFcn',{@SETTINGSKeyPress,G},'WindowKeyPressFcn',{@SETTINGSKeyPress,G} )
end

GUIcon = 1 ;
while GUIcon
    try
        set(0,'CurrentFigure',G.Settings_Fig) ;
        uiwait(gcf);
        select = get(gcf,'userdata');
    catch
        GUIcon = 0 ;
    end
    try
        switch select
            case 101 % ColorMap
                G.Settings.ColorMapIdx = get(h_ColorMap,'Value') ;
            case 102 % LineWidth
                G.Settings.LineWidthIdx = get(h_LineWidth,'Value') ;
                G.Settings.LineWidth = (G.Settings.LineWidthIdx-1)/2 ;
            case 103
                G.Settings.XgridFlag = get(h_Xgrid,'value') ;
            case 104
                G.Settings.YgridFlag = get(h_Ygrid,'value') ;
            case 105
                G.Settings.FixZLimitsFlag = get(h_FixZ,'value') ;
                if G.Settings.FixZLimitsFlag && G.Settings.SymZLimitsFlag
                    AbsMax = max([abs(G.Settings.ZlimitMin),G.Settings.ZlimitMax]) ;
                    G.Settings.ZlimitMin = G.Settings.ZlimitMin - G.Settings.ZlimitBuffer*AbsMax ;
                    G.Settings.ZlimitMax = G.Settings.ZlimitMax + G.Settings.ZlimitBuffer*AbsMax ;
                else
                    try
                        MinZ = min(G.Data(:));
                        MaxZ = max(G.Data(:));
                        AbsMax = max([abs(MinZ) , abs(MaxZ)]) ;
                        G.Settings.ZlimitMin = MinZ - G.Settings.ZlimitBuffer*AbsMax ;
                        G.Settings.ZlimitMax = MaxZ + G.Settings.ZlimitBuffer*AbsMax;
                    catch
                        set(G.h_message,'string','Error: Could not obtain Z limits!') ; %
                    end
                end
            case 106 % Coast LineWidth
                G.Settings.CoastLineWidthIdx = get(h_CoastLineWidth,'Value') ;
                G.Settings.CoastLineWidth = G.Settings.CoastLineWidthIdx/2 ;
            case 107 % Contouring
                G.Settings.ContourIdx = get(h_Contour,'Value') ;
            case 108 % Zlimit Buffer
                G.Settings.ZlimitBuffer =  str2double(get(h_Zbuffer,'string'))/100 ;
                if G.Settings.FixZLimitsFlag && G.Settings.SymZLimitsFlag
                    AbsMax = max([abs(G.Settings.ZlimitMin),G.Settings.ZlimitMax]) ;
                    G.Settings.ZlimitMin = G.Settings.ZlimitMin - G.Settings.ZlimitBuffer*AbsMax ;
                    G.Settings.ZlimitMax = G.Settings.ZlimitMax + G.Settings.ZlimitBuffer*AbsMax ;
                elseif G.Settings.FixZLimitsFlag
                    try
                        MinZ = min(G.Data(:));
                        MaxZ = max(G.Data(:));
                        AbsMax = max([abs(MinZ) , abs(MaxZ)]) ;
                        G.Settings.ZlimitMin = MinZ - G.Settings.ZlimitBuffer*AbsMax ;
                        G.Settings.ZlimitMax = MaxZ + G.Settings.ZlimitBuffer*AbsMax;
                    catch
                        set(G.h_message,'string','Error: Could not obtain Z limits!') ; %
                    end
                end
            case 109 % LogLevles
                G.Settings.LogLevelsFlag =  get(h_LogLevels,'value') ;
            case 110 % Contour Step Level
                try
                    ContourLevelStep = str2double(get(h_StepLevel,'string')) ;
                    MinZ = min(G.Data(:));
                    MaxZ = max(G.Data(:));
                    if isnan(ContourLevelStep)
                        G.Settings.ContourLevelStep = 0 ;
                        set(h_StepLevel,'string',num2str(G.Settings.ContourLevelStep,5))    ;
                    elseif  ContourLevelStep > (MaxZ - MinZ)/100 || ContourLevelStep == 0
                        G.Settings.ContourLevelStep = ContourLevelStep ;
                    else
                        set(h_StepLevel,'string',num2str(G.Settings.ContourLevelStep,5))    ;
                    end
                catch
                    set(h_StepLevel,'string',num2str(G.Settings.ContourLevelStep,5))    ;
                end
            case 111
                G.Settings.SymZLimitsFlag = get(h_SymZ,'value') ;
                try
                    MinZ = min(min(G.Data));
                    MaxZ = max(max(G.Data));
                    AbsMax = max([abs(MinZ) , abs(MaxZ)]) ;
                    G.Settings.ZlimitMin = - AbsMax ;
                    G.Settings.ZlimitMax =   AbsMax ;
                catch
                    set(G.h_message,'string','Error: Could not obtain Z limits!') ; %
                end
            case 171
                G.topo.TypeFlag = get(h_topography,'value') ;
                switch G.topo.TypeFlag
                    case {1,2}
                        G.topo.intervalmin = 0 ;
                        G.topo.intervalmax = 8000 ;
                    case 3
                        G.topo.intervalmin = -8000 ;
                        G.topo.intervalmax =  0 ;
                    case 4
                        G.topo.intervalmin = -8000 ;
                        G.topo.intervalmax = 8000 ;
                end
                G.topo.topovec = G.topo.intervalmin:G.topo.interval:G.topo.intervalmax ;
                zeroidx = dsearchn(G.topo.topovec',0) ;
                G.topo.topovec = G.topo.topovec(G.topo.topovec~=G.topo.topovec(zeroidx))  ;
            case 172
                G.topo.interval = round(str2double(get(h_topoint,'string'))) ;
                switch G.topo.TypeFlag
                    case {1,2}
                        G.topo.intervalmin = 0 ;
                        G.topo.intervalmax = 8000 ;
                    case 3
                        G.topo.intervalmin = -8000 ;
                        G.topo.intervalmax =  0 ;
                    case 4
                        G.topo.intervalmin = -8000 ;
                        G.topo.intervalmax = 8000 ;
                end
                G.topo.topovec = G.topo.intervalmin:G.topo.interval:G.topo.intervalmax ;
                zeroidx = dsearchn(G.topo.topovec',0) ;
                G.topo.topovec = G.topo.topovec(G.topo.topovec~=G.topo.topovec(zeroidx))  ;
            case 173
                G.topo.TopotLineWidthIdx = get(h_topoline,'value') ;
            case 199
                G.Limits.Default.DataClimSmooth = get(h_smoothClim,'value')*2-1 ;
            case 201 % Waitbar appearance # of loaded files cutoff
                G.Settings.WaitbarCutoff = round(str2double(get(h_WaitbarCutoff,'string'))) ;
            case 202 % Auto Time Stepping minimal Pause
                G.Settings.TimeStepPause = str2double(get(h_TimeStepPause,'string')) ;
            case 204 % NanMean Flag
                G.Settings.NanMeanFlag = get(h_NanMean,'value') ;
            case 205 % Levels Flag
                G.Settings.LevelsIdx = get(h_Levels,'value') ;
            case 206 % Update Consts
                try % BM1 Settings BookMark
                    prompt = {'Earth''s Radius [m]                 ' ; ...
                        'Gravitational Const. [m^2/s^2]      ' ; ...
                        'Rotational Frequency (Omega) [1/s]  ' ; ...
                        'Specific Heat at Const. Pr. [J/m^2K]' ; ...
                        'Moist Specific Heat at Const. Pr.   ' ; ...
                        'Latent Heat of Vaporization  [J/kg] ' ; ...
                        'Ideal Dry Gas Constant [J/kgK]      ' ; ...
                        'Ideal vapor Gas Constant [J/kgK]    ' } ;
                    
                    dlg_title = 'Default Consts'                   ;
                    num_lines = 1                                                   ;
                    def = {num2str(G.Consts.EarthRadius),num2str(G.Consts.GravitationalConst),num2str(G.Consts.Omega), ...
                        num2str(G.Consts.SpHeatConstPr),num2str(G.Consts.SpHeatConstPrMoist),num2str(G.Consts.LatentHeatOfVapor), ...
                        num2str(G.Consts.IdealGasRdry),num2str(G.Consts.IdealGasRvapor)}             ;  % Default values
                    options.Resize = 'on' ;
                    options.Interpreter = 'none' ;
                    options.WindowStyle = 'normal' ;
                    answer = inputdlg(prompt,dlg_title,num_lines,def,options)               ;
                    G.Consts.EarthRadius        = eval(cell2mat(answer(1,1))) ; %
                    G.Consts.GravitationalConst = eval(cell2mat(answer(2,1))) ; %
                    G.Consts.Omega              = eval(cell2mat(answer(3,1))) ; %
                    G.Consts.SpHeatConstPr      = eval(cell2mat(answer(4,1))) ; %
                    G.Consts.SpHeatConstPrMoist = eval(cell2mat(answer(5,1))) ; %
                    G.Consts.LatentHeatOfVapor  = eval(cell2mat(answer(6,1))) ; %
                    G.Consts.IdealGasRdry       = eval(cell2mat(answer(7,1))) ; %
                    G.Consts.IdealGasRvapor     = eval(cell2mat(answer(8,1))) ; %
                catch
                    % Nevermind, Keep Defaults
                end
            case 210 % Links
                LinkIdx = get(h_Links,'value') ;
                if LinkIdx <= length(G.Settings.functionLinks)
                    eval(cell2mat(G.Settings.functionLinks(LinkIdx))) ;
                else
                    web(cell2mat(G.Settings.webLinks(LinkIdx - length(G.Settings.functionLinks))),'-browser') ;
                end
            case 300 % General
                switch get(h_General,'value')
                    case 1 % GOAT GUIDE
                        try
                            GOAT_GUIDE_Address = fullfile('..','Documentation','GOAT_GUIDE.pdf') ;
                            open(GOAT_GUIDE_Address) ;
                        catch
                            disp(['Could not open ',GOAT_GUIDE_Address]) ;
                        end
                    case 2 % GOAT Update
                        Quest = sprintf(['Note:', ...
                            '\nThis requires closing the current GOAT session.', ...
                            ]) ;
                        button = questdlg(Quest,'GOAT UPDATE','Continue','Cancel','Cancel') ;
                        if strcmp(button,'Continue')
                            close all ;
                            GOAT_VersionUpdateCode ;
                        else
                            continue ;
                        end
                    case 3 % Edit Constants
                        try
                            prompt = {'Earth''s Radius [m]                 ' ; ...
                                'Gravitational Const. [m^2/s^2]      ' ; ...
                                'Rotational Frequency (Omega) [1/s]  ' ; ...
                                'Specific Heat at Const. Pr. [J/m^2K]' ; ...
                                'Moist Specific Heat at Const. Pr.   ' ; ...
                                'Latent Heat of Vaporization  [J/kg] ' ; ...
                                'Ideal Dry Gas Constant [J/kgK]      ' ; ...
                                'Ideal vapor Gas Constant [J/kgK]    ' } ;
                            
                            dlg_title = 'Default Consts'                   ;
                            num_lines = 1                                                   ;
                            def = {num2str(G.Consts.EarthRadius),num2str(G.Consts.GravitationalConst),num2str(G.Consts.Omega), ...
                                num2str(G.Consts.SpHeatConstPr),num2str(G.Consts.SpHeatConstPrMoist),num2str(G.Consts.LatentHeatOfVapor), ...
                                num2str(G.Consts.IdealGasRdry),num2str(G.Consts.IdealGasRvapor)}             ;  % Default values
                            options.Resize = 'on' ;
                            options.Interpreter = 'none' ;
                            options.WindowStyle = 'normal' ;
                            answer = inputdlg(prompt,dlg_title,num_lines,def,options)               ;
                            G.Consts.EarthRadius        = eval(cell2mat(answer(1,1))) ; %
                            G.Consts.GravitationalConst = eval(cell2mat(answer(2,1))) ; %
                            G.Consts.Omega              = eval(cell2mat(answer(3,1))) ; %
                            G.Consts.SpHeatConstPr      = eval(cell2mat(answer(4,1))) ; %
                            G.Consts.SpHeatConstPrMoist = eval(cell2mat(answer(5,1))) ; %
                            G.Consts.LatentHeatOfVapor  = eval(cell2mat(answer(6,1))) ; %
                            G.Consts.IdealGasRdry       = eval(cell2mat(answer(7,1))) ; %
                            G.Consts.IdealGasRvapor     = eval(cell2mat(answer(8,1))) ; %
                        catch
                            % Nevermind, Keep Defaults
                        end
                end
            case 999 % OK
                GUIcon = 0 ;
        end
    catch
        %nevermind
    end
end

try
    close(G.Settings_Fig) ;
catch
    % Nevermind
end

return ;
% KeyPressFunction
function GOATKeyRelease(~,event,G)
try
    switch  event.Key
        case {'control','ctrl'}
            uicontrol(G.h_DataSelectionT) ; % Place focus on data dropdown menu
            set(gcf,'userdata',0) ;
            return ;
    end
catch
    % disp 'keyrelease error!'
end
% KeyPressFunction
function GOATKeyPress(~,event,G)
try
    if verLessThan('matlab','8.4.0')
    switch event.Character
        case 'g'
            %if ~any(gco == ([G.h_DataSelection_handle,G.h_Projection_handle,G.h_TempRes_handle,G.h_Field_handle,G.h_rhs_Contour,G.h_rhs_ColorMap,G.h_rhs_LineWidth,G.h_rhs_CoastLineWidth]))
            if ~any(gco == ([G.h_DataSelection_handle,G.h_Projection_handle,G.h_TempRes_handle,G.h_Field_handle,G.h_rhs_ColorMap,G.h_rhs_LineWidth,G.h_rhs_CoastLineWidth]))
                set(gcf,'userdata',11) ;
                uiresume ;
                return ;
            end
        case 'x'
            %if ~any(gco == ([G.h_DataSelection_handle,G.h_Projection_handle,G.h_TempRes_handle,G.h_Field_handle,G.h_rhs_Contour,G.h_rhs_ColorMap,G.h_rhs_LineWidth,G.h_rhs_CoastLineWidth]))
            if ~any(gco == ([G.h_DataSelection_handle,G.h_Projection_handle,G.h_TempRes_handle,G.h_Field_handle,G.h_rhs_ColorMap,G.h_rhs_LineWidth,G.h_rhs_CoastLineWidth]))
                set(gcf,'userdata',900) ;
                uiresume ;
                return ;
            end
        case 's'
            %if ~any(gco == ([G.h_DataSelection_handle,G.h_Projection_handle,G.h_TempRes_handle,G.h_Field_handle,G.h_rhs_Contour,G.h_rhs_ColorMap,G.h_rhs_LineWidth,G.h_rhs_CoastLineWidth]))
            if ~any(gco == ([G.h_DataSelection_handle,G.h_Projection_handle,G.h_TempRes_handle,G.h_Field_handle,G.h_rhs_ColorMap,G.h_rhs_LineWidth,G.h_rhs_CoastLineWidth]))
                set(gcf,'userdata',901) ;
                uiresume ;
                return ;
            end
        case 'f'
            %if ~any(gco == ([G.h_DataSelection_handle,G.h_Projection_handle,G.h_TempRes_handle,G.h_Field_handle,G.h_rhs_Contour,G.h_rhs_ColorMap,G.h_rhs_LineWidth,G.h_rhs_CoastLineWidth]))
            if ~any(gco == ([G.h_DataSelection_handle,G.h_Projection_handle,G.h_TempRes_handle,G.h_Field_handle,G.h_rhs_ColorMap,G.h_rhs_LineWidth,G.h_rhs_CoastLineWidth]))
                set(gcf,'userdata',50) ;
                uiresume ;
                return ;
            end
        case 'd'
            %if ~any(gco == ([G.h_DataSelection_handle,G.h_Projection_handle,G.h_TempRes_handle,G.h_Field_handle,G.h_rhs_Contour,G.h_rhs_ColorMap,G.h_rhs_LineWidth,G.h_rhs_CoastLineWidth]))
            if ~any(gco == ([G.h_DataSelection_handle,G.h_Projection_handle,G.h_TempRes_handle,G.h_Field_handle,G.h_rhs_ColorMap,G.h_rhs_LineWidth,G.h_rhs_CoastLineWidth]))
                uicontrol(G.h_DataSelection) ; % Place focus on data dropdown menu
                set(gcf,'userdata',1) ;
            end
        case 't'
            %if ~any(gco == ([G.h_DataSelection_handle,G.h_Projection_handle,G.h_TempRes_handle,G.h_Field_handle,G.h_rhs_Contour,G.h_rhs_ColorMap,G.h_rhs_LineWidth,G.h_rhs_CoastLineWidth]))
            if ~any(gco == ([G.h_DataSelection_handle,G.h_Projection_handle,G.h_TempRes_handle,G.h_Field_handle,G.h_rhs_ColorMap,G.h_rhs_LineWidth,G.h_rhs_CoastLineWidth]))
                uicontrol(G.h_TempRes) ; % Place focus on data dropdown menu
                set(gcf,'userdata',300) ;
            end
        case 'b'
            %if ~any(gco == ([G.h_DataSelection_handle,G.h_Projection_handle,G.h_TempRes_handle,G.h_Field_handle,G.h_rhs_Contour,G.h_rhs_ColorMap,G.h_rhs_LineWidth,G.h_rhs_CoastLineWidth]))
            if ~any(gco == ([G.h_DataSelection_handle,G.h_Projection_handle,G.h_TempRes_handle,G.h_Field_handle,G.h_rhs_ColorMap,G.h_rhs_LineWidth,G.h_rhs_CoastLineWidth]))
                uicontrol(G.h_Field) ; % Place focus on data dropdown menu
                set(gcf,'userdata',200) ;
            end
        case 'p'
            %if ~any(gco == ([G.h_DataSelection_handle,G.h_Projection_handle,G.h_TempRes_handle,G.h_Field_handle,G.h_rhs_Contour,G.h_rhs_ColorMap,G.h_rhs_LineWidth,G.h_rhs_CoastLineWidth]))
            if ~any(gco == ([G.h_DataSelection_handle,G.h_Projection_handle,G.h_TempRes_handle,G.h_Field_handle,G.h_rhs_ColorMap,G.h_rhs_LineWidth,G.h_rhs_CoastLineWidth]))
                uicontrol(G.h_Projection) ; % Place focus on data dropdown menu
                set(gcf,'userdata',100) ;
            end
        case 'q'
            %if ~any(gco == ([G.h_DataSelection_handle,G.h_Projection_handle,G.h_TempRes_handle,G.h_Field_handle,G.h_rhs_Contour,G.h_rhs_ColorMap,G.h_rhs_LineWidth,G.h_rhs_CoastLineWidth]))
            if ~any(gco == ([G.h_DataSelection_handle,G.h_Projection_handle,G.h_TempRes_handle,G.h_Field_handle,G.h_rhs_ColorMap,G.h_rhs_LineWidth,G.h_rhs_CoastLineWidth]))
                try
                    close(G.GOAT_Fig) ; % Place focus on data dropdown menu
                catch
                    try
                        close(gcf)
                    catch
                        % Nevermind
                    end
                end
            end
    end
    else  % version >= 2014b
      switch event.Character
        case 'g'
            %if ~any(gco == ([G.h_DataSelection_handle,G.h_Projection_handle,G.h_TempRes_handle,G.h_Field_handle,G.h_rhs_Contour,G.h_rhs_ColorMap,G.h_rhs_LineWidth,G.h_rhs_CoastLineWidth]))
            if ~(isequal(gco,G.h_DataSelection_handle) || isequal(gco,G.h_Projection_handle) || isequal(gco,G.h_TempRes_handle)  || isequal(gco,G.h_Field_handle)  || isequal(gco,G.h_rhs_ColorMap)  || isequal(gco,G.h_rhs_LineWidth)   || isequal(gco,G.h_rhs_CoastLineWidth) )
                set(gcf,'userdata',11) ;
                uiresume ;
                return ;
            end
        case 'x'
            %if ~any(gco == ([G.h_DataSelection_handle,G.h_Projection_handle,G.h_TempRes_handle,G.h_Field_handle,G.h_rhs_Contour,G.h_rhs_ColorMap,G.h_rhs_LineWidth,G.h_rhs_CoastLineWidth]))
            if ~(isequal(gco,G.h_DataSelection_handle) || isequal(gco,G.h_Projection_handle) || isequal(gco,G.h_TempRes_handle)  || isequal(gco,G.h_Field_handle)  || isequal(gco,G.h_rhs_ColorMap)  || isequal(gco,G.h_rhs_LineWidth)   || isequal(gco,G.h_rhs_CoastLineWidth) )
                set(gcf,'userdata',900) ;
                uiresume ;
                return ;
            end
        case 's'
            if ~(isequal(gco,G.h_DataSelection_handle) || isequal(gco,G.h_Projection_handle) || isequal(gco,G.h_TempRes_handle)  || isequal(gco,G.h_Field_handle)  || isequal(gco,G.h_rhs_ColorMap)  || isequal(gco,G.h_rhs_LineWidth)   || isequal(gco,G.h_rhs_CoastLineWidth) )
                set(gcf,'userdata',901) ;
                uiresume ;
                return ;
            end
        case 'f'
            if ~(isequal(gco,G.h_DataSelection_handle) || isequal(gco,G.h_Projection_handle) || isequal(gco,G.h_TempRes_handle)  || isequal(gco,G.h_Field_handle)  || isequal(gco,G.h_rhs_ColorMap)  || isequal(gco,G.h_rhs_LineWidth)   || isequal(gco,G.h_rhs_CoastLineWidth) )
                set(gcf,'userdata',50) ;
                uiresume ;
                return ;
            end
        case 'd'
            if ~(isequal(gco,G.h_DataSelection_handle) || isequal(gco,G.h_Projection_handle) || isequal(gco,G.h_TempRes_handle)  || isequal(gco,G.h_Field_handle)  || isequal(gco,G.h_rhs_ColorMap)  || isequal(gco,G.h_rhs_LineWidth)   || isequal(gco,G.h_rhs_CoastLineWidth) )
                uicontrol(G.h_DataSelection) ; % Place focus on data dropdown menu
                set(gcf,'userdata',1) ;
            end
        case 't'
            if ~(isequal(gco,G.h_DataSelection_handle) || isequal(gco,G.h_Projection_handle) || isequal(gco,G.h_TempRes_handle)  || isequal(gco,G.h_Field_handle)  || isequal(gco,G.h_rhs_ColorMap)  || isequal(gco,G.h_rhs_LineWidth)   || isequal(gco,G.h_rhs_CoastLineWidth) )
                uicontrol(G.h_TempRes) ; % Place focus on data dropdown menu
                set(gcf,'userdata',300) ;
            end
        case 'b'
            if ~(isequal(gco,G.h_DataSelection_handle) || isequal(gco,G.h_Projection_handle) || isequal(gco,G.h_TempRes_handle)  || isequal(gco,G.h_Field_handle)  || isequal(gco,G.h_rhs_ColorMap)  || isequal(gco,G.h_rhs_LineWidth)   || isequal(gco,G.h_rhs_CoastLineWidth) )
                uicontrol(G.h_Field) ; % Place focus on data dropdown menu
                set(gcf,'userdata',200) ;
            end
        case 'p'
            if ~(isequal(gco,G.h_DataSelection_handle) || isequal(gco,G.h_Projection_handle) || isequal(gco,G.h_TempRes_handle)  || isequal(gco,G.h_Field_handle)  || isequal(gco,G.h_rhs_ColorMap)  || isequal(gco,G.h_rhs_LineWidth)   || isequal(gco,G.h_rhs_CoastLineWidth) )
                uicontrol(G.h_Projection) ; % Place focus on data dropdown menu
                set(gcf,'userdata',100) ;
            end
        case 'q'
            if ~(isequal(gco,G.h_DataSelection_handle) || isequal(gco,G.h_Projection_handle) || isequal(gco,G.h_TempRes_handle)  || isequal(gco,G.h_Field_handle)  || isequal(gco,G.h_rhs_ColorMap)  || isequal(gco,G.h_rhs_LineWidth)   || isequal(gco,G.h_rhs_CoastLineWidth) )
                try
                    close(G.GOAT_Fig) ; % Place focus on data dropdown menu
                catch
                    try
                        close(gcf)
                    catch
                        % Nevermind
                    end
                end
            end
    end  
end
catch        
end
try
    switch  event.Key
        case 'rightarrow'
            set(gcf,'userdata',232) ;
            uiresume ;
            return ;
        case  'leftarrow'
            set(gcf,'userdata',231) ;
             uiresume ;
            return ;
        case {'control','ctrl'}
            uicontrol(G.h_DataSelectionT) ; % Place focus on data dropdown menu
            uiresume(gcf) ;
            return ;
        case {'return'}
            uiresume ;
            return ;
    end
    
catch
    %disp(['keypress error!  Charecter = ',event.Character,'  Key = ',event.Key]) ;
end
% Settings KeyPressFunction
function SETTINGSKeyPress(~,event,G)
try
    switch event.Character
        case {'g','c'}
            set(gcf,'userdata',999) ;
            uiresume ;
            return ;
    end
catch
    % Ignore
end
% EXPORT KeyPressFunction
function EXPORTKeyPress(~,event,G)
try
    switch event.Character
        case 'g'
            if ~any(gco == ([G.Export.Name_handle,G.Export.Comment_handle,G.Export.Title_handle]))
                set(gcf,'userdata',100) ;
                uiresume ;
            end
            return ;
        case 'c'
            if ~any(gco == ([G.Export.Name_handle,G.Export.Comment_handle,G.Export.Title_handle]))
                set(gcf,'userdata',99) ;
                uiresume ;
                return ;
            end
    end
    switch  event.Key
        case {'control','ctrl'}
            uicontrol(G.Export.h_NameT) ; % Place focus on data dropdown menu
            uiresume(gcf) ;
            return ;
    end
catch
    % Ignore
end
% Conditions for green Filter Color
function G = FilterColor(G)
if G.Filter.WeightIdx          ~= 1 || ...
        sum(G.Filter.ZonalWaveNumberIdx) > -1 || ...
        G.Filter.RAIdx              ~= 1 || ...
        G.Filter.LowPassIdx         ~= 0 || ...
        G.Filter.HighPassIdx        ~= 0 || ...
        G.Filter.IndicesIdx         ~= 1 || ...
        G.Filter.ZEFlag
    set(G.h_Filters,'Backgroundcolor',G.GUI.ActiveColor)
else
    set(G.h_Filters,'Backgroundcolor',G.GUI.ButtonColor)
end
% Export Function
function G = GOAT_Export(G)
ClimatologyFlag = 0 ;
ExportName = 'Data_Export' ;
ExportAddress = fullfile(fileparts(cd),'Export',ExportName) ;
Comment = '' ;
Title   = G.message ;
pos        = G.GUI.pos.Export ;
G.Export_Fig = figure('name','Export Analysis','position',pos,'color',G.Display.FigBackgroudColor) ;
set(G.Export_Fig,'MenuBar','none')
NumLines = 8 ;
bl = 0.1 ; % Left Edge
bt = 0.05; % Top Edge
LineHeight = (1-2*bt)/NumLines ;
Column     = (1-2*bl)/1 ;
%Top Message
messageStr = {'Export Composite, Time-Series or Climatology.';'If seleced, Time Series and Climatology will be processd after the OK button is pressed.'} ;
h_message = uicontrol('Style', 'text','Units','normalized','pos',[0,1-LineHeight*.75,1,LineHeight*.75], 'string',messageStr);
uicontrol(h_message) ;  set(h_message,'ForegroundColor',G.GUI.messageColor,'FontSize',G.Display.TextSize_S,'FontName',G.GUI.Fonts.MessageFont) ;
% title
Line = 2 ;
G.Export.h_NameT = uicontrol('Style', 'text','Units','normalized','pos',[bl,1-bt-Line*LineHeight,Column/5,LineHeight], 'string','Output Adress');
uicontrol(G.Export.h_NameT) ;  set(G.Export.h_NameT,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',G.GUI.HeaderColor,'FontSize',G.Display.TextSize_L,'HorizontalAlignment','left','TooltipString','Output folder title','FontName',G.GUI.Fonts.uiHeaderFont) ;
Line = 2 ;
h_Name = uicontrol('style','edit','Units','normalized','pos',[bl*2+Column/5,1-bt-Line*LineHeight,Column*3/5-bl,LineHeight],...
    'string',ExportAddress,  'callback','set(gcbf,''userdata'',11); uiresume;');
uicontrol(h_Name) ;  set(h_Name,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',12,'FontName',G.GUI.Fonts.uiButtonFont,'HorizontalAlignment','left') ;
G.Export.Name_handle = h_Name ;
% Browse
h_Browse = uicontrol('Units','normalized','pos',[bl + Column*4/5,1-bt-Line*LineHeight,Column/5 ,LineHeight],...
    'string','Browse',  'callback','set(gcbf,''userdata'',112); uiresume;');
uicontrol(h_Browse) ;  set(h_Browse,'BackgroundColor',G.GUI.ButtonColor,'ForegroundColor',G.GUI.DropTextColor,'FontSize',G.Display.TextSize_L,'fontweight','bold','FontName',G.GUI.Fonts.uiButtonFont) ;
% comment
Line = 3 ;
h_CommentT = uicontrol('Style', 'text','Units','normalized','pos',[bl,1-bt-Line*LineHeight,Column/5,LineHeight], 'string','Comment');
uicontrol(h_CommentT) ;  set(h_CommentT,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',G.GUI.HeaderColor,'FontSize',G.Display.TextSize_L,'HorizontalAlignment','left','TooltipString','Comment','FontName',G.GUI.Fonts.uiHeaderFont) ;
Line = 3 ;
h_Comment = uicontrol('style','edit','Units','normalized','pos',[bl*2+Column/5,1-bt-Line*LineHeight,Column*4/5-bl,LineHeight],...
    'string',Comment,  'callback','set(gcbf,''userdata'',12); uiresume;');
uicontrol(h_Comment) ;  set(h_Comment,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',12,'FontName',G.GUI.Fonts.uiButtonFont,'HorizontalAlignment','left') ;
G.Export.Comment_handle = h_Comment ;
% Title
Line = 4 ;
h_TitleT = uicontrol('Style', 'text','Units','normalized','pos',[bl,1-bt-Line*LineHeight,Column/5,LineHeight], 'string','Fig Title');
uicontrol(h_TitleT) ;  set(h_TitleT,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',G.GUI.HeaderColor,'FontSize',G.Display.TextSize_L,'HorizontalAlignment','left','TooltipString','output figure title. strings beginning with $$ will be interpreted as latex.','FontName',G.GUI.Fonts.uiHeaderFont) ;
h_Title = uicontrol('style','edit','Units','normalized','pos',[bl*2+Column/5,1-bt-Line*LineHeight,Column*4/5-bl,LineHeight],...
    'string',Title,  'callback','set(gcbf,''userdata'',15); uiresume;');
uicontrol(h_Title) ;  set(h_Title,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',12,'FontName',G.GUI.Fonts.uiButtonFont,'HorizontalAlignment','left') ;
G.Export.Title_handle = h_Title ;
% Climatology Flags
Line = 5 ;
h_Climatology = uicontrol('style','checkbox','Units','normalized','pos',[bl,1-bt-Line*LineHeight,Column*.9,LineHeight],'string','Climatology',...
    'value',ClimatologyFlag,  'callback','set(gcbf,''userdata'',13); uiresume;');
uicontrol(h_Climatology) ;  set(h_Climatology,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_L,'TooltipString','Calculate Climatology','FontName',G.GUI.Fonts.uiHeaderFont) ;
if (G.ProjectionType > 4 && G.ProjectionType < 8)  % if Hovmoller
    set(h_Climatology ,'Enable','off') ;
end
Line = 6 ;
h_ExportG = uicontrol('style','checkbox','Units','normalized','pos',[bl,1-bt-Line*LineHeight,Column*.9,LineHeight], 'string','Export handles and structure',...
    'value',G.Settings.ExportGFlag,'callback','set(gcbf,''userdata'',16); uiresume;');
uicontrol(h_ExportG) ;  set(h_ExportG,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_L,'HorizontalAlignment','left', ...
    'TooltipString','include GOAT structure (G) in exported .mat file','FontName',G.GUI.Fonts.uiHeaderFont) ;
% Cancel
Line = NumLines  ;
h_Cancel = uicontrol('Units','normalized','pos',[bl*2 + Column*2/3,1-bt-Line*LineHeight,Column/3 - bl,LineHeight],...
    'string','Cancel',  'callback','set(gcbf,''userdata'',99); uiresume;');
uicontrol(h_Cancel) ;  set(h_Cancel,'BackgroundColor',G.GUI.ButtonColor,'ForegroundColor',G.GUI.DropTextColor,'FontSize',G.Display.TextSize_L,'fontweight','bold','FontName',G.GUI.Fonts.uiButtonFont) ;
% OK
Line = NumLines  ;
h_OK = uicontrol('Units','normalized','pos',[bl,1-bt-Line*LineHeight,Column*2/3,LineHeight],...
    'string','OK',  'callback','set(gcbf,''userdata'',100); uiresume;');
uicontrol(h_OK) ;  set(h_OK,'BackgroundColor',G.GUI.colors.celeste,'ForegroundColor',G.GUI.DropTextColor,'FontSize',16,'fontweight','bold','FontName',G.GUI.Fonts.uiButtonFont) ;

if G.Settings.KeyPressFlag
    %    set(G.Export_Fig,'KeyPressFcn',{@EXPORTKeyPress,G},'WindowKeyPressFcn',{@EXPORTKeyPress,G} )
end

GUIcon = 1 ;
while GUIcon
    try
        set(0,'CurrentFigure',G.Export_Fig) ;
        uiwait(gcf);
        select = get(gcf,'userdata');
    catch
        GUIcon = 0 ;
    end
    try
        switch select
            case 11 % Folder name of export
                ExportAddress          = get(h_Name,'string')            ;
                [Xpath,ExportName] = fileparts(ExportAddress) ;
            case 112
                try
                    ExportAddress = uigetdir ;
                    [Xpath,ExportName] = fileparts(ExportAddress) ;
                    set(h_Name,'string',ExportAddress)
                catch
                    % Nevermind
                end
            case 12 % Short comment regarding the export
                Comment          = get(h_Comment,'string')            ;
            case 15 % title text for the output figure
                Title          = get(h_Title,'string')            ;
            case 13 % Climatology Flag
                ClimatologyFlag = get(h_Climatology,'value')          ;
            case 14 % Obsolete. Time Series Flag
                TimeSeriesFlag = get(h_TimeSeries,'value')            ;
            case 16 % Export handles and G structure flag
                G.Settings.ExportGFlag = get(h_ExportG,'value') ;
            case 99 % Clear
                GUIcon = 0 ;
            case 100 % OK
                try
                    D.Hold = G.Hold ;
                end
                try
                    D.Comment = Comment                                             ;
                    try
                        D.data = G.Data                                                 ;
                    catch
                        D.data = [] ;
                    end
                    D.X    = G.X                                                    ;
                    D.x    = G.x                                                    ;
                    D.Y    = G.Y                                                    ;
                    D.y    = G.y                                                    ;
                    D.xlabel      = G.xlabel                                        ;
                    D.ylabel      = G.ylabel                                        ;
                    D.datalabel   = G.Datalabel                                     ;
                    D.Dates       = G.Dates                                         ;
                    try
                        D.FilteredDates = G.FilteredDates                               ;
                    end
                    D.Filter      = G.Filter                                        ;
                    try
                        D.infoMessage = dateMessage(G)                                  ;
                    catch
                        D.infoMessage = G.message                                       ;
                    end
                    D.ProjectionType = G.ProjectionType                             ;
                    D.Limits.Default.sameP = G.Limits.Default.sameP                 ;
                    D.Limits.Default.VerMeanInt = G.Limits.Default.VerMeanInt       ;
                    D.lon         = G.lon                                           ;
                    D.lat         = G.lat                                           ;
                    D.P           = G.P                                             ;
                    try
                        D.lonlabel    = G.lonlabel                                  ;
                        D.latlabel    = G.latlabel                                  ;
                        D.Plabel      = G.Plabel                                    ;
                    catch
                        D.lonlabel    = 'Longitude'                                 ;
                        D.latlabel    = 'Latitude'                                  ;
                        D.Plabel      = 'Level'                                     ;
                    end
                    try
                        D.Units       = G.DataFieldUnits                            ;
                        D.Description = G.FieldDescription                          ;
                    end
                    D.Limits.Default.Lat1 = G.Limits.Default.Lat1                   ;
                    D.Limits.Default.Lat2 = G.Limits.Default.Lat2                   ;
                    D.Limits.Default.Lon1 = G.Limits.Default.Lon1                   ;
                    D.Limits.Default.Lon2 = G.Limits.Default.Lon2                   ;
                    D.Limits.Default.Lev1 = G.Limits.Default.Lev1                   ;
                    D.Limits.Default.Lev2 = G.Limits.Default.Lev2                   ;
                    try  close(G.Export_Fig)                                        ;  end % close export figure and dim GO Button
                    set(G.h_Analyze,'Enable','off')                                 ; drawnow ;
                    if  ClimatologyFlag
                        set(G.h_message,'string',['Calculating Climatology.  ',G.message]) ; drawnow ;% Update Message
                        G = GOAT_Compile_Climatology(G)                             ; % calculated climatology (hourly daily or monthly) of the data timeseries
                        set(G.h_message,'string',['Calculating Anomalies.  ',G.message]) ; drawnow ; % Update Message
                        G = GOAT_Compile_ClimAnomaly(G)                             ; %
                    end
                    try
                        D.DataClim       = G.DataClim                                 ; % Climatology (hourly, daily or monthly)
                        D.DataSeriesAnom = G.DataSeriesAnom                           ; % time series of the climatological anomaly of the Time Series
                    end
                    mkdir(ExportAddress) ;
                    arraysize = 1 ;
                    for i = 1:length(size(G.DataSeries))
                        arraysize = arraysize*size(G.DataSeries,i) ;
                    end
                    ArraySizeCutoff = 2.5e8 ; % about 1GB per timeseries file
                    if arraysize<= ArraySizeCutoff
                        % no need to break down the time series
                        D.DataSeries     = single(G.DataSeries)     ; %
                        if  ClimatologyFlag
                            D.DataSeriesAnom = single(G.DataSeriesAnom)                 ; % 'DataSeriesAnom' field exists only if 'G.Limits.Default.DataSeriesAnomFlag' is set to 1 and 'GOAT_Compile_data_Climatology' has be run
                        end
                        if G.Settings.ExportGFlag
                            save(fullfile(ExportAddress,[ExportName,'_handles.mat']),'G')       ;
                        end
                        save(fullfile(ExportAddress,[ExportName,'.mat']),'D')           ;
                        
                    else
                        if G.Settings.ExportGFlag
                            set(G.h_message,'string',['Exporting handles . . .   ',G.message]) ; drawnow ; % Update Message
                            disp 'Note: Handles export may fail because file is too large'
                            save(fullfile(ExportAddress,[ExportName,'_handles.mat']),'G')           ;
                        end
                        save(fullfile(ExportAddress,[ExportName,'.mat']),'D')           ;
                        % the time series needs to be broken Down
                        set(G.h_message,'string',['Exporting . . . Breaking Timeseries into files . . .  ',G.message]) ; drawnow ; % Update Message
                        %disp('time series is too big and will be broken into several files')
                        Nfiles = ceil(arraysize/ArraySizeCutoff) ;
                        Tn = floor(size(G.DataSeries,1)/Nfiles) ;
                        Tidx = 1 ;
                        for f = 1:Nfiles
                            if f == Nfiles
                                Tn = size(G.DataSeries,1)+1 - Tidx ;
                            end
                            switch length(size(G.DataSeries))
                                case 1
                                    eval(['DataSeries_',int2str(f),' = single(G.DataSeries(Tidx:Tidx+Tn-1)) ;']) ;
                                    if  ClimatologyFlag
                                        eval(['DataSeriesAnom_',int2str(f),' = single(G.DataSeriesAnom(Tidx:Tidx+Tn-1)) ;']) ;
                                    end
                                case 2
                                    eval(['DataSeries_',int2str(f),' = single(G.DataSeries(Tidx:Tidx+Tn-1,:)) ;']) ;
                                    if  ClimatologyFlag
                                        eval(['DataSeriesAnom_',int2str(f),' = single(G.DataSeriesAnom(Tidx:Tidx+Tn-1,:)) ;']) ;
                                    end
                                case 3
                                    eval(['DataSeries_',int2str(f),' = single(G.DataSeries(Tidx:Tidx+Tn-1,:,:)) ;']) ;
                                    if  ClimatologyFlag
                                        eval(['DataSeriesAnom_',int2str(f),' = single(G.DataSeriesAnom(Tidx:Tidx+Tn-1,:,:)) ;']) ;
                                    end
                                case 4
                                    eval(['DataSeries_',int2str(f),' = single(G.DataSeries(Tidx:Tidx+Tn-1,:,:,:)) ;']) ;
                                    if  ClimatologyFlag
                                        eval(['DataSeriesAnom_',int2str(f),' = single(G.DataSeriesAnom(Tidx:Tidx+Tn-1,:,:,:)) ;']) ;
                                    end
                            end
                            eval(['Dates_',int2str(f),' = G.Dates(Tidx:Tidx+Tn-1) ;']) ;
                            save(fullfile(ExportAddress,[ExportName,'DataSeries_',int2str(f),'.mat']),['DataSeries_',int2str(f)],['Dates_',int2str(f)])           ;
                            if  ClimatologyFlag
                                save(fullfile(ExportAddress,[ExportName,'DataSeriesAnom_',int2str(f),'.mat']),['DataSeriesAnom_',int2str(f)])           ;
                            end
                            eval(['clear ''DataSeries_',int2str(f),'''  ''Dates_',int2str(f),''' ''DataSeriesAnom_',int2str(f),''''])
                            Tidx = Tidx+Tn ;
                        end
                    end
                    G.Plotting.GUI_PlotFlag = 0                                     ; % Informs GOAT_plotUpdate that the plotting is not in GUI format
                    Fig = figure('position',G.GUI.pos.Main)                                  ;
                    set(0,'CurrentFigure',Fig)                                      ; % New Figure Without GUI Properties
                    if G.Settings.HoldFlag && ~G.Settings.HoldDifFlag
                        % save current data in temp
                        temp_Data            = G.Data      ; G.Data      = G.Hold.Data       ;
                        temp_X               = G.X         ; G.X         = G.Hold.X          ;
                        temp_Y               = G.Y         ; G.Y         = G.Hold.Y          ;
                        temp_x               = G.x         ; G.x         = G.Hold.x          ;
                        temp_y               = G.y         ; G.y         = G.Hold.y          ;
                        temp_xlabel          = G.xlabel    ; G.xlabel    = G.Hold.xlabel     ;
                        temp_ylabel          = G.ylabel    ; G.ylabel    = G.Hold.ylabel     ;
                        temp_datalabel       = G.Datalabel ; G.Datalabel = G.Hold.datalabel  ;
                        temp_Dates           = G.Dates     ; G.Dates     = G.Hold.Dates      ;
                        temp_FilteredDates   = G.FilteredDates ; G.FilteredDates = G.Hold.FilteredDates  ;
                        temp_Filter          = G.Filter    ; G.Filter    = G.Hold.Filter     ;
                        temp_sameP           = G.Limits.Default.sameP ; G.Limits.Default.sameP = G.Hold.Limits.Default.sameP ;
                        temp_VerMeanInt      = G.Limits.Default.VerMeanInt ; G.Limits.Default.VerMeanInt = G.Hold.Limits.Default.VerMeanInt ;
                        temp_lon             = G.lon       ; G.lon       = G.Hold.lon        ;
                        temp_lat             = G.lat       ; G.lat       = G.Hold.lat        ;
                        temp_P               = G.P         ; G.P         = G.Hold.P          ;
                        temp_Units           = G.DataFieldUnits ; G.DataFieldUnits = G.Hold.Units   ;
                        temp_Description     = G.FieldDescription ; G.FieldDescription = G.Hold.Description ;
                        temp_Lat1            = G.Limits.Default.Lat1 ; G.Limits.Default.Lat1 = G.Hold.Limits.Default.Lat1 ;
                        temp_Lat2            = G.Limits.Default.Lat2 ; G.Limits.Default.Lat2 = G.Hold.Limits.Default.Lat2 ;
                        temp_Lon1            = G.Limits.Default.Lon1 ; G.Limits.Default.Lon1 = G.Hold.Limits.Default.Lon1 ;
                        temp_Lon2            = G.Limits.Default.Lon2 ; G.Limits.Default.Lon2 = G.Hold.Limits.Default.Lon2 ;
                        temp_P1              = G.Limits.Default.Lev1 ; G.Limits.Default.Lev1 = G.Hold.Limits.Default.Lev1 ;
                        temp_P2              = G.Limits.Default.Lev2 ; G.Limits.Default.Lev2 = G.Hold.Limits.Default.Lev2 ;
                        temp_ColorMapIdx       = G.Hold.Settings.ColorMapIdx            ;
                        temp_CoastLineWidthIdx = G.Hold.Settings.CoastLineWidthIdx      ;
                        temp_XgridFlag         = G.Hold.Settings.XgridFlag              ;
                        temp_YgridFlag         = G.Hold.Settings.YgridFlag              ;
                        temp_SymZLimitsFlag    = G.Hold.Settings.SymZLimitsFlag         ;
                        temp_FixZLimitsFlag    = G.Hold.Settings.FixZLimitsFlag         ;
                        temp_LogLevelsFlag     = G.Hold.Settings.LogLevelsFlag          ;
                        temp_ContourLevelStep  = G.Hold.Settings.ContourLevelStep       ;
                        temp_ReverseColorMap   = G.Hold.Settings.ReverseColorMap        ;
                        temp_TopoTypeFlag      = G.Hold.topo.TypeFlag                   ;
                        temp_LineWidthIdx      = G.Hold.Settings.LineWidthIdx           ;
                        temp_LineWidth         = G.Hold.Settings.LineWidth              ;
                        temp_CoastLineWidth    = G.Hold.Settings.CoastLineWidth         ;
                        temp_Clim              = G.Hold.Clim                            ;
                        temp_colorbarLimits    = G.Hold.colorbarLimits                  ;
                        temp_colorbarHandle    = G.Hold.colorbarHandle                  ;
                        G.Settings.ColorMapIdx       = G.Hold.Settings.ColorMapIdx            ;
                        G.Settings.CoastLineWidthIdx = G.Hold.Settings.CoastLineWidthIdx      ;
                        G.Settings.XgridFlag         = G.Hold.Settings.XgridFlag              ;
                        G.Settings.YgridFlag         = G.Hold.Settings.YgridFlag              ;
                        G.Settings.SymZLimitsFlag    = G.Hold.Settings.SymZLimitsFlag         ;
                        G.Settings.FixZLimitsFlag    = G.Hold.Settings.FixZLimitsFlag         ;
                        G.Settings.LogLevelsFlag     = G.Hold.Settings.LogLevelsFlag          ;
                        G.Settings.ContourLevelStep  = G.Hold.Settings.ContourLevelStep       ;
                        G.Settings.ReverseColorMap   = G.Hold.Settings.ReverseColorMap        ;
                        G.topo.TypeFlag              = G.Hold.topo.TypeFlag                   ;
                        G.Settings.LineWidthIdx      = G.Hold.Settings.LineWidthIdx           ;
                        G.Settings.LineWidth         = G.Hold.Settings.LineWidth              ;
                        G.Settings.CoastLineWidth    = G.Hold.Settings.CoastLineWidth         ;
                        GOAT_plotUpdate(G)  ; % Reproduce Plot without Buttons
                        G.Data            = temp_Data           ;
                        G.X               = temp_X              ;
                        G.Y               = temp_Y              ;
                        G.x               = temp_x              ;
                        G.y               = temp_y              ;
                        G.xlabel          = temp_xlabel         ;
                        G.ylabel          = temp_ylabel         ;
                        G.Datalabel       = temp_datalabel      ;
                        G.Dates           = temp_Dates          ;
                        G.FilteredDates   = temp_FilteredDates  ;
                        G.Filter          = temp_Filter         ;
                        G.Limits.Default.sameP = temp_sameP     ;
                        G.Limits.Default.VerMeanInt = temp_VerMeanInt ;
                        G.lon             = temp_lon            ;
                        G.lat             = temp_lat            ;
                        G.P               = temp_P              ;
                        G.DataFieldUnits  = temp_Units          ;
                        G.FieldDescription = temp_Description   ;
                        G.Limits.Default.Lat1 = temp_Lat1       ;
                        G.Limits.Default.Lat2 = temp_Lat2       ;
                        G.Limits.Default.Lon1 = temp_Lon1       ;
                        G.Limits.Default.Lon2 = temp_Lon2       ;
                        G.Limits.Default.Lev1 = temp_P1         ;
                        G.Limits.Default.Lev2 = temp_P2         ;
                        G.Settings.ColorMapIdx        = temp_ColorMapIdx        ;
                        G.Settings.CoastLineWidthIdx  = temp_CoastLineWidthIdx  ;
                        G.Settings.XgridFlag          = temp_XgridFlag          ;
                        G.Settings.YgridFlag          = temp_YgridFlag          ;
                        G.Settings.SymZLimitsFlag     = temp_SymZLimitsFlag     ;
                        G.Settings.FixZLimitsFlag     = temp_FixZLimitsFlag     ;
                        G.Settings.LogLevelsFlag      = temp_LogLevelsFlag      ;
                        G.Settings.ContourLevelStep   = temp_ContourLevelStep   ;
                        G.Settings.ReverseColorMap    = temp_ReverseColorMap    ;
                        G.topo.TypeFlag               = temp_TopoTypeFlag       ;
                        G.Settings.LineWidthIdx       = temp_LineWidthIdx       ;
                        G.Settings.LineWidth          = temp_LineWidth          ;
                        G.Settings.CoastLineWidth     = temp_CoastLineWidth     ;
                        G.ExportFlag = 1                                            ; % Flag indicating active export
                        G = GOAT_holdUpdate(G)  ; % Reproduce hold plot without Buttons
                        G.ExportFlag = 0                                            ; % Flag indicating active export
                    else
                        GOAT_plotUpdate(G)                                                   ; % Reproduce Plot without Buttons
                    end
                    try
                        if ~strcmp(Title,'')
                            if strncmp(Title,'$$',2)
                                title(Title,'fontsize',16,'interpreter','latex') ;
                            else
                                title(Title,'fontsize',16) ;
                            end
                        end
                    catch
                        disp 'error while writing title' ;
                    end
                    saveas(Fig,fullfile(ExportAddress,[ExportName,'.fig']))     ; % Save Buttonless Fig
                    set(gcf,'PaperPositionMode','auto','papersize',G.Saving.PapersizeGUI)  ;
                    print('-dpng',G.Saving.dpdfResolution,fullfile(ExportAddress,ExportName,'')) ;
                    %print('-dpdf',G.Saving.dpdfResolution,fullfile(ExportAddress,ExportName,'')) ;
                    set(0,'CurrentFigure',G.GOAT_Fig)                               ; % Override existing GUI Figure
                    close(Fig) ;
                    G.Plotting.GUI_PlotFlag = 1                                     ; % Informs GOAT_plotUpdate that plots are in GUI format
                    GUIcon = 0 ;
                catch
                    if isempty(D.data)
                        set(h_message,'string','Export cannot proceed. No data to export! ') ;
                    else
                        set(h_message,'string','Error while trying to export! ') ;
                    end
                end
        end
    catch
        %nevermind
    end
end

try
    set(G.h_Analyze,'Enable','on')  ; drawnow ; % re-enable GO button
    close(G.Export_Fig) ;
catch
    % Nevermind
end
set(G.h_message,'string',['Export Complete.        ',G.message]) ; % Update Message
drawnow ;
return ;
% hold analysis
function G = GOAT_holdAnalyze(G)
set(G.h_Analyze,'Enable','off')  ; drawnow ; % re-enable GO button

if ~G.StepGUIFlag
    G = GOAT_Limits(G) ;
end
drawnow ;
if ~G.continueFlag
    G = GOAT_Compile_data_Series(G) ;
    G.message = ['Held: ',G.Hold.infoMessage,' |  Overlay: ',dateMessage(G)] ;
    G.continueFlag = 0 ;
end
set(G.h_Analyze,'Enable','on')  ; drawnow ; % re-enable GO button
% Hold Data
function G = HoldData(G)
G.Hold.Data          = G.Data          ;
G.Hold.X             = G.X             ;
G.Hold.x             = G.x             ;
G.Hold.Y             = G.Y             ;
G.Hold.y             = G.y             ;
G.Hold.xlabel        = G.xlabel        ;
G.Hold.ylabel        = G.ylabel        ;
G.Hold.datalabel     = G.Datalabel     ;
G.Hold.Dates         = G.Dates         ;
G.Hold.FilteredDates = G.FilteredDates ;
G.Hold.Filter        = G.Filter        ;
try
    G.Hold.infoMessage = dateMessage(G)                              ;
catch
    G.Hold.infoMessage = G.message                                   ;
end
G.Hold.Limits.Default.sameP = G.Limits.Default.sameP                 ;
G.Hold.Limits.Default.VerMeanInt = G.Limits.Default.VerMeanInt       ;
G.Hold.lon         = G.lon                                           ;
G.Hold.lat         = G.lat                                           ;
G.Hold.P           = G.P                                             ;
try
    G.Hold.Units       = G.DataFieldUnits                            ;
    G.Hold.Description = G.FieldDescription                          ;
end
G.Hold.Limits.Default.Lat1  = G.Limits.Default.Lat1                  ;
G.Hold.Limits.Default.Lat2  = G.Limits.Default.Lat2                  ;
G.Hold.Limits.Default.Lon1  = G.Limits.Default.Lon1                  ;
G.Hold.Limits.Default.Lon2  = G.Limits.Default.Lon2                  ;
G.Hold.Limits.Default.Lev1  = G.Limits.Default.Lev1                  ;
G.Hold.Limits.Default.Lev2  = G.Limits.Default.Lev2                  ;
G.Hold.Settings.ColorMapIdx = G.Settings.ColorMapIdx                 ;
G.Hold.Settings.CoastLineWidthIdx = G.Settings.CoastLineWidthIdx     ;
G.Hold.Settings.XgridFlag   = G.Settings.XgridFlag                   ;
G.Hold.Settings.YgridFlag   = G.Settings.YgridFlag                   ;
G.Hold.Settings.SymZLimitsFlag = G.Settings.SymZLimitsFlag           ;
G.Hold.Settings.FixZLimitsFlag = G.Settings.FixZLimitsFlag           ;
G.Hold.Settings.LogLevelsFlag  = G.Settings.LogLevelsFlag            ;
G.Hold.Settings.SinLatFlag     = G.Settings.SinLatFlag               ;
G.Hold.Settings.ContourLevelStep = G.Settings.ContourLevelStep       ;
G.Hold.Settings.ReverseColorMap  = G.Settings.ReverseColorMap        ;
G.Hold.topo.TypeFlag             = G.topo.TypeFlag                   ;
G.Hold.Settings.LineWidthIdx     = G.Settings.LineWidthIdx           ;
G.Hold.Settings.LineWidth        = G.Settings.LineWidth              ;
G.Hold.Settings.CoastLineWidth   = G.Settings.CoastLineWidth         ;
try  G.Hold.DataSeries      = single(G.DataSeries)                   ; end
try  G.Hold.DataClim        = G.DataClim                             ; end
try  G.Hold.DataSeriesAnom  = single(G.DataSeriesAnom)               ; end
%G.Hold.SaveDataFlag = 0 ;
G.Hold.Clim = get(gca,'CLim')                                        ;
G.Hold.colorbarLimits = G.Display.colorbarLimits                     ;
G.Hold.colorbarHandle = G.Display.colorbarHandle                     ;
% Diff Calculation
function G = Analyze_Diff(G)
%  interpolate if necessary
if ~(isequal(G.X,G.Hold.X) && isequal (G.Y,G.Hold.Y))
    if min(size(G.X)) > 1 % 2D interpolation
        Data = interp2(G.X,G.Y,G.Data',G.Hold.X,G.Hold.Y) ;
        G.Data = Data' - G.Hold.Data ;
    else % 1D interpolation
        switch G.ProjectionType
            case {3,4}
                if G.Limits.Default.Lev1==G.Limits.Default.Lev2 || G.Limits.Default.sameP
                    Data = interp1(G.X,G.Data,G.Hold.X) ;
                    try
                        G.Data = Data' - G.Hold.Data ;
                    catch
                        G.Data = Data - G.Hold.Data ;
                    end
                else
                    Data = interp1(G.Y,G.Data,G.Hold.Y) ;
                    try
                        G.Data = Data' - G.Hold.Data ;
                    catch
                        G.Data = Data - G.Hold.Data ;
                    end
                end
            otherwise
                disp 'Diff Option not supported for this configuration'
        end
    end
    G.X = G.Hold.X ;
    G.Y = G.Hold.Y ;
else
    G.Data = G.Data - G.Hold.Data ;
end
% Add Datasets
function [G,Flag] = FindDatasets(G) 

NF0 = size(G.DataFolders,1)  -  1;  
NF  = NF0 ; 
% first check whether the selected folder contains valid calendar folders
DataFolderDir = dir(G.DataFolder) ;
DataFolderDir = DataFolderDir([DataFolderDir(:).isdir],1) ;
if  (sum(strncmpi({DataFolderDir.name},'Daily',5))  || ...
     sum(strncmpi({DataFolderDir.name},'Julian',6)) || ...
     sum(ismember({DataFolderDir.name},{'Monthly','List','monthly','list'})))
     % Add folder
     NF = NF + 1 ; 
     [DataFolderPath,DataFolderName] = fileparts(G.DataFolder) ; 
     G.DataFolders(NF,1) = {DataFolderName} ;
     G.DataFolderAddress(NF,1) = {DataFolderPath} ;
else % the selected datafolder was not a dataset folder. Look recursively for dataset folders below the selected filder
       Folders = GOAT_FileFolderTrees(G.DataFolder,1) ; 
       for d = 1:length(Folders)
        if strcmp(Folders{d},'n')
            continue
        end
        DataFolderDir = dir(Folders{d}) ;
        DataFolderDir = DataFolderDir([DataFolderDir(:).isdir],1) ;    
        if  (sum(strncmpi({DataFolderDir.name},'Daily',5))  || ...
             sum(strncmpi({DataFolderDir.name},'Julian',6)) || ...
             sum(ismember({DataFolderDir.name},{'Monthly','List','monthly','list'})))
             % Add folder
             NF = NF + 1 ; 
             [DataFolderPath,DataFolderName] = fileparts(Folders{d}) ; 
             G.DataFolders(NF,1) = {DataFolderName} ;
             G.DataFolderAddress(NF,1) = {DataFolderPath} ;
             % remove all subfolders of the dataset folder from the Folders list
             Folders(strncmp(Folders{d},Folders,length(Folders{d}))) = {'n'} ;
        end
      end
end
% set Flag to 0 if no new datasets were added
if NF == NF0
    Flag = 0 ;
else
    Flag = 1 ;
end

if Flag % if a new dataset was added
    NF = NF + 1 ; G.DataFolders(NF,1) = {'Browse'} ;
    % Create data folders string for popup menu
    G.DataFoldersString = '' ;
    for j = 1:NF
        G.DataFoldersString = [G.DataFoldersString,cell2mat(G.DataFolders(j)),' |'] ;
    end
    G.DataFoldersString = G.DataFoldersString(1:end-1) ;
    G.DataSelected = NF0 + 1 ;
    G.DataFolder = fullfile(cell2mat(G.DataFolderAddress(G.DataSelected)),cell2mat(G.DataFolders(G.DataSelected)),'') ;
    set(G.h_DataSelection,'string',G.DataFoldersString) ;
    set(G.h_DataSelection,'Value',G.DataSelected) ;
end