% Link netCDF MIP files to GOAT.
% Compatible with CMIP5 standard archived data following Data Reference Syntax (DRS) 1.3.1

function G = GOAT_CMIP5_LINK_DRS_1_3_1(G)
% Copyright (C) Ori Adam, Apr.2014.
if ~nargin
    G = GOAT_Environment ; % set up GOAT, and clean java heap memory
end
G = LINK_GUI(G) ;
if G.LINK.CancelFlag
    return
end
try
    close(G.LINK.Fig) ;
end
try
    close(G.GOAT_Fig) ;
end
%close all
drawnow ;
G.EnvironmentFlag = 1 ;
%% Step 1: Define CMIP archive and Link directories, scan archive and construct 'link_build_dataset.csv' 
mip_dirs      = G.LINK.MIPFolder     ; % Main MIP Directories
link_root_dir = G.LINK.StaticLink    ; % This is preferably a static directory. All linked datasets will be created in this directory
% scan MIP archive
% FLAGS
flag_erase_null_files   = 1 ; % 1/0 erase zero kb files
flag_erase_null_folders = 0 ; % 1/0 erase zero kb folders
GOAT_Display_message(G,'scanning for files in MIP directories . . .') ;
[folders,files] = GOAT_FileFolderTrees(mip_dirs,1) ;
%
if flag_erase_null_folders
    GOAT_Display_message(G,'Removing empty folders from MIP directories . . .')
    qt = 0 ;
    Cond = 1 ;
    while Cond > 0
        qr = 0 ;
        for i = 1:length(folders)
            try
                dirInfo = dir(folders{i});
                if isempty(dirInfo)
                    continue ;
                end
            catch
                continue ;
            end
            isDir = 0 ;
            for j = 1:length(dirInfo)
                isDir = isDir + dirInfo(j).isdir;
            end
            if isDir < 3 % only end folders that do not have subfolders
                dirSize = 0 ;
                for j = 1:length(dirInfo)
                    dirSize = dirSize + dirInfo(j).bytes;
                end
                if dirSize == 0
                    disp(['removing ',folders{i}])
                    %pause ;
                    try
                        [status, message, messageid] = rmdir(folders{i},'s') ;
                        if status
                            qr = qr+1 ;
                        end
                    end
                end
            end
        end
        qt = qt + qr ;
        Cond = qr ;
    end
    if qt == 0
        GOAT_Display_message(G,'No empty folders were found in MIP directories')
    else
        if length(mip_dirs) > 1
            GOAT_Display_message(G,[int2str(qt), ' empty folders were removed from MIP directories'])
        else
            GOAT_Display_message(G,[int2str(qt), ' empty folders were removed from ',mip_dirs,' . . .'])
        end
    end
end
%
CB    = cell(1,25) ;
[CB(1,:)] = deal([{'Temporal Resolution'},{'link type'},{'file path'},{'varname'},{'x dim #'},{'y dim #'},{'z dim #'},{'t dim #'},{'year1'},{'month1'},{'day1'},{'hour1'},{'year2'},{'month2'},{'day2'},{'hour2'},{'stride'},{''},{'Units'},{'long-name'},{'Realm'},{'Experiment'},{'Ensemble'},{'modeling realm'},{'Model'}]) ;
attributes = cell(1,1) ;
lon_dims   = {'lon','longitude','X','x','long','i','rlon'}      ;
lat_dims   = {'lat','latitude','Y','y','j','rlat'}              ;
level_dims = {'plev','lev','level','height','depth'} ;
time_dims  = {'time','type','loc','line','sza'}      ;

q = 1 ; e = 0 ;
NF = length(files) ;
if NF < 10
    Nmod = 1 ;
else
    Nmod = round(length(files)/10) ;
end
GOAT_Display_message(G,'Retrieving attributes from NetCDF MIP files . . .')
GOAT_Display_message(G,'','init')
for i = 1:NF
    if mod(i,Nmod) == 0
        GOAT_Display_message(G,['Retrieving attributes from files . . . ',num2str(i/NF*100,'%02.0f'),' %'],'stat') ;
    end
    [Path,Name,Ext] = fileparts(files{i}) ;
    if ~strcmp(Ext,'.nc')
        continue
    end
    try
        Parse =  GOAT_CMIP5_parseName(Name) ;
    catch
        e = e + 1 ;
        GOAT_Display_message(G,['Error parsing file name:',files{i}]) ;
        GOAT_Display_message(G,'','init')
        continue ;
    end
    try
        a = ncinfo(files{i},Parse.VarName) ;
    catch
        e = e + 1 ;
        fileInfo = dir(files{i});
        fileSize = fileInfo.bytes;
        if fileSize == 0
            GOAT_Display_message(G,['Size = Zero! could not retrieve attributes from ',files{i}])
            GOAT_Display_message(G,'','init')
            if flag_erase_null_files
                delete(files{i}) ;
            end
        else
            GOAT_Display_message(G,['Unknown Error. Could not retrieve attributes from ',files{i}])
            GOAT_Display_message(G,'','init')
        end
        continue
    end
    q = q + 1 ;
    [CB(q,1:4)] = deal([{Parse.TimeResolution},{1},files(i),{Parse.VarName}]) ;
    CB(q,5:8) = {0}                 ; % default dimension number
    for j = 1:length(a.Dimensions)
        if sum(strcmpi(a.Dimensions(j).Name,lon_dims))   ; CB(q,5) = {j} ; end
        if sum(strcmpi(a.Dimensions(j).Name,lat_dims))   ; CB(q,6) = {j} ; end
        if sum(strcmpi(a.Dimensions(j).Name,level_dims)) ; CB(q,7) = {j} ; end
        if sum(strcmpi(a.Dimensions(j).Name,time_dims))  ; CB(q,8) = {j} ; end
    end
    Hour1 = 0 ;
    switch Parse.TimeResolution
        case 'DailyX4'
            Hour2 = 18 ;
        case 'DailyX8'
            Hour2 = 21 ;
        otherwise
            Hour2 = 0 ;
    end
    [CB(q,9:17)] = deal([{Parse.Year1},{Parse.Month1},{Parse.Day1},{Hour1},{Parse.Year2},{Parse.Month2},{Parse.Day2},{Hour2},{1}]) ;
    longname = '' ;
    for j = 1:length(a.Attributes)
        if strcmpi(a.Attributes(j).Name,'Units')
            if strcmp(a.Attributes(j).Value,'%')
                CB(q,19) = {' % '} ;
            else
                CB(q,19) = {a.Attributes(j).Value} ;
            end
        end
        if sum(strcmpi(a.Attributes(j).Name,{'long-name','long_name'}))
            CB(q,20) = {a.Attributes(j).Value} ;
            longname = a.Attributes(j).Value ;
        end
    end
    if  isempty(longname)
        for j = 1:length(a.Attributes)
            if sum(strcmpi(a.Attributes(j).Name,{'standard-name','standard_name'}))
                CB(q,20) = {a.Attributes(j).Value} ;
                break ;
            end
        end
    end
    CB(q,21) = {Parse.RealmName} ;
    CB(q,22) = {Parse.ExperimentName} ;
    CB(q,23) = {Parse.EnsembleName} ;
    for j = 1:size(CB,2)
        if isnan(cell2mat(CB(q,j)))
            CB(q,j) = {''} ;
        end
    end
    a = ncinfo(files{i}) ;
    l = 0 ;
    CB(q,24) = {''} ;
    attributes(q) = {a.Attributes} ;
    for j = 1:length(a.Attributes)
        if sum(strcmpi(a.Attributes(j).Name,{'modeling-realm','modeling_realm'}))
            CB(q,24) = {regexprep(a.Attributes(j).Value,' ','_')} ;
        else
            switch lower(Parse.RealmName)
                case {'omon','oyr','oclim'}
                    CB(q,24) = {'ocean'} ;
                case {'lmon'}
                    CB(q,24) = {'land'} ;
                case {'limon'}
                    CB(q,24) = {'land_ice'} ;
                case {'oimon'}
                    CB(q,24) = {'ocean_ice'} ;
                otherwise
                    CB(q,24) = {'atmos'} ;
            end
        end
        l = l + 1 ;
    end
    % refering to 'reanalysis' as an experiment and the name of thr reanalysis as a model
    if strcmpi(Parse.ModelName,'reanalysis')
        CB(q,25) = CB(q,22)       ;
        CB(q,22) = {Parse.ModelName} ;
    else
        CB(q,25) = {Parse.ModelName} ;
    end
    
end
if e > 0
    GOAT_Display_message(G,['Retrieving attributes . . . Done. Attributes from ',int2str(e),' *.nc files out of ',int2str(q),' could not be retrieved.'])
else
    GOAT_Display_message(G,['Retrieving attributes . . . Done.'])
end

% now build links using link_build and link_convert

%'link_field' columns:
% 1.  nc variable name
% 2.  file path
% 3. type: 1) opendap  2) nc file 3) hdf file (TBD)  4) GRIB1 file (TBD) 5) GRIB2 file (TBD)
% 4.  lon   (x) dimension number (1,2,3,4) or 0 if does not exist
% 5.  lat   (y) dimention number (1,2,3,4) or 0 if does not exist
% 6.  level (z) dimension number (1,2,3,4) or 0 if does not exist
% 7.  time  (t) dimensoon number (1,2,3,4) or 0 if does not exist
% example1: if variable has size [128,64,17,365]  with dimensions (longitude,latitude,pressure,time)
% then lon dimension number = 1, lat dimension number  = 2, level dimension number = 3, time dimension number = 4
% example2: if variable has size [365,128,64]  with dimensions (time,longitude,latitude)
% then lon dimension number = 2, lat dimension number  = 3, level dimension number = 0, time dimension number = 1
% 8.  first year
% 9.  first month
% 10. first day
% 11. first hour
% 12. last year
% 13. last month
% 14. last day
% 15. last hour
% 16. stride. for example if temporal resolution of the field is 4XDaily but the nc file contains only 12:00 hours, stride = 4
% 17. (optional) linear folder name
% 18. (optional) 'assimilated' | 'analyzed' | 'forecast'
% 19. (optional) 'accumilated fields'

% linking Fields
Data_Link = link_root_dir ;
if ~exist(Data_Link,'dir')
    mkdir(Data_Link) ;
end
% Linked data is archived in the following way: CMIP5_Link/Experiment/Model/Frequency/Ensemble/Fields . . .
Errors = cell(1,3) ; er = 1 ;
[Errors(1,:)] = deal([{'Message'},{'Identifier'},{'Supporting data'}]) ;
Warnings = cell(1,3) ; wr = 1 ;
[Warnings(1,:)] = deal([{'Message'},{'Identifier'},{'Supporting data'}]) ;
Experiments     = unique(CB(2:end,22)) ;
EXT = 'ncf' ;
AttributesToInfo = {'title','institution','forcing','physics_version','comment','version','creation_date'} ;
tic ;
for e = 1:length(Experiments)
    CE = CB(strcmp(Experiments(e),CB(:,22)),:) ;
    Models  = unique(CE(:,25)) ;
    for m = 1:length(Models)
        CM = CE(strcmp(Models(m),CE(:,25)),:) ;
        TimeResolutions = unique(CM(:,1)) ;
        for t = 1:length(TimeResolutions)
            CT = CM(strcmp(TimeResolutions(t),CM(:,1)),:) ;
            if sum(strcmpi(TimeResolutions{t},{'Oyr','Oclim'}))
                Message = 'Unspported Frequency (at present, GOAT does not link to ''Oyr'', and ''Oclim'' frequencies)' ;
                Identifier = TimeResolutions{t} ;
                GOAT_Display_message(G,[Message,'  ',Identifier]) ;
                er = er + 1 ;
                Errors(er,1) = {Message} ;
                Errors(er,2) = {Identifier} ;
                Errors(er,3) = {[CB(1,:) ; CT]} ;
                continue ;
            end
            ENSEMBLES = unique(CT(:,23)) ;
            for r = 1:length(ENSEMBLES)
                Cr = CT(strcmp(ENSEMBLES(r),CT(:,23)),:) ;
                if strcmpi(ENSEMBLES(r),'None')
                    Ens = '' ;
                else
                    Ens = ENSEMBLES{r} ;
                end
                REALMS = unique(Cr(:,24)) ;
                for re = 1:length(REALMS)
                    C = Cr(strcmp(REALMS(re),Cr(:,24)),:) ;
                    % Create Info String
                    %save(fullfile(Data_Link,TimeResolutions{t},'info.mat'),'infostr') ;
                    % Identify Fields
                    Fields = unique(C(:,4)) ;
                    for f = 1:length(Fields)
                        CF = C(strcmp(Fields(f),C(:,4)),:) ;
                        TimeResolutionsField = CF(:,1);
                        FieldName = (Fields{f}) ;
                        if ismember(TimeResolutions{t},{'Daily','DailyX4','DailyX8'})
                            % Check for temporal consistency (no leap yeara . . .)
                            [YYY,X] = GOAT_ParseTimeResolution(TimeResolutions{t})  ;
                            for ly = 1:size(CF,1)
                                avar = ncinfo(CF{ly,3},Fields{f}) ;
                                TimeLength = avar.Dimensions(CF{ly,8}).Length ;
                                Sbeg = GOAT_getSerial(CF{ly, 9},CF{ly,10},CF{ly,11},CF{ly,12},TimeResolutions{t},CF{ly,9}) ;
                                Send = GOAT_getSerial(CF{ly,13},CF{ly,14},CF{ly,15},CF{ly,16},TimeResolutions{t},CF{ly,9}) ;
                                if Send - Sbeg + 1 ~= TimeLength
                                    CalendarOK = 0 ;
                                    % try Daily365XXX
                                    %disp(['timespan inconsistency in ',CF{ly,3}])
                                    TimeResolution365 = regexprep(TimeResolutions{t},'Daily','Daily365') ;
                                    Sbegly = GOAT_getSerial(CF{ly, 9},CF{ly,10},CF{ly,11},CF{ly,12},TimeResolution365,CF{ly,9}) ;
                                    Sendly = GOAT_getSerial(CF{ly,13},CF{ly,14},CF{ly,15},CF{ly,16},TimeResolution365,CF{ly,9}) ;
                                    if Sendly - Sbegly + 1 == TimeLength
                                        % disp(['found Non-LeapYear calendar in ',CF{ly,3}])
                                        % disp(['Saving Field as ',TimeResolution365])
                                        TimeResolutionsField(ly) = {TimeResolution365} ;
                                        CalendarOK = 1 ;
                                    end
                                    if ~CalendarOK
                                        % try Daily360 (as is the case in historical/HadGEM/day)
                                        TimeResolution360 = regexprep(TimeResolutions{t},'Daily','Daily360') ;
                                        Sbegly = GOAT_getSerial(CF{ly, 9},CF{ly,10},CF{ly,11},CF{ly,12},TimeResolution360,CF{ly,9}) ;
                                        Sendly = GOAT_getSerial(CF{ly,13},CF{ly,14},CF{ly,15},CF{ly,16},TimeResolution360,CF{ly,9}) ;
                                        if Sendly - Sbegly + 1 == TimeLength
                                            % disp(['found Non-LeapYear calendar in ',CF{ly,3}])
                                            % disp(['Saving Field as ',TimeResolution365])
                                            TimeResolutionsField(ly) = {TimeResolution360} ;
                                            CalendarOK = 1 ;
                                        end
                                    end
                                    if ~CalendarOK
                                        % try Julian 360
                                        TimeResolution360 = regexprep(TimeResolutions{t},'Daily','Julian360') ;
                                        Jd1 = GOAT_JulianDay(CF{ly, 9},CF{ly,10},CF{ly,11},TimeResolution360) ;
                                        Jd2 = GOAT_JulianDay(CF{ly,13},CF{ly,14},CF{ly,15},TimeResolution360) ;
                                        Sbegly = GOAT_getSerial(CF{ly, 9},0,Jd1,0,TimeResolution360,CF{ly,9}) ;
                                        Sendly = GOAT_getSerial(CF{ly,13},0,Jd2,0,TimeResolution360,CF{ly,9}) ;
                                        if Sendly - Sbegly + 1 == TimeLength
                                            %disp(['360-Calendar timespan in ',CF{ly,3}])
                                            %disp(['Saving Field as ',TimeResolution360])
                                            TimeResolutionsField(ly) = {TimeResolution360} ;
                                            CalendarOK = 1 ;
                                        end
                                    end
                                    if ~CalendarOK % Could Not identify Calendar
                                        Message = 'Link Aborted. Cannot identify calendar.' ;
                                        Identifier = CF{ly,3} ;
                                        GOAT_Display_message(G,[Message,'  ',Identifier]) ;
                                        er = er + 1 ;
                                        Errors(er,1) = {Message} ;
                                        Errors(er,2) = {Identifier} ;
                                        continue ;
                                    end
                                end
                            end
                            % If all DailyXXX are Non-Leap Years make them Daily365XXX as well
                            if length(unique(TimeResolutionsField)) > 1
                                for ly = 1:length(TimeResolutionsField)
                                    if ~strncmp(TimeResolutionsField(ly),'Daily365',8)
                                        %check if all years are non leap years
                                        lys = GOAT_leapyear(CF{ly,9}:CF{ly,13},TimeResolutionsField{ly}) ;
                                        if sum(lys) == 0 % No leap years
                                            % No harm done in joining non-leapyear files to Daily365XXX files
                                            TimeResolutionsField(ly) = {TimeResolution365} ;
                                        end
                                    end
                                end
                            end
                        end
                        % Check for spatial consistency
                        LonLength = zeros(size(CF,1),1) ;
                        for dl = 1:length(LonLength)
                            avar = ncinfo(CF{dl,3},FieldName) ;
                            LonLength(dl) = avar.Dimensions(CF{dl,5}).Length ;
                        end
                        if length(unique(LonLength)) > 1
                            Message = 'Inconsistent Dimensions. Spatial dimentions vary between files of same field.' ;
                            Identifier = fullfile(Experiments{e},Models{m},Ens,TimeResolutions{t},FieldName) ;
                            GOAT_Display_message(G,[Message,'  ',Identifier]) ;
                            er = er + 1 ;
                            Errors(er,1) = {Message} ;
                            Errors(er,2) = {Identifier} ;
                            Errors(er,3) = {[CB(1,:) ; CF]}      ;
                        end
                        
                        GOAT_Display_message(G,['linking experiment ',int2str(e),'/',int2str(length(Experiments)),' ',Experiments{e},' | model ',int2str(m),'/',int2str(length(Models)),' ',Models{m},' | ',Ens,'/',REALMS{re},'/',TimeResolutions{t},' field ',int2str(f),'/',int2str(length(Fields)),':  ',FieldName,' . . . '])
                        UniqueTimeResolutions = unique(TimeResolutionsField) ;
                        for df = 1:length(UniqueTimeResolutions)
                            CFU = CF(strcmp(UniqueTimeResolutions{df},TimeResolutionsField),:) ;
                            Units            = CFU{1,19} ;
                            switch CFU{1,24}
                                case 'atmos'
                                    VerticalOrdinate = 'Pressure (mb)' ;
                                otherwise
                                    VerticalOrdinate = 'Depth (m)' ;
                            end
                            FieldDescription = CFU{1,20} ;
                            % retrieve spatial vectors
                            filename = CFU{1,3} ;
                            XdimNum = CFU{1,5} ;
                            YdimNum = CFU{1,6} ;
                            ZdimNum = CFU{1,7} ;
                            DimsSize = sum(sign([XdimNum,YdimNum,ZdimNum])) ;
                            if ~exist(fullfile(Data_Link,Experiments{e},Models{m},REALMS{re},Ens,UniqueTimeResolutions{df},FieldName,'Dims.mat'),'file') ;
                                a = ncinfo(filename) ;
                                for j = 1:length(lon_dims)
                                    try
                                        lon = double(ncread(filename,lon_dims{j})) ;
                                        break ;
                                    catch
                                        % move on to next dim name
                                    end
                                end
                                for j = 1:length(lat_dims)
                                    try
                                        lat = double(ncread(filename,lat_dims{j})) ;
                                        break ;
                                    catch
                                        % move on to next dim name
                                    end
                                end
                                if ~any(size(lat)==1)
                                    UnstructuredGridFlag = 1 ; 
                                    % checking for meshed grid
                                    if lat(1,1) == lat(2,1) && lon(1,1) == lon(1,2)
                                        lat = lat(1,:) ; 
                                        lon = lon(:,1) ;
                                        UnstructuredGridFlag = 0 ; 
                                    elseif lat(1,1) == lat(1,2) && lon(1,1) == lon(2,1)
                                        lat = lat(:,1) ; 
                                        lon = lon(1,:) ;
                                        UnstructuredGridFlag = 0 ; 
                                    end
                                    % Assuming that if 2D grid matrices are not meshed, they are unstructured.
                                    if UnstructuredGridFlag
                                    Message = 'Error: Unstructured Grid: unstructured grid not supported by GOAT' ;
                                    Identifier = [Experiments{e},'/',Models{m},'/',Ens,'/',TimeResolutions{t},'/',FieldName] ;
                                    GOAT_Display_message(G,[Message,'  ',Identifier]) ;
                                    er = er + 1 ;
                                    Errors(er,1) = {Message} ;
                                    Errors(er,2) = {Identifier} ;
                                    Errors(er,3) = {[CB(1,:) ; CF]}      ;
                                    lat = [] ; lon = [] ;
                                    continue
                                    end
                                end
                                switch DimsSize
                                    case 2
                                        P = 1 ;
                                    otherwise
                                        for j = 1:length(lon_dims)
                                            try
                                                P = double(ncread(filename,level_dims{j})) ;
                                                aP = ncinfo(filename,level_dims{j}) ;
                                                if strcmpi(aP.Attributes(find(strcmpi({aP.Attributes.Name},'Units'))).Value,'Pa')
                                                    P = P/100 ;
                                                end
                                                break ;
                                            catch
                                                % move on to next dim name
                                            end
                                        end
                                        
                                end
                                if ~exist(fullfile(Data_Link,Experiments{e},Models{m},REALMS{re},Ens,UniqueTimeResolutions{df},FieldName),'dir')
                                    mkdir(fullfile(Data_Link,Experiments{e},Models{m},REALMS{re},Ens,UniqueTimeResolutions{df},FieldName)) ;
                                end
                                lev = P ;
                                save(fullfile(Data_Link,Experiments{e},Models{m},REALMS{re},Ens,UniqueTimeResolutions{df},FieldName,'Dims.mat'),'lev','lon','lat','Units','VerticalOrdinate','FieldDescription') ;
                            end
                            if ~exist(fullfile(Data_Link,Experiments{e},Models{m},REALMS{re},Ens,UniqueTimeResolutions{df},'info.mat'),'file') ;
                                a = ncinfo(filename) ;
                                infostr = '' ;
                                for i = 1:length(AttributesToInfo)
                                    for j = 1:length(a.Attributes)
                                        if strcmpi(a.Attributes(j).Name,AttributesToInfo{i})
                                            infostr = [infostr,'\n',AttributesToInfo{i},':'] ;
                                            infostr = [infostr,'\n',a.Attributes(j).Value] ;
                                        end
                                    end
                                end
                                try
                                    infostr = sprintf(infostr(3:end)) ;
                                catch
                                    % no attributes
                                end
                                save(fullfile(Data_Link,Experiments{e},Models{m},REALMS{re},Ens,UniqueTimeResolutions{df},'info.mat'),'infostr') ;
                            end
                            % move on to creating Attributes
                            dates1 = cell2mat(CFU(:,9 ))*1.e10 + cell2mat(CFU(:,10)).*1.e4 + cell2mat(CFU(:,11)).*1.e2 + cell2mat(CFU(:,12))  ;
                            dates2 = cell2mat(CFU(:,13))*1.e10 + cell2mat(CFU(:,14)).*1.e4 + cell2mat(CFU(:,15)).*1.e2 + cell2mat(CFU(:,16))  ;
                            [S, Si] = sort(dates1) ;
                            year1 = min(cell2mat(CFU(:,9 ))) ;
                            year2 = max(cell2mat(CFU(:,13))) ;
                            if strcmp(UniqueTimeResolutions{df},'List')
                                G.Addresses.Default      = cell(2,1)                   ;
                                G.Addresses.Default(1,1) = {fullfile('fx','data.mat')} ;
                                G.Addresses.Default(2,1) = {fullfile('List')}          ;
                            else
                                G = GOAT_Produce_DefaultAddress(G,year1,year2,UniqueTimeResolutions{df},EXT) ;
                            end
                            Addresses = G.Addresses.Default ;
                            Attributes = cell(length(Addresses),7) ;
                            % Attributes(i,1) = filename ;
                            % Attributes(i,2) = varname  ;
                            % Attributes(i,3) = start    ;
                            % Attributes(i,4) = count    ;
                            % Attributes(i,5) = factor   ;
                            % Attributes(i,6) = permute   ; < Normally, no need for CMIP5 variables
                            % Attributes(i,7) = reshape (add dimensions)  ; < Normally, no need for CMIP5 variables
                            % Assign Attributes to each date
                            for k = 1:length(Addresses) - 1 ;
                                if strcmp(UniqueTimeResolutions{df},'List')
                                    Idf = 1 ;
                                    S   = 1 ;
                                    Sf  = 1 ;
                                else
                                    [year,month,day,hour] = GOAT_serial2date(Addresses,k) ;
                                    DATE = year*1.e10 + month.*1.e4 + day.*1.e2 + hour  ;
                                    if DATE < min(dates1) ||DATE > max(dates2)
                                        continue ;
                                    end
                                    % find appropriate file
                                    Idf = find(dates1 <= DATE & dates2 >= DATE) ;
                                    if isempty(Idf)
                                        [Attributes(k,:)] = deal([{'No matching file'},CFU(1,4),{[]},{[]},{[]},{[]},{[]}]) ;
                                        continue ;
                                    elseif length(Idf) > 1
                                        %[Attributes(k,:)] = deal([{'More than one matching file'},CFU(1,4),{[]},{[]},{[]},{[]},{[]}]) ;
                                        Message = 'Warning: more than one matching file for date. Look in ''Attributes'' to determine which of the files with overlapping dates is used. ' ;
                                        Identifier = [int2str(DATE),'  ',Experiments{e},'/',Models{m},'/',Ens,'/',TimeResolutions{t},'/',FieldName] ;
                                        %GOAT_Display_message(G,[Message,Identifier]) ;
                                        wr = wr + 1 ;
                                        Warnings(wr,1) = {Message} ;
                                        Warnings(wr,2) = {Identifier} ;
                                        Warnings(wr,3) = {[CB(1,:) ; CFU(Idf,:)]}      ;
                                        Idf = Idf(1) ;
                                    end
                                    if strncmp(UniqueTimeResolutions{df},'Julian360',9)
                                        Jd = GOAT_JulianDay(year,month,day,TimeResolution360) ;
                                        S   = GOAT_getSerial(year,0,Jd,hour,UniqueTimeResolutions{df},year1) ;
                                        Jdf = GOAT_JulianDay(CFU{Idf,9},CFU{Idf,10},CFU{Idf,11},TimeResolution360) ;
                                        Sf  = GOAT_getSerial(CFU{Idf,9},0,Jdf,CFU{Idf,12},UniqueTimeResolutions{df},year1) ;
                                    else
                                        S  = GOAT_getSerial(year,month,day,hour,UniqueTimeResolutions{df},year1) ;
                                        Sf = GOAT_getSerial(CFU{Idf,9},CFU{Idf,10},CFU{Idf,11},CFU{Idf,12},UniqueTimeResolutions{df},year1) ;
                                    end
                                end
                                Attributes(k,1) = CFU(Idf,3) ;
                                Attributes(k,2) = CFU(Idf,4) ;
                                Attributes(k,5) = {1}       ;
                                dims = cell2mat(CFU(Idf,5:8)) ;
                                dims = dims(dims>0) ;
                                if ~(isequal(dims,[1 2 3]) || isequal(dims,[1 2 3 4]))
                                    %                                 Attributes(k,6) = {[]}   ;
                                    %                             else
                                    Attributes(k,6) = {dims}   ;
                                end
                                switch length(dims)
                                    case 1 % 1D
                                        start = 1 ;
                                        count = inf ;
                                    case 2 % lat,lon
                                        start = [1 1] ;
                                        count = [inf inf] ;
                                    case 3 % lat,lon,time
                                        start = [1 1 1] ;
                                        start(dims(3)) = (S-Sf)/CFU{Idf,17}+1 ;
                                        count = [1 1 1] ;
                                        count(dims(1)) = inf ;
                                        count(dims(2)) = inf ;
                                    case 4 % lat lon level time
                                        start = [1 1 1 1] ;
                                        start(dims(4)) = (S-Sf)/CFU{Idf,17}+1 ;
                                        count = [1 1 1 1] ;
                                        count(dims(1)) = inf ;
                                        count(dims(2)) = inf ;
                                        count(dims(3)) = inf ;
                                end
                                Attributes(k,3) = {start} ;
                                Attributes(k,4) = {count} ;
                            end
                            % Attributes(end,1) =
                            Attributes(end,1) = {EXT} ; % 'mat' | 'ncf' | 'dap' | 'gb1' | 'gb2'
                            Attributes(end,2) = {[year1 year2]} ;
                            Attributes(end,3) = {[]} ;
                            Attributes(end,4) = UniqueTimeResolutions(df) ;
                            % Compress Attributes
                            Attributes = GOAT_ALP_CompressAttributes(Attributes) ;
                            J = 1 ; clear Text
                            Text(J,1) = {['Variable name = ',FieldName]} ; J = J + 1 ;
                            Text(J,1) = {['Field Description = ',FieldDescription]} ; J = J + 1 ;
                            Text(J,1) = {['Units = ',Units]} ; J = J + 1 ;
                            Text(J,1) = {['LINK Type = ',EXT]} ; J = J + 1 ;
                            Text(J,1) = {['Calendar = ',UniqueTimeResolutions{df}]} ; J = J + 1 ;
                            Text(J,1) = {['First Year = ',int2str(year1)]} ; J = J + 1 ;
                            Text(J,1) = {['Last Year = ',int2str(year2)]} ; J = J + 1 ;
                            Text(J,1) = {['Experiment = ',Experiments{e}]} ; J = J + 1 ;
                            Text(J,1) = {['Model = ',Models{m}]} ; J = J + 1 ;
                            if ~isempty(Ens)
                                Text(J,1) = {['Ensemble = ',Ens]} ; J = J + 1 ;
                            end
                            Text(J,1) = {['Created by   GOAT_Script_MIP_LINK_DRS_1_3_1.m']} ; J = J + 1 ;
                            Text(J,1) = {['File list:']} ; J = J + 1 ;
                            Text(J:J+size(CFU,1)-1) = CFU(:,3) ;
                            if strcmpi(UniqueTimeResolutions{df},'List')
                                mkdir(fullfile(Data_Link,Experiments{e},Models{m},REALMS{re},Ens,UniqueTimeResolutions{df},FieldName,'fx')) ;
                                save(fullfile(Data_Link,Experiments{e},Models{m},REALMS{re},Ens,UniqueTimeResolutions{df},FieldName,'fx','link_field.mat'),'Attributes') ;
                                GOAT_cell2txt(fullfile(Data_Link,Experiments{e},Models{m},REALMS{re},Ens,UniqueTimeResolutions{df},FieldName,'fx','link_info.txt'),Text) ;
                            else
                                save(fullfile(Data_Link,Experiments{e},Models{m},REALMS{re},Ens,UniqueTimeResolutions{df},FieldName,'link_field.mat'),'Attributes') ;
                                GOAT_cell2txt(fullfile(Data_Link,Experiments{e},Models{m},REALMS{re},Ens,UniqueTimeResolutions{df},FieldName,'link_info.txt'),Text) ;
                            end
                            if strcmp(FieldName,'ps')
                                if ~exist(fullfile(Data_Link,Experiments{e},Models{m},REALMS{re},Ens,UniqueTimeResolutions{df},'BoerBeta'),'dir') ;
                                    mkdir(fullfile(Data_Link,Experiments{e},Models{m},REALMS{re},Ens,UniqueTimeResolutions{df},'BoerBeta')) ;
                                end
                                Attributes(end,1) = {'bns'} ;
                                save(fullfile(Data_Link,Experiments{e},Models{m},REALMS{re},Ens,UniqueTimeResolutions{df},'BoerBeta','link_field.mat'),'Attributes') ;
                            end
                        end
                    end
                end
            end
        end
    end
end

% after all field installations are done, locate BoerBeta Fields and set appropriate Dims
GOAT_Display_message(G,'setting Dims for BoerBeta functions . . .') ;
[folders] = GOAT_FileFolderTrees(Data_Link,1) ;

for i = 1:length(folders)
    [Path,Name] = fileparts(folders{i}) ;
    if strcmp(Name,'BoerBeta')
        fieldsdir = dir(Path) ;
        Fields = cell(1,1) ;
        NF = 0 ;
        for f = 1:length(fieldsdir)
            if fieldsdir(f,1).isdir && ~strncmp(fieldsdir(f,1).name,'.',1) && ~strcmp(fieldsdir(f,1).name,'BoerBeta')
                NF = NF + 1 ;
                Fields(NF,1) = {fieldsdir(f,1).name} ;
            end
        end
        % use lon and lat from surface pressure
        load(fullfile(Path,'ps','Dims.mat')) ;
        lat_ps = lat ; lon_ps = lon ; 
        % look for 3D fields
        OK = 0 ;
        for f = 1:length(Fields)
            clear lev P lon lat levlabel Plabel lonlabel latlabel Units VerticalOrdinate FieldDescription
            load(fullfile(Path,Fields{f},'Dims.mat')) ;
            if ~exist('lev','var')
                lev = P ; % compatibility with older GOAT versions
            end
            if length(lev) > 1 && max(lev) > 1.01 %exclude 3D sigma coordinate variables
                lat = lat_ps ; 
                lon = lon_ps ; 
                FieldDescription = 'Boer Beta function (0/1 below/above surface pressure level)' ;
                Units = '1' ;
                levlabel = 'Pressure (mb)' ; 
                try
                    save(fullfile(Path,'BoerBeta','Dims.mat'),'lev','lon','lat','levlabel','latlabel','lonlabel','Units','VerticalOrdinate','FieldDescription') ;
                catch
                    save(fullfile(Path,'BoerBeta','Dims.mat'),'lev','lon','lat','Units','VerticalOrdinate','FieldDescription') ;
                end
                OK = 1 ;
                break ;
            end
        end
        if ~OK
            GOAT_Display_message(G,['could not determine appropriate Dims.mat for ',folders{i}])
            % No 3D fields --> remove Boer Beta Function
            %rmdir(folders{i},'s') ;
        end
    end
end


%
t = toc ;
h = floor(toc/3600) ;
m = floor((t - h*3600)/60) ;
s = t - h*3600 - m*60 ;
if h > 0
    TimeString = [int2str(h),' hours, ',int2str(m),' minutes and ',int2str(s),' seconds.'] ;
elseif m > 0
    TimeString = [int2str(m),' minutes and ',int2str(s),' seconds.'] ;
elseif s > 0
    TimeString = [int2str(s),' seconds.'] ;
end
GOAT_Display_message(G,['Linking MIP archive completed in ',TimeString]) ;
c = clock ;
Today = date ;
TimeStamp = [Today,'-',int2str(c(4)),'-',int2str(c(5))] ;
% Errors and Warnings Report
UniqueErrors = unique(Errors(:,1)) ;
UniqueWarnings = unique(Warnings(:,1)) ;
if size(UniqueErrors,1) > 1 && size(UniqueWarnings,1) > 1
    GOAT_Display_message(G,['LINK complete in ',TimeString,'. A total of ',int2str(size(UniqueErrors,1)-1),' Errors and ',int2str(size(UniqueWarnings,1)-1),' Warnings found. See Error report at ',fullfile(Data_Link,['LINK_Report_',TimeStamp,'.mat'])]) ;
    save(fullfile(Data_Link,['LINK_Report_',TimeStamp,'.mat']),'Errors','Warnings') ;
elseif size(UniqueErrors,1) > 1
    GOAT_Display_message(G,['LINK complete in ',TimeString,'. A total of ',int2str(size(UniqueErrors,1)-1),' Errors found. See Error report at ',fullfile(Data_Link,['LINK_Report_',TimeStamp,'.mat'])]) ;
    save(fullfile(Data_Link,['LINK_Report_',TimeStamp,'.mat']),'Errors') ;
elseif size(UniqueWarnings,1) > 1
    GOAT_Display_message(G,['LINK complete in ',TimeString,'. A total of ',int2str(size(UniqueWarnings,1)-1),' Warnings found. See Error report at ',fullfile(Data_Link,['LINK_Report_',TimeStamp,'.mat'])]) ;
    save(fullfile(Data_Link,['LINK_Report_',TimeStamp,'.mat']),'Warnings') ;
else
    GOAT_Display_message(G,['LINK complete in ',TimeString]) ;
end

return

function G = LINK_GUI(G)

G.LINK.StaticLink = fullfile(fileparts(fileparts(which('GOAT.m'))),'Data_LINK','MIP_LINK') ;
G.LINK.MIPFolder = fullfile(fileparts(cd),'MIP_Folder_Address') ;
G.LINK.CancelFlag = 0 ;
pos        = G.GUI.pos.Export ;
G.LINK.Fig = figure('name','Link MIP netCDF files','position',pos,'color',G.Display.FigBackgroudColor) ;
set(G.LINK.Fig,'MenuBar','none')
NumLines = 6 ;
bl = 0.02 ; % Left Edge
bt = 0.05; % Top Edge
LineHeight = (1-2*bt)/NumLines ;
Column     = (1-2*bl)/1 ;
%Top Message
messageStr = {'LINK MIP NetCDF files.' ; 'Note: this script will end the current GOAT session.'} ;
G.h_message = uicontrol('Style', 'text','Units','normalized','pos',[0,1-LineHeight*.75,1,LineHeight*.75], 'string',messageStr);
uicontrol(G.h_message) ;  set(G.h_message,'ForegroundColor',G.GUI.messageColor,'FontSize',G.Display.TextSize_S,'FontName',G.GUI.Fonts.MessageFont) ;
% title
Line = 2 ;
h_NameT = uicontrol('Style', 'text','Units','normalized','pos',[bl,1-bt-Line*LineHeight,Column/5,LineHeight], 'string','NetCDF MIP directory');
uicontrol(h_NameT) ;  set(h_NameT,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',G.GUI.HeaderColor,'FontSize',G.Display.TextSize_L,'HorizontalAlignment','left','FontName',G.GUI.Fonts.uiHeaderFont) ;
Line = 2 ;
G.LINK.h_Name = uicontrol('style','edit','Units','normalized','pos',[bl*2+Column/5,1-bt-Line*LineHeight,Column*3/5-bl,LineHeight],...
    'string',G.LINK.MIPFolder,  'callback','set(gcbf,''userdata'',11); uiresume;');
uicontrol(G.LINK.h_Name) ;  set(G.LINK.h_Name,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',12,'FontName',G.GUI.Fonts.uiButtonFont,'HorizontalAlignment','left','TooltipString','CMIP5 NetCDF files in this directory and its subdirectories will be LINKed.') ;
% Browse
G.LINK.h_Browse = uicontrol('Units','normalized','pos',[bl + Column*4/5,1-bt-Line*LineHeight,Column/5 ,LineHeight],...
    'string','Browse',  'callback','set(gcbf,''userdata'',112); uiresume;');
uicontrol(G.LINK.h_Browse) ;  set(G.LINK.h_Browse,'BackgroundColor',G.GUI.ButtonColor,'ForegroundColor',G.GUI.DropTextColor,'FontSize',G.Display.TextSize_L,'fontweight','bold','FontName',G.GUI.Fonts.uiButtonFont) ;
% title
Line = 4 ;
h_NameT = uicontrol('Style', 'text','Units','normalized','pos',[bl,1-bt-Line*LineHeight,Column/5,LineHeight], 'string','LINK directory');
uicontrol(h_NameT) ;  set(h_NameT,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',G.GUI.HeaderColor,'FontSize',G.Display.TextSize_L,'HorizontalAlignment','left','FontName',G.GUI.Fonts.uiHeaderFont) ;
Line = 4 ;
G.LINK.h_Name2 = uicontrol('style','edit','Units','normalized','pos',[bl*2+Column/5,1-bt-Line*LineHeight,Column*3/5-bl,LineHeight],...
    'string',G.LINK.StaticLink,  'callback','set(gcbf,''userdata'',21); uiresume;');
ToolTip = sprintf(['Preferably a static directory. ', ...
                '\nAll LINKs will be arranged in this directory according to:', ...
                '\nExperiment   /   Realm   /   Model   /   Ensemble   /   Calendar   /   Field' , ... 
                '\nThe Ensemble directory should be used as the ''GOAT Dataset'' directory']) ; 
uicontrol(G.LINK.h_Name2) ;  set(G.LINK.h_Name2,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',12,'FontName',G.GUI.Fonts.uiButtonFont,'HorizontalAlignment','left','TooltipString',ToolTip) ;
% Browse
G.LINK.h_Browse2 = uicontrol('Units','normalized','pos',[bl + Column*4/5,1-bt-Line*LineHeight,Column/5 ,LineHeight],...
    'string','Browse',  'callback','set(gcbf,''userdata'',212); uiresume;');
uicontrol(G.LINK.h_Browse2) ;  set(G.LINK.h_Browse2,'BackgroundColor',G.GUI.ButtonColor,'ForegroundColor',G.GUI.DropTextColor,'FontSize',G.Display.TextSize_L,'fontweight','bold','FontName',G.GUI.Fonts.uiButtonFont) ;


% Cancel
Line = NumLines  ;
h_Cancel = uicontrol('Units','normalized','pos',[bl*2 + Column*2/3,1-bt-Line*LineHeight,Column/3 - bl,LineHeight],...
    'string','Cancel',  'callback','set(gcbf,''userdata'',99); uiresume;');
uicontrol(h_Cancel) ;  set(h_Cancel,'BackgroundColor',G.GUI.ButtonColor,'ForegroundColor',G.GUI.DropTextColor,'FontSize',G.Display.TextSize_L,'fontweight','bold','FontName',G.GUI.Fonts.uiButtonFont) ;
% OK
Line = NumLines  ;
h_OK = uicontrol('Units','normalized','pos',[bl,1-bt-Line*LineHeight,Column*2/3,LineHeight],...
    'string','OK',  'callback','set(gcbf,''userdata'',100); uiresume;');
uicontrol(h_OK) ;  set(h_OK,'BackgroundColor',G.GUI.colors.celeste,'ForegroundColor',G.GUI.DropTextColor,'FontSize',16,'fontweight','bold','FontName',G.GUI.Fonts.uiButtonFont) ;


GUIcon = 1 ;
while GUIcon
    try
        set(0,'CurrentFigure',G.LINK.Fig) ;
        uiwait(gcf);
        select = get(gcf,'userdata');
    catch
        GUIcon = 0 ;
    end
    try
        switch select
            case 11 % Folder name of export
                G.LINK.MIPFolder          = get(G.LINK.h_Name,'string')            ;
            case 112 % Browse
                try
                    ExportAddress = uigetdir ;
                    try
                        if ExportAddress == 0
                            continue ;
                        end
                    end
                    G.LINK.MIPFolder = ExportAddress ;
                    set(G.LINK.h_Name,'string',G.LINK.MIPFolder)
                catch
                    % Nevermind
                end
            case 21 % Folder name of export
                G.LINK.StaticLink          = get(G.LINK.h_Name2,'string')            ;
            case 212 % Browse
                try
                    StaticAddress = uigetdir ;
                    try
                        if StaticAddress == 0
                            continue ;
                        end
                    end
                    G.LINK.StaticLink = StaticAddress ;
                    set(G.LINK.h_Name2,'string',G.LINK.StaticLink)
                catch
                    % Nevermind
                end
            case 99 % Clear
                G.LINK.CancelFlag = 1 ;
                GUIcon = 0 ;
            case 100 % OK
                GUIcon = 0 ;
        end
    catch
        %nevermind
    end
end

%GOAT_Display_message(G,'','init') ;
if ~G.LINK.CancelFlag
    %GOAT_Display_message(G,'Preparing to linked MIP archive . . .        ') ;
    %set(h_OK,'Enable','off') ;
    %set(h_Cancel,'Enable','off') ;
    try
        close(G.LINK.Fig) ; drawnow
    end
    try
        close(G.GOAT_Fig) ; drawnow
    end
    %set(0,'CurrentFigure',G.LINK.Fig) ;
    %drawnow ;
else
    try
        close(G.LINK.Fig) ; drawnow
    end
end

