% Generic code for creating derived 20CR fields. Field specifications need to be specified in t
% 1. Dims specifications (search 'Dims specifications')
% 2. data frame calculation (search 'data frame calculation')

G = GOAT_Environment ; % Initiate GOAT Environment
Calendar = 'Daily' ; 
DataFolder = '/Volumes/data3/GOAT/Data/R20C' ;         
Field = 'MSEI' ; 
year1  = 1871 ; % yyyy
month1 = 1    ; % 1:12
day1   = 1    ; % 1:31 
hour1  = 0    ; % 0:3:21
year2  = 2012 ; 
month2 = 12   ; 
day2   = 31   ; 
hour2  = 18   ; 
G = GOAT_Initialize_Dataset(G,DataFolder,Calendar) ; % Go through contents of Data folder and Calendar. Link existing fields, temporal span, etc.
% Dims specifications
switch Field
    case 'MSEI'    
        G = GOAT_Initialize_Field(G,'ta')        ; % get spatial and temporal dimensions of field, display settings, topography, and file addresses. Link if necessary
        lon = G.lon ;
        lat = G.lat ;
        lev = 1 ;
        VerticalOrdinate = 'Pressure (mb)' ;
        Units = 'J' ;
        FieldDescription = 'Vertically integrated Moist Static Energy' ;
    case 'LEIv'    
        G = GOAT_Initialize_Field(G,'ta')        ; % get spatial and temporal dimensions of field, display settings, topography, and file addresses. Link if necessary
        lon = G.lon ;
        lat = G.lat ;
        lev = 1 ;
        VerticalOrdinate = 'Pressure (mb)' ;
        Units = 'J' ;
        FieldDescription = 'Vertically integrated Meridional Latent Energy Flux' ;    
    case 'MKEI'    
        G = GOAT_Initialize_Field(G,'ta')        ; % get spatial and temporal dimensions of field, display settings, topography, and file addresses. Link if necessary
        lon = G.lon ;
        lat = G.lat ;
        lev = 1 ;
        VerticalOrdinate = 'Pressure (mb)' ;
        Units = 'J' ;
        FieldDescription = 'Vertically integrated Moist Kinetic Energy' ;
    case 'MSEIv'
        G = GOAT_Initialize_Field(G,'ta')        ; % get spatial and temporal dimensions of field, display settings, topography, and file addresses. Link if necessary
        lon = G.lon ;
        lat = G.lat ;
        lev = 1 ;
        VerticalOrdinate = 'Pressure (mb)' ;
        Units = 'W/m' ;
        FieldDescription = 'Vertically integrated Meridional Moist Static Energy flux' ;
    case 'tph'
        G = GOAT_Initialize_Field(G,'ta')        ; % get spatial and temporal dimensions of field, display settings, topography, and file addresses. Link if necessary
        lon = G.lon ;
        lat = G.lat ;
        lev = 1 ;
        VerticalOrdinate = 'Pressure (mb)' ;
        Units = 'Pa' ;
        FieldDescription = 'Tropopause Height' ;    
end
mkdir(fullfile(DataFolder,Calendar,Field)) ;
save(fullfile(DataFolder,Calendar,Field,'Dims.mat'),'lon','lat','lev','FieldDescription','Units','VerticalOrdinate') ;
% Temporal Checkboxes/Flags
G.Limits.Default.Day2same   = 0 ; % 1/0 Set Date 2 same as Date 1
G.Limits.Default.Sequential = 1 ; % 1/0 Sequential or Seasonal averaging Flag
G.Limits.Default.ClimatologyContinuous = 0 ; % present seasonal data sequentially in Hovmoller plots
G.Limits.Default.Anom  = 0      ; % display anomalies in Hovmoller plots
% G.Limits.S1 and G.Limits.S1 are the temporal indices used by GOAT to locate files in the GOAT archiving standard
G.Limits.S1 = GOAT_getSerial(year1,month1,day1,hour1,G.TimeFieldVirtual,G.Limits.Year0) ; 
G.Limits.S2 = GOAT_getSerial(year2,month2,day2,hour2,G.TimeFieldVirtual,G.Limits.Year0) ; 
G.ProjectionType = 1 ; % 1) lat-lon with coast 2) lat-lon 3) lat-height 4)lon-height 5) hovmoller-lon 6) hovmoller-lat 7) hovmoller height 8/9) NH/SH polar stereographic
% Filters, Weights and Indices: 
% No Filters, Weights or Indices Selected 
G.Filter.Index = ones(length(G.Addresses.Default),1) ; % default index
G.DataStringVec = GOAT_Data_string_vec(G)                      ; % A vecotor of the Time Series File Indices
% retrieve and compile data 
NF = length(G.DataStringVec) ; 
Nmod = ceil(NF/1000) ;
GOAT_Display_message(G,'','init') ;
zeros_hus = zeros(length(G.lon),length(G.lat),24-19) ; 
%
for i = 1:NF
    
if mod(i,Nmod) == 0
   GOAT_Display_message(G,['Creating ',Field,' . . .   ',num2str(i/NF*100,'%03.1f'),' %'],'stat')
end
datafile = cell2mat(G.Addresses.Default(G.DataStringVec(i))) ;
if ~exist(fullfile(DataFolder,Calendar,Field,fileparts(datafile)),'dir')
    mkdir(fullfile(DataFolder,Calendar,Field,fileparts(datafile)))
end
% data frame calculation
switch Field
    case 'MSEIv'
        [ta  ,G] = GOAT_LoadData(G,DataFolder,Calendar,'ta',datafile,i) ;
        [hus ,G] = GOAT_LoadData(G,DataFolder,Calendar,'hus',datafile,i)   ; hus  = cat(3,hus,zeros_hus) ; 
        [zg  ,G] = GOAT_LoadData(G,DataFolder,Calendar,'zg',datafile,i)  ;
        [va  ,G] = GOAT_LoadData(G,DataFolder,Calendar,'va',datafile,i) ;
        bata = GOAT_load_bata(fullfile(DataFolder,Calendar,'BoerBeta',datafile))  ;
        L = GOAT_Calculate_L(ta,G,'era') ; % Calculate Latent heat of vaporization
        MSE = G.Consts.SpHeatConstPr*ta + zg*G.Consts.GravitationalConst + L.*hus ;
        data = single(sign(G.lev(end)-G.lev(1))*trapz(G.lev*100/G.Consts.GravitationalConst,va.*MSE.*bata,3)) ;
    case 'LEIv'
        [ta  ,G] = GOAT_LoadData(G,DataFolder,Calendar,'ta',datafile,i) ;
        [hus ,G] = GOAT_LoadData(G,DataFolder,Calendar,'hus',datafile,i)   ; hus  = cat(3,hus,zeros_hus) ; 
        [va  ,G] = GOAT_LoadData(G,DataFolder,Calendar,'va',datafile,i) ;
        bata = GOAT_load_bata(fullfile(DataFolder,Calendar,'BoerBeta',datafile))  ;
        L = GOAT_Calculate_L(ta,G,'era') ; % Calculate Latent heat of vaporization
        LE =  L.*hus ;
        data = single(sign(G.lev(end)-G.lev(1))*trapz(G.lev*100/G.Consts.GravitationalConst,va.*LE.*bata,3)) ;    
    case 'MSEI'
        [ta  ,G] = GOAT_LoadData(G,DataFolder,Calendar,'ta',datafile,i) ;
        [hus ,G] = GOAT_LoadData(G,DataFolder,Calendar,'hus',datafile,i)   ;hus  = cat(3,hus,zeros_hus) ;
        [zg  ,G] = GOAT_LoadData(G,DataFolder,Calendar,'zg',datafile,i)  ;
        bata = GOAT_load_bata(fullfile(DataFolder,Calendar,'BoerBeta',datafile))  ;
        L = GOAT_Calculate_L(ta,G,'era') ; % Calculate Latent heat of vaporization
        MSE = G.Consts.SpHeatConstPr*ta + zg*G.Consts.GravitationalConst + L.*hus ;
        data = single(sign(G.lev(end)-G.lev(1))*trapz(G.lev*100/G.Consts.GravitationalConst,MSE.*bata,3)) ;
     case 'MKEI'
        [ta  ,G] = GOAT_LoadData(G,DataFolder,Calendar,'ta',datafile,i) ;
        [hus ,G] = GOAT_LoadData(G,DataFolder,Calendar,'hus',datafile,i)   ;hus  = cat(3,hus,zeros_hus) ;
        [zg  ,G] = GOAT_LoadData(G,DataFolder,Calendar,'zg',datafile,i)  ;
        [ua  ,G] = GOAT_LoadData(G,DataFolder,Calendar,'ua',datafile,i) ;
        [va  ,G] = GOAT_LoadData(G,DataFolder,Calendar,'va',datafile,i) ;
        bata = GOAT_load_bata(fullfile(DataFolder,Calendar,'BoerBeta',datafile))  ;
        L = GOAT_Calculate_L(ta,G,'era') ; % Calculate Latent heat of vaporization
        MKE = G.Consts.SpHeatConstPr*ta + zg*G.Consts.GravitationalConst + L.*hus + (ua.^2 + va.^2)/2 ;
        data = single(sign(G.lev(end)-G.lev(1))*trapz(G.lev*100/G.Consts.GravitationalConst,MKE.*bata,3)) ;   
     case 'tph'
        [ta  ,G] = GOAT_LoadData(G,DataFolder,Calendar,'ta',datafile,i) ;
        data = single(GOAT_Analysis_Tropopause_Height(ta,G.lev,G)) ; % Calculate tropopause height (Pa)   
end

save(fullfile(DataFolder,Calendar,Field,datafile),'data') ;
end

disp([Field,' ',Calendar,' calculation complete!'])