% GOAT_ALP_ERA40_LINK link code for ERA-40 Daily + Monthly via apdrc

G = GOAT_Environment ;
link_dir = fullfile(fileparts(which('GOAT.m')),'Utils','DataManagementCode','ALP','ERA40') ; % MERRA ALP code folder
Data_ALP = fullfile(fileparts(fileparts(which('GOAT.m'))),'Data_LINK') ; % LINK Data folder

file_root = {'http://apdrc.soest.hawaii.edu:80/dods/public_data/ERA-40/daily-pressure','Daily' ; ...
    'http://apdrc.soest.hawaii.edu:80/dods/public_data/ERA-40/daily-surface'          ,'Daily' ; ...
    'http://apdrc.soest.hawaii.edu:80/dods/public_data/ERA-40/monthly-pressure'       ,'Monthly' ; ...
    'http://apdrc.soest.hawaii.edu:80/dods/public_data/ERA-40/monthly-surface'        ,'Monthly' ...
    } ;



% Units and Name Conversion Array
CV = cell(1,5) ; qV = 1 ;
[CV(qV,:)] = deal([{'name'}    ,{'stndrd_name'},{'Factor'},{'Units'}     ,{'Description'}])                   ; qV = qV + 1 ;
[CV(qV,:)] = deal([{'hgt'}     ,{'zg'}         ,{1/9.81}  ,{'m'}         ,{'Geopotential'}])                  ; qV = qV + 1 ;
[CV(qV,:)] = deal([{'air'}     ,{'ta'}         ,{1}       ,{'K'}         ,{'Temperature'}])                   ; qV = qV + 1 ;
[CV(qV,:)] = deal([{'uwnd'}    ,{'ua'}         ,{1}       ,{'m/s'}       ,{'Zonal velocity'}])                ; qV = qV + 1 ;
[CV(qV,:)] = deal([{'vwnd'}    ,{'va'}         ,{1}       ,{'m/s'}       ,{'Meridional velocity'}])           ; qV = qV + 1 ;
[CV(qV,:)] = deal([{'omega'}   ,{'wap'}        ,{1}       ,{'Pa/s'}      ,{'Vertical pressure velocity'}])    ; qV = qV + 1 ;
[CV(qV,:)] = deal([{'ozone'}   ,{'o3'}         ,{1}       ,{'kg/kg'}     ,{'Ozone mass mixing ratio'} ])            ; qV = qV + 1 ;
[CV(qV,:)] = deal([{'shum'}    ,{'hus'}        ,{1}       ,{'kg/kg'}     ,{'Specific humidity'} ])            ; qV = qV + 1 ;
[CV(qV,:)] = deal([{'rhum'}    ,{'hur'}        ,{1}       ,{'%'}         ,{'Specific humidity'} ])            ; qV = qV + 1 ;
[CV(qV,:)] = deal([{'pvor'}    ,{'pvor'}       ,{1}       ,{'Km^2/kg/s'} ,{'Potential vorticity'}])           ; qV = qV + 1 ;
[CV(qV,:)] = deal([{'vor'}     ,{'vor'}        ,{1}       ,{'1/s'}       ,{'Vorticity'}])                     ; qV = qV + 1 ;
[CV(qV,:)] = deal([{'div'}     ,{'div'}        ,{1}       ,{'1/s'}       ,{'Divergence'}])                     ; qV = qV + 1 ;
[CV(qV,:)] = deal([{'param01'} ,{'blh'}        ,{1}       ,{'m'}         ,{'Boundary layer height'}])         ; qV = qV + 1 ;
[CV(qV,:)] = deal([{'param02'} ,{'pr_convective'},{1}     ,{'m of water'},{'Convective precipitation'}])      ; qV = qV + 1 ;
[CV(qV,:)] = deal([{'param03'} ,{'er'}        ,{1}        ,{'m of water'},{'Evaporation'}])                   ; qV = qV + 1 ;
[CV(qV,:)] = deal([{'param04'} ,{'clch'}      ,{1}        ,{'1'}         ,{'High cloud cover'}])              ; qV = qV + 1 ;
[CV(qV,:)] = deal([{'param05'} ,{'icet1'}     ,{1}        ,{'K'}         ,{'ice surface temperature layer 1'}]); qV = qV + 1 ;
[CV(qV,:)] = deal([{'param06'} ,{'ugws'}      ,{1}        ,{'Ns/m^2'}    ,{'Longitudinal component of gravity wave stress'}]); qV = qV + 1 ;
[CV(qV,:)] = deal([{'param07'} ,{'hfls'}      ,{1}        ,{'Ws/m^2'}    ,{'Surface latent heat flux'}])      ; qV = qV + 1 ;
[CV(qV,:)] = deal([{'param08'} ,{'vgws'}      ,{1}        ,{'Ns/m^2'}    ,{'Meridional component of gravity wave stress'}]); qV = qV + 1 ;
[CV(qV,:)] = deal([{'param09'} ,{'clcl'}      ,{1}        ,{'1'}         ,{'Low cloud cover'}])               ; qV = qV + 1 ;
[CV(qV,:)] = deal([{'param10'} ,{'clcm'}      ,{1}        ,{'1'}         ,{'Medium cloud cover'}])            ; qV = qV + 1 ;
[CV(qV,:)] = deal([{'param11'} ,{'runoff'}    ,{1}        ,{'m'}         ,{'Runoff'}])                        ; qV = qV + 1 ;
[CV(qV,:)] = deal([{'param12'} ,{'sicec'}     ,{1}        ,{'1'}         ,{'Sea ice cover'}])                 ; qV = qV + 1 ;
[CV(qV,:)] = deal([{'param13'} ,{'hfss'}      ,{1}        ,{'Ws/m^2'}    ,{'Surface sensible heat flux'}])    ; qV = qV + 1 ;
[CV(qV,:)] = deal([{'param14'} ,{'slp'}       ,{1}        ,{'Pa'}        ,{'Mean sea level pressure'}])       ; qV = qV + 1 ;
[CV(qV,:)] = deal([{'param15'} ,{'snd'}       ,{1}        ,{'m'}         ,{'Snow depth'}])                    ; qV = qV + 1 ;
[CV(qV,:)] = deal([{'param16'} ,{'snf'}       ,{1}        ,{'m'}         ,{'Snowfall (convective + stratiform)'}]); qV = qV + 1 ;
[CV(qV,:)] = deal([{'param17'} ,{'st1'}       ,{1}        ,{'K'}         ,{'Soil temperature level 1'}])      ; qV = qV + 1 ;
[CV(qV,:)] = deal([{'param18'} ,{'rsnscs'}    ,{1}        ,{'W/m^2'}     ,{'Surface net solar radiation, clear sky'}]); qV = qV + 1 ;
[CV(qV,:)] = deal([{'param19'} ,{'rsns'}      ,{1}        ,{'Ws/m^2'}    ,{'Surface solar radiation'}])       ; qV = qV + 1 ;
[CV(qV,:)] = deal([{'param20'} ,{'rsds'}      ,{1}        ,{'Ws/m^2'}    ,{'Surface solar radiation downwards'}]); qV = qV + 1 ;
[CV(qV,:)] = deal([{'param21'} ,{'pr_strat'}  ,{1}        ,{'m of water'},{'Stratiform precipitation (large-scale precip)'}]); qV = qV + 1 ;
[CV(qV,:)] = deal([{'param22'} ,{'uss'}       ,{1}        ,{'Ns/m^2'}    ,{'East-west surface stress'}])      ; qV = qV + 1 ;
[CV(qV,:)] = deal([{'param23'} ,{'vss'}       ,{1}        ,{'Ns/m^2'}    ,{'Morth-south surface stress'}])    ; qV = qV + 1 ;
[CV(qV,:)] = deal([{'param24'} ,{'ta2m'}      ,{1}        ,{'K'}         ,{'2m temperature'}])                ; qV = qV + 1 ;
[CV(qV,:)] = deal([{'param25'} ,{'rlnscs'}    ,{1}        ,{'W/m^2'}     ,{'Surface net thermal radiation, clear sky'}]); qV = qV + 1 ;
[CV(qV,:)] = deal([{'param26'} ,{'rlns'}      ,{1}        ,{'Ws/m^2'}    ,{'Surface thermal radiation'}])     ; qV = qV + 1 ;
[CV(qV,:)] = deal([{'param27'} ,{'rlds'}      ,{1}        ,{'Ws/m^2'}    ,{'Surface thermal radiation downwards'}]); qV = qV + 1 ;
[CV(qV,:)] = deal([{'param28'} ,{'rsntcs'}    ,{1}        ,{'W/m^2'}     ,{'Top net solar radiation, clear sky'}]); qV = qV + 1 ;
[CV(qV,:)] = deal([{'param29'} ,{'rsnt'}      ,{1}        ,{'Ws/m^2'}    ,{'Top solar radiation'}])           ; qV = qV + 1 ;
[CV(qV,:)] = deal([{'param30'} ,{'rlntcs'}    ,{1}        ,{'W/m^2'}     ,{'Top net thermal radiation, clear sky'}]); qV = qV + 1 ;
[CV(qV,:)] = deal([{'param31'} ,{'rlnt'}      ,{1}        ,{'Ws/m^2'}    ,{'Top thermal radiation'}])         ; qV = qV + 1 ;
[CV(qV,:)] = deal([{'param32'} ,{'tco3'}      ,{1}        ,{'kg/m^2'}    ,{'Total column ozone'}])            ; qV = qV + 1 ;
[CV(qV,:)] = deal([{'param33'} ,{'tcw'}       ,{1}        ,{'kg/m^2'}    ,{'Total column water'}])            ; qV = qV + 1 ;
[CV(qV,:)] = deal([{'param34'} ,{'tcwv'}      ,{1}        ,{'kg/m^2'}    ,{'Total column water vapor'}])      ; qV = qV + 1 ;
[CV(qV,:)] = deal([{'param35'} ,{'clc'}       ,{1}        ,{'1'}         ,{'Total cloud cover'}])             ; qV = qV + 1 ;
[CV(qV,:)] = deal([{'param36'} ,{'ua10'}      ,{1}        ,{'m/s'}       ,{'10m Zonal velocity'}])            ; qV = qV + 1 ;
[CV(qV,:)] = deal([{'param37'} ,{'vswl1'}     ,{1}        ,{'m^3/m^3'}   ,{'Volumetric soil water layer 1'}]) ; qV = qV + 1 ;
[CV(qV,:)] = deal([{'param38'} ,{'va10'}      ,{1}        ,{'m/s'}       ,{'10m Meridional wind'}])           ; qV = qV + 1 ;
[CV(qV,:)] = deal([{'param39'} ,{'mwd'}       ,{1}        ,{'degrees'}   ,{'Mean wave direction'}])           ; qV = qV + 1 ;
[CV(qV,:)] = deal([{'param40'} ,{'swh'}       ,{1}        ,{'m'}         ,{'Significant wave height'}])       ; qV = qV + 1 ;
[CV(qV,:)] = deal([{'param41'} ,{'mwp'}       ,{1}        ,{'s'}         ,{'mean wave period'}])              ; qV = qV + 1 ;

totaltoc = 0 ;
Ext = 'dap' ;
longitude_dimname = 'lon' ;
latitude_dimname  = 'lat' ;
level_dimname     = 'lev' ;
time_dimname      = 'time';
today = date ;
First_Year = 1957 ;
First_Month = 9 ;
First_Day = 1 ;
First_Hour = 0 ;
Last_Year = 2002 ;
Last_Month = 8 ;
Last_Day = 31 ;
Last_Hour = 0 ;
%
for s = 1:size(file_root,1)
    tic
    Data_LINK = fullfile(Data_ALP,'ERA40_LINK',file_root{s,2}) ;
    infostr = sprintf(['About ERA-40 (source: UCAR Climate Data Guide)', ...
        '\nERA 40 is a second generation reanalysis. ', ...
        '\nIt is the first reanalysis to directly assimilate satellite radiance data (TOVS, SSM/I, ERS and ATOVS). ', ...
        '\nCloud Motion Winds are also used. ', ...
        '\nThe result is better circulation over the tropics and southern hemisphere. ', ...
        '\nHowever, unless comparing  to previous ERA-40 based results, '  , ...
        '\nit is recommended that the 3rd generation reanalyses, ' , ...
        '\nERA-Interim or MERRA, be used for new research.' ...
        ]) ;
    
    
    mkdir(Data_LINK) ;
    save(fullfile(Data_LINK,'info.mat'),'infostr')
    opendap_address = file_root{s,1} ;
    a = ncinfo(opendap_address) ;
    
    VerticalOrdinate = 'Pressure (mb)' ;
    
    disp(['Building link table ',file_root{s,1},' . . .'])
    year1 = First_Year ; month1 = First_Month ; day1 = First_Day ; hour1 = First_Hour ;
    year2 = Last_Year  ; month2 = Last_Month  ; day2 = Last_Day  ;
    switch file_root{s,2}
        case {'Monthly','Daily'}
            hour2 = 00 ;
        case 'DailyX4'
            hour2 = 18 ;
        case 'DailyX8'
            hour2 = 21 ;
    end
    
    varlist = cell(1,1) ;
    q = 0 ;
    for i = 1:length(a.Variables)
        if ~ismember(a.Variables(i).Name,{longitude_dimname,latitude_dimname,level_dimname,time_dimname})
            q = q + 1 ;
            varlist(q) = {a.Variables(i).Name} ;
        end
    end
    
    CB    = cell(1,20) ;
    [CB(1,:)] = deal([{'Calendar'},{'link type'},{'file path'},{'varname'},{'x dim #'},{'y dim #'},{'z dim #'},{'t dim #'},{'year1'},{'month1'},{'day1'},{'hour1'},{'year2'},{'month2'},{'day2'},{'hour2'},{'Description'},{'Units'},{'stndrd_name'},{'Factor'}]) ;
    q = 1 ;
    GOAT_Display_message(G,'','init')
    
    for d = 1:length(varlist)
        GOAT_Display_message(G,['Surveying variables . . . ',num2str(d/length(varlist)*100,'%02.0f')],'stat')
        a = ncinfo(opendap_address,varlist{d}) ;
        varfilename = varlist{d} ;
        q = q + 1 ;
        CB(q,1) = file_root(s,2)  ; % Calendar
        CB(q,2) = {1}                   ; % type
        CB(q,3) = {opendap_address}     ; % opendap url
        CB(q,4) = {varfilename}            ; % variable name (short)
        CB(q,5:8) = {0}                 ; % default dimension number
        for j = 1:length(a.Dimensions)
            if strcmpi(a.Dimensions(j).Name,longitude_dimname) ; CB(q,5) = {j} ; end
            if strcmpi(a.Dimensions(j).Name,latitude_dimname)  ; CB(q,6) = {j} ; end
            if strcmpi(a.Dimensions(j).Name,level_dimname)     ; CB(q,7) = {j} ; end
            if strcmpi(a.Dimensions(j).Name,time_dimname)      ; CB(q,8) = {j} ; end
        end
        [CB(q,9:16)] = deal([{year1},{month1},{day1},{hour1},{year2},{month2},{day2},{hour2}]) ;
        ParamIdx = find(strcmp(CV(:,1),varfilename)) ;
        CB(q,17) = CV(ParamIdx,5) ; % Description
        CB(q,18) = CV(ParamIdx,4) ; % Units
        CB(q,19) = CV(ParamIdx,2) ; % Variable standard name
        CB(q,20) = CV(ParamIdx,3) ; % Factor
    end
    
    % Identify Fields
    Fields = unique(CB(2:end,19)) ;
    for f = 1:length(Fields)
        Defs = [] ;
        CF = CB(strcmp(Fields(f),CB(:,19)),:) ;
        % Find standard field name
        FieldName = CF{1,19} ;
        if strncmp(FieldName,'pr',2)
            Defs.ReferenceSettingsField = 'pr' ;
        end
        
        disp(['linking field ',file_root{s,2},'  ',int2str(f),'/',int2str(length(Fields)),' :  ',FieldName,' . . . '])
        mkdir(fullfile(Data_LINK,FieldName)) ;
        %VerticalOrdinate =
        Units            = CF{1,18} ;
        FieldDescription = CF{1,17} ;
        % retrieve spatial vectors
        lon = double(ncread(opendap_address,longitude_dimname)) ;
        lat = double(ncread(opendap_address,latitude_dimname)) ;
        av = ncinfo(opendap_address,CF{1,4}) ;
        
        if cell2mat(CF(1,7)) == 0
            P = 1000 ;
        else
            P = double(ncread(opendap_address,level_dimname)) ;
        end
        
        save(fullfile(Data_LINK,FieldName,'Dims.mat'),'P','lon','lat','Units','VerticalOrdinate','FieldDescription') ;
        % move on to creating Attributes
        dates1 = cell2mat(CF(:,9 ))*1.e10 + cell2mat(CF(:,10)).*1.e4 + cell2mat(CF(:,11)).*1.e2 + cell2mat(CF(:,12))  ;
        dates2 = cell2mat(CF(:,13))*1.e10 + cell2mat(CF(:,14)).*1.e4 + cell2mat(CF(:,15)).*1.e2 + cell2mat(CF(:,16))  ;
        [S, Si] = sort(dates1) ;
        year1 = min(cell2mat(CF(:,9 ))) ;
        year2 = max(cell2mat(CF(:,13))) ;
        if strcmp(file_root{s,2},'List')
            G.Addresses.Default      = cell(2,1)                   ;
            G.Addresses.Default(1,1) = {fullfile('Invariant','data.mat')} ;
            G.Addresses.Default(2,1) = {'List'}          ;
        else
            G = GOAT_Produce_DefaultAddress(G,year1,year2,file_root{s,2},Ext) ;
        end
        Addresses = G.Addresses.Default ;
        Idf = 1 ;
        q0 = GOAT_getSerial(CF{Idf,9},CF{Idf,10},CF{Idf,11},CF{Idf,12},file_root{s,2},year1) - 1  ;
        if q0
            qIdx = 2 ;
        else
            qIdx = 1 ;
        end
        qc = GOAT_getSerial(CF{Idf,13},CF{Idf,14},CF{Idf,15},CF{Idf,16},file_root{s,2},year1)  -q0  ;
        dims = cell2mat(CF(Idf,5:8)) ;
        dims = dims(dims>0) ;
        if (isequal(dims,[1 2]) || isequal(dims,[1 2 3]) || isequal(dims,[1 2 3 4]))
            Permute = {[]}   ;
        else
            Permute = {dims}   ;
        end
        Attributes = cell(2,6) ;
        Attributes(end,5) = {qc+q0} ;
        Col = cell(qIdx,2) ;
        if q0
            Col(1,1) = {[]} ;
            Col(1,2) = {q0} ;
        end
        % Column 1: OPeNDAP Address
        Col(qIdx,1) = CF(Idf,3) ;
        Col(qIdx,2) = {qc} ;
        Attributes(1,1) = {Col} ;
        % Column 2: source varname
        Col(qIdx,1) = CF(Idf,4) ;
        Col(qIdx,2) = {qc} ;
        Attributes(1,2) = {Col} ;
        switch length(dims)
            case 3 % lat,lon,time
                % Column 3: start
                Col(qIdx,1) = {['Start = ones(',int2str(qc),',3) ; ', ...
                    'Start(:,',int2str(dims(3)),') = (1:',int2str(qc),')'' ; ', ...
                    'Col = num2cell(Start,2) ; ']} ;
                Col(qIdx,2) = {'eval'} ;
                Attributes(1,3) = {Col} ;
                % Column 4: count
                count = [1 1 1] ;
                count(dims(1)) = inf ;
                count(dims(2)) = inf ;
                Col(qIdx,1) = {count} ;
                Col(qIdx,2) = {qc} ;
                Attributes(1,4) = {Col} ;
            case 4 % lat lon level time
                % Column 3: start
                Col(qIdx,1) = {['Start = ones(',int2str(qc),',4) ; ', ...
                    'Start(:,',int2str(dims(4)),') = (1:',int2str(qc),')'' ; ', ...
                    'Col = num2cell(Start,2) ; ']} ;
                Col(qIdx,2) = {'eval'} ;
                Attributes(1,3) = {Col} ;
                % Column 4: count
                count = [1 1 1 1] ;
                count(dims(1)) = inf ;
                count(dims(2)) = inf ;
                count(dims(3)) = inf ;
                Col(qIdx,1) = {count} ;
                Col(qIdx,2) = {qc} ;
                Attributes(1,4) = {Col} ;
        end
        % Column 5: Factor
        Col(qIdx,2) = {qc} ;
        Col(qIdx,1) = CF(Idf,20)  ;
        Attributes(1,5) = {Col} ;
        % Column 6: Permute
        if ~isempty(Permute{1})
            Col(qIdx,2) = {qc} ;
            Col(qIdx,1) = Permute  ;
            Attributes(1,6) = {Col} ;
        end
        
        Attributes(end,1) = {Ext} ; % 'mat' | 'ncf' | 'dap' | 'gb1' | 'gb2'
        Attributes(end,2) = {[First_Year Last_Year]} ;
        Attributes(end,3) = {Defs} ; % Defs Struct
        Attributes(end,4) = file_root(s,2) ;
        Attributes = GOAT_ALP_CompressAttributes(Attributes) ;
        if strcmp(file_root{s,2},'List')
            mkdir(fullfile(Data_LINK,FieldName,'Invariant')) ;
            save(fullfile(Data_LINK,FieldName,'Invariant','link_field.mat'),'Attributes') ;
        else
            save(fullfile(Data_LINK,FieldName,'link_field.mat'),'Attributes') ;
        end
        if strcmp(FieldName,'ps')
            if ~exist(fullfile(Data_LINK,'BoerBeta'),'dir') ;
                mkdir(fullfile(Data_LINK,'BoerBeta')) ;
            end
            Attributes(end,1) = {'bds'} ;
            save(fullfile(Data_LINK,'BoerBeta','link_field.mat'),'Attributes') ;
            FieldDescription = 'Boer Beta Function' ;
            Units = '1' ;
            try
                P = double(ncread(opendap_address,level_dimname)) ;
                save(fullfile(Data_LINK,'BoerBeta','Dims.mat'),'P','lon','lat','Units','VerticalOrdinate','FieldDescription') ;
            catch
                % 2D dataset Boer Beta is a place holder for other 3D datasets
            end
            
        end
    end
    TT = toc ;
    totaltoc = totaltoc + TT ;
    HH = floor(TT/3600) ;
    MM = floor((TT - HH*3600)/60) ;
    SS = TT - HH*3600 - MM*60 ;
    if HH > 0
        TimeString = [int2str(HH),' hours, ',int2str(MM),' minutes and ',int2str(SS),' seconds.'] ;
    elseif MM > 0
        TimeString = [int2str(MM),' minutes and ',int2str(SS),' seconds.'] ;
    elseif SS > 0
        TimeString = [int2str(SS),' seconds.'] ;
    end
    disp(['Linking ERA40 ',int2str(s),' of ',int2str(size(file_root,1)),' -- ',file_root{s,2},' completed in ',TimeString]) ;
end % s


TT = totaltoc ;
HH = floor(TT/3600) ;
MM = floor((TT - HH*3600)/60) ;
SS = TT - HH*3600 - MM*60 ;
if HH > 0
    TimeString = [int2str(HH),' hours, ',int2str(MM),' minutes and ',int2str(SS),' seconds.'] ;
elseif MM > 0
    TimeString = [int2str(MM),' minutes and ',int2str(SS),' seconds.'] ;
elseif SS > 0
    TimeString = [int2str(SS),' seconds.'] ;
end
disp(['Linking (partial) ERA-40 OPeNDAP dataset completed in ',TimeString]) ;














