% GOAT_ALP_ERSST_LINK
G = GOAT_Environment ;
Data_ALP = fullfile(fileparts(fileparts(which('GOAT.m'))),'Data_LINK') ; % LINK Data folder
opendap_address = 'http://nomads.ncdc.noaa.gov/thredds/dodsC/ersstv3Agg' ;

Ext = 'dap' ;
longitude_dimname = 'lon' ;
latitude_dimname  = 'lat' ;
level_dimname     = 'lev' ;
time_dimname      = 'time';
today = date ;
First_Year = 1854 ;
First_Month = 1 ;
First_Day = 15 ;
First_Hour = 0 ;
Last_Year = 2020 ; % To be later updated by evalStr_load
Last_Month = 12 ;
Last_Day = 15 ;
Last_Hour = 0 ;
%
tic
Data_LINK = fullfile(Data_ALP,'ERSST_LINK','Monthly') ;
infostr = sprintf(['About ERSST (source: UCAR Climate Data Guide)', ...
    '\nThe NOAA Extended Reconstruction Sea Surface Temperature (ERSST) provides global, ', ...
    '\nspatially complete SST data at a monthly timestep for 1854-present.', ...
    '\nIt is based upon statistical interpolation of the ICOADS release 2.4 data. ', ...
    '\nVersion 3 is the latest release and includes satellite AVHRR SST data for 1985 onwards. ', ...
    '\nVersion 3b does not include satellite data due to a cold bias in the satellite-derived SSTs that proved difficult to correct. '  , ...
    '\nThe data are distributed as anomalies; climatologies (absolute values) are available for 1971-2000. ' , ...
    '\nERSST v3 forms the basis for NOAA''s merged global land-ocean surface temperature analysis.' ...
    ]) ;

mkdir(Data_LINK) ;
save(fullfile(Data_LINK,'info.mat'),'infostr')

VerticalOrdinate = 'Depth (m)' ;

year1 = First_Year ; month1 = First_Month ; day1 = First_Day ; hour1 = First_Hour ;
year2 = Last_Year  ; month2 = Last_Month  ; day2 = Last_Day  ; hour2 = Last_Hour ;

CB    = cell(4,20) ;
[CB(1,:)] = deal([{'Calendar'},{'link type'},{'file path'}    ,{'varname'},{'x dim #'},{'y dim #'},{'z dim #'},{'t dim #'},{'year1'},{'month1'},{'day1'},{'hour1'},{'year2'},{'month2'},{'day2'},{'hour2'},{'Description'},{'Units'},{'stndrd_name'},{'Factor'}]) ;
[CB(2,:)] = deal([{'Monthly'} ,{1}          ,{opendap_address},{'sst'}    ,{1}        ,{2}        ,{3}        ,{4}        ,{year1}  ,{month1}  ,{day1}  ,{hour1}  ,{year2}  ,{month2}  ,{day2}  ,{hour2}  ,{'Extended reconstructed sea surface temperature'},{'C'}    ,{''},{1}]) ;
[CB(3,:)] = deal([{'Monthly'} ,{1}          ,{opendap_address},{'err'}    ,{1}        ,{2}        ,{3}        ,{4}        ,{year1}  ,{month1}  ,{day1}  ,{hour1}  ,{year2}  ,{month2}  ,{day2}  ,{hour2}  ,{'Estimated error standard deviation of analyzed_ersst'},{'C'}    ,{''},{1}]) ;
[CB(4,:)] = deal([{'Monthly'} ,{1}          ,{opendap_address},{'anom'}   ,{1}        ,{2}        ,{3}        ,{4}        ,{year1}  ,{month1}  ,{day1}  ,{hour1}  ,{year2}  ,{month2}  ,{day2}  ,{hour2}  ,{'Extended reconstructed SST anomaliese'},{'C'}    ,{''},{1}]) ;

P = 0 ;
for f = 2:4
    Defs.evalStr_load = ['Today = date ; ', ...
        'ThisYear = round(str2double(Today(end-3:end))) ; ', ...
        'Q0 = GOAT_getSerial(2020,12,1,0,''Monthly'',1854) ;', ...
        'Qc = GOAT_getSerial(ThisYear,12,1,0,''Monthly'',1854) ;', ... 
        'Attributes{1,3}{1,1} = regexprep(Attributes{1,3}{1,1},int2str(Q0),int2str(Qc)) ; ' , ...
        'Attributes{1,1}{1,2} = Qc ; ' , ...
        'Attributes{1,2}{1,2} = Qc ; ' , ...
        'Attributes{1,1}{1,2} = Qc ; ' , ...
        'Attributes{1,4}{1,2} = Qc ; ' , ...
        'Attributes{1,5}{1,2} = Qc ; ' , ...
        'Attributes{1,8}{1,2} = Qc ; ' , ...
        'Attributes(end,2) = {[1854 ThisYear]} ;' , ...
        ] ;
    
    CF = CB(f,:) ;
    % Find standard field name
    FieldName = CF{1,4} ;
    disp(['linking field  Monthly   ',int2str(f-1),'/3 :  ',FieldName,' . . . '])
    mkdir(fullfile(Data_LINK,FieldName)) ;
    %VerticalOrdinate =
    Units            = CF{1,18} ;
    FieldDescription = CF{1,17} ;
    % retrieve spatial vectors
    lon = double(ncread(opendap_address,longitude_dimname)) ;
    lat = double(ncread(opendap_address,latitude_dimname)) ;    
    save(fullfile(Data_LINK,FieldName,'Dims.mat'),'P','lon','lat','Units','VerticalOrdinate','FieldDescription') ;
    % move on to creating Attributes
    Idf = 1 ;
    qc = GOAT_getSerial(CF{Idf,13},CF{Idf,14},CF{Idf,15},CF{Idf,16},'Monthly',year1)    ;
    dims = cell2mat(CF(Idf,5:8)) ;
    dims = dims(dims>0) ;
    if (isequal(dims,[1 2]) || isequal(dims,[1 2 3]) || isequal(dims,[1 2 3 4]))
        Permute = {[]}   ;
    else
        Permute = {dims}   ;
    end
    Attributes = cell(2,8) ;
    Attributes(end,5) = {qc} ;
    Col = cell(1,2) ;
    % Column 1: OPeNDAP Address
    Col(1,1) = CF(Idf,3) ;
    Col(1,2) = {qc} ;
    Attributes(1,1) = {Col} ;
    % Column 2: source varname
    Col(1,1) = CF(Idf,4) ;
    Col(1,2) = {qc} ;
    Attributes(1,2) = {Col} ;
    switch length(dims)
        case 3 % lat,lon,time
            % Column 3: start
            Col(1,1) = {['Start = ones(',int2str(qc),',3) ; ', ...
                'Start(:,',int2str(dims(3)),') = (1:',int2str(qc),')'' ; ', ...
                'Col = num2cell(Start,2) ; ']} ;
            Col(1,2) = {'eval'} ;
            Attributes(1,3) = {Col} ;
            % Column 4: count
            count = [1 1 1] ;
            count(dims(1)) = inf ;
            count(dims(2)) = inf ;
            Col(1,1) = {count} ;
            Col(1,2) = {qc} ;
            Attributes(1,4) = {Col} ;
        case 4 % lat lon level time
            % Column 3: start
            Col(1,1) = {['Start = ones(',int2str(qc),',4) ; ', ...
                'Start(:,',int2str(dims(4)),') = (1:',int2str(qc),')'' ; ', ...
                'Col = num2cell(Start,2) ; ']} ;
            Col(1,2) = {'eval'} ;
            Attributes(1,3) = {Col} ;
            % Column 4: count
            count = [1 1 1 1] ;
            count(dims(1)) = inf ;
            count(dims(2)) = inf ;
            count(dims(3)) = inf ;
            Col(1,1) = {count} ;
            Col(1,2) = {qc} ;
            Attributes(1,4) = {Col} ;
    end
    % Column 5: Factor
    Col(1,2) = {qc} ;
    Col(1,1) = CF(Idf,20)  ;
    Attributes(1,5) = {Col} ;
    % Column 6: Permute
    if ~isempty(Permute{1})
        Col(1,2) = {qc} ;
        Col(1,1) = Permute  ;
        Attributes(1,6) = {Col} ;
    end
    % Column 8: Nan
    Col(1,2) = {qc} ;
    Col(1,1) = {-99}  ; % Lower limit
    Attributes(1,8) = {Col} ;
    
    Attributes(end,1) = {Ext} ; % 'mat' | 'ncf' | 'dap' | 'gb1' | 'gb2'
    Attributes(end,2) = {[First_Year Last_Year]} ;
    Attributes(end,3) = {Defs} ; % Defs Struct
    Attributes(end,4) = {'Monthly'} ;
    save(fullfile(Data_LINK,FieldName,'link_field.mat'),'Attributes') ;
end
TT = toc ;
HH = floor(TT/3600) ;
MM = floor((TT - HH*3600)/60) ;
SS = TT - HH*3600 - MM*60 ;
if HH > 0
    TimeString = [int2str(HH),' hours, ',int2str(MM),' minutes and ',int2str(SS),' seconds.'] ;
elseif MM > 0
    TimeString = [int2str(MM),' minutes and ',int2str(SS),' seconds.'] ;
elseif SS > 0
    TimeString = [int2str(SS),' seconds.'] ;
end
disp(['Linking ERSST  completed in ',TimeString]) ;















