% GOAT_ALP_ERSST_LINK
G = GOAT_Environment ;
Data_ALP = fullfile(fileparts(fileparts(which('GOAT.m'))),'Data_LINK') ; % LINK Data folder
opendap_address = 'http://www.esrl.noaa.gov/psd/thredds/dodsC/Datasets/gpcp/precip.mon.mean.nc' ;

Ext = 'dap' ;
longitude_dimname = 'lon' ;
latitude_dimname  = 'lat' ;
level_dimname     = 'lev' ;
time_dimname      = 'time';
today = date ;
First_Year = 1979 ;
First_Month = 1 ;
First_Day = 1 ;
First_Hour = 0 ;
Last_Year = 2020 ; % To be later updated by evalStr_load
Last_Month = 12 ;
Last_Day = 31 ;
Last_Hour = 0 ;
%
tic
Data_LINK = fullfile(Data_ALP,'GPCP_LINK','Monthly') ;
infostr = sprintf(['About GPCP v2.2 (source: UCAR Climate Data Guide)', ...
    '\nData from rain gauge stations, satellites, and sounding observations have been merged to estimate monthly rainfall on a 2.5-degree global grid from 1979 to the present.', ...
    '\nThe careful combination of satellite-based rainfall estimates provides the most complete analysis of rainfall available to date over the global oceans,', ...
    '\nand adds necessary spatial detail to the rainfall analyses over land. In addition to the combination of these data sets,', ...
    '\nestimates of the uncertainties in the rainfall analysis are provided as a part of the GPCP products.', ...
    '\nThe August 2012 GPCP v2.2 uses upgraded emission and scattering algorithms, the GPCC precipitation gauge analysis, and inclusion of the DMSP F17 SSMIS. '  , ...
    '\nThe December 2012 update contains "recomputed" October 2012 values.' , ...
    '\- See more at: https://climatedataguide.ucar.edu/climate-data/gpcp-monthly-global-precipitation-climatology-project#sthash.vT9AFeiX.dpuf' ...
    ]) ;

     

mkdir(Data_LINK) ;
save(fullfile(Data_LINK,'info.mat'),'infostr')

VerticalOrdinate = 'Pressure (mb)' ;

year1 = First_Year ; month1 = First_Month ; day1 = First_Day ; hour1 = First_Hour ;
year2 = Last_Year  ; month2 = Last_Month  ; day2 = Last_Day  ; hour2 = Last_Hour ;

CB    = cell(4,20) ;
[CB(1,:)] = deal([{'Calendar'},{'link type'},{'file path'}    ,{'varname'},{'x dim #'},{'y dim #'},{'z dim #'},{'t dim #'},{'year1'},{'month1'},{'day1'},{'hour1'},{'year2'},{'month2'},{'day2'},{'hour2'},{'Description'},{'Units'},{'stndrd_name'},{'Factor'}]) ;
[CB(2,:)] = deal([{'Monthly'} ,{1}          ,{opendap_address},{'precip'}    ,{1}        ,{2}        ,{0}        ,{3}        ,{year1}  ,{month1}  ,{day1}  ,{hour1}  ,{year2}  ,{month2}  ,{day2}  ,{hour2}  ,{'Precipitation'},{'kg/m^2/s'}    ,{'pr'},{1/86400}]) ;

P = 1000 ;
for f = 2:2
    Defs.evalStr_load = ['Today = date ; ', ...
        'ThisYear = round(str2double(Today(end-3:end))) ; ', ...
        'Q0 = GOAT_getSerial(2020,12,1,0,''Monthly'',1979) ;', ...
        'Qc = GOAT_getSerial(ThisYear,12,1,0,''Monthly'',1979) ;', ... 
        'Attributes{1,3}{1,1} = regexprep(Attributes{1,3}{1,1},int2str(Q0),int2str(Qc)) ; ' , ...
        'Attributes{1,1}{1,2} = Qc ; ' , ...
        'Attributes{1,2}{1,2} = Qc ; ' , ...
        'Attributes{1,1}{1,2} = Qc ; ' , ...
        'Attributes{1,4}{1,2} = Qc ; ' , ...
        'Attributes{1,5}{1,2} = Qc ; ' , ...
        'Attributes{1,8}{1,2} = Qc ; ' , ...
        'Attributes(end,2) = {[1979 ThisYear]} ;' , ...
        ] ;
    
    CF = CB(f,:) ;
    % Find standard field name
    FieldName = CF{1,4} ;
    StandardName = CF{1,19} ;
    disp(['linking field  Monthly:  ',FieldName,' . . . '])
    mkdir(fullfile(Data_LINK,StandardName)) ;
    %VerticalOrdinate =
    Units            = CF{1,18} ;
    FieldDescription = CF{1,17} ;
    % retrieve spatial vectors
    lon = double(ncread(opendap_address,longitude_dimname)) ;
    lat = double(ncread(opendap_address,latitude_dimname)) ;   
    lev = 1000 ; 
    save(fullfile(Data_LINK,StandardName,'Dims.mat'),'lev','lon','lat','Units','VerticalOrdinate','FieldDescription') ;
    % move on to creating Attributes
    Idf = 1 ;
    qc = GOAT_getSerial(CF{Idf,13},CF{Idf,14},CF{Idf,15},CF{Idf,16},'Monthly',year1)    ;
    dims = cell2mat(CF(Idf,5:8)) ;
    dims = dims(dims>0) ;
    if (isequal(dims,[1 2]) || isequal(dims,[1 2 3]) || isequal(dims,[1 2 3 4]))
        Permute = {[]}   ;
    else
        Permute = {dims}   ;
    end
    Attributes = cell(2,8) ;
    Attributes(end,5) = {qc} ;
    Col = cell(1,2) ;
    % Column 1: OPeNDAP Address
    Col(1,1) = CF(Idf,3) ;
    Col(1,2) = {qc} ;
    Attributes(1,1) = {Col} ;
    % Column 2: source varname
    Col(1,1) = CF(Idf,4) ;
    Col(1,2) = {qc} ;
    Attributes(1,2) = {Col} ;
    switch length(dims)
        case 3 % lat,lon,time
            % Column 3: start
            Col(1,1) = {['Start = ones(',int2str(qc),',3) ; ', ...
                'Start(:,',int2str(dims(3)),') = (1:',int2str(qc),')'' ; ', ...
                'Col = num2cell(Start,2) ; ']} ;
            Col(1,2) = {'eval'} ;
            Attributes(1,3) = {Col} ;
            % Column 4: count
            count = [1 1 1] ;
            count(dims(1)) = inf ;
            count(dims(2)) = inf ;
            Col(1,1) = {count} ;
            Col(1,2) = {qc} ;
            Attributes(1,4) = {Col} ;
        case 4 % lat lon level time
            % Column 3: start
            Col(1,1) = {['Start = ones(',int2str(qc),',4) ; ', ...
                'Start(:,',int2str(dims(4)),') = (1:',int2str(qc),')'' ; ', ...
                'Col = num2cell(Start,2) ; ']} ;
            Col(1,2) = {'eval'} ;
            Attributes(1,3) = {Col} ;
            % Column 4: count
            count = [1 1 1 1] ;
            count(dims(1)) = inf ;
            count(dims(2)) = inf ;
            count(dims(3)) = inf ;
            Col(1,1) = {count} ;
            Col(1,2) = {qc} ;
            Attributes(1,4) = {Col} ;
    end
    % Column 5: Factor
    Col(1,2) = {qc} ;
    Col(1,1) = CF(Idf,20)  ;
    Attributes(1,5) = {Col} ;
    % Column 6: Permute
    if ~isempty(Permute{1})
        Col(1,2) = {qc} ;
        Col(1,1) = Permute  ;
        Attributes(1,6) = {Col} ;
    end
    % Column 8: Nan
    Col(1,2) = {qc} ;
    Col(1,1) = {-99}  ; % Lower limit
    Attributes(1,8) = {Col} ;
    
    Attributes(end,1) = {Ext} ; % 'mat' | 'ncf' | 'dap' | 'gb1' | 'gb2'
    Attributes(end,2) = {[First_Year Last_Year]} ;
    Attributes(end,3) = {Defs} ; % Defs Struct
    Attributes(end,4) = {'Monthly'} ;
    save(fullfile(Data_LINK,StandardName,'link_field.mat'),'Attributes') ;
    J = 1 ; clear Text
    Text(J,1) = {['GPCP variable name = ',FieldName]} ; J = J + 1 ;
    Text(J,1) = {['GPCP variable units = ','mm/day']} ; J = J + 1 ;
    Text(J,1) = {['GOAT variable name = ',StandardName]} ; J = J + 1 ;
    Text(J,1) = {['GOAT variable units = ','kg/m^2/s']} ; J = J + 1 ;
    Text(J,1) = {['OPeNDAP address = ',opendap_address]} ; J = J + 1 ;
    Text(J,1) = {['First Year = ',int2str(First_Year)]} ; J = J + 1 ;
    Text(J,1) = {['Last Year = ',int2str(Last_Year)]} ; J = J + 1 ;
    Text(J,1) = {['Calendar = ','Monthly']} ; J = J + 1 ;
    Text(J,1) = {['LINK Type = ',Ext]} ; J = J + 1 ;
    GOAT_cell2txt(fullfile(Data_LINK,StandardName,'link_info.txt'),Text) ;
end
TT = toc ;
HH = floor(TT/3600) ;
MM = floor((TT - HH*3600)/60) ;
SS = TT - HH*3600 - MM*60 ;
if HH > 0
    TimeString = [int2str(HH),' hours, ',int2str(MM),' minutes and ',int2str(SS),' seconds.'] ;
elseif MM > 0
    TimeString = [int2str(MM),' minutes and ',int2str(SS),' seconds.'] ;
elseif SS > 0
    TimeString = [int2str(SS),' seconds.'] ;
end
disp(['Linking GPCP  completed in ',TimeString]) ;















