% General Code for OPeNDAP LINKing
% NCEP_I (Sig and Pressure)
% NCEP_II (Sig and Pressure)
% as appearing on the URL: http://nomad3.ncep.noaa.gov/ncep_data/
% Note: a possible mirror is nomad1  --> http://nomad1.ncep.noaa.gov/ncep_data/
% Note: This LINK code calls the function GOAT_ALP_NCEP_evalStr_load

G = GOAT_Environment ;
link_dir = fullfile(fileparts(which('GOAT.m')),'Utils','DataManagementCode','ALP','NCEP') ; % MERRA ALP code folder
Data_ALP = fullfile(fileparts(fileparts(which('GOAT.m'))),'Data_LINK') ; % LINK Data folder


% 1. NCEP-I + NCEP-II from NOMADS3

OPeNDAP_URLs_I = {'http://nomad3.ncep.noaa.gov:9090/dods/reanalyses/reanalysis-1/6hr/grb2d/grb2d'              ,'DailyX4','2D_flx' ,'NCEP/NCAR Reanalysis 4x daily flux files'             ; ...
    'http://nomad3.ncep.noaa.gov:9090/dods/reanalyses/reanalysis-1/6hr/pgb/pgb'                              ,'DailyX4','3DPressure' ,'NCEP/NCAR Reanalysis 4x daily pgb files'         ; ...
    'http://nomad3.ncep.noaa.gov:9090/dods/reanalyses/reanalysis-1/month/flx.var/flx.lola.grib.var.y1958-cur','Monthly','flx_variance_1958_Current' ,'Variance (about monthly mean) of selected 6-hrly surface data'         ; ...
    'http://nomad3.ncep.noaa.gov:9090/dods/reanalyses/reanalysis-1/month/grb2d.gau/flx.gau'                  ,'Monthly','flx_gaus' ,'NCEP/NCAR reanalysis monthly mean gaussian grid data (flux)'           ; ...
    'http://nomad3.ncep.noaa.gov:9090/dods/reanalyses/reanalysis-1/month/grb2d.gau/flx.gau.grib.mean.clim.y1958-1997','Monthly','Clim_gaus_1958_1997' ,'flx.gau.grib.mean.clim.y1958-1997'; ...
    'http://nomad3.ncep.noaa.gov:9090/dods/reanalyses/reanalysis-1/month/grb2d.gau/flx.gau.grib.mean.clim.y1971-2000','Monthly','Clim_gaus_1971_2000','flx.gau.grib.mean.clim.y1971-2000' ; ...
    'http://nomad3.ncep.noaa.gov:9090/dods/reanalyses/reanalysis-1/month/grb2d.gau/flx.gau.grib.mean.clim.y1979-1995','Monthly','Clim_gaus_CDAS_1979_1995','CDAS CLIMATOLOGY (1979-1995 MEANS)' ; ...
    'http://nomad3.ncep.noaa.gov:9090/dods/reanalyses/reanalysis-1/month/grb2d.gau/flx.gau.grib.mean.clim.y1981-2010','Monthly','Clim_gaus_1981_2010','flx.gau.grib.mean.clim.y1981-2010' ; ...
    'http://nomad3.ncep.noaa.gov:9090/dods/reanalyses/reanalysis-1/month/grb2d.gau/flx.gau.grib.mean.toga.coare','Monthly','CDAS_flx_gau','NCEP/NCAR ReANL PROJECT: CDAS: Monthly Means and Anomalies' ; ...
    'http://nomad3.ncep.noaa.gov:9090/dods/reanalyses/reanalysis-1/month/grb2d.gau/flx.lola.grib.mean.toga.coare','Monthly','CDAS_flx_lola','NCEP/NCAR ReANL PROJECT: CDAS: Monthly Means and Anomalies' ; ...
    'http://nomad3.ncep.noaa.gov:9090/dods/reanalyses/reanalysis-1/month/grb2d.lola/flx.lola'                ,'Monthly','flx_lola' ,'NCEP/NCAR Reanalysis monthly mean non-pressure level lat-lon data'           ; ...
    'http://nomad3.ncep.noaa.gov:9090/dods/reanalyses/reanalysis-1/month/grb2d.lola/flx.lola.grib.mean.clim.y1958-1997','Monthly','Clim_lola_1958_1997','flx.lola.grib.mean.clim.y1958-1997' ; ...
    'http://nomad3.ncep.noaa.gov:9090/dods/reanalyses/reanalysis-1/month/grb2d.lola/flx.lola.grib.mean.clim.y1971-2000','Monthly','Clim_lola_1971_2000','flx.lola.grib.mean.clim.y1971-2000' ; ...
    'http://nomad3.ncep.noaa.gov:9090/dods/reanalyses/reanalysis-1/month/grb2d.lola/flx.lola.grib.mean.clim.y1979-1995','Monthly','Clim_lola_CDAS_1979_1995','CDAS CLIMATOLOGY (1979-1995 MEANS)' ; ...
    'http://nomad3.ncep.noaa.gov:9090/dods/reanalyses/reanalysis-1/month/grb2d.lola/flx.lola.grib.mean.clim.y1981-2010','Monthly','Clim_lola_1981_2010','flx.lola.grib.mean.clim.y1981-2010' ; ...
    'http://nomad3.ncep.noaa.gov:9090/dods/reanalyses/reanalysis-1/obs_count/obs_density'                    ,'Monthly','Obs_Count' ,'Reanalysis observation density'; ...
    'http://nomad3.ncep.noaa.gov:9090/dods/reanalyses/reanalysis-1/month/prs/prs'                            ,'Monthly','3DPressure' ,'NCEP/NCAR Reanalysis monthly mean pressure level data'         ; ...
    'http://nomad3.ncep.noaa.gov:9090/dods/reanalyses/reanalysis-1/month/prs/prs.grib.mean.clim.y1958-1997'  ,'Monthly','3DClim_1958_1997' ,'prs.grib.mean.clim.y1958-1997'; ...
    'http://nomad3.ncep.noaa.gov:9090/dods/reanalyses/reanalysis-1/month/prs/prs.grib.mean.clim.y1971-2000'  ,'Monthly','3DClim_1971_2000' ,'prs.grib.mean.clim.y1971-2000'; ...
    'http://nomad3.ncep.noaa.gov:9090/dods/reanalyses/reanalysis-1/month/prs/prs.grib.mean.clim.y1979-1995'  ,'Monthly','3DClim_CDAS_1979_1995' ,'CDAS CLIMATOLOGY (1979-95 Means)'; ...
    'http://nomad3.ncep.noaa.gov:9090/dods/reanalyses/reanalysis-1/month/prs/prs.grib.mean.clim.y1981-2010'  ,'Monthly','3DClim_1981_2010' ,'prs.grib.mean.clim.y1981-2010'; ...
    'http://nomad3.ncep.noaa.gov:9090/dods/reanalyses/reanalysis-1/month/prs.var/prs.grib.var.y1949-cur'     ,'Monthly','3DVariance_1949_Current' ,'Variance (about monthly mean) of 6-hrly pres.level data'; ...
    'http://nomad3.ncep.noaa.gov:9090/dods/reanalyses/reanalysis-1/month/prs.var/prs.grib.var.y1958-cur'     ,'Monthly','3DVariance_1958_Current' ,'Variance (about monthly mean) of 6-hrly pres.level data'; ...
    'http://nomad3.ncep.noaa.gov:9090/dods/reanalyses/reanalysis-1/month/prs.var/prs.grib.var.y1979-cur'     ,'Monthly','3DVariance_1979_Current' ,'Variance (about monthly mean) of 6-hrly pres.level data'; ...
    'http://nomad3.ncep.noaa.gov:9090/dods/reanalyses/reanalysis-1/6hr/rotating/grb2d'                       ,'DailyX4','2D_flx_Rotating' ,'NCEP/NCAR Reanalysis (CDAS) Rotating (latest ~ 2years)'    ; ...
    'http://nomad3.ncep.noaa.gov:9090/dods/reanalyses/reanalysis-1/6hr/rotating/pgb'                         ,'DailyX4','3DPressure_Rotating','NCEP/NCAR Reanalysis (CDAS) Rotating (latest ~ 2years)' ; ...
    'http://nomad3.ncep.noaa.gov:9090/dods/reanalyses/reanalysis-1/daily/rotating/grb2d'                     ,'Daily'  ,'2D_flx_Rotating' ,'NCEP/NCAR Reanalysis (CDAS) flx Rotating (latest ~ 2years)'     ; ...
    'http://nomad3.ncep.noaa.gov:9090/dods/reanalyses/reanalysis-1/daily/rotating/pgb'                       ,'Daily'  ,'3DPressure_Rotating' ,'NCEP/NCAR Reanalysis (CDAS) pgb Rotating (latest ~ 2years)' 
    } ; 
OPeNDAP_URLs_II = {'http://nomad3.ncep.noaa.gov:9090/dods/reanalyses/reanalysis-2/6hr/flx/flx','DailyX4','2D_flx','4x daily flx file from NCEP/DOE Reanalysis (R-2)' ;  ...
    'http://nomad2.ncep.noaa.gov:9090/dods/reanalyses/reanalysis-2/6hr/pgb/pgb','DailyX4','3DPressure','NCEP/DOE Reanalysis (R-2) 4x daily pressure-level analyses' ; ...
    'http://nomad3.ncep.noaa.gov:9090/dods/reanalyses/reanalysis-2/6hr/pgb.ft06/pgb','DailyX4','6HForecast','Reanalysis-2 6 hour forecasts (time=start of forecast)' ; ...
    'http://nomad3.ncep.noaa.gov:9090/dods/reanalyses/reanalysis-2/6hr/rotating/flx','DailyX4','2D_flx_Rotating','NCEP/DOE Reanalysis (R-2) 4x daily flux file rotating archive' ; ...
    'http://nomad3.ncep.noaa.gov:9090/dods/reanalyses/reanalysis-2/6hr/rotating/pgb','DailyX4','3DPressure_Rotating','NCEP/DOE Reanalysis (R-2), 4x daily rotating archive' ; ...
    'http://nomad3.ncep.noaa.gov:9090/dods/reanalyses/reanalysis-2/month/dg3/dg3','Monthly','3DSigma','NCEP/DOE Reanalysis-2 sig level fields' ; ...
    'http://nomad3.ncep.noaa.gov:9090/dods/reanalyses/reanalysis-2/month/flx/cmporain','Monthly','cmpo_rain','cmporain.sfc' ; ...
    'http://nomad3.ncep.noaa.gov:9090/dods/reanalyses/reanalysis-2/month/flx/flx','Monthly','2D_flx','flx.ft06.197901.avrg.grib' ; ...
    'http://nomad3.ncep.noaa.gov:9090/dods/reanalyses/reanalysis-2/month/flx/flx.ft06.clim.1980-2009','Monthly','Clim_flx_1980_2009','flx.ft06.clim.1980-2009.grib' ; ...
    'http://nomad3.ncep.noaa.gov:9090/dods/reanalyses/reanalysis-2/month/flx/flx.ft06.climo.197901-199812','Monthly','Clim_flx_1979_1998','197901-199812 R2 flx.ft06 climatology' ; ...
    'http://nomad3.ncep.noaa.gov:9090/dods/reanalyses/reanalysis-2/month/flx/flxA','Monthly','2D_flxA','flx.ft06.197901.avrg.grib' ; ...
    'http://nomad3.ncep.noaa.gov:9090/dods/reanalyses/reanalysis-2/month/flx/gpcprain','Monthly','GPCP_rain','gpcprain.sfc' ; ...
    'http://nomad3.ncep.noaa.gov:9090/dods/reanalyses/reanalysis-2/month/pgb/pgb','Monthly','3DPressure','NCEP/DOE Reanalysis monthly pressure grib files' ; ...
    'http://nomad3.ncep.noaa.gov:9090/dods/reanalyses/reanalysis-2/month/pgb/pgb.anl.clim.1980-2009','Monthly','3DClim_1980_2009','R2 pgb climo 1980-2009' ; ...
    'http://nomad3.ncep.noaa.gov:9090/dods/reanalyses/reanalysis-2/month/pgb/pgb.anl.climo','Monthly','3DClim_1979_1998','Reanalysis-2 (version C) 20 year climatology 197901-199812' ; ...
    } ; 
OPeNDAP_URLs = [OPeNDAP_URLs_I ; OPeNDAP_URLs_II] ; 
% Units and Name Conversion Array
CV = cell(1,4) ; qV = 1 ;
[CV(qV,:)] = deal([{'name'}    ,{'standard_name'},{'Factor'},{'Units'}]) ; qV = qV + 1 ;
[CV(qV,:)] = deal([{'hgtsfc'}  ,{'zgs'}          ,{1}       ,{[]}])      ; qV = qV + 1 ;
[CV(qV,:)] = deal([{'hgtprs'}  ,{'zg'}           ,{1}       ,{[]}])      ; qV = qV + 1 ;
[CV(qV,:)] = deal([{'hgt'}     ,{'zg'}           ,{1}       ,{[]}])      ; qV = qV + 1 ;
[CV(qV,:)] = deal([{'pratesfc'},{'pr'}           ,{1}       ,{[]}])      ; qV = qV + 1 ;
[CV(qV,:)] = deal([{'pressfc'} ,{'ps'}           ,{1}       ,{[]}])      ; qV = qV + 1 ;
[CV(qV,:)] = deal([{'rhprs'}   ,{'hur'}          ,{1}       ,{[]}])      ; qV = qV + 1 ;
[CV(qV,:)] = deal([{'rh'}      ,{'hur'}          ,{1}       ,{[]}])      ; qV = qV + 1 ;
[CV(qV,:)] = deal([{'spfh'}    ,{'hus'}          ,{1}       ,{[]}])      ; qV = qV + 1 ;
[CV(qV,:)] = deal([{'tmpprs'}  ,{'ta'}           ,{1}       ,{[]}])      ; qV = qV + 1 ;
[CV(qV,:)] = deal([{'ugrdprs'} ,{'ua'}           ,{1}       ,{[]}])      ; qV = qV + 1 ;
[CV(qV,:)] = deal([{'vgrdprs'} ,{'va'}           ,{1}       ,{[]}])      ; qV = qV + 1 ;
[CV(qV,:)] = deal([{'vvelprs'} ,{'wap'}          ,{1}       ,{[]}])      ; qV = qV + 1 ;
[CV(qV,:)] = deal([{'ugrd'}    ,{'ua'}           ,{1}       ,{[]}])      ; qV = qV + 1 ;
[CV(qV,:)] = deal([{'vgrd'}    ,{'va'}           ,{1}       ,{[]}])      ; qV = qV + 1 ;
[CV(qV,:)] = deal([{'vvel'}    ,{'wap'}          ,{1}       ,{[]}])      ; qV = qV + 1 ;
[CV(qV,:)] = deal([{'tmpsfc'}  ,{'tas'}          ,{1}       ,{[]}])      ; qV = qV + 1 ;
[CV(qV,:)] = deal([{'tmp'}     ,{'ta'}           ,{1}       ,{[]}])      ; qV = qV + 1 ;

CompressFlag = 1 ;
totaltoc = 0 ;
Ext = 'dap' ;
longitude_dimname = 'lon' ;
latitude_dimname  = 'lat' ;
level_dimname     = 'lev' ;
time_dimname      = 'time';
today = date ;
%
for s = 1:size(OPeNDAP_URLs,1)
    tic
    Add_Year = 0 ; % add a year to Rotating datasets to allow shifting of temporal span
    switch OPeNDAP_URLs{s,3}
        case {'2DFlux_Rotating','3DPressure_Rotating'}
            Add_Year = 1 ;
        otherwise
            
    end % switch
    switch OPeNDAP_URLs{s,1}
        case 'http://nomad3.ncep.noaa.gov:9090/dods/reanalyses/reanalysis-1/6hr/rotating/grb2d'
            Defs.evalStr_load = 'Attributes = GOAT_ALP_NCEP_evalStr_load(Attributes,1) ; ' ; 
        case 'http://nomad3.ncep.noaa.gov:9090/dods/reanalyses/reanalysis-1/6hr/rotating/pgb'
            Defs.evalStr_load = 'Attributes = GOAT_ALP_NCEP_evalStr_load(Attributes,2) ; ' ; 
        case 'http://nomad3.ncep.noaa.gov:9090/dods/reanalyses/reanalysis-1/daily/rotating/grb2d'
            Defs.evalStr_load = 'Attributes = GOAT_ALP_NCEP_evalStr_load(Attributes,3) ; ' ;     
        case 'http://nomad3.ncep.noaa.gov:9090/dods/reanalyses/reanalysis-1/daily/rotating/pgb'
            Defs.evalStr_load = 'Attributes = GOAT_ALP_NCEP_evalStr_load(Attributes,4) ; ' ; 
        otherwise
            Defs = [] ; 
    
    end
    
    if s>size(OPeNDAP_URLs_I,1)
    Data_LINK = fullfile(Data_ALP,'NCEP-II_LINK',OPeNDAP_URLs{s,3},OPeNDAP_URLs{s,2}) ;
    infostr = sprintf([OPeNDAP_URLs{s,4}, ...
                '\nAbout NCEP-DOE Reanalysis-2 (source: UCAR Climate Data Guide)', ...
                '\nNCEP-DOE Reanalysis II (R2) is an improved version of the NCEP-NCAR Reanalysis I (R1).', ...
                '\nThe improvements include an updated model with better physical parameterizations,', ...
                '\nassorted data assimilation errors were fixed and additional data were included.', ...
                '\n Still, it is still a ''first generation'' product.' ...
                ]) ;
    else
    Data_LINK = fullfile(Data_ALP,'NCEP-I_LINK',OPeNDAP_URLs{s,3},OPeNDAP_URLs{s,2}) ;
    infostr = sprintf([OPeNDAP_URLs{s,4}, ...
                '\nAbout NCEP-NCAR Reanalysis-1 (source: UCAR Climate Data Guide)', ...
                '\nNCEP-NCAR (R1) is the original reanalysis effort. ', ...
                '\nIt uses a frozen global state-of-the-art global data assimilation system (as of 11 January 1995).', ...
                '\nThe original database was enhanced (additional, quality checked datasets) by NCAR''s Data Support Section.', ...
                '\nOriginally planned to span 1957-96 ("40-Year Reanalysis Project"), it was extended back to 1948 and continues to this day.' ...
                ]) ;
    end
    mkdir(Data_LINK) ;
    save(fullfile(Data_LINK,'info.mat'),'infostr')
    opendap_address = OPeNDAP_URLs{s,1} ;
    a = ncinfo(opendap_address) ;
    Minimum_Time = a.Variables(find(strcmp({a.Variables.Name},'time'))).Attributes(find(strcmp({a.Variables(find(strcmp({a.Variables.Name},'time'))).Attributes.Name},'minimum'))).Value ;
    months_strings = {'jan','feb','mar','apr','may','jun','jul','aug','sep','oct','nov','dec'} ;
    First_Hour  = str2double(Minimum_Time(1:2)) ;
    First_Day   = str2double(Minimum_Time(4:5)) ;
    First_Month = find(strcmpi(Minimum_Time(6:8),months_strings)) ;
    First_Year  = str2double(Minimum_Time(9:end)) ;
    Maximum_Time = a.Variables(find(strcmp({a.Variables.Name},'time'))).Attributes(find(strcmp({a.Variables(find(strcmp({a.Variables.Name},'time'))).Attributes.Name},'maximum'))).Value ;
    Last_Year   = str2double(Maximum_Time(9:end)) ;
    This_Year   = round(str2double(today(end-3:end))) ;
    if Last_Year < This_Year
        Final_Year = Last_Year ; 
    else
        Final_Year = This_Year + Add_Year ; 
    end
    VerticalOrdinate = 'Pressure (mb)' ;
    
    varlist = cell(1,1) ;
    q = 0 ;
    for i = 1:length(a.Variables)
        if ~ismember(a.Variables(i).Name,{longitude_dimname,latitude_dimname,level_dimname,time_dimname})
            q = q + 1 ;
            varlist(q) = {a.Variables(i).Name} ;
        end
    end
    disp(['Building link table ',OPeNDAP_URLs{s,3},' . . .'])
    year1 = First_Year ; month1 = First_Month ; day1 = First_Day ; hour1 = First_Hour ;
    year2 = Final_Year ; month2 = 12 ; day2 = 31 ;
    switch OPeNDAP_URLs{s,2}
        case {'Monthly','Daily'}
            hour2 = 00 ;
        case 'DailyX4'
            hour2 = 18 ;
    end
    
    CB    = cell(1,20) ;
    [CB(1,:)] = deal([{'Calendar'},{'link type'},{'file path'},{'varname'},{'x dim #'},{'y dim #'},{'z dim #'},{'t dim #'},{'year1'},{'month1'},{'day1'},{'hour1'},{'year2'},{'month2'},{'day2'},{'hour2'},{'Description'},{'Units'},{''},{''}]) ;
    q = 1 ;
    GOAT_Display_message(G,'','init')
    for d = 1:length(varlist)
        GOAT_Display_message(G,['Surveying variables . . . ',num2str(d/length(varlist)*100,'%02.0f')],'stat')
        a = ncinfo(opendap_address,varlist{d}) ;
        q = q + 1 ;
        CB(q,1) = OPeNDAP_URLs(s,2)  ; % Temporal Resolution
        CB(q,2) = {1}                   ; % type
        CB(q,3) = {opendap_address}     ; % opendap url
        CB(q,4) = varlist(d)            ; % variable name (short)
        CB(q,5:8) = {0}                 ; % default dimension number
        for j = 1:length(a.Dimensions)
            if strcmpi(a.Dimensions(j).Name,longitude_dimname) ; CB(q,5) = {j} ; end
            if strcmpi(a.Dimensions(j).Name,latitude_dimname)  ; CB(q,6) = {j} ; end
            if strcmpi(a.Dimensions(j).Name,level_dimname)     ; CB(q,7) = {j} ; end
            if strcmpi(a.Dimensions(j).Name,time_dimname)      ; CB(q,8) = {j} ; end
        end
        [CB(q,9:16)] = deal([{year1},{month1},{day1},{hour1},{year2},{month2},{day2},{hour2}]) ;
        long_name = a.Attributes(1,3).Value ;
        long_name = regexprep(long_name,'0 **','') ;
        long_name = regexprep(long_name,'0**','') ;
        star_idx = regexp(long_name,'*') ;
        units_idx = regexp(long_name,'[') ;
        if ~isempty(star_idx) 
            if units_idx > star_idx(1)
            long_name = regexprep(long_name,'**','') ;
            end
        end
        units_idx = regexp(long_name,'[') ;
        CB(q,17) = {long_name(1:units_idx-1)} ;
        units_idx_end = regexp(long_name,']') ;
        CB(q,18) = {long_name(units_idx+1:units_idx_end-1)} ;
    end
    
    % Identify Fields
    Fields = unique(CB(2:end,4)) ;
    for f = 1:length(Fields)
        CF = CB(strcmp(Fields(f),CB(:,4)),:) ;
        % Find standard field name
        Idx = find(strcmpi(Fields(f),CV(:,1))) ;
        if isempty(Idx)
            FieldName = lower(Fields{f}) ;
        else
            FieldName = lower(CV{Idx,2}) ;
        end
        disp(['linking field ',OPeNDAP_URLs{s,3},'/',OPeNDAP_URLs{s,2},'  ',int2str(f),'/',int2str(length(Fields)),' :  ',FieldName,' . . . '])
        mkdir(fullfile(Data_LINK,FieldName)) ;
        %VerticalOrdinate =
        Units            = CF{1,18} ;
        FieldDescription = CF{1,17} ;
        % retrieve spatial vectors
        lon = double(ncread(opendap_address,longitude_dimname)) ;
        lat = double(ncread(opendap_address,latitude_dimname)) ;
        av = ncinfo(opendap_address,CF{1,4}) ;
        if cell2mat(CF(1,7)) == 0
            P = 1000 ;
        else
            P = double(ncread(opendap_address,level_dimname)) ;
        end
        save(fullfile(Data_LINK,FieldName,'Dims.mat'),'P','lon','lat','Units','VerticalOrdinate','FieldDescription') ;
        % move on to creating Attributes
        dates1 = cell2mat(CF(:,9 ))*1.e10 + cell2mat(CF(:,10)).*1.e4 + cell2mat(CF(:,11)).*1.e2 + cell2mat(CF(:,12))  ;
        dates2 = cell2mat(CF(:,13))*1.e10 + cell2mat(CF(:,14)).*1.e4 + cell2mat(CF(:,15)).*1.e2 + cell2mat(CF(:,16))  ;
        [S, Si] = sort(dates1) ;
        year1 = min(cell2mat(CF(:,9 ))) ;
        year2 = max(cell2mat(CF(:,13))) ;
        if strcmp(OPeNDAP_URLs{s,2},'List')
            G.Addresses.Default      = cell(2,1)                   ;
            G.Addresses.Default(1,1) = {fullfile('Invariant','data.mat')} ;
            G.Addresses.Default(2,1) = {'List'}          ;
        else
            G = GOAT_Produce_DefaultAddress(G,year1,year2,OPeNDAP_URLs{s,2},Ext) ;
        end
        Addresses = G.Addresses.Default ;
        if ~CompressFlag && ~strcmp(OPeNDAP_URLs{s,2},'List')
            Attributes = cell(length(Addresses),6) ;
            % Attributes(i,1) = filename ;
            % Attributes(i,2) = varname  ;
            % Attributes(i,3) = start    ;
            % Attributes(i,4) = count    ;
            % Attributes(i,5) = factor   ;
            % Attributes(i,6) = permute   ;
            % Attributes(i,7) = reshape (add dimensions)   ;
            % Assign Attributes to each date
            for k = 1:length(Addresses) - 1 ;
                if strcmp(OPeNDAP_URLs{s,2},'List')
                    Idf = 1 ;
                    S   = 1 ;
                    Sf  = 1 ;
                else
                    [year,month,day,hour] = GOAT_serial2date(Addresses,k) ;
                    Idf = 1 ;
                    S  = (GOAT_getSerial(year,month,day,hour,OPeNDAP_URLs{s,2},year1)-1)*SeqFileNum + 1 ;
                    Sf = GOAT_getSerial(CF{Idf,9},CF{Idf,10},CF{Idf,11},CF{Idf,12},OPeNDAP_URLs{s,2},year1)  ;
                end
                Attributes(k,1) = CF(1,3) ;
                Attributes(k,2) = CF(1,4) ;
                Attributes(k,5) = {1}  ;
                
                % locate dimensions
                dims = cell2mat(CF(1,5:8)) ;
                dims = dims(dims>0) ;
                
                if ~(isequal(dims,[1 2]) || isequal(dims,[1 2 3]) || isequal(dims,[1 2 3 4]))
                    %                         Attributes(k,6) = {0}   ;
                    %                     else
                    Attributes(k,6) = {dims}   ;
                end
                
                switch length(dims)
                    case 2 % lat,lon
                        start = [1 1] ;
                        count = [inf inf] ;
                    case 3 % lat,lon,time
                        start = [1 1 1] ;
                        start(dims(3)) = (S-Sf)+1 ;
                        count = [1 1 1] ;
                        count(dims(1)) = inf ;
                        count(dims(2)) = inf ;
                    case 4 % lat lon level time
                        start = [1 1 1 1] ;
                        start(dims(4)) = (S-Sf)+1 ;
                        count = [1 1 1 1] ;
                        count(dims(1)) = inf ;
                        count(dims(2)) = inf ;
                        count(dims(3)) = inf ;
                end
                
                Attributes(k,3) = {start} ;
                Attributes(k,4) = {count} ;
            end
            
        else
            Idf = 1 ;
            q0 = GOAT_getSerial(CF{Idf,9},CF{Idf,10},CF{Idf,11},CF{Idf,12},OPeNDAP_URLs{s,2},year1) - 1  ;
            if q0
                qIdx = 2 ;
            else
                qIdx = 1 ;
            end
            qc = length(Addresses)-1 -q0  ;
            dims = cell2mat(CF(Idf,5:8)) ;
            dims = dims(dims>0) ;
            if (isequal(dims,[1 2]) || isequal(dims,[1 2 3]) || isequal(dims,[1 2 3 4]))
                Permute = {[]}   ;
            else
                Permute = {dims}   ;
            end
            Attributes = cell(2,6) ;
            Attributes(end,5) = {qc+q0} ;
            Col = cell(qIdx,2) ;
            if q0
                Col(1,1) = {[]} ;
                Col(1,2) = {q0} ;
            end
            % Column 1: OPeNDAP Address
            Col(qIdx,1) = CF(Idf,3) ;
            Col(qIdx,2) = {qc} ;
            Attributes(1,1) = {Col} ;
            % Column 2: source varname
            Col(qIdx,1) = CF(Idf,4) ;
            Col(qIdx,2) = {qc} ;
            Attributes(1,2) = {Col} ;
            switch length(dims)
                case 3 % lat,lon,time
                    % Column 3: start
                    Col(qIdx,1) = {['Start = ones(',int2str(qc),',3) ; ', ...
                        'Start(:,',int2str(dims(3)),') = (1:',int2str(qc),')'' ; ', ...
                        'Col = num2cell(Start,2) ; ']} ;
                    Col(qIdx,2) = {'eval'} ;
                    Attributes(1,3) = {Col} ;
                    % Column 4: count
                    count = [1 1 1] ;
                    count(dims(1)) = inf ;
                    count(dims(2)) = inf ;
                    Col(qIdx,1) = {count} ;
                    Col(qIdx,2) = {qc} ;
                    Attributes(1,4) = {Col} ;
                case 4 % lat lon level time
                    % Column 3: start
                    Col(qIdx,1) = {['Start = ones(',int2str(qc),',4) ; ', ...
                        'Start(:,',int2str(dims(4)),') = (1:',int2str(qc),')'' ; ', ...
                        'Col = num2cell(Start,2) ; ']} ;
                    Col(qIdx,2) = {'eval'} ;
                    Attributes(1,3) = {Col} ;
                    % Column 4: count
                    count = [1 1 1 1] ;
                    count(dims(1)) = inf ;
                    count(dims(2)) = inf ;
                    count(dims(3)) = inf ;
                    Col(qIdx,1) = {count} ;
                    Col(qIdx,2) = {qc} ;
                    Attributes(1,4) = {Col} ;
            end
            % Column 5: Factor
            Col(qIdx,2) = {qc} ;
            Col(qIdx,1) = {1}  ;
            Attributes(1,5) = {Col} ;
            % Column 6: Permute
            if ~isempty(Permute{1})
                Col(qIdx,2) = {qc} ;
                Col(qIdx,1) = Permute  ;
                Attributes(1,6) = {Col} ;
            end
        end
        Attributes(end,1) = {Ext} ; % 'mat' | 'ncf' | 'dap' | 'gb1' | 'gb2'
        Attributes(end,2) = {[year1 year2]} ;
        Attributes(end,3) = {Defs} ; % Defs Struct
        Attributes(end,4) = OPeNDAP_URLs(s,2) ;
        %Attributes = GOAT_ALP_CompressAttributes(Attributes) ;
        if strcmp(OPeNDAP_URLs{s,2},'List')
            mkdir(fullfile(Data_LINK,FieldName,'Invariant')) ;
            save(fullfile(Data_LINK,FieldName,'Invariant','link_field.mat'),'Attributes') ;
        else
            save(fullfile(Data_LINK,FieldName,'link_field.mat'),'Attributes') ;
        end
        if strcmp(FieldName,'ps')
            if ~exist(fullfile(Data_LINK,'BoerBeta'),'dir') ;
                mkdir(fullfile(Data_LINK,'BoerBeta')) ;
            end
            Attributes(end,1) = {'bds'} ;
            save(fullfile(Data_LINK,'BoerBeta','link_field.mat'),'Attributes') ;
            FieldDescription = 'Boer Beta Function' ;
            Units = '1' ;
            try
            P = double(ncread(opendap_address,level_dimname)) ;
            save(fullfile(Data_LINK,'BoerBeta','Dims.mat'),'P','lon','lat','Units','VerticalOrdinate','FieldDescription') ;
            catch
                % 2D dataset Boer Beta is a place holder for other 3D datasets
            end
        end
    end
    TT = toc ;
    totaltoc = totaltoc + TT ;
    HH = floor(TT/3600) ;
    MM = floor((TT - HH*3600)/60) ;
    SS = TT - HH*3600 - MM*60 ;
    if HH > 0
        TimeString = [int2str(HH),' hours, ',int2str(MM),' minutes and ',int2str(SS),' seconds.'] ;
    elseif MM > 0
        TimeString = [int2str(MM),' minutes and ',int2str(SS),' seconds.'] ;
    elseif SS > 0
        TimeString = [int2str(SS),' seconds.'] ;
    end
    disp(['Linking NCEP ',int2str(s),' of ',int2str(size(OPeNDAP_URLs,1)),' -- ',OPeNDAP_URLs{s,3},'/',OPeNDAP_URLs{s,2},' completed in ',TimeString]) ;
end % s


TT = totaltoc ;
HH = floor(TT/3600) ;
MM = floor((TT - HH*3600)/60) ;
SS = TT - HH*3600 - MM*60 ;
if HH > 0
    TimeString = [int2str(HH),' hours, ',int2str(MM),' minutes and ',int2str(SS),' seconds.'] ;
elseif MM > 0
    TimeString = [int2str(MM),' minutes and ',int2str(SS),' seconds.'] ;
elseif SS > 0
    TimeString = [int2str(SS),' seconds.'] ;
end
disp(['Linking entire NCEP OPeNDAP dataset completed in ',TimeString]) ;



