% GOAT_ALP_ORAS4_LINK link code for )RA-S4 Ocean Reanalysis

G = GOAT_Environment ;
link_dir = fullfile(fileparts(which('GOAT.m')),'Utils','DataManagementCode','ALP','ORAS4') ; % MERRA ALP code folder
Data_ALP = fullfile(fileparts(fileparts(which('GOAT.m'))),'Data_LINK') ; % LINK Data folder

Years = 1871:2012 ;

file_root = 'http://apdrc.soest.hawaii.edu:80/dods/public_data/Reanalysis_Data/ORAS4/' ; 
 
Calendar = 'Monthly' ;

varlist = {'ild_t02','ild_t05','mask','mld_s030','mld_s125','so','thetao','uo','vo','ssh'} ;


CompressFlag = 1 ;
totaltoc = 0 ;
Ext = 'dap' ;
longitude_dimname = 'lon' ;
latitude_dimname  = 'lat' ;
level_dimname     = 'lev' ;
time_dimname      = 'time';
today = date ;
First_Year = 1958 ;
First_Month = 1 ;
First_Day = 15 ;
First_Hour = 0 ;
Last_Year = 2014 ;
Last_Month = 12 ;
Last_Day = 15 ;
Last_Hour = 0 ;
%%
    tic
    Data_LINK = fullfile(Data_ALP,'ORAS4_LINK',Calendar) ;
    infostr = sprintf(['About ORA-S4 (source: UCAR Climate Data Guide)', ...
        '\nECMWF''s current ocean reanalysis uses a sophisticated data assimilation methodology which includes a model bias correction.', ...
        '\nThe ocean model used is forced by atmospheric daily surface fluxes, relaxed to SST and bias corrected.', ...
        '\nBalmaseda et al. (QJRMS 2013) provides excellent peer reviewed documentation and evaluation.', ...
        ]) ;
    mkdir(Data_LINK) ; 
    save(fullfile(Data_LINK,'info.mat'),'infostr')
    
    
    VerticalOrdinate = 'Depth (m)' ;
    disp(['Building link table ',file_root,' . . .'])
    year1 = First_Year ; month1 = First_Month ; day1 = First_Day ; hour1 = First_Hour ;
    year2 = Last_Year  ; month2 = Last_Month  ; day2 = Last_Day  ; hour2 = Last_Hour  ; 
    
    CB    = cell(1,20) ;
    [CB(1,:)] = deal([{'Calendar'},{'link type'},{'file path'},{'varname'},{'x dim #'},{'y dim #'},{'z dim #'},{'t dim #'},{'year1'},{'month1'},{'day1'},{'hour1'},{'year2'},{'month2'},{'day2'},{'hour2'},{'Description'},{'Units'},{''},{''}]) ;
    q = 1 ;
    GOAT_Display_message(G,'','init')
    for d = 1:length(varlist)
        opendap_address = [file_root,varlist{d}] ;
        GOAT_Display_message(G,['Surveying variables . . . ',num2str(d/length(varlist)*100,'%02.0f')],'stat')
        if strcmp(varlist{d},'ssh')
        opendap_address = [file_root,'zos'] ;
        end
        a = ncinfo(opendap_address,varlist{d}) ;
        varfilename = varlist{d} ; 
        q = q + 1 ;
        CB(q,1) = {file_root}  ; % Temporal Resolution
        CB(q,2) = {1}                   ; % type
        CB(q,3) = {opendap_address}     ; % opendap url
        CB(q,4) = {varfilename}            ; % variable name (short)
        CB(q,5:8) = {0}                 ; % default dimension number
        for j = 1:length(a.Dimensions)
            if strcmpi(a.Dimensions(j).Name,longitude_dimname) ; CB(q,5) = {j} ; end
            if strcmpi(a.Dimensions(j).Name,latitude_dimname)  ; CB(q,6) = {j} ; end
            if strcmpi(a.Dimensions(j).Name,level_dimname)     ; CB(q,7) = {j} ; end
            if strcmpi(a.Dimensions(j).Name,time_dimname)      ; CB(q,8) = {j} ; end
        end
        [CB(q,9:16)] = deal([{year1},{month1},{day1},{hour1},{year2},{month2},{day2},{hour2}]) ;
        long_name = a.Attributes(find(strcmp({a.Attributes.Name},'long_name'))).Value ;
        if regexp(long_name,'(m')
            Units = 'm' ; 
            long_name = long_name(1:regexp(long_name,'(m')-1) ; 
        elseif regexp(long_name,'[m s')
            Units = 'm/s' ; 
            long_name = long_name(1:regexp(long_name,'[m s')-1) ; 
        elseif regexp(long_name,'[m')
            Units = 'm' ; 
            long_name = long_name(1:regexp(long_name,'[m')-1) ; 
        elseif ismember(varfilename,{'so','mask'})
            Units = '1' ; 
        elseif regexp(long_name,'[degc')
            Units = 'C' ; 
            long_name = long_name(1:regexp(long_name,'[degc')-1) ;
        else
            disp 'could not identify units' ; 
            Units = '' ; 
        end
        CB(q,17) = {long_name} ;
        CB(q,18) = {Units} ;
    end
    
    % Identify Fields
    Fields = unique(CB(2:end,4)) ;
    for f = 1:length(Fields)
        Defs = [] ; 
        CF = CB(strcmp(Fields(f),CB(:,4)),:) ;
        FieldName = lower(Fields{f}) ;
        disp(['linking field ',file_root,FieldName,'  ',int2str(f),'/',int2str(length(Fields)),':  ',Calendar,' . . . '])
        mkdir(fullfile(Data_LINK,FieldName)) ;
        %VerticalOrdinate =
        Units            = CF{1,18} ;
        FieldDescription = CF{1,17} ;
        % retrieve spatial vectors
        lon = double(ncread(CF{1,3},longitude_dimname)) ;
        lat = double(ncread(CF{1,3},latitude_dimname)) ;
        av = ncinfo(CF{1,3},CF{1,4}) ;
        if cell2mat(CF(1,7)) == 0
                P = 0 ;
            else
                P = double(ncread(CF{1,3},level_dimname)) ;
        end
        
        save(fullfile(Data_LINK,FieldName,'Dims.mat'),'P','lon','lat','Units','VerticalOrdinate','FieldDescription') ;
        % move on to creating Attributes
        dates1 = cell2mat(CF(:,9 ))*1.e10 + cell2mat(CF(:,10)).*1.e4 + cell2mat(CF(:,11)).*1.e2 + cell2mat(CF(:,12))  ;
        dates2 = cell2mat(CF(:,13))*1.e10 + cell2mat(CF(:,14)).*1.e4 + cell2mat(CF(:,15)).*1.e2 + cell2mat(CF(:,16))  ;
        [S, Si] = sort(dates1) ;
        year1 = min(cell2mat(CF(:,9 ))) ;
        year2 = max(cell2mat(CF(:,13))) ;
        if strcmp(Calendar,'List')
            G.Addresses.Default      = cell(2,1)                   ;
            G.Addresses.Default(1,1) = {fullfile('Invariant','data.mat')} ;
            G.Addresses.Default(2,1) = {'List'}          ;
        else
            G = GOAT_Produce_DefaultAddress(G,year1,year2,Calendar,Ext) ;
        end
        Addresses = G.Addresses.Default ;
        Idf = 1 ;
            q0 = GOAT_getSerial(CF{Idf,9},CF{Idf,10},CF{Idf,11},CF{Idf,12},Calendar,year1) - 1  ;
            if q0
                qIdx = 2 ;
            else
                qIdx = 1 ;
            end
            qc = length(Addresses)-1 -q0  ;
            dims = cell2mat(CF(Idf,5:8)) ;
            dims = dims(dims>0) ;
            if (isequal(dims,[1 2]) || isequal(dims,[1 2 3]) || isequal(dims,[1 2 3 4]))
                Permute = {[]}   ;
            else
                Permute = {dims}   ;
            end
            Attributes = cell(2,6) ;
            Attributes(end,5) = {qc+q0} ;
            Col = cell(qIdx,2) ;
            if q0
                Col(1,1) = {[]} ;
                Col(1,2) = {q0} ;
            end
            % Column 1: OPeNDAP Address
            Col(qIdx,1) = CF(Idf,3) ;
            Col(qIdx,2) = {qc} ;
            Attributes(1,1) = {Col} ;
            % Column 2: source varname
            Col(qIdx,1) = CF(Idf,4) ;
            Col(qIdx,2) = {qc} ;
            Attributes(1,2) = {Col} ;
            switch length(dims)
                case 3 % lat,lon,time
                    % Column 3: start
                    Col(qIdx,1) = {['Start = ones(',int2str(qc),',3) ; ', ...
                        'Start(:,',int2str(dims(3)),') = (1:',int2str(qc),')'' ; ', ...
                        'Col = num2cell(Start,2) ; ']} ;
                    Col(qIdx,2) = {'eval'} ;
                    Attributes(1,3) = {Col} ;
                    % Column 4: count
                    count = [1 1 1] ;
                    count(dims(1)) = inf ;
                    count(dims(2)) = inf ;
                    Col(qIdx,1) = {count} ;
                    Col(qIdx,2) = {qc} ;
                    Attributes(1,4) = {Col} ;
                case 4 % lat lon level time
                    % Column 3: start
                    Col(qIdx,1) = {['Start = ones(',int2str(qc),',4) ; ', ...
                        'Start(:,',int2str(dims(4)),') = (1:',int2str(qc),')'' ; ', ...
                        'Col = num2cell(Start,2) ; ']} ;
                    Col(qIdx,2) = {'eval'} ;
                    Attributes(1,3) = {Col} ;
                    % Column 4: count
                    count = [1 1 1 1] ;
                    count(dims(1)) = inf ;
                    count(dims(2)) = inf ;
                    count(dims(3)) = inf ;
                    Col(qIdx,1) = {count} ;
                    Col(qIdx,2) = {qc} ;
                    Attributes(1,4) = {Col} ;
            end
            % Column 5: Factor
            Col(qIdx,2) = {qc} ;
            Col(qIdx,1) = {1}  ;
            Attributes(1,5) = {Col} ;
            % Column 6: Permute
            if ~isempty(Permute{1})
                Col(qIdx,2) = {qc} ;
                Col(qIdx,1) = Permute  ;
                Attributes(1,6) = {Col} ;
            end
        Attributes(end,1) = {Ext} ; % 'mat' | 'ncf' | 'dap' | 'gb1' | 'gb2'
        Attributes(end,2) = {[First_Year Last_Year]} ;
        Attributes(end,3) = {Defs} ; % Defs Struct
        Attributes(end,4) = {Calendar} ;
        Attributes = GOAT_ALP_CompressAttributes(Attributes) ;
        if strcmp(Calendar,'List')
            mkdir(fullfile(Data_LINK,FieldName,'Invariant')) ;
            save(fullfile(Data_LINK,FieldName,'Invariant','link_field.mat'),'Attributes') ;
        else
            save(fullfile(Data_LINK,FieldName,'link_field.mat'),'Attributes') ;
        end
    end
    TT = toc ;
    HH = floor(TT/3600) ;
    MM = floor((TT - HH*3600)/60) ;
    SS = TT - HH*3600 - MM*60 ;
    if HH > 0
        TimeString = [int2str(HH),' hours, ',int2str(MM),' minutes and ',int2str(SS),' seconds.'] ;
    elseif MM > 0
        TimeString = [int2str(MM),' minutes and ',int2str(SS),' seconds.'] ;
    elseif SS > 0
        TimeString = [int2str(SS),' seconds.'] ;
    end
    disp(['Linking ORAS4  --  completed in ',TimeString]) ;










