% GOAT_AutoDownload_CloudSatCalipso
% Copyright (C) Ori Adam Dec.2012. Revised Dec.2013. Compatible with version 2013L or later.
% Input:
% Mode = 'Meta': obtain the following global dataset properties
% D.AutoDownloadDataset.ActiveFlag: 1/0 (yes/no) whether to include the Dataset in the list of datasets appearing in the GUI Fields dropdown menu.
%                  when 0 the dataset will not appear in the GUI list.
% D.AutoDownloadDataset.Name: returns How the dataset name appears in the GUI
% D.AutoDownloadDataset.FolderName: returns the name of the default physical
%                            folder name where the dataset is saved. Also used for substructure name for this dataset
% D.AutoDownloadDataset.DescriptionTip: a brief description of the dataset
% D.AutoDownloadDataset.CurrentYear:  1/0 Flag. Allow downloading files from current year (1 = yes | 0 = no)
% D.AutoDownloadDataset.TimeSelect: 1/0 (yes/no) whether to allow selection of years or not. set to 1 when raw data is given in years and 0 when the data
%                      the Dataset is downloaded as one block.
% D.AutoDownloadDataset.Fields.Names: a cell string list of the fields available for download
% Mode = 'Field': returns the field specifications
% Mode = 'Download': the actual field download code.
% D = GOAT_AutoDownload main structure
% There is no limit to the number of Fields or datasets that can be added.
%
% see also GOAT_VirtualFields, GOAT_AutoDownload

function D = GOAT_AutoDownload_CloudSatCalipso(Mode,D,Field)
warning off all
% This function contains all the operations required for Auto Downloading
% geophysical data and installing it in GOAT Format
switch Mode
    case 'Meta'
        D.AutoDownloadDataset.ActiveFlag =  1 ; % | 1 for 'yes' and 0 for 'no'
        D.AutoDownloadDataset.Name = 'CloudSatCalipso' ;
        D.AutoDownloadDataset.FolderName = 'CloudSatCalipso' ;
        D.AutoDownloadDataset.DescriptionTip = sprintf(['Combined CloudSat (a 94 GHz radar) and CALIOP (a 532/1064 nm depolarization lidar)', ...
            '\nglobal cloud distributions. R04 release over the period June 2006 to February 2011.', ...
            '\nproduct derived by Jennifer E. Kay. more information at:', ...
            '\nhttp://www.cgd.ucar.edu/staff/jenkay/cloudsatcaliop/methods.html']) ;
        D.AutoDownloadDataset.CurrentYear = 0 ;
        D.AutoDownloadDataset.TimeSelect = 0 ;
        D.AutoDownloadDataset.Fields.Names = {'Profile','VerticallyIntegrated','Climatology'} ;
    case 'Field'
        % Defaults
        D.AutoDownloadDataset.VerticalOrdinate = 'Height (m)' ; % Pressure (mb) | Height (m) | Depth (m) | Sigma | Potential Temperature (K) | Level
        
        D.AutoDownloadDataset.FirstYear = 2006 ;
        D.AutoDownloadDataset.LastYear = 2011 ; % GOAT will use year before present if LastYear is later then the current year
        D.AutoDownloadDataset.TimeResolution = 'Monthly' ;
        switch Field
            case 'Profile'
                D.AutoDownloadDataset.FieldDescription = '3D profiles of cloud count and fraction' ;
                D.AutoDownloadDataset.Units = '%' ;
                D.AutoDownloadDataset.LevelType = '3D' ; % 3D | Surface | TOA
                D.AutoDownloadDataset.url = 'http://www.cgd.ucar.edu/staff/jenkay/cloudsatcaliop/CloudSatCALIPSO_for_distribution/cloudsatcalipso/profile/geoprof_stats_global_YYYYMM_2deg_asl_nobin1tobin3_R04_addlidar.nc' ;
            case 'VerticallyIntegrated'
                D.AutoDownloadDataset.FieldDescription = sprintf(['vertically integrated cloud fraction and count', ...
                    '\nmid, high, low and total levels']) ;
                D.AutoDownloadDataset.Units = '' ;
                D.AutoDownloadDataset.LevelType = 'Surface' ; % 3D | Surface | TOA
                D.AutoDownloadDataset.url = 'http://www.cgd.ucar.edu/staff/jenkay/cloudsatcaliop/CloudSatCALIPSO_for_distribution/cloudsatcalipso/verticallyintegrated/geoprof_stats_bbp_global_YYYYMM_2deg_asl_ccsmhigh_2bins_nobin1tobin3_R04_addlidar.nc' ;
                % 'http://www.cgd.ucar.edu/staff/jenkay/cloudsatcaliop/CloudSatCALIPSO_for_distribution/cloudsatcalipso/verticallyintegrated/geoprof_stats_bbp_global_YYYYMM_2deg_asl_ccsmlow_2bins_nobin1tobin3_R04_addlidar.nc
            case 'Climatology'
                D.AutoDownloadDataset.FieldDescription = 'cloud count and fraction climatology' ;
                D.AutoDownloadDataset.Units = '%' ;
                D.AutoDownloadDataset.LevelType = '3D' ; % 3D | Surface | TOA
                D.AutoDownloadDataset.url = 'http://www.cgd.ucar.edu/staff/jenkay/cloudsatcaliop/CloudSatCALIPSO_for_distribution/cloudsatcalipso/verticallyintegrated/geoprof_stats_bbp_global_MMM_LLL.2deg_asl_nobin1tobin3_R04_addlidar.nc' ;
                % http://www.cgd.ucar.edu/staff/jenkay/cloudsatcaliop/CloudSatCALIPSO_for_distribution/cloudsatcalipso/profile/geoprof_stats_global_ANN_2deg_asl_nobine1tobin3_R04_addlidar.nc
        end
    case 'Download'
        warning off all
        DownloadFolder = fullfile('..','Downloads','temp','') ;
        mkdir(DownloadFolder) ;
        % Boer Beta geography
        load('Topography.mat') ; 
        %topolat = [-89.5:89.5];     % CREATE LAT ARRAY FOR TOPO MATRIX
        %topolon = [0.5:359.5];      % CREATE LON ARRAY FOR TOPO MATRIX
        topo = topo' ;
        topo = (topo(1:end-1,:) + topo(2:end,:))/2 ;
        topo = (topo(:,1:end-1) + topo(:,2:end))/2 ;
        topo = topo(1:2:end,1:2:end) ;
        topolat = [-89:2:89];     % decimated lat
        topolon = [-179:2:179];    % decimated lon
        n79 = dsearchn(topolon',179)  ;
        trepN = 40 ;
        topo = [topo(n79+1:end,:) ; topo(1:n79,:)] ;
        topoRep = repmat(reshape(topo,size(topo,1),size(topo,2),1),[1 1 trepN] ) ;
        if strcmp(D.Field.Name,'Climatology')
            Year1 = 1 ;
            Year2 = 1 ;
            Times = {'Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep','Oct','Nov','Dec','ANN','DJF','JJA','MAM','SON'} ;
            mm = length(Times) ;
            D.Field.TimeResolution = 'List' ;
        else
            Year1 = min([D.Field.BeginYear,D.Field.EndYear]) ;
            Year2 = max([D.Field.BeginYear,D.Field.EndYear]) ;
            mm  = 12 ;
        end
        for y = Year1:Year2
            for m = 1:mm
                switch D.Field.Name
                    case 'Profile'
                        URL = regexprep(D.Field.url, 'YYYY', int2str(y)) ;
                        URL = regexprep(URL, 'MM', num2str(m,'%02.0f')) ;
                        DimsFlag = 1 ;
                        FileName = [int2str(y),D.Field.Name,'Salipso.nc'] ;
                        if (y==2006 && m < 7) || (y==2011 && m > 2)
                            continue ;
                        end
                        set(D.h_message,'string',['Downloading profile variables: ',int2str(y),'.',int2str(m),' . . .']) ; drawnow ;
                        for da = 1:10
                            try
                                urlwrite(URL,fullfile(DownloadFolder,FileName)) ;
                                break ;
                            catch
                                if da < 10
                                    set(D.h_message,'string',['Download attempt ',int2str(da),' of ',D.Field.Name,': ',int2str(y),' Failed. Waiting 5 min and Retrying . . .']) ; drawnow ;
                                    pause(300) ;
                                else
                                    set(D.h_message,'string',['Download attempt ',int2str(da),' of ',D.Field.Name,': ',int2str(y),' Failed. Download Terminated.']) ; drawnow ;
                                    return ;
                                end
                            end
                        end
                        Data = GOAT_nc2mat(fullfile(DownloadFolder,FileName)) ;
                        GOATAddress = fullfile(D.DataSet.Address,D.Field.TimeResolution) ;
                        if DimsFlag
                            lon = Data.lon ;
                            lat = Data.lat ;
                            P = Data.dim4*1000 ;
                            ht = repmat(reshape(P,1,1,length(P)),[180 90 1]) ;
                            bata = ht*0 + 1 ;
                            bata(ht<topoRep) = 0 ;
                            Units = 'Unitless' ;
                            VerticalOrdinate = D.Field.VerticalOrdinate ;
                            FieldDescription = 'cloud count'  ;
                            mkdir(fullfile(GOATAddress,'cloudcount')) ;
                            save(fullfile(GOATAddress,'cloudcount','Dims.mat'),'lon','lat','P','Units','FieldDescription','VerticalOrdinate') ;
                            FieldDescription = 'total count'  ;
                            mkdir(fullfile(GOATAddress,'totalcount')) ;
                            save(fullfile(GOATAddress,'totalcount','Dims.mat'),'lon','lat','P','Units','FieldDescription','VerticalOrdinate') ;
                            FieldDescription = 'cloud fraction'  ;
                            Units = '%' ;
                            mkdir(fullfile(GOATAddress,'cl')) ;
                            save(fullfile(GOATAddress,'cl','Dims.mat'),'lon','lat','P','Units','FieldDescription','VerticalOrdinate') ;
                            FieldDescription = 'Boer Beta Function'  ;
                            Units = 'Unitless' ;
                            mkdir(fullfile(GOATAddress,'BoerBeta')) ;
                            save(fullfile(GOATAddress,'BoerBeta','Dims.mat'),'lon','lat','P','Units','FieldDescription','VerticalOrdinate') ;
                            DimsFlag = 0  ;
                        end
                        set(D.h_message,'string',['Installing cloud fields: ',int2str(y),'.',int2str(m),' . . .']) ; drawnow;
                        data = permute(Data.dim1,[2 1 3]) ;
                        if ~exist(fullfile(GOATAddress,'cloudcount',int2str(y)),'dir')
                            mkdir(fullfile(GOATAddress,'cloudcount',int2str(y))) ;
                        end
                        save(fullfile(GOATAddress,'cloudcount',int2str(y),[int2str(m*10000),'.mat']),'data') ;
                        data = permute(Data.dim2,[2 1 3]) ;
                        if ~exist(fullfile(GOATAddress,'totalcount',int2str(y)),'dir')
                            mkdir(fullfile(GOATAddress,'totalcount',int2str(y))) ;
                        end
                        save(fullfile(GOATAddress,'totalcount',int2str(y),[int2str(m*10000),'.mat']),'data') ;
                        data = permute(Data.dim3,[2 1 3])*100 ;
                        if ~exist(fullfile(GOATAddress,'cl',int2str(y)),'dir')
                            mkdir(fullfile(GOATAddress,'cl',int2str(y))) ;
                        end
                        save(fullfile(GOATAddress,'cl',int2str(y),[int2str(m*10000),'.mat']),'data') ;
                        if ~exist(fullfile(GOATAddress,'BoerBeta',int2str(y)),'dir')
                            mkdir(fullfile(GOATAddress,'BoerBeta',int2str(y))) ;
                        end
                        save(fullfile(GOATAddress,'BoerBeta',int2str(y),[int2str(m*10000),'.mat']),'bata') ;
                        delete(fullfile(DownloadFolder,FileName)) ;
                    case 'VerticallyIntegrated'
                        DimsFlag = 1 ;
                        URL = regexprep(D.Field.url, 'YYYY', int2str(y)) ;
                        URL = regexprep(URL, 'MM', num2str(m,'%02.0f')) ;
                        URL0 = URL ;
                        for s = 1:4
                            switch s
                                case 1
                                    level = 'high' ;
                                    Plevel = 14.75*1000 ;
                                    LevelDescription = 'averaged between 7-22.5 km' ;
                                    URL = URL0 ;
                                case 2
                                    level = 'low' ;
                                    Plevel = 1.855*1000 ;
                                    LevelDescription = 'averaged between 0.96-2.75 km' ;
                                    URL = regexprep(URL0, 'high','low') ;
                                case 3
                                    level = 'mid' ;
                                    Plevel = 4.875*1000 ;
                                    LevelDescription = 'averaged between 2.75-7 km' ;
                                    URL = regexprep(URL0, 'high','mid') ;
                                case 4
                                    level = 'total' ;
                                    URL = regexprep(URL0, 'ccsmhigh_2bins','total_4bins') ;
                            end
                            
                            FileName = [int2str(y),D.Field.Name,'Salipso.nc'] ;
                            if (y==2006 && m < 7) || (y==2011 && m > 2)
                                continue ;
                            end
                            set(D.h_message,'string',['Downloading vertically integrated ',level,' variables: ',int2str(y),'.',int2str(m),' . . .']) ; drawnow ;
                            for da = 1:10
                                try
                                    urlwrite(URL,fullfile(DownloadFolder,FileName)) ;
                                    break ;
                                catch
                                    if da < 10
                                        set(D.h_message,'string',['Download attempt ',int2str(da),' of ',D.Field.Name,': ',int2str(y),'.',int2str(m),' Failed. Waiting 5 min and Retrying . . .']) ; drawnow ;
                                        pause(300) ;
                                    else
                                        set(D.h_message,'string',['Download attempt ',int2str(da),' of ',D.Field.Name,': ',int2str(y),'.',int2str(m),' Failed. Download Terminated.']) ; drawnow ;
                                        return ;
                                    end
                                end
                            end
                            Data = GOAT_nc2mat(fullfile(DownloadFolder,FileName)) ;
                            GOATAddress = fullfile(D.DataSet.Address,D.Field.TimeResolution) ;
                            if DimsFlag
                                lon = Data.lon ;
                                lat = Data.lat ;
                                P = Plevel ;
                                Units = 'Unitless' ;
                                VerticalOrdinate = D.Field.VerticalOrdinate ;
                                FieldDescription = ['cloud count ',LevelDescription]  ;
                                mkdir(fullfile(GOATAddress,['cloudcount_',level])) ;
                                save(fullfile(GOATAddress,['cloudcount_',level],'Dims.mat'),'lon','lat','P','Units','FieldDescription','VerticalOrdinate') ;
                                FieldDescription = ['total count ',LevelDescription]  ;
                                mkdir(fullfile(GOATAddress,['totalcount_',level])) ;
                                save(fullfile(GOATAddress,['totalcount_',level],'Dims.mat'),'lon','lat','P','Units','FieldDescription','VerticalOrdinate') ;
                                FieldDescription = ['cloud fraction ',LevelDescription]  ;
                                Units = '%' ;
                                mkdir(fullfile(GOATAddress,['cl_',level])) ;
                                save(fullfile(GOATAddress,['cl_',level],'Dims.mat'),'lon','lat','P','Units','FieldDescription','VerticalOrdinate') ;
                                if s==4
                                    DimsFlag = 0  ;
                                end
                            end
                            set(D.h_message,'string',['Installing ',level,' level cloud fields: ',int2str(y),'.',int2str(m),' . . .']) ; drawnow;
                            data = Data.dim1' ;
                            if ~exist(fullfile(GOATAddress,['cloudcount_',level],int2str(y)),'dir')
                                mkdir(fullfile(GOATAddress,['cloudcount_',level],int2str(y))) ;
                            end
                            save(fullfile(GOATAddress,['cloudcount_',level],int2str(y),[int2str(m*10000),'.mat']),'data') ;
                            data = Data.dim2' ;
                            if ~exist(fullfile(GOATAddress,['totalcount_',level],int2str(y)),'dir')
                                mkdir(fullfile(GOATAddress,['totalcount_',level],int2str(y))) ;
                            end
                            save(fullfile(GOATAddress,['totalcount_',level],int2str(y),[int2str(m*10000),'.mat']),'data') ;
                            data = Data.dim3'*100 ;
                            if ~exist(fullfile(GOATAddress,['cl_',level],int2str(y)),'dir')
                                mkdir(fullfile(GOATAddress,['cl_',level],int2str(y))) ;
                            end
                            save(fullfile(GOATAddress,['cl_',level],int2str(y),[int2str(m*10000),'.mat']),'data') ;
                            delete(fullfile(DownloadFolder,FileName)) ;
                        end
                    case 'Climatology'
                        DimsFlag = 1 ;
                        URL = regexprep(D.Field.url, 'MMM', Times{m}) ;
                        for s = 1:5  % 4 level averages low mid high total
                            switch s
                                case 1
                                    level = 'high' ;
                                    Plevel = 14.75*1000 ;
                                    LevelDescription = 'averaged between 7-22.5 km' ;
                                    URL = regexprep(URL, 'LLL','ccsmhigh') ;
                                case 2
                                    level = 'low' ;
                                    Plevel = 1.855*1000 ;
                                    LevelDescription = 'averaged between 0.96-2.75 km' ;
                                    URL = regexprep(URL, 'LLL','ccsmlow') ;
                                case 3
                                    level = 'mid' ;
                                    Plevel = 4.875*1000 ;
                                    LevelDescription = 'averaged between 2.75-7 km' ;
                                    URL = regexprep(URL, 'LLL','ccsmmid') ;
                                case 4
                                    level = 'total' ;
                                    URL = regexprep(URL, 'LLL','total') ;
                                case 5
                                    level = 'profile' ;
                                    LevelDescription = 'per level climatology' ;
                                    if m <  13
                                    URL = regexprep(D.Field.url, 'verticallyintegrated/geoprof_stats_bbp_global_MMM_LLL.2deg_asl_nobin1tobin3_R04_addlidar','profile/geoprof_stats_global_MMM_2deg_asl_nobin1tobin3_R04_addlidar') ;
                                    else
                                    URL = regexprep(D.Field.url, 'verticallyintegrated/geoprof_stats_bbp_global_MMM_LLL.2deg_asl_nobin1tobin3_R04_addlidar','profile/geoprof_stats_global_MMM_2deg_asl_nobine1tobin3_R04_addlidar') ;    
                                    end
                                    URL = regexprep(URL, 'MMM', Times{m}) ;
                            end
                            FileName = [D.Field.Name,'Salipso.nc'] ;
                            set(D.h_message,'string',['Downloading ',level,' ',Times{m},' climatology . . .']) ; drawnow ;
                            for da = 1:10
                                try
                                    urlwrite(URL,fullfile(DownloadFolder,FileName)) ;
                                    break ;
                                catch
                                    if da < 10
                                        set(D.h_message,'string',['Download attempt ',int2str(da),' of ',D.Field.Name,':  Failed. Waiting 5 min and Retrying . . .']) ; drawnow ;
                                        pause(300) ;
                                    else
                                        set(D.h_message,'string',['Download attempt ',int2str(da),' of ',D.Field.Name,':  Failed. Download Terminated.']) ; drawnow ;
                                        return ;
                                    end
                                end
                            end
                            Data = GOAT_nc2mat(fullfile(DownloadFolder,FileName)) ;
                            GOATAddress = fullfile(D.DataSet.Address,D.Field.TimeResolution) ;
                            if DimsFlag
                                lon = Data.lon ;
                                lat = Data.lat ;
                                if s~=5
                                    P = Plevel ;
                                else
                                    P = Data.var1*1000 ;
                                end
                                VerticalOrdinate = D.Field.VerticalOrdinate ;
                                FieldDescription = ['cloud fraction ',LevelDescription]  ;
                                Units = '%' ;
                                mkdir(fullfile(GOATAddress,['cl_',level])) ;
                                save(fullfile(GOATAddress,['cl_',level],'Dims.mat'),'lon','lat','P','Units','FieldDescription','VerticalOrdinate') ;
                                if s==5
                                    DimsFlag = 0  ;
                                end
                            end
                            set(D.h_message,'string',['Installing ',level,' level cloud fields: ',int2str(y),'.',int2str(m),' . . .']) ; drawnow;
%                             if s < 5
%                                 data = Data.dim1' ;
%                             else
%                                 data = permute(Data.dim1,[2 1 3]) ;
%                             end
%                             if m < 13
%                                 if ~exist(fullfile(GOATAddress,['cloudcount_',level],'Months'),'dir')
%                                     mkdir(fullfile(GOATAddress,['cloudcount_',level],'Months')) ;
%                                     save(fullfile(GOATAddress,['cloudcount_',level],'Months',[int2str(m*10000),'.mat']),'data') ;
%                                 end
%                             else
%                                 if ~exist(fullfile(GOATAddress,['cloudcount_',level],'Seasons'),'dir')
%                                     mkdir(fullfile(GOATAddress,['cloudcount_',level],'Seasons')) ;
%                                     save(fullfile(GOATAddress,['cloudcount_',level],'Seasons',[Times{m},'.mat']),'data') ;
%                                 end
%                             end
%                             if s < 5
%                                 data = Data.dim2' ;
%                             else
%                                 data = permute(Data.dim2,[2 1 3]) ;
%                             end
%                             if m < 13
%                                 if ~exist(fullfile(GOATAddress,['totalcount_',level],'Months'),'dir')
%                                     mkdir(fullfile(GOATAddress,['totalcount_',level],'Months')) ;
%                                     save(fullfile(GOATAddress,['totalcount_',level],'Months',[int2str(m*10000),'.mat']),'data') ;
%                                 end
%                             else
%                                 if ~exist(fullfile(GOATAddress,['totalcount_',level],'Seasons'),'dir')
%                                     mkdir(fullfile(GOATAddress,['totalcount_',level],'Seasons')) ;
%                                     save(fullfile(GOATAddress,['totalcount_',level],'Seasons',[Times{m},'.mat']),'data') ;
%                                 end
%                             end
                            
                            if s < 5
                                data = Data.dim1'*100 ;
                            else
                                data = permute(Data.dim1,[2 1 3])*100 ;
                            end
                            if m < 13
                                if ~exist(fullfile(GOATAddress,['cl_',level],'Months'),'dir')
                                    mkdir(fullfile(GOATAddress,['cl_',level],'Months')) ;
                                end
                                    save(fullfile(GOATAddress,['cl_',level],'Months',[int2str(m*10000),'.mat']),'data') ;
                            else
                                if ~exist(fullfile(GOATAddress,['cl_',level],'Seasons'),'dir')
                                    mkdir(fullfile(GOATAddress,['cl_',level],'Seasons')) ;
                                end
                                    save(fullfile(GOATAddress,['cl_',level],'Seasons',[Times{m},'.mat']),'data') ;
                            end    
                            delete(fullfile(DownloadFolder,FileName)) ;
                        end                 
                end
            end
        end
        set(D.h_message,'string',[D.Field.Name,': Installation Complete.']) ; drawnow ;
end

return