% GOAT_AutoDownload_MERRA DailyX24
% Copyright (C) Ori Adam Dec.2012. Revised Dec.2013. Compatible with version 2013L or later.
% Input:
% Mode = 'Meta': obtain the following global dataset properties
% D.AutoDownloadDataset.ActiveFlag: 1/0 (yes/no) whether to include the Dataset in the list of datasets appearing in the GUI Fields dropdown menu.
%                  when 0 the dataset will not appear in the GUI list.
% D.AutoDownloadDataset.Name: returns How the dataset name appears in the GUI
% D.AutoDownloadDataset.FolderName: returns the name of the default physical
%                            folder name where the dataset is saved. Also used for substructure name for this dataset
% D.AutoDownloadDataset.DescriptionTip: a brief description of the dataset
% % D.AutoDownloadDataset.CurrentYear:  1/0 Flag. Allow downloading files from current year (1 = yes | 0 = no)
% D.AutoDownloadDataset.TimeSelect: 1/0 (yes/no) whether to allow selection of years or not. set to 1 when raw data is given in years and 0 when the data
%                      the Dataset is downloaded as one block.
% D.AutoDownloadDataset.Fields.Names: a cell string list of the fields available for download
% Mode = 'Field': returns the field specifications
% Mode = 'Download': the actual field download code.
% D = GOAT_AutoDownload main structure
% There is no limit to the number of Fields or datasets that can be added.
% NOTE: on line 153, change InpaintFlag to 1 if you want nan values ot be interpolated using inpaintn
%
% see also GOAT_VirtualFields, GOAT_AutoDownload

function D = GOAT_AutoDownload_MERRAX24(Mode,D,Field)
warning off all
% This function contains all the operations required for Auto Downloading
% geophysical data and installing it in GOAT Format
switch Mode
    case 'Meta'
        D.AutoDownloadDataset.ActiveFlag =  1 ; % | 1 for 'yes' and 0 for 'no'
        D.AutoDownloadDataset.Name = 'MERRA DailyX24' ; % comment this and use name below for interpolated nan data
        D.AutoDownloadDataset.FolderName = 'MERRAX24' ;
        D.AutoDownloadDataset.DescriptionTip = sprintf(['The Modern Era Retrospective-Analysis for Research and Applications (MERRA)', ...
            '\nwas undertaken by NASA?s Global Modeling and Assimilation Office with two primary objectives:', ...
            '\nto place observations from NASA?s Earth Observing System satellites into a climate context', ...
            '\nand to improve upon the hydrologic cycle represented in earlier generations of reanalyses.', ...
            '\nMERRA was generated with version 5.2.0 of the Goddard Earth Observing System (GEOS) atmospheric model and data assimilation system (DAS),', ...
            '\nand covers the modern satellite era from 1979 to the present.', ...
            '\nSpecifically, the GEOS-DAS Version 5  implements Incremental Analysis Updates (IAU) to slowly adjust the model states toward the observed state.', ...
            '\n(Description Source: NCAR-UCAR Climate Data Guide)']) ;
        D.AutoDownloadDataset.CurrentYear = 1 ; %  1/0 Flag. Allow downloading files from current year (1 = yes | 0 = no)
        D.AutoDownloadDataset.TimeSelect = 1 ;
        D.AutoDownloadDataset.Fields.Names = {'pr','prML','er','erML','hfls','hflsML','hfss','hfssML','ts','tsML','ps','swdwn'} ; % 
    case 'Field'
        % Defaults
        D.AutoDownloadDataset.VerticalOrdinate = 'Pressure (mb)' ; % Pressure (mb) | Height (m) | Depth (m) | Sigma | Potential Temperature (K) | Level
        D.AutoDownloadDataset.LevelType = '3D' ; % 3D | Surface | TOA
        D.AutoDownloadDataset.FirstYear = 1979 ;
        D.AutoDownloadDataset.LastYear = 2020 ; % GOAT will use year before present if LastYear is later then the current year
        D.AutoDownloadDataset.TimeResolution = 'DailyX24' ;
        switch Field
            case 'swdwn'                                
                D.AutoDownloadDataset.FieldDescription = 'Total Downward Shortwave radiation at surface' ;
                D.AutoDownloadDataset.Units = 'W/m^2' ;
                D.AutoDownloadDataset.FirstYear = 1979 ;
                D.AutoDownloadDataset.LevelType = 'Surface' ; % 3D | Surface | TOA
                D.AutoDownloadDataset.url = 'http://goldsmr2.sci.gsfc.nasa.gov/daac-bin/OTF/HTTP_services.cgi?FILENAME=%2Fdata%2Fs4pa%2FMERRA%2FMAT1NXRAD.5.2.0%2Fyyyy%2Fmm%2FMERRAMMM.prod.assim.tavg1_2d_rad_Nx.yyyymmdd.hdf&FORMAT=TmV0Q0RGLw&BBOX=-90%2C-180%2C90%2C180&LABEL=MERRAMMM.prod.assim.tavg1_2d_rad_Nx.yyyymmdd.SUB.nc&FLAGS=&SHORTNAME=MAT1NXRAD&SERVICE=SUBSET_LATS4D&LAYERS=&VERSION=1.02&VARIABLES=swgdn' ;        
                %                            http://goldsmr2.sci.gsfc.nasa.gov/daac-bin/OTF/HTTP_services.cgi?FILENAME=%2Fdata%2Fs4pa%2FMERRA%2FMAT1NXRAD.5.2.0%2F1979%2F01%2FMERRA101.prod.assim.tavg1_2d_rad_Nx.19790101.hdf&FORMAT=TmV0Q0RGLw&BBOX=-90%2C-180%2C90%2C180&LABEL=MERRA101.prod.assim.tavg1_2d_rad_Nx.19790101.SUB.nc&FLAGS=&SHORTNAME=MAT1NXRAD&SERVICE=SUBSET_LATS4D&LAYERS=&VERSION=1.02&VARIABLES=swgdn
            case 'pr'
                D.AutoDownloadDataset.FieldDescription = 'Precipitation rate (assimilated)' ;
                D.AutoDownloadDataset.Units = 'kg/m^2/s' ;
                D.AutoDownloadDataset.LevelType = 'Surface' ; % 3D | Surface | TOA
                D.AutoDownloadDataset.url = 'http://goldsmr2.sci.gsfc.nasa.gov/daac-bin/OTF/HTTP_services.cgi?FILENAME=%2Fdata%2Fs4pa%2FMERRA%2FMAT1NXFLX.5.2.0%2Fyyyy%2Fmm%2FMERRAMMM.prod.assim.tavg1_2d_flx_Nx.yyyymmdd.hdf&FORMAT=TmV0Q0RGLw&BBOX=-90%2C-180%2C90%2C180&LABEL=MERRAMMM.prod.assim.tavg1_2d_flx_Nx.yyyymmdd.SUB.nc&FLAGS=&SHORTNAME=MAT1NXFLX&SERVICE=SUBSET_LATS4D&LAYERS=&VERSION=1.02&VARIABLES=prectot' ;
            case 'hfls'
                D.AutoDownloadDataset.FieldDescription = 'Latent heat flux (assimilated)' ;
                D.AutoDownloadDataset.Units = 'W/m^2' ;
                D.AutoDownloadDataset.LevelType = 'Surface' ; % 3D | Surface | TOA
                D.AutoDownloadDataset.url = 'http://goldsmr2.sci.gsfc.nasa.gov/daac-bin/OTF/HTTP_services.cgi?FILENAME=%2Fdata%2Fs4pa%2FMERRA%2FMAT1NXFLX.5.2.0%2Fyyyy%2Fmm%2FMERRAMMM.prod.assim.tavg1_2d_flx_Nx.yyyymmdd.hdf&FORMAT=TmV0Q0RGLw&BBOX=-90%2C-180%2C90%2C180&LABEL=MERRAMMM.prod.assim.tavg1_2d_flx_Nx.yyyymmdd.SUB.nc&FLAGS=&SHORTNAME=MAT1NXFLX&SERVICE=SUBSET_LATS4D&LAYERS=&VERSION=1.02&VARIABLES=eflux' ;
            case 'hfss'
                D.AutoDownloadDataset.FieldDescription = 'Sensible heat flux (assimilated)' ;
                D.AutoDownloadDataset.Units = 'W/m^2' ;
                D.AutoDownloadDataset.LevelType = 'Surface' ; % 3D | Surface | TOA
                D.AutoDownloadDataset.url = 'http://goldsmr2.sci.gsfc.nasa.gov/daac-bin/OTF/HTTP_services.cgi?FILENAME=%2Fdata%2Fs4pa%2FMERRA%2FMAT1NXFLX.5.2.0%2Fyyyy%2Fmm%2FMERRAMMM.prod.assim.tavg1_2d_flx_Nx.yyyymmdd.hdf&FORMAT=TmV0Q0RGLw&BBOX=-90%2C-180%2C90%2C180&LABEL=MERRAMMM.prod.assim.tavg1_2d_flx_Nx.yyyymmdd.SUB.nc&FLAGS=&SHORTNAME=MAT1NXFLX&SERVICE=SUBSET_LATS4D&LAYERS=&VERSION=1.02&VARIABLES=hflux' ;    
            case 'er'
                D.AutoDownloadDataset.FieldDescription = 'Evaporation rate (assimilated)' ;
                D.AutoDownloadDataset.Units = 'kg/m^2/s' ;
                D.AutoDownloadDataset.LevelType = 'Surface' ; % 3D | Surface | TOA
                D.AutoDownloadDataset.url = 'http://goldsmr2.sci.gsfc.nasa.gov/daac-bin/OTF/HTTP_services.cgi?FILENAME=%2Fdata%2Fs4pa%2FMERRA%2FMAT1NXFLX.5.2.0%2Fyyyy%2Fmm%2FMERRAMMM.prod.assim.tavg1_2d_flx_Nx.yyyymmdd.hdf&FORMAT=TmV0Q0RGLw&BBOX=-90%2C-180%2C90%2C180&LABEL=MERRAMMM.prod.assim.tavg1_2d_flx_Nx.yyyymmdd.SUB.nc&FLAGS=&SHORTNAME=MAT1NXFLX&SERVICE=SUBSET_LATS4D&LAYERS=&VERSION=1.02&VARIABLES=evap' ;
            case 'erML'   
                D.AutoDownloadDataset.FieldDescription = 'Evaporation rate (MERRA Land - simulated)' ;
                D.AutoDownloadDataset.Units = 'kg/m^2/s' ;
                D.AutoDownloadDataset.LevelType = 'Surface' ; % 3D | Surface | TOA
                D.AutoDownloadDataset.url = 'http://goldsmr2.sci.gsfc.nasa.gov/daac-bin/OTF/HTTP_services.cgi?FILENAME=%2Fdata%2Fs4pa%2FMERRA%2FMAT1NXFLX.5.2.0%2Fyyyy%2Fmm%2FMERRAMMM.prod.simul.tavg1_2d_mld_Nx.yyyymmdd.hdf&FORMAT=TmV0Q0RGLw&BBOX=-90%2C-180%2C90%2C180&LABEL=MERRAMMM.prod.assim.tavg1_2d_flx_Nx.yyyymmdd.SUB.nc&FLAGS=&SHORTNAME=MAT1NXFLX&SERVICE=SUBSET_LATS4D&LAYERS=&VERSION=1.02&VARIABLES=evland' ; 
            case 'ps'
                D.AutoDownloadDataset.FieldDescription = 'Surface pressure (assimilated)' ;
                D.AutoDownloadDataset.Units = 'Pa' ;
                D.AutoDownloadDataset.LevelType = 'Surface' ; % 3D | Surface | TOA
                D.AutoDownloadDataset.url = 'http://goldsmr3.sci.gsfc.nasa.gov/daac-bin/OTF/HTTP_services.cgi?FILENAME=%2Fdata%2Fs4pa%2FMERRA%2FMAI3CPASM.5.2.0%2Fyyyy%2Fmm%2FMERRAMMM.prod.assim.inst3_3d_asm_Cp.yyyymmdd.hdf&FORMAT=TmV0Q0RGLw&BBOX=-90%2C-180%2C90%2C180&LABEL=MERRAMMM.prod.assim.inst3_3d_asm_Cp.yyyymmdd.SUB.nc&FLAGS=&SHORTNAME=MAI3CPASM&SERVICE=SUBSET_LATS4D&LAYERS=&VERSION=1.02&VARIABLES=ps' ;
            case 'prML'
                D.AutoDownloadDataset.FieldDescription = 'Precipitation rate (MERRA Land - simulated)' ;
                D.AutoDownloadDataset.Units = 'kg/m^2/s' ;
                D.AutoDownloadDataset.FirstYear = 1980 ;
                D.AutoDownloadDataset.LevelType = 'Surface' ; % 3D | Surface | TOA
                D.AutoDownloadDataset.url = 'http://goldsmr2.sci.gsfc.nasa.gov/daac-bin/OTF/HTTP_services.cgi?FILENAME=%2Fdata%2Fs4pa%2FMERRA%2FMST1NXMLD.5.2.0%2Fyyyy%2Fmm%2FMERRAMMM.prod.simul.tavg1_2d_mld_Nx.yyyymmdd.hdf&FORMAT=TmV0Q0RGLw&BBOX=-90%2C-180%2C90%2C180&LABEL=MERRAMMM.prod.simul.tavg1_2d_mld_Nx.yyyymmdd.SUB.nc&FLAGS=&SHORTNAME=MST1NXMLD&SERVICE=SUBSET_LATS4D&LAYERS=&VERSION=1.02&VARIABLES=prectot' ;
            case 'hflsML'                                
                D.AutoDownloadDataset.FieldDescription = 'Latent heat flux (MERRA Land - simulated)' ;
                D.AutoDownloadDataset.Units = 'W/m^2' ;
                D.AutoDownloadDataset.FirstYear = 1980 ;
                D.AutoDownloadDataset.LevelType = 'Surface' ; % 3D | Surface | TOA
                D.AutoDownloadDataset.url = 'http://goldsmr2.sci.gsfc.nasa.gov/daac-bin/OTF/HTTP_services.cgi?FILENAME=%2Fdata%2Fs4pa%2FMERRA%2FMST1NXMLD.5.2.0%2Fyyyy%2Fmm%2FMERRAMMM.prod.simul.tavg1_2d_mld_Nx.yyyymmdd.hdf&FORMAT=TmV0Q0RGLw&BBOX=-90%2C-180%2C90%2C180&LABEL=MERRAMMM.prod.simul.tavg1_2d_mld_Nx.yyyymmdd.SUB.nc&FLAGS=&SHORTNAME=MST1NXMLD&SERVICE=SUBSET_LATS4D&LAYERS=&VERSION=1.02&VARIABLES=lhland' ;
            case 'hfssML'                                
                D.AutoDownloadDataset.FieldDescription = 'Sensible heat flux (MERRA Land - simulated)' ;
                D.AutoDownloadDataset.Units = 'W/m^2' ;
                D.AutoDownloadDataset.FirstYear = 1980 ;
                D.AutoDownloadDataset.LevelType = 'Surface' ; % 3D | Surface | TOA
                D.AutoDownloadDataset.url = 'http://goldsmr2.sci.gsfc.nasa.gov/daac-bin/OTF/HTTP_services.cgi?FILENAME=%2Fdata%2Fs4pa%2FMERRA%2FMST1NXMLD.5.2.0%2Fyyyy%2Fmm%2FMERRAMMM.prod.simul.tavg1_2d_mld_Nx.yyyymmdd.hdf&FORMAT=TmV0Q0RGLw&BBOX=-90%2C-180%2C90%2C180&LABEL=MERRAMMM.prod.simul.tavg1_2d_mld_Nx.yyyymmdd.SUB.nc&FLAGS=&SHORTNAME=MST1NXMLD&SERVICE=SUBSET_LATS4D&LAYERS=&VERSION=1.02&VARIABLES=shland' ;    
            case 'swdownML'                                
                D.AutoDownloadDataset.FieldDescription = 'Downward Shortwave radiation (MERRA Land - simulated)' ;
                D.AutoDownloadDataset.Units = 'W/m^2' ;
                D.AutoDownloadDataset.FirstYear = 1980 ;
                D.AutoDownloadDataset.LevelType = 'Surface' ; % 3D | Surface | TOA
                D.AutoDownloadDataset.url = 'http://goldsmr2.sci.gsfc.nasa.gov/daac-bin/OTF/HTTP_services.cgi?FILENAME=%2Fdata%2Fs4pa%2FMERRA%2FMST1NXMLD.5.2.0%2Fyyyy%2Fmm%2FMERRAMMM.prod.simul.tavg1_2d_mld_Nx.yyyymmdd.hdf&FORMAT=TmV0Q0RGLw&BBOX=-90%2C-180%2C90%2C180&LABEL=MERRAMMM.prod.simul.tavg1_2d_mld_Nx.yyyymmdd.SUB.nc&FLAGS=&SHORTNAME=MST1NXMLD&SERVICE=SUBSET_LATS4D&LAYERS=&VERSION=1.02&VARIABLES=swland' ;        
            case 'tsML'
                D.AutoDownloadDataset.FieldDescription = 'Surface temeprature (MERRA Land - simulated)' ;
                D.AutoDownloadDataset.Units = 'K' ;
                D.AutoDownloadDataset.FirstYear = 1980 ;
                D.AutoDownloadDataset.LevelType = 'Surface' ; % 3D | Surface | TOA
                D.AutoDownloadDataset.url = 'http://goldsmr2.sci.gsfc.nasa.gov/daac-bin/OTF/HTTP_services.cgi?FILENAME=%2Fdata%2Fs4pa%2FMERRA%2FMST1NXMLD.5.2.0%2Fyyyy%2Fmm%2FMERRAMMM.prod.simul.tavg1_2d_mld_Nx.yyyymmdd.hdf&FORMAT=TmV0Q0RGLw&BBOX=-90%2C-180%2C90%2C180&LABEL=MERRAMMM.prod.simul.tavg1_2d_mld_Nx.yyyymmdd.SUB.nc&FLAGS=&SHORTNAME=MST1NXMLD&SERVICE=SUBSET_LATS4D&LAYERS=&VERSION=1.02&VARIABLES=tsurf' ;    
            case 'ts'
                D.AutoDownloadDataset.FieldDescription = 'Surface skin temperature (assimilated)' ;
                D.AutoDownloadDataset.Units = 'K' ;
                D.AutoDownloadDataset.LevelType = 'Surface' ; % 3D | Surface | TOA
                D.AutoDownloadDataset.url = 'http://goldsmr2.sci.gsfc.nasa.gov/daac-bin/OTF/HTTP_services.cgi?FILENAME=%2Fdata%2Fs4pa%2FMERRA%2FMAT1NXSLV.5.2.0%2Fyyyy%2Fmm%2FMERRAMMM.prod.assim.tavg1_2d_slv_Nx.yyyymmdd.hdf&FORMAT=TmV0Q0RGLw&BBOX=-90%2C-180%2C90%2C180&LABEL=MERRAMMM.prod.assim.tavg1_2d_slv_Nx.yyyymmdd.SUB.nc&FLAGS=&SHORTNAME=MAT1NXSLV&SERVICE=SUBSET_LATS4D&LAYERS=&VERSION=1.02&VARIABLES=ts' ;
        end
    case 'Download'
        warning off all
        InpaintFlag = 0 ; % 1/0 inpaintn(data)
        DownloadFolder = fullfile('..','Downloads','temp','') ;
        mkdir(DownloadFolder) ;
        % Boer Beta
        if strcmp(D.Field.Name,'zg')
            load('Topography.mat') ; 
            topolat = [-89.5:89.5];     % CREATE LAT ARRAY FOR TOPO MATRIX
            topolon = [0.5:359.5];      % CREATE LON ARRAY FOR TOPO MATRIX
            topo = topo' ;
            [X1, X2] = ndgrid(topolon,topolat);
            F = griddedInterpolant(X1,X2,topo);
            lat = single(linspace(-89.375,89.375,144))' ;
            lon = single(linspace(-179.375,179.375,288))' ;
            trepN = 42 ;
            latf = lat*-sign(lat(1)) ;
            [lonG, latG] = ndgrid(lon,latf);
            topoG = F(lonG,latG);
            topo90N = (topo(:,end)*1.5 - topo(:,end-1)*.5) ;
            topo90S = (topo(:,1  )*1.5 - topo(:,2)*.5) ;
            topoGN = interp1(topolon',topo90N,lon) ;
            topoGS = interp1(topolon',topo90S,lon) ;
            topoG(:,1) = topoGS ;
            topoG(:,end) = topoGN ;
            topo0 = (topo(1,:) + topo(end,:))/2 ;
            topoG0 = interp1(topolat,topo0,latf) ;
            topoG(1,:) = topoG0 ;
            topoG(1,  1) = (topo90S(1) + topo90S(end))/2 ;
            topoG(1,end) = (topo90N(1) + topo90N(end))/2 ;
            topoG = flipdim(topoG,2) ;
            topoRep = repmat(reshape(topoG,size(topoG,1),size(topoG,2),1),[1 1 trepN] ) ;
        end
        ResumeFlag = 1 ;
        DimsFlag   = 1 ;
        Year1 = min([D.Field.BeginYear,D.Field.EndYear]) ;
        Year2 = max([D.Field.BeginYear,D.Field.EndYear]) ;
        MEnd     = [0, 31, 59, 90 ,120, 151, 181, 212, 243, 273, 304, 334, 365]' ;
        MEndLeap = [0, 31, 60, 91 ,121, 152, 182, 213, 244, 274, 305, 335, 366]' ;
        GOATAddress = fullfile(D.DataSet.Address,D.Field.TimeResolution,D.Field.Name) ;
        for y = Year1:Year2
            clear Data data DataM
            if GOAT_leapyear(y)
                ME = MEndLeap ;
                JJ = 366 ;
            else
                ME = MEnd ;
                JJ = 365 ;
            end
            
            for j = 1:JJ ;
                m = dsearchn(ME,j) ;
                if j-ME(m) <= 0 ;
                    m = m - 1 ;
                end
                d = j - ME(m) ;
                mm = num2str(m,'%02.0f') ;
                dd = num2str(d,'%02.0f') ;
                if y == 2010 && any( m == [6,7,8])
                    MMM = '301' ;
                elseif y > 2000
                    MMM = '300' ;
                elseif y > 1992
                    MMM = '200' ;
                else
                    MMM = '100' ;
                end
                % Exceptions
                if y == 1980 && m == 8 && d == 12 &&  sum(strcmp(D.Field.Name,{'pr','er'}))
                    MMM = '101' ;
                end
                if strcmp(D.Field.Name,'prML')
                    if y == 2012 && any(m == [1,2,3])
                        MMM = '301' ;
                    elseif y > 2000
                        MMM = '300' ;
                    end
                end
                if strcmp(D.Field.Name,{'swdwn'})
                MMM = '101' ; 
                end
                URL = regexprep(D.Field.url, 'yyyy', int2str(y)) ;
                URL = regexprep(URL, 'MMM', MMM) ;
                URL = regexprep(URL, 'mm', mm) ;
                URL = regexprep(URL, 'dd', dd) ;
                set(D.h_message,'string',['Downloading ',D.Field.Name,': ',int2str(y),'.',mm,'.',dd,' . . .']) ; drawnow ;
                FileName = [int2str(y),mm,dd,D.Field.Name,'MERRA.nc'] ;
                if ResumeFlag
                    if exist(fullfile(GOATAddress,int2str(y),[int2str(m*10000+d*100),'.mat']),'file');
                        continue ; % If load is successful, continue
                    end
                end
                for da = 1:10
                    try
                        urlwrite(URL,fullfile(DownloadFolder,FileName)) ;
                        break ;
                    catch
                        if da < 10
                            set(D.h_message,'string',['Download attempt ',int2str(da),' of ',D.Field.Name,': ',int2str(y),'.',mm,'.',dd,' Failed. Waiting 5 min and Retrying . . .']) ; drawnow ;
                            pause(300) ;
                        else
                            set(D.h_message,'string',['Download attempt ',int2str(da),' of ',D.Field.Name,': ',int2str(y),'.',mm,'.',dd,' Failed. Download Terminated.']) ; drawnow ;
                            return ;
                        end
                    end
                end
                Data = GOAT_nc2mat(fullfile(DownloadFolder,FileName)) ;
                if DimsFlag
                    try
                        load(fullfile(GOATAddress,'Dims.mat'))
                    catch
                        lon = Data.lon ;
                        lat = Data.lat ;
                        switch D.Field.LevelType
                            case '3D'
                                P = Data.P ;
                            case 'Surface'
                                P = 1000 ;
                            case 'TOA'
                                P = 0    ;
                        end
                        Units = D.Field.Units ;
                        FieldDescription = D.Field.FieldDescription ;
                        VerticalOrdinate = D.Field.VerticalOrdinate ;
                        try
                            save(fullfile(GOATAddress,'Dims.mat'),'lon','lat','P','Units','FieldDescription','VerticalOrdinate') ;
                        catch
                            mkdir(GOATAddress) ;
                            save(fullfile(GOATAddress,'Dims.mat'),'lon','lat','P','Units','FieldDescription','VerticalOrdinate') ;
                        end
                        DimsFlag = 0 ;
                    end
                end
                %set(D.h_message,'string',['Installing ',D.Field.Name,': ',int2str(y),mm,dd,' . . .']) ; drawnow;
                Nh = size(Data.var1,length(size(Data.var1))) ; 
                for n = 1:Nh
                    
                    h = (n-1) ; 
                    
                    if strcmp(D.Field.LevelType,'3D')
                    data = squeeze(Data.var1(:,:,:,n)) ;
                    if strcmp(D.Field.Name,'zg')
                        bata = data*0+1 ;
                        bata(isnan(data)) = 0 ;
                    end
                    if InpaintFlag
                        data = single(inpaintn(data)) ;
                    else
                        data = single(data) ;
                    end
                else
                    data = single(squeeze(Data.var1(:,:,n))) ;
                end
                
                FileAddress = fullfile(GOATAddress,int2str(y),[int2str(m*10000+d*100+h),'.mat']);
                try
                    save(FileAddress,'data') ;
                catch
                    pathstr = fileparts(FileAddress) ;
                    mkdir(pathstr) ;
                    save(FileAddress,'data') ;
                end
                if strcmp(D.Field.Name,'zg')
                    %                             bata = data*0 ;
                    %                             bata(data>=topoRep) = 1 ;
                    FileAddress = fullfile(D.DataSet.Address,D.Field.TimeResolution,'BoerBeta',int2str(y),[int2str(m*10000+ d*100 + h),'.mat']) ;
                    pathstr = fileparts(FileAddress) ;
                    try
                        save(FileAddress,'bata') ;
                    catch
                        mkdir(pathstr) ;
                        save(FileAddress,'bata') ;
                    end
                end
                end
                delete(fullfile(DownloadFolder,FileName)) ;
            end
            
            
        end
        if strcmp(D.Field.Name,'zg')
            FieldDescription = 'Boer Beta Function' ;
            save(fullfile(D.DataSet.Address,D.Field.TimeResolution,'BoerBeta','Dims.mat'),'lon','lat','P','Units','FieldDescription','VerticalOrdinate') ;
        end
        set(D.h_message,'string',[D.Field.Name,': Installation Complete.']) ; drawnow ;
end

return