% GOAT_AutoDownload_WOA
% Copyright (C) Ori Adam Dec.2012. Revised Dec.2013. Compatible with version 2013L or later.
% Input:
% Mode = 'Meta': obtain the following global dataset properties
% D.AutoDownloadDataset.ActiveFlag: 1/0 (yes/no) whether to include the Dataset in the list of datasets appearing in the GUI Fields dropdown menu.
%                  when 0 the dataset will not appear in the GUI list.
% D.AutoDownloadDataset.Name: returns How the dataset name appears in the GUI
% D.AutoDownloadDataset.FolderName: returns the name of the default physical
%                            folder name where the dataset is saved. Also used for substructure name for this dataset
% D.AutoDownloadDataset.DescriptionTip: a brief description of the dataset
% D.AutoDownloadDataset.CurrentYear:  1/0 Flag. Allow downloading files from current year (1 = yes | 0 = no)
% D.AutoDownloadDataset.TimeSelect: 1/0 (yes/no) whether to allow selection of years or not. set to 1 when raw data is given in years and 0 when the data
%                      the Dataset is downloaded as one block.
% D.AutoDownloadDataset.Fields.Names: a cell string list of the fields available for download
% Mode = 'Field': returns the field specifications
% Mode = 'Download': the actual field download code.
% D = GOAT_AutoDownload main structure
% There is no limit to the number of Fields or datasets that can be added.
%
% see also GOAT_VirtualFields, GOAT_AutoDownload

function D = GOAT_AutoDownload_WOA(Mode,D,Field)
warning off all
% This function contains all the operations required for Auto Downloading
% geophysical data and installing it in GOAT Format
switch Mode
    case 'Meta'
        D.AutoDownloadDataset.ActiveFlag =  1 ; % | 1 for 'yes' and 0 for 'no'
        D.AutoDownloadDataset.Name = 'World Ocean Atlas 2013' ;
        D.AutoDownloadDataset.FolderName = 'WOA13' ;
        D.AutoDownloadDataset.DescriptionTip = sprintf(['The World Ocean Atlas (2013) is a set of objectively analyzed (1 grid) climatological fields of ', ...
            '\nin situ temperature, salinity, dissolved oxygen, Apparent Oxygen Utilization (AOU), percent oxygen saturation, ', ...
            '\nphosphate, silicate, and nitrate at standard depth levels for annual, seasonal, and monthly compositing periods for the World Ocean.', ...
            '\nIt also includes associated statistical fields of observed oceanographic profile data interpolated to standard depth levels on both 1 and 5 grids.', ...
            '\n(Source: http://www.nodc.noaa.gov/OC5/woa13/)']) ;
        D.AutoDownloadDataset.TimeSelect = 0 ;
        D.AutoDownloadDataset.Fields.Names = {'to','so','o2','no3','si','po4'} ;
    case 'Field'
        % Defaults
        D.AutoDownloadDataset.VerticalOrdinate = 'Depth (m)' ; % Pressure (mb) | Height (m) | Depth (m) | Sigma | Potential Temperature (K) | Level
        D.AutoDownloadDataset.LevelType = '3D' ; % 3D | Surface | TOA
        D.AutoDownloadDataset.FirstYear = 2013 ;
        D.AutoDownloadDataset.LastYear = 2013 ; % GOAT will use year before present if LastYear is later then the current year
        D.AutoDownloadDataset.TimeResolution = 'List' ;
        switch Field
            case 'to'
                D.AutoDownloadDataset.FieldDescription = 'Ocean temperature' ;
                D.AutoDownloadDataset.Units = 'C' ;
                D.AutoDownloadDataset.url = 'http://data.nodc.noaa.gov/thredds/fileServer/woa/WOA13/DATA/temperature/netcdf/decav/1.00/woa13_decav_tmm_01.nc' ;
            case 'so'
                D.AutoDownloadDataset.FieldDescription = 'Salinity' ;
                D.AutoDownloadDataset.Units = 'Unitless' ;
                D.AutoDownloadDataset.url = 'http://data.nodc.noaa.gov/thredds/fileServer/woa/WOA13/DATA/salinity/netcdf/decav/1.00/woa13_decav_smm_01.nc' ;
            case 'o2'
                D.AutoDownloadDataset.FieldDescription = 'Dissolved oxygen' ;
                D.AutoDownloadDataset.Units = '' ;
                D.AutoDownloadDataset.url = 'http://data.nodc.noaa.gov/thredds/fileServer/woa/WOA13/DATA/oxygen/netcdf/decav/1.00/woa13_decav_omm_01.nc' ;
                %case 'o2_sat'
                %    D.AutoDownloadDataset.FieldDescription = 'Oxygen saturation' ;
                %    D.AutoDownloadDataset.Units = '' ;
                %    D.AutoDownloadDataset.url = 'http://data.nodc.noaa.gov/thredds/fileServer/woa/WOA09/NetCDFdata/oxygen_saturation_monthly_1deg.nc' ;
                %case 'o2_util'
                %    D.AutoDownloadDataset.FieldDescription = 'Apparent oxygen utilization' ;
                %    D.AutoDownloadDataset.Units = '' ;
                %    D.AutoDownloadDataset.url = 'http://data.nodc.noaa.gov/thredds/fileServer/woa/WOA09/NetCDFdata/apparent_oxygen_utilization_monthly_1deg.nc' ;
            case 'no3'
                D.AutoDownloadDataset.FieldDescription = 'Nitrate' ;
                D.AutoDownloadDataset.Units = '' ;
                D.AutoDownloadDataset.url = 'http://data.nodc.noaa.gov/thredds/fileServer/woa/WOA13/DATA/nitrogen/netcdf/decav/1.00/woa13_decav_nmm_01.nc' ;
            case 'si'
                D.AutoDownloadDataset.FieldDescription = 'Silicate' ;
                D.AutoDownloadDataset.Units = '' ;
                D.AutoDownloadDataset.url = 'http://data.nodc.noaa.gov/thredds/fileServer/woa/WOA13/DATA/oxygen/netcdf/decav/1.00/woa13_decav_imm_01.nc' ;
            case 'po4'
                D.AutoDownloadDataset.FieldDescription = 'Phosphate' ;
                D.AutoDownloadDataset.Units = '' ;
                D.AutoDownloadDataset.url = 'http://data.nodc.noaa.gov/thredds/fileServer/woa/WOA13/DATA/oxygen/netcdf/decav/1.00/woa13_decav_pmm_01.nc' ;
        end
    case 'Download'
        warning off all
        DownloadFolder = fullfile('..','Downloads','temp','') ;
        mkdir(DownloadFolder) ;
        
        Year = D.Field.EndYear ;
        URL = regexprep(D.Field.url, 'mm', num2str(1,'%02.1g')) ;
        set(D.h_message,'string',['Downloading ',D.Field.Name,': ',' . . .']) ; drawnow ;
        FileName = [D.Field.Name,'WOA13.nc'] ;
        for da = 1:10
            try
                urlwrite(URL,fullfile(DownloadFolder,FileName)) ;
                break ;
            catch
                if da < 10
                    set(D.h_message,'string',['Download attempt ',int2str(da),' of ',D.Field.Name,': Failed. Waiting 5 min and Retrying . . .']) ; drawnow ;
                    pause(300) ;
                else
                    set(D.h_message,'string',['Download attempt ',int2str(da),' of ',D.Field.Name,': Failed. Download Terminated.']) ; drawnow ;
                    return ;
                end
            end
        end
        Data = GOAT_nc2mat(fullfile(DownloadFolder,FileName)) ;
        GOATAddress = fullfile(D.DataSet.Address,D.Field.TimeResolution,D.Field.Name) ;
        types =     {'an','mn','dd','ma','sd','se','oa','gp'} ;
        typesFlag = [ 1    1     1    1    1   1     1    1] ;
        try
            load(fullfile(GOATAddress,'Dims.mat'))
        catch
            lon = Data.lon ;
            lat = Data.lat ;
            switch D.Field.LevelType
                case '3D'
                    P = Data.P ;
                case 'Surface'
                    P = 1000 ;
                case 'TOA'
                    P = 0    ;
            end
            Units = D.Field.Units ;
            FieldDescription0 = D.Field.FieldDescription ;
            VerticalOrdinate = D.Field.VerticalOrdinate ;
            for j = find(typesFlag==1)
                switch types{j}
                    case 'an'
                        FieldDescription = [D.Field.FieldDescription,' - Objectively analyzed climatology'] ;
                    case 'mn'
                        FieldDescription = [D.Field.FieldDescription,' - Statistical mean'] ;
                    case 'dd'
                        FieldDescription = [D.Field.FieldDescription,' - Number of observations'] ;
                    case 'ma'
                        FieldDescription = [D.Field.FieldDescription,' - monthly climatology minus annual climatology'] ;
                    case 'sd'
                        FieldDescription = [D.Field.FieldDescription,' - Standard deviation from statistical mean'] ;
                    case 'se'
                        FieldDescription = [D.Field.FieldDescription,' - Standard error of the statistical mean'] ;
                    case 'oa'
                        FieldDescription = [D.Field.FieldDescription,' - Statistical mean minus objectively analyzed climatology'] ;
                    case 'gp'
                        FieldDescription = [D.Field.FieldDescription,' - Number of mean values within radius of influence'] ;
                end
                try
                    save(fullfile([GOATAddress,'_',types{j}],'Dims.mat'),'lon','lat','P','Units','FieldDescription','VerticalOrdinate') ;
                catch
                    mkdir([GOATAddress,'_',types{j}]) ;
                    save(fullfile([GOATAddress,'_',types{j}],'Dims.mat'),'lon','lat','P','Units','FieldDescription','VerticalOrdinate') ;
                end
            end
        end
        Months = {'Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep','Oct','Nov','Dec'} ; 
        for T = 1:7
            switch T
                case 1
                    Period = 'decav' ;
                    %Year = '2013' ;
                case 2
                    Period = '5564' ;
                    %Year = '1964' ;
                case 3
                    Period = '6574' ;
                    %Year = '1974' ;
                case 4
                    Period = '7584' ;
                    %Year = '1984' ;
                case 5
                    Period = '8594' ;
                    %Year = '1994' ;
                case 6
                    Period = '95A4' ;
                    %Year = '2004' ;
                case 7
                    Period = 'A5B2' ;
                    %Year = '2012' ;
            end
            delete(fullfile(DownloadFolder,FileName)) ;
            for m = 1:12
                URL = regexprep(D.Field.url, 'mm', num2str(m,'%02.0f')) ;
                URL = regexprep(URL, 'decav', Period) ;
                set(D.h_message,'string',['Downloading ',D.Field.Name,': ',Period,' ',Months{m},' . . .']) ; drawnow ;
                FileName = [D.Field.Name,'WOA13.nc'] ;
                for da = 1:10
                    try
                        urlwrite(URL,fullfile(DownloadFolder,FileName)) ;
                        break ;
                    catch
                        if da < 10
                            set(D.h_message,'string',['Download attempt ',int2str(da),' of ',D.Field.Name,': ',Period,' Failed. Waiting 5 min and Retrying . . .']) ; drawnow ;
                            pause(300) ;
                        else
                            set(D.h_message,'string',['Download attempt ',int2str(da),' of ',D.Field.Name,': ',Period,' Failed. Download Terminated.']) ; drawnow ;
                            return ;
                        end
                    end
                end
                Data = GOAT_nc2mat(fullfile(DownloadFolder,FileName)) ;
                for i = 1:20
                    Idx = [] ;
                    try
                        eval(['Idx = find(strcmpi(Data.var',int2str(i),'name(end-1:end),types))  ;']) ;
                        clear data
                        if~isempty(Idx) && typesFlag(Idx)
                            eval(['data = Data.var',int2str(i),' ; ']) ;
                            FileAddress = fullfile([GOATAddress,'_',types{Idx}],Period,[int2str(m*10000),'.mat']);
                            data(data>1.e15) = nan ; 
                            data = single(data) ; 
                            try
                                save(FileAddress,'data') ;
                            catch
                                pathstr = fileparts(FileAddress) ;
                                mkdir(pathstr) ;
                                save(FileAddress,'data') ;
                            end
                        end
                    end
                end
            end
            delete(fullfile(DownloadFolder,FileName)) ;
        end
        set(D.h_message,'string',[D.Field.Name,': Installation Complete.']) ; drawnow ;
end

return