% GOAT_ALP_ERAI_LINK link code for ERA-Interim DailyX4 , Daily  and  Monthly by using perl scripts to send batch jobs to the MARS API server


G = GOAT_Environment ;
Data_LINK = fullfile(fileparts(fileparts(which('GOAT.m'))),'Data_LINK','ERAI_Derived_LINK','Monthly') ; % LINK Data folder
DerivedFieldsDirectory = '/Volumes/data2/ERA-Interim_Derived' ;
totaltoc = 0 ;
Ext = 'ncf' ;
longitude_dimname = 'lon' ;
latitude_dimname  = 'lat' ;
level_dimname     = 'level' ;
time_dimname      = 'time';
dimnames = {longitude_dimname,latitude_dimname,level_dimname,time_dimname} ;
First_Year = 1979 ;
Last_Year = 2012 ;
%
FieldFiles = dir(DerivedFieldsDirectory) ;
FieldFiles = {FieldFiles(~cell2mat({FieldFiles(:).isdir}),1).name}' ;
FieldFiles = FieldFiles(~strncmpi(FieldFiles,'.',1)) ;
Calendar = 'Monthly' ;
tic
for f = 1:length(FieldFiles)
    
    infostr = sprintf(['\nMonthly Means from Daily Means', ...
        '\nEra-Interim data are used to derive monthly mass, moisture and energy budget products.', ...
        '\nEach term is available.', ...
        '\nAtmospheric energy, mass, and moisture budgets can be computed by vertically integrating reanalysis fields and employing suitable corrections (e.g. mass).', ...
        '\nThe budgets are uniquely valuable for interpreting the climate''s mean state and its variability.', ...
        '\nThey can also be used in conjunction with complementary data (i.e. top of atmosphere fluxes)', ...
        '\nto diagnose the surface energy budget and over large scales these estimates can rival or surpass other methods for doing so.', ...
        '\n(NCAR / K. Trenberth, John Fasullo)', ...
        '\n(reference: NCAR Climate Data Guide https://climatedataguide.ucar.edu/climate-data/era-interim-derived-components)', ...
        ]) ;
    
    
    if ~exist(Data_LINK,'dir')
        mkdir(Data_LINK) ;
    end
    save(fullfile(Data_LINK,'info.mat'),'infostr')
    VerticalOrdinate = 'Pressure (mb)' ;
    GOAT_Display_message(G,'','init')
    filename = fullfile(DerivedFieldsDirectory,FieldFiles{f}) ;
    a = ncinfo(filename) ;
    Variables = {a.Variables(:).Name} ;
    FieldName = Variables{find(~ismember(Variables,dimnames))} ;
    Attributes = a.Variables(find(~ismember(Variables,dimnames))).Attributes ;
    Units      = Attributes(find(strcmpi('units',{Attributes.Name}))).Value ;
    FieldDescription = Attributes(find(strcmpi('long_name',{Attributes.Name}))).Value ;
    if f == 1
        Time = ncread(filename,'time') ;
        lon = ncread(filename,'lon') ;
        lat = ncread(filename,'lat') ;
        qc = length(Time) ;
    end
    disp(['linking derived Monthly mean of ',FieldName,'  ',int2str(f),'/',num2str(length(FieldFiles),3),'  . . . '])
    mkdir(fullfile(Data_LINK,FieldName)) ;
    P = 1 ;
    save(fullfile(Data_LINK,FieldName,'Dims.mat'),'P','lon','lat','Units','VerticalOrdinate','FieldDescription') ;
    Attributes = cell(2,5) ;
    Attributes(end,5) = {qc} ;
    Col = cell(1,2) ;
    % Column 1: OPeNDAP Address
    Col(1,1) = {filename} ;
    Col(1,2) = {qc} ;
    Attributes(1,1) = {Col} ;
    % Column 2: source varname
    Col(1,1) = {FieldName} ;
    Col(1,2) = {qc} ;
    Attributes(1,2) = {Col} ;
    % Column 3: Start
    Col(1,1) = {['Start = ones(',int2str(qc),',3) ; ', ...
                'Start(:,',int2str(3),') = (1:',int2str(qc),')'' ; ', ...
                'Col = num2cell(Start,2) ; ']} ;
    Col(1,2) = {'eval'} ;
    Attributes(1,3) = {Col} ;
    % Column 4: Count
    Col(1,1) = {[inf inf 1]} ;
    Col(1,2) = {qc} ;
    Attributes(1,4) = {Col} ;
    
    Attributes(end,1) = {Ext} ; % 'eri'
    Attributes(end,2) = {[First_Year Last_Year]} ;
    Attributes(end,3) = {[]} ; % Defs Struct
    Attributes(end,4) = {Calendar} ;
    
    Attributes_check = GOAT_ALP_CompressAttributes(Attributes) ;
    
    save(fullfile(Data_LINK,FieldName,'link_field.mat'),'Attributes') ;
    J = 1 ; clear Text
    Text(J,1) = {['source file = ',filename]} ; J = J + 1 ;
    Text(J,1) = {['Variable name = ',FieldName]} ; J = J + 1 ;
    Text(J,1) = {['First Year = ',int2str(First_Year)]} ; J = J + 1 ;
    Text(J,1) = {['Last Year = ',int2str(Last_Year)]} ; J = J + 1 ;
    Text(J,1) = {['Calendar = ',Calendar]} ; J = J + 1 ;
    Text(J,1) = {['LINK Type = ',Ext]} ; J = J + 1 ;
    GOAT_cell2txt(fullfile(Data_LINK,FieldName,'link_info.txt'),Text) ;
end 

TT = toc ;
HH = floor(TT/3600) ;
MM = floor((TT - HH*3600)/60) ;
SS = TT - HH*3600 - MM*60 ;
if HH > 0
    TimeString = [int2str(HH),' hours, ',int2str(MM),' minutes and ',int2str(SS),' seconds.'] ;
elseif MM > 0
    TimeString = [int2str(MM),' minutes and ',int2str(SS),' seconds.'] ;
elseif SS > 0
    TimeString = [int2str(SS),' seconds.'] ;
end
disp(['Linking Derived Monthly ERA-Interim fields completed in ',TimeString]) ;



