% Code for auto downloading and installing LINKed ERA-Interim fields from the MARS server
DataSet_LINK    = '/Users/oradam/Documents/MATLAB/GOAT/Data_LINK/ERA20C_LINK/1_5X1_5' ;
DataSet_Install = '/Volumes/data3/GOAT/Data/ERA20C'                                   ;
Calendar  = 'Monthly'                                                                  ;
% Fields = dir(fullfile(DataSet_Install,Calendar)) ;
% Fields = {Fields([Fields(:).isdir],1).name}' ;
% Fields = Fields(~logical(strncmp(Fields,'.',1))) ;
% Fields = Fields(~logical(ismember(Fields,{'BoerBeta','Indices'}))) ;
FieldsToExclude = {'MSEI','MSEIv','vac','uac','mres','mres_pe','mres_pten','mres_div'} ;
Fields         = {'pr','ps','prw','zg','va','ua','ta','hus','wap'}                                                                       ;
% Span
%year1  = 1900  ; % yyyy
year1  = 1900  ; % yyyy
month1 = 1     ; % 1:12
day1   = 1     ; % 1:31
hour1  = 0     ; % 0:1:23
%year2  = 2010  ;
year2  = 2010  ;
month2 = 12    ;
day2   = 31    ;
hour2  = 21    ;
%
Ntry = 50 ; Nt = 1 ; 
for f = 1:length(Fields)
    Field = Fields{f} ;
    if ismember(Field,FieldsToExclude)
        continue
    end
    ExportAddress = fullfile(DataSet_Install,Calendar,Field)  ; %
    G = GOAT_Environment('enter_project_path') ; % Initiate GOAT Environment
    switch Field
        case {'albedo','albedosnow','at2m','cc','charnock','ci','ciwc','clwc','d2m','div','hcc','hur','hus','istl1','istl2','istl3','istl4','lcc','lsrh','mcc', ...
                'mwd','mwp','o3','prw','ps','psl','pv','rsn','sd','skt','sr','src','stl1','stl2','stl3','stl4','swh', ...
                'swvl1','swvl2','swvl3','swvl4','ta','tcc','tcdivfw','tcdivhef','tcdivke','tcdivl1','tcdivmf','tcdivo3','tcdivqf','tcdivtef','tcdivzg','tcdse','tcec' ...
                'tcecfwf','tceclwf','tcehf','tcekef','tcemf','tceo3f','tcetef','tcewvf','tcezgf','tche','tcke','tclfw','tcllw','tcmass','tcmse','tcmt','tcncfwf','tcnclwf', ...
                'tcnhf','tcnkef','tcnmf','tcno3f','tcntef','tcnwvf','tcnzgf','tco3','tcta','tcte','tcw','tcwv','tos','tsn','ua','ua10m','va','va10m','vor','wap','zg'}
            Type = 'Analysis' ;
        otherwise
            Type = 'Forecast' ;
    end
    G = GOAT_Initialize_Dataset(G,fullfile(DataSet_LINK,Type),Calendar) ; % Go through contents of Data folder and Calendar. Link existing fields, temporal span, etc.
    disp(['Installing ',int2str(f),'/',int2str(length(Fields)),' ',Type,' field ',Field,' . . .']) ;
    try
        G = GOAT_Initialize_Field(G,Field)        ; % get spatial and temporal dimensions of field, display settings, topography, and file addresses. Link if necessary
    catch
        disp(['The field ',Field,' could not be initialized in ',fullfile(DataSet_LINK,Type,Calendar)])
        continue
    end
    G.Limits.Default.Day2same   = 0 ; % 1/0 Set Date 2 same as Date 1
    G.Limits.Default.Sequential = 1 ; % 1/0 Sequential or Seasonal averaging Flag
    G.Limits.Default.ClimatologyContinuous = 0 ; % present seasonal data sequentially in Hovmoller plots
    G.Limits.Default.Anom  = 0      ; % display anomalies in Hovmoller plots
    G.Limits.S1 = GOAT_getSerial(year1,month1,day1,hour1,G.TimeFieldVirtual,G.Limits.Year0) ;
    G.Limits.S2 = GOAT_getSerial(year2,month2,day2,hour2,G.TimeFieldVirtual,G.Limits.Year0) ;
    G.ProjectionType = 10 ; %
    G.ALP.ExportAddress  = ExportAddress ; %
    G.ALP.Calendar       = Calendar ; % Calendar
    G.ALP.OverWriteFlag  = 1 ; % when checked, data will NOT be overwritten
    if strcmp(Field,'zg')
        G.ALP.BoerBetaFlag   = 1 ; % create BoerBeta function
    else
        G.ALP.BoerBetaFlag   = 0 ; % create BoerBeta function
    end
    G.ALP.SaveSingle     = 1 ; % save data as single precision
    if G.ALP.OverWriteFlag
    % Try Ntry times. MARS server is unstable
    while Nt <=Ntry
        try
            G = GOAT_ALP_Install(G) ; % Install as MAT archive format from OPeNDAP server or nc file
            Nt = Ntry + 1 ; 
        catch
            disp(['Try ',int2str(Nt),'/',int2str(Ntry),' failed. Retrying . . .'])
            Nt = Nt + 1 ;
        end
    end
    Nt = 1 ;
    else
        G = GOAT_ALP_Install(G) ; % Install as MAT archive format from OPeNDAP server or nc file
    end
end

%disp 'ERA-Interim Install Field Complete.'
disp 'ERA20C Install Field Complete.'