% GOAT_FileFolderTrees create file and folder trees of the contents of the Folder 'Datadir'
% Copyright (C) Ori Adam Nov.2010. Updated Dec.29.2013.
% This is an updated version of the CreateTrees.m function
% syntax:
% >> [FoldTree,FileTree] = GOAT_FileFolderTrees(Datadir,Flag)
% Input:
% Datadir = directory or string array of directories
% Flag = optional (Dummy flag for nargin > 1). When present, File and Folder trees are outputed as
% single colum values. Without the flag, Folder and File trees are outputed
% in 2 columns: path and directory-name/filename, respectively 
%
%

function  [FoldTree,FileTree] = GOAT_FileFolderTrees(Datadir,Flag)
D = 2      ; % number of subfolders in tree
Filenum = 0 ;
L = 200    ; % length of initial tree folder list
FoldTree = cell(L,D) ; % a cell array of all folders and their subfolders
if nargout > 1
    FileTreeFlag = 1 ;
    FileTree = cell(L,D) ; % a cell array of all of the files in the folder Datadir and its subfolders
else
    FileTreeFlag = 0 ;
end
FoldTree (1,1) = {''} ;
FoldTree (1,2) = {''} ;
LastDir = 1 ;
NewDir  = 1 ;
N = 1 ;
% Create a Folder List and a File list of all folders and files in the main directory - Datadir
while (N  > 0 || LastDir == 1)
    N = 0 ;
    for i = LastDir: NewDir
        FDir = fullfile(Datadir,cell2mat(FoldTree(i,1)),cell2mat(FoldTree(i,2)),'') ;
        if NewDir == 1
            SDir = '' ;
        else
            SDir = fullfile(cell2mat(FoldTree(i,1)),cell2mat(FoldTree(i,2)),'') ;
        end
        FolderDir = dir(FDir) ;
        for j = 1:length(FolderDir)
            if FolderDir(j,1).isdir && ~strncmp(FolderDir(j,1).name,'.',1)
                N = N + 1 ;
                FoldTree(NewDir + N,1) = {SDir} ;
                FoldTree(NewDir + N,2) = {FolderDir(j,1).name} ;
            elseif ~strncmp(FolderDir(j,1).name,'.',1) && FileTreeFlag
                Filenum = Filenum + 1 ;
                FileTree(Filenum,1) = {SDir} ;
                FileTree(Filenum,2) = {FolderDir(j,1).name} ;
            end
        end
    end
    LastDir = NewDir +1 ;
    NewDir = NewDir + N ;
end

% Trim Cell Arrays
FoldT = cell(NewDir,D) ;
if isempty(FoldTree{1,1}) && isempty(FoldTree{1,2})
    FoldT(1:NewDir-1,1) = FoldTree(2:NewDir,1) ;
    FoldT(1:NewDir-1,2) = FoldTree(2:NewDir,2) ;
else
    FoldT(1:NewDir,1) = FoldTree(1:NewDir,1) ;
    FoldT(1:NewDir,2) = FoldTree(1:NewDir,2) ;
end
if FileTreeFlag
    FileT = cell(Filenum,D) ;
    FileT(1:Filenum,1) = FileTree(1:Filenum,1) ;
    FileT(1:Filenum,2) = FileTree(1:Filenum,2) ;
end
if nargin > 1
    FoldTree = cell(size(FoldT,1),1) ;
    if FileTreeFlag
        FileTree = cell(size(FileT,1),1) ;
    end
    for i = 1:size(FoldT,1)
        FoldTree(i) = {fullfile(Datadir,FoldT{i,1},FoldT{i,2},'')}  ;
    end
    if FileTreeFlag
        for i = 1:size(FileT,1)
            FileTree(i) = {fullfile(Datadir,FileT{i,1},FileT{i,2},'')}  ;
        end
    end
else
    FoldTree = FoldT ;
    if FileTreeFlag
        FileTree = FileT ;
    end
end

