function NAM_Interpolation
% Organize and distribute data using the GOAT2012 standard
% GOAT2012 standard: 'Monthly --> 'Field' --> 'year' --> month*10000 + Day*100 + Hour(00:23)
% Example: Nov.28.75, 12:00 sits at: '1975' --> '112812.mat'
% Example: November monthly mean sits at: '1975' --> '110000.mat'
ReplaceFlag = 0 ; % 1 = replace Files. 0 = Create Copy ;
TimeField = 'DailyX4' ; 
P = (50 : 25 : 1000)' ;
lat = linspace(16.6,60,428) ;
lon = linspace(-143.3,-61.9,614) ; 
[qLon,qLat] = meshgrid(lon,lat) ; 
VerticalOrdinate = 'Pressure (mb)' ;
% Manually Load the Lon and Lat NAM File
%[fid,fidf] = uigetfile ;  load([fidf,fid]) ; 
load('NAM_Lon_Lat.mat') ; 
LON = double(Lon(:)) ; 
LAT = double(Lat(:)) ; 
try
    Dailydir = uigetdir(cd,'NAM Data') ;
    [NAMPath, DailyName, ext] = fileparts(Dailydir) ;
    Interpdir = fullfile(NAMPath, 'NAM_Interpolated','') ;
catch
    return ;
end
tic ;
% Locate Existing Fields
Fieldsdir = dir(fullfile(Dailydir,TimeField,'')) ;
Fields = cell(1,1) ;
NF = 0 ;
for j = 1:length(Fieldsdir)
    if Fieldsdir(j,1).isdir && ...
            ~strcmp(Fieldsdir(j,1).name,'.') && ...
            ~strcmp(Fieldsdir(j,1).name,'..')
        NF = NF + 1 ;
        Fields(NF,1) = {Fieldsdir(j,1).name} ;
    end
end

for f = 1 : length(Fields)
    yearsdir = dir(fullfile(Dailydir,TimeField,cell2mat(Fields(f)),'')) ;
    NF = 0 ;
    years = [] ;
    for j = 1:length(yearsdir)
        if yearsdir(j,1).isdir && ...
                ~strcmp(yearsdir(j,1).name,'.') && ...
                ~strcmp(yearsdir(j,1).name,'..')
            NF = NF + 1 ;
            years(NF) = round(str2double(yearsdir(j,1).name)) ;
        end
    end
    for y = 1: length(years)
        %disp(['Converting: ',fullfile(Dailydir,cell2mat(Fields(f)),num2str(years(y)),'')]) ;
        for m = 1 : 12
            switch m
                case {1,3,5,7,8,10,12}
                    mdays = 31 ;
                case {4,6,9,11}
                    mdays = 30 ;
                case 2
                    if GOAT_leapyear(years(y))
                        mdays = 29 ;
                    else
                        mdays = 28 ;
                    end
            end
            clear data
            for k = 1 : mdays
                for j = 1:8
                    clear data
                    try
                        load(fullfile(Dailydir,TimeField,cell2mat(Fields(f)),num2str(years(y)),[int2str(m*10000+k*100 + (j-1)*3),'.mat'])) ;
                        disp(['Interpolating: ',fullfile(Dailydir,cell2mat(Fields(f)),num2str(years(y)),[int2str(m*10000+k*100 + (j-1)*3),'.mat'])]) ;
                        data = InterpNAM(data,qLat,qLon,LAT,LON) ;
                        if ReplaceFlag
                            FileAddress = fullfile(Dailydir ,TimeField,cell2mat(Fields(f)),num2str(years(y)),[int2str(m*10000+k*100 + (j-1)*3),'.mat']) ;
                        else
                            FileAddress = fullfile(Interpdir,TimeField,cell2mat(Fields(f)),num2str(years(y)),[int2str(m*10000+k*100 + (j-1)*3),'.mat']) ;
                        end
                        [pathstr, name, ext] = fileparts(FileAddress) ;
                        try
                            save(FileAddress,'data') ;
                        catch
                            mkdir(pathstr) ;
                            save(FileAddress,'data') ;
                        end
                        
                    end
                end
            end
            
        end
    end
    % Transfer Spatial Files
    switch cell2mat(Fields(f))
        case 'HGT'
            Units = 'm'            ;
            FieldDescription = 'geopotential height divided by g' ;
        case 'TMP'
            Units = 'K'            ;
            FieldDescription = 'Temperature' ;
        case 'UGRD'
            Units = 'm/s'            ;
            FieldDescription = 'Zonal velocity' ;
        case 'VGRD'
            Units = 'm/s'            ;
            FieldDescription = 'Meridional velocity' ;
        case 'VVEL'
            Units = 'mb/s'            ;
            FieldDescription = 'Pressure vertical velocity' ;
        case 'DZDT'
            Units = 'm/s'            ;
            FieldDescription = 'Vertical velocity' ;
        case 'RH'
            Units = '%'            ;
            FieldDescription = 'Relative humidity' ;    
    end
    if ReplaceFlag
    save(fullfile(Dailydir,TimeField,cell2mat(Fields(f)),'Dims.mat'),'P','lat','lon','Units','FieldDescription','VerticalOrdinate') ; 
    else
    save(fullfile(Interpdir,TimeField,cell2mat(Fields(f)),'Dims.mat'),'P','lat','lon','Units','FieldDescription','VerticalOrdinate') ;
    end
end
t = toc ;
h = floor(toc/3600) ;
m = floor((t - h*3600)/60) ;
s = t - h*3600 - m*60 ;
if h > 0
    TimeString = [int2str(h),' hours, ',int2str(m),' minutes and ',int2str(s),' seconds.'] ;
elseif m > 0
    TimeString = [int2str(m),' minutes and ',int2str(s),' seconds.'] ;
elseif s > 0
    TimeString = [int2str(s),' seconds.'] ;
end
if ReplaceFlag
    mkdir(fullfile(Dailydir,TimeField,'Indices','')) ;
else
    mkdir(fullfile(Interpdir,TimeField,'Indices','')) ;    
end

disp(['Conversion Completed in ',TimeString]) ;
return ; 

function data = InterpNAM(data,qLat,qLon,LAT,LON)

for i = 1: size(data,3)
    Data = data(:,:,i)' ;    
    F = TriScatteredInterp(LON, LAT, double(Data(:))) ;
    data(:,:,i) = F(qLon,qLat)' ; 
end
