% code for creating mass residual, mres, field for ERA-Interim
G = GOAT_Environment ; % Initiate GOAT Environment
Calendar = 'Daily' ; 
DataFolder = '/Volumes/data3/GOAT/Data/NCEP-I' ;         
G = GOAT_Initialize_Dataset(G,DataFolder,Calendar) ; % Go through contents of Data folder and Calendar. Link existing fields, temporal span, etc.

G = GOAT_Initialize_Field(G,'ps')        ; % get spatial and temporal dimensions of field, display settings, topography, and file addresses. Link if necessary
lon_ps = G.lon ; 
lat_ps = G.lat ;
G.Limits.Default.Day2same   = 0 ; % 1/0 Set Date 2 same as Date 1
G.Limits.Default.Sequential = 1 ; % 1/0 Sequential or Seasonal averaging Flag
G.Limits.Default.ClimatologyContinuous = 0 ; % present seasonal data sequentially in Hovmoller plots
G.Limits.Default.Anom  = 0      ; % display anomalies in Hovmoller plots
year1  = 1999 ; % yyyy
month1 = 1    ; % 1:12
day1   = 1    ; % 1:31 
hour1  = 0    ; % 0:3:21
year2  = 2014 ; 
month2 = 12    ; 
day2   = 31    ; 
hour2  = 0    ; 
G.Limits.S1 = GOAT_getSerial(year1,month1,day1,hour1,G.TimeFieldVirtual,G.Limits.Year0) ; 
if G.Limits.Default.Day2same && G.Limits.Default.Sequential
    G.Limits.S2 = G.Limits.S1 ;
else
G.Limits.S2 = GOAT_getSerial(year2,month2,day2,hour2,G.TimeFieldVirtual,G.Limits.Year0) ; 
end 
G.ProjectionType = 1 ; % 1) lat-lon with coast 2) lat-lon 3) lat-height 4)lon-height 5) hovmoller-lon 6) hovmoller-lat 7) hovmoller height 8/9) NH/SH polar stereographic
G.Limits.Default.sameP = 1      ; % Projection 1,2,8,9) 1/0 set level2 = level1 | Projection 3,4,7) 1/0 composite levels
G.Limits.Default.VerMeanInt = 1 ; % 1) Vertical Mean 2) Integral using Boer Beta 3) set Nan values to zero and vertically integrate
G.Settings.NanMeanFlag = 0  ; % use mean(x) when 0 and nanmean(x) when 1
Level1  = 0 ; % GOAT will use the nearest level to this value
Level2  = 1000 ; % GOAT will use the nearest level to this value
Lon1    = -360   ; % use values from -360 to 360. GOAT will use the nearest values to the ones specified
Lon2    = 360   ; % use values from -360 to 360. 
Lat1    = 90   ; % use values from -90 to 90. GOAT will use the nearest values to the ones specified.
Lat2    = -90   ; % Lat2 does not have to be larger than Lat1.
G.Limits.Default.Lev1 = dsearchn(double(G.lev),Level1) ; 
G.Limits.Default.Lev2 = dsearchn(double(G.lev),Level2) ; 
G.Limits.Default.Lat1 = dsearchn(G.lat,Lat1)  ; 
G.Limits.Default.Lat2 = dsearchn(G.lat,Lat2)  ; 
G.Limits.Default.Lon1 = dsearchn(G.lon,Lon1)  ; 
G.Limits.Default.Lon2 = dsearchn(G.lon,Lon2)  ; 
G.Filter.Index = ones(length(G.Addresses.Default),1) ;
G = GOAT_Compile_data_Series(G) ;
G = GOAT_Compile_Climatology(G) ; 
ps_Clim = G.DataClim ; 



G = GOAT_Initialize_Field(G,'ua')        ; % get spatial and temporal dimensions of field, display settings, topography, and file addresses. Link if necessary
lon = G.lon ; 
lat = G.lat ;
lev0 = G.lev ;
G = GOAT_Initialize_Field(G,'mres')        ; % get spatial and temporal dimensions of field, display settings, topography, and file addresses. Link if necessary
VerticalOrdinate = 'Pressure (mb)' ; 
Units = 'm/s' ;
FieldDescription = 'Meridional mass flux correction ' ; 
mkdir(fullfile(DataFolder,Calendar,'vac')) ; 
mkdir(fullfile(DataFolder,Calendar,'uac')) ; 
lev = G.lev ; 
save(fullfile(DataFolder,Calendar,'vac','Dims.mat'),'lon','lat','lev','FieldDescription','Units','VerticalOrdinate') ; 
FieldDescription = 'Zonal mass flux correction ' ; 
save(fullfile(DataFolder,Calendar,'uac','Dims.mat'),'lon','lat','lev','FieldDescription','Units','VerticalOrdinate') ; 
% cell-2: Temporal limits
% Temporal Checkboxes/Flags
G.Limits.Default.Day2same   = 0 ; % 1/0 Set Date 2 same as Date 1
G.Limits.Default.Sequential = 1 ; % 1/0 Sequential or Seasonal averaging Flag
G.Limits.Default.ClimatologyContinuous = 0 ; % present seasonal data sequentially in Hovmoller plots
G.Limits.Default.Anom  = 0      ; % display anomalies in Hovmoller plots
year1  = 1979 ; % yyyy
month1 = 1    ; % 1:12
day1   = 1    ; % 1:31 
hour1  = 0    ; % 0:3:21
year2  = 2014 ; 
month2 = 12   ; 
day2   = 31   ; 
hour2  = 18   ; 
% G.Limits.S1 and G.Limits.S1 are the temporal indices used by GOAT to locate files in the GOAT archiving standard
G.Limits.S1 = GOAT_getSerial(year1,month1,day1,hour1,G.TimeFieldVirtual,G.Limits.Year0) ; 
if G.Limits.Default.Day2same && G.Limits.Default.Sequential
    G.Limits.S2 = G.Limits.S1 ;
else
G.Limits.S2 = GOAT_getSerial(year2,month2,day2,hour2,G.TimeFieldVirtual,G.Limits.Year0) ; 
end 
%
G.ProjectionType = 1 ; % 1) lat-lon with coast 2) lat-lon 3) lat-height 4)lon-height 5) hovmoller-lon 6) hovmoller-lat 7) hovmoller height 8/9) NH/SH polar stereographic
G.Limits.Default.sameP = 0      ; % Projection 1,2,8,9) 1/0 set level2 = level1 | Projection 3,4,7) 1/0 composite levels
G.Limits.Default.VerMeanInt = 1 ; % 1) Vertical Mean 2) Integral using Boer Beta 3) set Nan values to zero and vertically integrate
G.Settings.NanMeanFlag = 0  ; % use mean(x) when 0 and nanmean(x) when 1
Level1  = 1000 ; % GOAT will use the nearest level to this value
Level2  = 0 ; % GOAT will use the nearest level to this value
Lon1    = 0   ; % use values from -360 to 360. GOAT will use the nearest values to the ones specified
Lon2    = 358.5   ; % use values from -360 to 360. 
Lat1    = 90   ; % use values from -90 to 90. GOAT will use the nearest values to the ones specified.
Lat2    = -90   ; % Lat2 does not have to be larger than Lat1.
G.Limits.Default.Lev1 = dsearchn(double(G.lev),Level1) ; 
G.Limits.Default.Lev2 = dsearchn(double(G.lev),Level2) ; 
G.Limits.Default.Lat1 = dsearchn(G.lat,Lat1)  ; 
G.Limits.Default.Lat2 = dsearchn(G.lat,Lat2)  ; 
G.Limits.Default.Lon1 = dsearchn(G.lon,Lon1)  ; 
G.Limits.Default.Lon2 = dsearchn(G.lon,Lon2)  ; 
% Filters, Weights and Indices: 
% No Filters, Weights or Indices Selected 
G.Filter.Index = ones(length(G.Addresses.Default),1) ; % default index
G.DataStringVec = GOAT_Data_string_vec(G)                      ; % A vecotor of the Time Series File Indices
% retrieve and compile data 
NF = length(G.DataStringVec) ; 
Nmod = ceil(NF/100) ;
GOAT_Display_message(G,'','init') ;
Z = zeros(length(lon),length(lat),1) ; 
vac = Z ; 
uac   = Z ; 

%


nj = length(G.lat) ; 
T  = gauss_grid('T',nj - mod(nj,2))  ; 
aradius = 6.37122e6                  ; % units - m (earth radius)
T = set(T,'radius',aradius)          ;  % set the Earths radius for the grid operators
ni = get(T,'ni')                     ; % number of longitude grid points
xg = get(T,'xg')                     ; 
yg = get(T,'yg')                     ;
DIV = spectral_field('Divergence',T) ;
VOR = spectral_field('Vorticity',T)  ;
VOR = set(VOR,'gp',zeros(nj*2,nj))   ;
LAT = flipud(lat) ; 




%
for i = 1:NF
    
if mod(i,Nmod) == 0
   GOAT_Display_message(G,[' . . . Mass corrected u and v fluxes  . . .  ',num2str(i/NF*100,'%02.1f'),' %'],'stat')
end
datafile = cell2mat(G.Addresses.Default(G.DataStringVec(i))) ;
[y,m,d,h] = GOAT_serial2date(G.Addresses.Default,G.DataStringVec(i)) ;
if ~exist(fullfile(DataFolder,Calendar,'vac',fileparts(datafile)),'dir')
    mkdir(fullfile(DataFolder,Calendar,'vac',fileparts(datafile)))
end
if ~exist(fullfile(DataFolder,Calendar,'uac',fileparts(datafile)),'dir')
    mkdir(fullfile(DataFolder,Calendar,'uac',fileparts(datafile)))
end
[mres ,G] = GOAT_LoadData(G,DataFolder,Calendar,'mres',datafile,i) ; mres = flip(mres,2) ;  
mres(:,[1 end]) = 0 ; 

[prw  ,G] = GOAT_LoadData(G,DataFolder,Calendar,'prw',datafile,i)        ; prw  = flip(prw,2)  ;  

try
[ps   ,G] = GOAT_LoadData(G,DataFolder,Calendar,'ps',datafile,i)         ;   
catch
    Jd = GOAT_JulianDay(y,m,d) ;
    if Jd == 366
    ps = squeeze(ps_Clim(365,:,:)) ;   
    else
    ps = squeeze(ps_Clim(Jd,:,:)) ; 
    end
end
psi = interp2(lon_ps,lat_ps',ps',lon,lat')' ; 
psi   = flip(psi,2)   ;  
mg = interp2(double([G.lon-360 ; G.lon ; G.lon + 360]),double(LAT'),double([mres ; mres ; mres]'),double(180/pi*xg'),double(asind(yg)))' ;



DIV = set(DIV,'gp',mg)        ;

[U,V] = UVinv(VOR,DIV) ;  

uac = interp2(double(180/pi*xg'),double(asind(yg)),get(U,'gp')',G.lon,LAT')'./G.Vectors.cosmat ; uac(:,[1 end]) = 0 ; 
vac = interp2(double(180/pi*xg'),double(asind(yg)),get(V,'gp')',G.lon,LAT')'./G.Vectors.cosmat ; vac(:,[1 end]) = 0 ; 
data = single(flip(vac./(psi - G.Consts.GravitationalConst*prw),2)) ; 
save(fullfile(DataFolder,Calendar,'vac',datafile),'data') ; 
data = single(flip(uac./(psi - G.Consts.GravitationalConst*prw),2)) ;  
save(fullfile(DataFolder,Calendar,'uac',datafile),'data') ; 
end

disp(['vac and uac ',Calendar,' calculation complete!'])