% Produce BoerBeta from surface pressure and a reference 3D field
% syntax: ProduceBoerBeta_ps(FieldDir,SurfPDir) -- FieldDir = Reference 3D Field Folder , SurfPDir = Surface Pressure Field Folder
% syntax: ProduceBoerBeta_ps(FieldDir)          -- FieldDir = Reference 3D Field Folder , SurfPDir = Assumes is is named 'ps' and sits at the same Calendar folder as FieldDir
% syntax: ProduceBoerBeta_ps                    -- FieldDir = uigetdir                  , SurfPDir = Assumes is is named 'ps' and sits at the same Calendar folder as FieldDir
% Copyright (C) Ori Adam, May.2014


function ProduceBoerBeta_ps(FieldDir,SurfPDir)
G = GOAT_Environment ; 
Span = 'all'     ;
if nargin < 1
FieldDir = uigetdir ; % Some reference 3D field folder 
end
FieldPath = fileparts(FieldDir) ;
if nargin < 2
SurfPDir = fullfile(FieldPath,'ps','') ; % Assumes Surface Pressure field exists and that it is named 'ps'
end
try
load(fullfile(SurfPDir,'Dims.mat')) ;
catch
 disp(['Could not load ',fullfile(SurfPDir,'Dims.mat'),'. Process Aborted.']) ; 
 return
end
BoerDir = fullfile(FieldPath,'BoerBeta','') ;
load(fullfile(FieldDir,'Dims.mat')) ;
mkdir(BoerDir) ;
FieldDescription = 'Boer Beta function (0/1  for P >/< Surface P)'  ;
Units = '1' ;
VerticalOrdinate = 'Pressure (mb)' ;
save(fullfile(BoerDir,'Dims.mat'),'lon','lat','P','Units','FieldDescription','VerticalOrdinate') ;
O = ones(length(lon), length(lat), length(P)) ; 
Pmat = repmat(reshape(P,1,1,length(P)),[length(lon) length(lat) 1])*100    ;

if strcmp(Span,'all')
    yearsdir = dir(SurfPDir) ;
    Ny = 0 ;
    years = [] ;
    for j = 1:length(yearsdir)
        if yearsdir(j,1).isdir && ...
                ~strncmp(yearsdir(j,1).name,'.',1)
            Ny = Ny + 1 ;
            years(Ny) = round(str2double(yearsdir(j,1).name)) ;
        end
    end
    Pyearsdir = dir(SurfPDir) ;
    Ny = 0 ;
    Pyears = [] ;
    for j = 1:length(Pyearsdir)
        if Pyearsdir(j,1).isdir && ...
                ~strncmp(Pyearsdir(j,1).name,'.',1)
            Ny = Ny + 1 ;
            Pyears(Ny) = round(str2double(Pyearsdir(j,1).name)) ;
        end
    end
    y1 = max([min(years),min(Pyears)]) ;
    y2 = min([max(years),max(Pyears)]) ;
else
    y1 = Span(1) ;
    y2 = Span(end) ;
end
tic ;
q =0 ;
[Folders,Files] = GOAT_FileFolderTrees(SurfPDir,1) ; 
LF = length(Files) ; 
GOAT_Display_message(G,'','init')
for y = y1:y2
    for m = 1:12
        for d = 1:32
            for h = 1:24
                try
                    clear data
                    if ~exist(fullfile(SurfPDir,int2str(y),[int2str(m*10000 + (d-1)*100 + (h-1)*1),'.mat']),'file')
                        continue ; 
                    end
                    load(fullfile(SurfPDir,int2str(y),[int2str(m*10000 + (d-1)*100 + (h-1)*1),'.mat'])) ;
                    SurfPmat = repmat(data,[1 1 length(P)]) ;
                    bata = O ; 
                    GOAT_Display_message(G,['Creating ',fullfile(BoerDir,int2str(y),[int2str(m*10000 + (d-1)*100 + (h-1)*1),'.mat']),'  . . . ',num2str((q+1)/LF*100,'%04.1f'),' % complete . . .'],'stat')
                    bata(SurfPmat < Pmat) = 0 ;
                    if ~exist(fullfile(BoerDir,int2str(y)),'dir')
                        mkdir(fullfile(BoerDir,int2str(y),'')) ;
                    end
                        save(fullfile(BoerDir,int2str(y),[int2str(m*10000 + (d-1)*100 + (h-1)*1),'.mat']),'bata') ;
                    q = q + 1 ;
                catch
                    % No Such File
                end
            end
        end
    end
end
te = toc ;
H = floor(te/3600) ;
M = (te - H*3600)/60 ;
disp '   '
disp '   '
if q
disp(['BoerBeta field: ',BoerDir,'  -- completed in ',int2str(H),' hours and ',int2str(M),' minutes.'])
else
disp(['No BoerBeta files were created. Check reference field -- ',FieldDir])    
end