% GOAT_ALP_CompressAttributes  Compress Attributes cell array in order to reduce link_field.mat file size
% The Attrubutes cell array is recreated using the function GOAT_ALP_DecompressAttributes
% Copyright (C) Ori Adam, Apr.2014.
%
% see also GOAT_ALP_DecompressAttributes

function CompressedAttributes = GOAT_ALP_CompressAttributes(Attributes)
%%
if ~isempty(Attributes{end,5})
    % Decompress and then compress again. Useful if latest Compress code is more efficient
    Attributes = GOAT_ALP_DecompressAttributes(Attributes) ; 
end
J = size(Attributes,2) ; 
% Remove null columns if possible
if J > 5
 for jj = 1:size(Attributes,2)-5
  if sum(cellfun(@isempty,Attributes(1:end-1,size(Attributes,2)-jj+1))) == size(Attributes,1)-1
      J = J - 1 ; 
  else
      break ; 
  end
 end
end
CompressedAttributes = cell(2,J) ;
CompressedAttributes(end,:) = Attributes(end,1:J) ;
CompressedAttributes(end,5) = {size(Attributes,1)-1} ;


for j = 1:J
    Col = cell(1,2) ; % (value,NumberOfRepetitions)
    c = 0 ;
    LastValue = Attributes(1,j) ;
    q = 1 ;
    for i = 2:size(Attributes,1)-1
        if isequal(Attributes(i,j),LastValue)
            q = q+1 ;
        else
            c = c + 1 ;
            Col(c,1) = Attributes(i-1,j) ;
            if q>1
            Col(c,2) = {q} ;
            end
            q = 1 ;
            LastValue = Attributes(i,j) ;
        end
    end
    c = c + 1 ;
    if ~isempty(LastValue{1})
    Col(c,1) = LastValue ;
    if q > 1
    Col(c,2) = {q} ;
    end
    end
    CompressedAttributes(1,j) = {Col} ;
end

%% Compression Eval examples
% Single file per frame example
% Start = ones(315576,3) ; 
% Start(:,3) = (1:315576)' ; 
% EvalStr = 'Start = ones(315576,3) ; Start(:,3) = (1:315576)'' ; Col = num2cell(Start,2) ; '  ; 
% 
% Multiple files per frame example (daily averages from X24)
%  Start = ones(315576,3) ; 
%  Start(:,3) = (1:315576)' ; 
%  Col = cell(315576/24,1) ; 
%  for i = 1:315576/24
%  Col(i) = {num2cell(Start((i-1)*24+1:i*24,:,:),2)} ; 
%  end
%  EvalStr = 'Start = ones(315576,3) ; Start(:,3) = (1:315576)'' ; Col = cell(315576/24,1) ; ; for ijk = 1:315576/24 ; Col(ijk) = {num2cell(Start((ijk-1)*24+1:ijk*24,:,:),2)} ; end ;  '  ; 
% 
% EvalCell = cell(1,2) ;
% EvalCell(1,2) = {'eval'} ; 
% EvalCell(1,1) = {EvalStr} ; 
% CompressedAttributes(1,3) = {EvalCell} ; 

