% ProduceFileAddressesStrings produce a vector of address indices according to TimeFiels and Limits Settings.
% written by Ori Adam 2012. compatible with version 2013h
% Syntax:
% >> Addresses = GOAT_AddressesStrings(year1,year2,Calendar)
%
% see also, GOAT_Data_string_vec

function Addresses = GOAT_AddressesStrings(year1,year2,Calendar)

switch Calendar
    case 'Monthly'
        Addresses = cell((year2-year1+1)*12,1) ;
        q = 0 ;
        for y = year1: year2
            for m = 1:12
                q = q + 1 ;
                Addresses(q) = {fullfile(int2str(y),[int2str(m*10000),'.mat'])} ;
            end
        end
        return ;
    case {'Daily','DailyX1'}
        Xd = 1 ;
    case {'DailyX2'}
        Xd = 2 ;
    case {'DailyX4'}
        Xd = 4 ;
    case {'DailyX6'}
        Xd = 6 ;
    case {'DailyX12'}
        Xd = 12 ;
    case {'DailyX24'}
        Xd = 24 ;
    otherwise
        disp(['usupported temporal resolution ',Calendar])
        Addresses = [] ;
        return
end

H = 24/Xd ;
Addresses = cell((year2-year1+1)*365*Xd,1) ;
q = 0 ;
for y = year1: year2
    for m = 1:12
        switch m
            case {1,3,5,7,8,10,12}
                for d = 1:31
                    for x = 1:Xd
                        q = q + 1 ;
                        Addresses(q) = {fullfile(int2str(y),[int2str((m*100 + d)*100 + (x-1)*H),'.mat'])} ;
                    end
                end
            case {4,6,9,11}
                for d = 1:30
                    for x = 1:Xd
                        q = q + 1 ;
                        Addresses(q) = {fullfile(int2str(y),[int2str((m*100 + d)*100 + (x-1)*H),'.mat'])} ;
                    end
                end
            case 2
                if GOAT_leapyear(y,Calendar)
                    for d = 1:29
                        for x = 1:Xd
                            q = q + 1 ;
                            Addresses(q) = {fullfile(int2str(y),[int2str((m*100 + d)*100 + (x-1)*H),'.mat'])} ;
                        end
                    end
                else
                    for d = 1:28
                        for x = 1:Xd
                            q = q + 1 ;
                            Addresses(q) = {fullfile(int2str(y),[int2str((m*100 + d)*100 + (x-1)*H),'.mat'])} ;
                        end
                    end
                end
        end
    end
end
Addresses = Addresses(1:q) ;
