% GOAT_Compile_data_Climatology Compile data into daily or monthly climatological mean
% Copyright (C) Ori Adam Aug.2013. Compatible with version 2013i or later
%
% see also GOAT_Compile_ClimAnomaly, GOAT_Compile_data_Series

function G = GOAT_Compile_Climatology(G)

if strcmpi(G.TimeFieldVirtual,'List')
    GOAT_Display_message(G,'Aborting climatology calculation: Cannot calculate climatology for List Calendar') ;
    return ;
end
if ~isfield(G,'DataSeries')
    GOAT_Display_message(G,'Aborting climatology calculation: No G.DataSeries field') ;
    return ;
end
Nf = length(G.DataStringVec) ;
TimeStamp = zeros(Nf,1) ;
[YYY , X] = GOAT_ParseTimeResolution(G.TimeFieldVirtual) ;
switch G.TimeFieldVirtual
    case 'Monthly'
        BaseNum = 12 ;
    otherwise
        BaseNum = YYY*X ;
end
G.DataClim = zeros(BaseNum,size(G.DataSeries,2),size(G.DataSeries,3),size(G.DataSeries,4)) ;
GOAT_Display_message(G,'','init') ;
GOAT_Display_message(G,'Gathering Data Series dates for climatology calculation . . .','stat') ;
qF = 0 ;
for i = 1:Nf
    [year,month,day,hour] = GOAT_serial2date(G.Addresses.Default,G.DataStringVec(i)) ;
    if X
        if X == 1
            hour = 0 ; 
        end
        if G.Filter.Index(G.DataStringVec(i))
            qF = qF + 1 ;
            if GOAT_leapyear(year,G.TimeFieldVirtual) && month == 2 && day == 29
                TimeStamp(qF) = 0 ;
            else
                TimeStamp(qF) = GOAT_JulianDay(year,month,day,G.TimeFieldVirtual)*100 + hour ;
            end
        end
    else
        if G.Filter.Index(G.DataStringVec(i))
            qF = qF + 1 ;
            TimeStamp(qF) = month ;
        end
    end
end
TimeStamp = TimeStamp(1:qF) ;
Nmod = ceil(BaseNum/100) ; 
if X
    q = 0 ; qc = 0 ;
    for i = 1:YYY
        for h = 1:X
            q = q + 1 ;
            if mod(i,Nmod) == 0
            GOAT_Display_message(G,['Processing Climatology . . .  ',num2str(q/BaseNum*100,'%02.0f'),' %'],'stat')
            end
            idx = find(TimeStamp==(i*100 + (h-1)*24/X)) ;
            if ~isempty(idx)
                G.DataClim(q,:,:,:) = squeeze(nanmean(G.DataSeries(idx,:,:,:),1)) ;
                qc = qc + 1 ;
            end
        end
    end
else
    for i = 1:12
        GOAT_Display_message(G,['Processing Climatology . . .  ',num2str(i/BaseNum*100,'%02.0f'),' %'],'stat')
        idx = find(TimeStamp==i) ;
        if ~isempty(idx)
            G.DataClim(i,:,:,:) = squeeze(nanmean(G.DataSeries(idx,:,:,:),1)) ;
        end
    end
    qc = 12 ;
end

if qc ==BaseNum && G.Limits.Default.DataClimSmooth > 1 % Do not smooth if there are missing days
    if ~mod(G.Limits.Default.DataClimSmooth,2)
        G.Limits.Default.DataClimSmooth = round(G.Limits.Default.DataClimSmooth) + 1 ;
        %GOAT_Display_message(G,['Smoothing: G.Limits.Default.DataClimSmooth must be an odd number: Was changed from ',int2str(G.Limits.Default.DataClimSmooth-1),' to ',int2str(G.Limits.Default.DataClimSmooth)]) ;
    end
    if ~X && G.Limits.Default.DataClimSmooth > 11
        %GOAT_Display_message(G,['Smoothing: G.Limits.Default.DataClimSmooth cannot exceed 11 for Monthly climatology: was changed from ',int2str(G.Limits.Default.DataClimSmooth),' to 11']) ;
        G.Limits.Default.DataClimSmooth = 11 ;
    end
    GOAT_Display_message(G,['Smoothing: G.Limits.Default.DataClimSmooth = ',int2str(G.Limits.Default.DataClimSmooth)]) ;
    N = (G.Limits.Default.DataClimSmooth - 1)/2 ;
    if X
        BaseNum = YYY*X ;
    else
        BaseNum = 12 ;
    end
    DataClim = zeros(BaseNum + 2*N,size(G.DataSeries,2),size(G.DataSeries,3),size(G.DataSeries,4)) ;
    DataClim(N+1:N+BaseNum,:,:,:) = G.DataClim ;
    for i = 1:N
        DataClim(N+1-i,:,:,:) = G.DataClim(end-i+1,:,:,:) ;
        DataClim(end-N+i,:,:,:) = G.DataClim(i,:,:,:) ;
    end
    DataClim = cumsum(DataClim,1) ;
    G.DataClim = [DataClim(2*N+1,:,:,:) ; (DataClim(2*N+2:end,:,:,:) - DataClim(1:BaseNum-1,:,:,:)) ]./ (2*N+1) ;
    GOAT_Display_message(G,['Processing Climatology . . Done. '])
else
    GOAT_Display_message(G,['Processing Climatology . . Done. '],'stat')
end
return
