% GOAT_Compile_data_Series creates a time series of the selected data
% Copyright (C) Ori Adam Jan.2012. Compatible with GOAT version 2013i or later
%
% see also GOAT_Compile_data, GOAT_Compile_data_Climatology

function G = GOAT_Compile_data_Series(G)

G.DataLimits.Lon1 = G.Limits.Default.Lon1 ; % min([G.Limits.Default.Lon1,G.Limits.Default.Lon2]) ;
G.DataLimits.Lon2 = G.Limits.Default.Lon2 ; % max([G.Limits.Default.Lon1,G.Limits.Default.Lon2]) ;
G.DataLimits.Lat1 = min([G.Limits.Default.Lat1,G.Limits.Default.Lat2]) ;
G.DataLimits.Lat2 = max([G.Limits.Default.Lat1,G.Limits.Default.Lat2]) ;
G.DataLimits.Lev1 = min([G.Limits.Default.Lev1,G.Limits.Default.Lev2]) ;
G.DataLimits.Lev2 = max([G.Limits.Default.Lev1,G.Limits.Default.Lev2]) ;
if G.DataLimits.Lon2 >= G.DataLimits.Lon1
    SizeLon0 = (G.DataLimits.Lon2-G.DataLimits.Lon1+1) ;
else
    SizeLon0 = (G.DataLimits.Lon2+1 + length(G.lon) -G.DataLimits.Lon1) ;
end
SizeLat0 = (G.DataLimits.Lat2-G.DataLimits.Lat1+1) ;
if G.Limits.Default.sameP
    SizeP0   = 1 ;
else
    SizeP0   = G.DataLimits.Lev2 - G.DataLimits.Lev1 + 1 ;
end
%G.DataLimits.p    = double(G.lev(G.DataLimits.Lev1:G.DataLimits.Lev2)/(G.lev(G.DataLimits.Lev2) - G.lev(G.DataLimits.Lev1)))   ; % Weighted Pressure In Case of vertical Averaging
switch G.Limits.Default.VerMeanInt
    case 1
        G.DataLimits.p    = double(G.lev(G.DataLimits.Lev1:G.DataLimits.Lev2)/(G.lev(G.DataLimits.Lev2) - G.lev(G.DataLimits.Lev1)))   ; % Weighted Pressure In Case of vertical Averaging
    case {2,3}
        if length(G.lev)==1
            G.DataLimits.p    = double(G.lev(G.DataLimits.Lev1))   ; % Pressure vector In Case of vertical integral
        else
            switch G.VerticalOrdinate
                case 'Pressure (mb)'
                    G.DataLimits.p    = double(G.lev(G.DataLimits.Lev1:G.DataLimits.Lev2)*sign(G.lev(end)-G.lev(1)))*100/G.Consts.GravitationalConst   ; % mass weighted vertical integral
                case 'Depth (m)'
                    G.DataLimits.p    = double(G.lev(G.DataLimits.Lev1:G.DataLimits.Lev2)*sign(G.lev(end)-G.lev(1)))*G.Consts.WaterDensity         ; % mass weighted vertical integral
                otherwise
                    G.DataLimits.p    = double(G.lev(G.DataLimits.Lev1:G.DataLimits.Lev2)*sign(G.lev(end)-G.lev(1))) ; % vertical integral with no mass weight
            end
        end
end
G.DataStringVec = GOAT_Data_string_vec(G)                      ; % A vecotor of the Time Series File Indices
NF  = length(G.DataStringVec) ;
if NF < G.Settings.Default.DisplayProgressCutoff
    Nmod = 0            ;
else
    Nmod = ceil(NF/100) ;
    GOAT_Display_message(G,'','init') ;
end

% Spatial Weights
G.Filter.weight = WeightFunc(G) ;
if Nmod
    GOAT_Display_message(G,' . . . Allocating memory space . . .  ','stat') ;
end

switch G.ProjectionType
    case {1,2,8,9} % Long-Lat
        Event_Data = zeros(length(G.DataStringVec),SizeLon0*SizeLat0) ;
        if G.Limits.Default.DataSeriesAnomFlag
            Event_Data_Anom = Event_Data ;
        end
        qF = 0 ;
        for i = 1:NF
            if mod(i,G.Settings.breakSteps) && G.Flags.breakFlag
                break ;
            end
            if mod(i,Nmod) == 0
                GOAT_Display_message(G,[' . . . Processing Time Series  . . .  ',num2str(i/NF*100,'%02.0f'),' %'],'stat')
            end
            if G.Filter.Index(G.DataStringVec(i))
                qF = qF + 1 ;
                [data,G] = GOAT_LoadDataAndTimeFilter(G,i) ; % Loads Data and Applies Time Filters
                % Zonal Wave Number
                if sum (G.Filter.ZonalWaveNumberIdx) == -1
                    if G.DataLimits.Lon2 >= G.DataLimits.Lon1
                        data = data(G.DataLimits.Lon1:G.DataLimits.Lon2,:) ;
                    else
                        data = data([G.DataLimits.Lon1:end,1:G.DataLimits.Lon2],:) ;
                    end
                else
                    data = ZonalWaveFilter(data,G.Filter.ZonalWaveNumberIdx) ;
                    if G.DataLimits.Lon2 >= G.DataLimits.Lon1
                        data = data(G.DataLimits.Lon1:G.DataLimits.Lon2,:) ;
                    else
                        data = data([G.DataLimits.Lon1:end,1:G.DataLimits.Lon2],:) ;
                    end
                end
                Event_Data(qF,:) = data(:)  ;
                if G.Limits.Default.DataSeriesAnomFlag
                    [year,month,day] = GOAT_serial2date(G.Addresses.Default,G.DataStringVec(i)) ;
                    switch G.TimeField
                        case 'Monthly'
                            dayIdx = month ;
                        otherwise
                            dayIdx = monthday2dayIdx(month,day) ;
                    end
                    MeanField = squeeze(G.DataClim(dayIdx,:,:)) ;
                    MeanField = MeanField(:) ;
                    Event_Data_Anom(qF,:) = (data(:) - MeanField(:)) ;
                end
            end
        end
        Event_Data      =      Event_Data(1:qF,:) ;
        if G.Limits.Default.DataSeriesAnomFlag
            Event_Data_Anom = Event_Data_Anom(1:qF,:) ;
        end
        if G.DataLimits.Lon2 >= G.DataLimits.Lon1
            G.x = G.lon(G.DataLimits.Lon1:G.DataLimits.Lon2) ;
        else
            if G.lon(G.DataLimits.Lon2) > 0
                if size(G.lon,2) < size(G.lon,1)
                    G.x = [ (G.lon(G.DataLimits.Lon1:end) -360) ; G.lon(1:G.DataLimits.Lon2)] ;
                else
                    G.x = [ (G.lon(G.DataLimits.Lon1:end) -360) , G.lon(1:G.DataLimits.Lon2)] ;
                end
            else
                if size(G.lon,2) < size(G.lon,1)
                    G.x = [ G.lon(G.DataLimits.Lon1:end) ; (G.lon(1:G.DataLimits.Lon2) + 360)] ;
                else
                    G.x = [ G.lon(G.DataLimits.Lon1:end) , (G.lon(1:G.DataLimits.Lon2) + 360)] ;
                end
            end
        end
        G.xlabel = G.lonlabel ;
        G.y = G.lat(G.DataLimits.Lat1:G.DataLimits.Lat2) ;
        G.ylabel = G.latlabel  ;
        [G.X,G.Y] = meshgrid(G.x,G.y) ;
        G.Datalabel = G.DataField ;
    case 3 % Lat-Pressure
        Event_Data = zeros(length(G.DataStringVec),SizeLat0*SizeP0) ;
        if G.Limits.Default.DataSeriesAnomFlag
            Event_Data_Anom = Event_Data ;
        end
        if G.Limits.Default.sameP || G.DataLimits.Lev1 == G.DataLimits.Lev2 % This Flag Means Vertical Averaging In this Projection
            qF = 0 ;
            for i = 1:NF
                if mod(i,G.Settings.breakSteps) && G.Flags.breakFlag
                    break ;
                end
                if mod(i,Nmod) == 0
                    GOAT_Display_message(G,[' . . . Processing Time Series  . . .  ',num2str(i/NF*100,'%02.0f'),' %'],'stat')
                end
                if G.Filter.Index(G.DataStringVec(i))
                    qF = qF + 1 ;
                    [data,G] = GOAT_LoadDataAndTimeFilter(G,i) ; % Loads Data and Applies Time Filters
                    Event_Data(qF,:) = data(:)  ;
                    if G.Limits.Default.DataSeriesAnomFlag
                        [year,month,day] = GOAT_serial2date(G.Addresses.Default,G.DataStringVec(i)) ;
                        switch G.TimeField
                            case 'Monthly'
                                dayIdx = month ;
                            otherwise
                                dayIdx = monthday2dayIdx(month,day) ;
                        end
                        MeanField = squeeze(G.DataClim(dayIdx,:,:)) ;
                        Event_Data_Anom(qF,:) = (data(:) - MeanField(:)) ;
                    end
                end
            end
            G.x = G.lat(G.DataLimits.Lat1:G.DataLimits.Lat2) ;
            G.X = G.lat(G.DataLimits.Lat1:G.DataLimits.Lat2) ;
            G.xlabel = G.latlabel  ;
            G.ylabel = ''          ;
            G.y = [] ;
            G.Y = [] ;
            G.Datalabel = G.DataField ;
        else
            qF = 0 ;
            for i = 1:NF
                if mod(i,G.Settings.breakSteps) && G.Flags.breakFlag
                    break ;
                end
                
                if mod(i,Nmod) == 0
                    GOAT_Display_message(G,[' . . . Processing Time Series  . . .  ',num2str(i/NF*100,'%02.0f'),' %'],'stat')
                end
                qF = qF + 1 ;
                [data,G] = GOAT_LoadDataAndTimeFilter(G,i) ; % Loads Data and Applies Time Filters
                Event_Data(qF,:) = data(:)  ;
                if G.Limits.Default.DataSeriesAnomFlag
                    [year,month,day] = GOAT_serial2date(G.Addresses.Default,G.DataStringVec(i)) ;
                    switch G.TimeField
                        case 'Monthly'
                            dayIdx = month ;
                        otherwise
                            dayIdx = monthday2dayIdx(month,day) ;
                    end
                    MeanField = squeeze(G.DataClim(dayIdx,:,:)) ;
                    Event_Data_Anom(qF,:) = (data(:) - MeanField(:)) ;
                end
            end
            G.x = G.lat(G.DataLimits.Lat1:G.DataLimits.Lat2) ;
            G.xlabel = G.latlabel ;
            G.y = G.lev(G.DataLimits.Lev1:G.DataLimits.Lev2) ;
            G.ylabel = G.Plabel ;
            [G.X,G.Y] = meshgrid(G.x, G.y) ;
            G.Datalabel = G.DataField ;
        end
        Event_Data      =      Event_Data(1:qF,:) ;
        if G.Limits.Default.DataSeriesAnomFlag
            Event_Data_Anom = Event_Data_Anom(1:qF,:) ;
        end
    case 4 % Lon-Pressure
        Event_Data = zeros(length(G.DataStringVec),SizeLon0*SizeP0) ;
        if G.Limits.Default.DataSeriesAnomFlag
            Event_Data_Anom = Event_Data ;
        end
        qF = 0 ;
        for i = 1:NF
            if mod(i,G.Settings.breakSteps) && G.Flags.breakFlag
                break ;
            end
            if mod(i,Nmod) == 0
                GOAT_Display_message(G,[' . . . Processing Time Series  . . .  ',num2str(i/NF*100,'%02.0f'),' %'],'stat')
            end            
            if G.Filter.Index(G.DataStringVec(i))
                qF = qF + 1 ;
                [data,G] = GOAT_LoadDataAndTimeFilter(G,i) ; % Loads Data and Applies Time Filters
                if sum (G.Filter.ZonalWaveNumberIdx) == -1
                    if G.DataLimits.Lon2 >= G.DataLimits.Lon1
                        data = data(G.DataLimits.Lon1:G.DataLimits.Lon2,:) ;
                    else
                        data = data([G.DataLimits.Lon1:end,1:G.DataLimits.Lon2],:) ;
                    end
                else
                    data = ZonalWaveFilter(data,G.Filter.ZonalWaveNumberIdx) ;
                    if G.DataLimits.Lon2 >= G.DataLimits.Lon1
                        data = data(G.DataLimits.Lon1:G.DataLimits.Lon2,:) ;
                    else
                        data = data([G.DataLimits.Lon1:end,1:G.DataLimits.Lon2],:) ;
                    end
                end
                Event_Data(qF,:) = data(:)  ;
                if G.Limits.Default.DataSeriesAnomFlag
                    [year,month,day] = GOAT_serial2date(G.Addresses.Default,G.DataStringVec(i)) ;
                    switch G.TimeField
                        case 'Monthly'
                            dayIdx = month ;
                        otherwise
                            dayIdx = monthday2dayIdx(month,day) ;
                    end
                    MeanField = squeeze(G.DataClim(dayIdx,:,:)) ;
                    Event_Data_Anom(qF,:) = (data(:) - MeanField(:)) ;
                end
            end
        end
        if G.DataLimits.Lon2 >= G.DataLimits.Lon1
            G.x = G.lon(G.DataLimits.Lon1:G.DataLimits.Lon2) ;
        else
            if G.lon(G.DataLimits.Lon2) > 0
                if size(G.lon,2) < size(G.lon,1)
                    G.x = [ (G.lon(G.DataLimits.Lon1:end) -360) ; G.lon(1:G.DataLimits.Lon2)] ;
                else
                    G.x = [ (G.lon(G.DataLimits.Lon1:end) -360) , G.lon(1:G.DataLimits.Lon2)] ;
                end
            else
                if size(G.lon,2) < size(G.lon,1)
                    G.x = [ G.lon(G.DataLimits.Lon1:end) ; (G.lon(1:G.DataLimits.Lon2) + 360)] ;
                else
                    G.x = [ G.lon(G.DataLimits.Lon1:end) , (G.lon(1:G.DataLimits.Lon2) + 360)] ;
                end
            end
        end
        G.xlabel = G.lonlabel ;
        G.ylabel = G.Plabel ;
        if G.Limits.Default.sameP || G.DataLimits.Lev1 == G.DataLimits.Lev2
            G.y = G.lev(G.DataLimits.Lev1) ;
            G.X = G.x ; G.Y = G.y ;
        else
            G.y = G.lev(G.DataLimits.Lev1:G.DataLimits.Lev2) ;
            [G.X,G.Y] = meshgrid(G.x, G.y) ;
        end
        G.Datalabel = G.DataField ;
        Event_Data      =      Event_Data(1:qF,:) ;
        if G.Limits.Default.DataSeriesAnomFlag
            Event_Data_Anom = Event_Data_Anom(1:qF,:) ;
        end
    case 5 % Hovmoller Longitude
        % Sequential or Seasonal Hovmoller
        NPeriod = AveragingType(G,NF) ;
        
        if G.DataLimits.Lon2 >= G.DataLimits.Lon1
            G.Data = zeros(G.DataLimits.Lon2 - G.DataLimits.Lon1 +1, NPeriod) ;
            weight = ones(G.DataLimits.Lon2 - G.DataLimits.Lon1 +1, NPeriod)*round(NF/NPeriod) ;
        else
            G.Data = zeros(length(G.lon) - G.DataLimits.Lon1 + G.DataLimits.Lon2 +1, NPeriod) ;
            weight = ones( length(G.lon) - G.DataLimits.Lon1 + G.DataLimits.Lon2 +1, NPeriod)*round(NF/NPeriod) ;
        end
        q = 0 ;
        for i = 1:NF
            if mod(i,G.Settings.breakSteps) && G.Flags.breakFlag
                break ;
            end
            if mod(i,Nmod) == 0
                GOAT_Display_message(G,[' . . . Processing Time Series  . . .  ',num2str(i/NF*100,'%02.0f'),' %'],'stat')
            end
            if G.Filter.Index(G.DataStringVec(i))
                [data,G] = GOAT_LoadDataAndTimeFilter(G,i) ; % Loads Data and Applies Time Filters
                if sum(G.Filter.ZonalWaveNumberIdx) == -1
                    if G.DataLimits.Lon2 >= G.DataLimits.Lon1
                        data = data(G.DataLimits.Lon1:G.DataLimits.Lon2) ;
                    else
                        data = data([G.DataLimits.Lon1:end , 1:G.DataLimits.Lon2]) ;
                    end
                else
                    data = ZonalWaveFilter(data,G.Filter.ZonalWaveNumberIdx) ;
                    if G.DataLimits.Lon2 >= G.DataLimits.Lon1
                        data = data(G.DataLimits.Lon1:G.DataLimits.Lon2) ;
                    else
                        data = data([G.DataLimits.Lon1:end , 1:G.DataLimits.Lon2]) ;
                    end
                    clear Datatoifft Datafft
                end
            end
            if G.Limits.Default.Sequential || (G.Limits.Default.Sequential == 0 && G.Limits.Default.ClimatologyContinuous ==1)
                
                if G.Filter.Index(G.DataStringVec(i))
                    G.Data(:,i) =  data ;
                else
                    G.Data(:,i) =  nan ;
                end
            else
                if GOAT_is_Gregorian(G.TimeFieldVirtual)
                    [~,month,day] = GOAT_serial2date(G.Addresses.Default,G.DataStringVec(i)) ;
                    if ~(month == 2 && day == 29)
                        q = q + 1 ;
                        if G.Filter.Index(G.DataStringVec(i))
                            G.Data(:,mod(q,NPeriod) + NPeriod*~mod(q,NPeriod)) = G.Data(:,mod(q,NPeriod) + NPeriod*~mod(q,NPeriod)) + data ;
                        else
                            weight(:,mod(q,NPeriod) + NPeriod*~mod(q,NPeriod)) = weight(:,mod(q,NPeriod) + NPeriod*~mod(q,NPeriod)) - 1 ;
                        end
                    end
                else
                    q = q + 1 ;
                    if G.Filter.Index(G.DataStringVec(i))
                        G.Data(:,mod(q,NPeriod) + NPeriod*~mod(q,NPeriod)) = G.Data(:,mod(q,NPeriod) + NPeriod*~mod(q,NPeriod)) + data ;
                    else
                        weight(:,mod(q,NPeriod) + NPeriod*~mod(q,NPeriod)) = weight(:,mod(q,NPeriod) + NPeriod*~mod(q,NPeriod)) - 1 ;
                    end
                end
            end
        end
        if ~G.Limits.Default.Sequential && ~G.Limits.Default.ClimatologyContinuous
            G.Data = G.Data./weight ;
        end
        if G.DataLimits.Lon2 >= G.DataLimits.Lon1
            G.x = G.lon(G.DataLimits.Lon1:G.DataLimits.Lon2) ;
        else
            if G.lon(G.DataLimits.Lon2) > 0
                if size(G.lon,2) < size(G.lon,1)
                    G.x = [ (G.lon(G.DataLimits.Lon1:end) -360) ; G.lon(1:G.DataLimits.Lon2)] ;
                else
                    G.x = [ (G.lon(G.DataLimits.Lon1:end) -360) , G.lon(1:G.DataLimits.Lon2)] ;
                end
            else
                if size(G.lon,2) < size(G.lon,1)
                    G.x = [ G.lon(G.DataLimits.Lon1:end) ; (G.lon(1:G.DataLimits.Lon2) + 360)] ;
                else
                    G.x = [ G.lon(G.DataLimits.Lon1:end) , (G.lon(1:G.DataLimits.Lon2) + 360)] ;
                end
            end
        end
        G.xlabel = G.lonlabel ;
        G.y = 1:NPeriod ;
        G.ylabel = 'Time Intervals'  ;
        [G.X,G.Y] = meshgrid(G.x,G.y) ;
        G.Datalabel = G.DataField ;
    case 6 % Hovmoller Latitude
        % Sequential or Seasonal Hovmoller
        NPeriod = AveragingType(G,NF) ;
        G.Data = zeros(G.DataLimits.Lat2-G.DataLimits.Lat1+1 , NPeriod) ;
        weight =  ones(G.DataLimits.Lat2-G.DataLimits.Lat1+1 , NPeriod)*round(NF/NPeriod) ;
        q = 0 ;
        for i = 1:NF
            if mod(i,G.Settings.breakSteps) && G.Flags.breakFlag
                break ;
            end
            if mod(i,Nmod) == 0
                GOAT_Display_message(G,[' . . . Processing Time Series  . . .  ',num2str(i/NF*100,'%02.0f'),' %'],'stat')
            end
            if G.Filter.Index(G.DataStringVec(i))
                [data,G] = GOAT_LoadDataAndTimeFilter(G,i) ; % Loads Data and Applies Time Filters
            end
            if G.Limits.Default.Sequential || (G.Limits.Default.Sequential == 0 && G.Limits.Default.ClimatologyContinuous ==1)
                if G.Filter.Index(G.DataStringVec(i))
                    try
                        G.Data(:,i) =  data' ;
                    catch
                        G.Data(:,i) =  data ;
                    end
                else
                    G.Data(:,i) =  nan ;
                end
            else
                if GOAT_is_Gregorian(G.TimeFieldVirtual)
                    [~,month,day] = GOAT_serial2date(G.Addresses.Default,G.DataStringVec(i)) ;
                    if ~(month == 2 && day == 29)
                        q = q + 1 ;
                        if G.Filter.Index(G.DataStringVec(i))
                            try
                                G.Data(:,mod(q,NPeriod) + NPeriod*~mod(q,NPeriod)) = G.Data(:,mod(q,NPeriod) + NPeriod*~mod(q,NPeriod)) + data' ;
                            catch
                                G.Data(:,mod(q,NPeriod) + NPeriod*~mod(q,NPeriod)) = G.Data(:,mod(q,NPeriod) + NPeriod*~mod(q,NPeriod)) + data ;
                            end
                        else
                            weight(:,mod(q,NPeriod) + NPeriod*~mod(q,NPeriod)) = weight(:,mod(q,NPeriod) + NPeriod*~mod(q,NPeriod)) - 1 ;
                        end
                    end
                else
                    q = q + 1 ;
                    if G.Filter.Index(G.DataStringVec(i))
                        try
                            G.Data(:,mod(q,NPeriod) + NPeriod*~mod(q,NPeriod)) = G.Data(:,mod(q,NPeriod) + NPeriod*~mod(q,NPeriod)) + data' ;
                        catch
                            G.Data(:,mod(q,NPeriod) + NPeriod*~mod(q,NPeriod)) = G.Data(:,mod(q,NPeriod) + NPeriod*~mod(q,NPeriod)) + data ;
                        end
                    else
                        weight(:,mod(q,NPeriod) + NPeriod*~mod(q,NPeriod)) = weight(:,mod(q,NPeriod) + NPeriod*~mod(q,NPeriod)) - 1 ;
                    end
                end
            end
        end
        if ~G.Limits.Default.Sequential && ~G.Limits.Default.ClimatologyContinuous
            G.Data = G.Data./weight ;
        end
        G.x = G.lat(G.DataLimits.Lat1:G.DataLimits.Lat2) ;
        G.xlabel = G.latlabel ;
        G.y = 1:NPeriod ;
        G.ylabel = 'Time Intervals'  ;
        [G.X,G.Y] = meshgrid(G.x,G.y) ;
        G.Datalabel = G.DataField ;
    case 7 % Hovmoller Pressure
        % Sequential or Seasonal Hovmoller
        NPeriod = AveragingType(G,NF) ;
        if G.Limits.Default.sameP
            G.Data = zeros(1, NPeriod) ;
            weight =  ones(1, NPeriod)*round(NF/NPeriod) ;
        else
            G.Data = zeros(G.DataLimits.Lev2-G.DataLimits.Lev1+1, NPeriod) ;
            weight =  ones(G.DataLimits.Lev2-G.DataLimits.Lev1+1, NPeriod)*round(NF/NPeriod) ;
        end
        q = 0 ;
        for i = 1:NF
            if mod(i,G.Settings.breakSteps) && G.Flags.breakFlag
                break ;
            end
            if mod(i,Nmod) == 0
                GOAT_Display_message(G,[' . . . Processing Time Series  . . .  ',num2str(i/NF*100,'%02.0f'),' %'],'stat')
            end
            if G.Filter.Index(G.DataStringVec(i))
                [data,G] = GOAT_LoadDataAndTimeFilter(G,i) ; % Loads Data and Applies Time Filters
            end
            if G.Limits.Default.Sequential || (G.Limits.Default.Sequential == 0 && G.Limits.Default.ClimatologyContinuous ==1)
                if G.Filter.Index(G.DataStringVec(i))
                    G.Data(:,i) =  data ;
                else
                    G.Data(:,i) =  nan ;
                end
            else
                if GOAT_is_Gregorian(G.TimeFieldVirtual)
                    [~,month,day] = GOAT_serial2date(G.Addresses.Default,G.DataStringVec(i)) ;
                    if ~(month == 2 && day == 29)
                        q = q + 1 ;
                        if G.Filter.Index(G.DataStringVec(i))
                            G.Data(:,mod(q,NPeriod) + NPeriod*~mod(q,NPeriod)) = G.Data(:,mod(q,NPeriod) + NPeriod*~mod(q,NPeriod)) + data ;
                        else
                            weight(:,mod(q,NPeriod) + NPeriod*~mod(q,NPeriod)) = weight(:,mod(q,NPeriod) + NPeriod*~mod(q,NPeriod)) - 1 ;
                        end
                    end
                else
                    q = q + 1 ;
                    if G.Filter.Index(G.DataStringVec(i))
                        G.Data(:,mod(q,NPeriod) + NPeriod*~mod(q,NPeriod)) = G.Data(:,mod(q,NPeriod) + NPeriod*~mod(q,NPeriod)) + data ;
                    else
                        weight(:,mod(q,NPeriod) + NPeriod*~mod(q,NPeriod)) = weight(:,mod(q,NPeriod) + NPeriod*~mod(q,NPeriod)) - 1 ;
                    end
                end
            end
        end
        if ~G.Limits.Default.Sequential && ~G.Limits.Default.ClimatologyContinuous
            G.Data = G.Data./weight ;
        end
        G.x = G.lev(G.DataLimits.Lev1:G.DataLimits.Lev2) ;
        G.xlabel = G.Plabel ;
        G.y = 1:NPeriod ;
        G.ylabel = 'Time Intervals'  ;
        [G.X,G.Y] = meshgrid(G.x,G.y) ;
        G.Datalabel = G.DataField ;
end
if Nmod && ~G.Flags.breakFlag
    GOAT_Display_message(G,[' . . . Processing Time Series  . . .  Done.'],'stat') ;
end
if G.Flags.breakFlag
    set(G.h_message,'string','Process Terminated.') ; drawnow ; % Update Message
    G.Flags.breakFlag = 0 ;
    return ;
end

switch G.ProjectionType
    case {1,2,8,9} % Long-Lat
        if G.DataLimits.Lon2 >= G.DataLimits.Lon1
            G.DataSeries     = reshape(Event_Data,qF,G.DataLimits.Lon2-G.DataLimits.Lon1+1,G.DataLimits.Lat2-G.DataLimits.Lat1+1) ;
            if G.Limits.Default.DataSeriesAnomFlag
                G.DataSeriesAnom = reshape(Event_Data_Anom,qF,G.DataLimits.Lon2-G.DataLimits.Lon1+1,G.DataLimits.Lat2-G.DataLimits.Lat1+1) ;
            end
        else
            G.DataSeries     = reshape(Event_Data,qF,G.DataLimits.Lon2+1 + length(G.lon) -G.DataLimits.Lon1,G.DataLimits.Lat2-G.DataLimits.Lat1+1) ;
            if G.Limits.Default.DataSeriesAnomFlag
                G.DataSeriesAnom = reshape(Event_Data_Anom,qF,G.DataLimits.Lon2+1 + length(G.lon) -G.DataLimits.Lon1,G.DataLimits.Lat2-G.DataLimits.Lat1+1) ;
            end
        end
    case 3
        G.DataSeries     = reshape(Event_Data,qF,SizeLat0,SizeP0) ;
        if G.Limits.Default.DataSeriesAnomFlag
            G.DataSeriesAnom = reshape(Event_Data_Anom,qF,SizeLat0,SizeP0) ;
        end
    case 4
        G.DataSeries     = reshape(Event_Data,qF,SizeLon0,SizeP0) ;
        if G.Limits.Default.DataSeriesAnomFlag
            G.DataSeriesAnom = reshape(Event_Data_Anom,qF,SizeLon0,SizeP0) ;
        end
    otherwise
        G.DataSeries = reshape(G.Data,[1 size(G.Data)]) ;
        G.DataSeries = permute(G.DataSeries,[3 1 2]) ;
        if  G.Limits.Default.Anom
            G = GOAT_Compile_Climatology(G) ;
            G = GOAT_Compile_ClimAnomaly(G) ;
            G.Data = squeeze(G.DataSeriesAnom)' ;
        end
end
if any(G.ProjectionType == [1,2,3,4,8,9])
    G.Data = squeeze(nanmean(G.DataSeries,1)) ;
end
G.Dates          = G.Addresses.Default(G.DataStringVec) ;
G.FilteredDates  = G.Dates(G.Filter.Index(G.DataStringVec)==1) ;