% GOAT_Filters GOAT Filters and Weights GUI
% Copyright (C) Ori Adam 2012. compatible with version 2013i or later
%
% see also WeightFunc

function G = GOAT_Filters(G)
pos        = G.GUI.pos.Filter ; 
G.Filter_Fig = figure('name','GOAT: Filters and Weights','position',pos,'color',G.Display.FigBackgroudColor,'KeyPressFcn',{@FILTERKeyPress,G},'WindowKeyPressFcn',{@FILTERKeyPress,G}) ;
set(G.Filter_Fig,'MenuBar','none')
NumLines = 12 ;
bl = 0.1 ; % Left Edge
bt = 0.05; % Top Edge
LineHeight = (1-2*bt)/NumLines ;
Column     = (1-2*bl)/2 ;
%Top Message
messageStr = {'Select Filter / Weight';'Zonal Wave # is disabled in zonally averaged projections'} ;
h_message = uicontrol('Style', 'text','Units','normalized','pos',[0,1-LineHeight*.75,1,LineHeight*.75], 'string',messageStr);
uicontrol(h_message) ;  set(h_message,'ForegroundColor',G.GUI.messageColor,'FontSize',G.Display.TextSize_M,'FontName',G.GUI.Fonts.MessageFont) ;
% Weights
Line = 2 ;
h_WeightsT = uicontrol('Style', 'text','Units','normalized','pos',[bl,1-bt-Line*LineHeight,Column,LineHeight], 'string','Weights');
uicontrol(h_WeightsT) ;  set(h_WeightsT,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',G.GUI.HeaderColor,'FontSize',16,'HorizontalAlignment','left','TooltipString','Spatial Weights','FontName',G.GUI.Fonts.uiHeaderFont) ;
Line = 2 ;
h_Weights = uicontrol('style','popup','Units','normalized','pos',[bl+Column,1-bt-Line*LineHeight,Column,LineHeight],...
    'string','None | Factor | cos(lat) | -cos(lat) | 1/(a*cos) | -1/(a*cos)',  'callback','set(gcbf,''userdata'',11); uiresume;');
uicontrol(h_Weights) ;  set(h_Weights,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'TooltipString','Spatial Weights','FontName',G.GUI.Fonts.uiButtonFont) ;
set(h_Weights,'value',G.Filter.WeightIdx) ;
% Zonal Wave Number
Line = 3 ;
h_ZWNLowT = uicontrol('Style', 'text','Units','normalized','pos',[bl,1-bt-Line*LineHeight,Column,LineHeight], 'string','Low Zonal Wave #');
uicontrol(h_ZWNLowT) ;  set(h_ZWNLowT,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',G.GUI.HeaderColor,'FontSize',G.Display.TextSize_L,'HorizontalAlignment','left','TooltipString','Low range of Zonal Wave Number using FFT','FontName',G.GUI.Fonts.uiHeaderFont) ;
h_ZWNLow = uicontrol('style','popup','Units','normalized','pos',[bl+Column,1-bt-Line*LineHeight,Column,LineHeight],...
    'string','None | 0 | 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9 | 10 | 11 | 12 | 13 | 14 | 15 | 16 | 17 | 18 | 19 | 20 ', ...
    'callback','set(gcbf,''userdata'',12); uiresume;');
uicontrol(h_ZWNLow) ;  set(h_ZWNLow,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'TooltipString','Low range of Zonal Wave Number using FFT','FontName',G.GUI.Fonts.uiButtonFont) ;
set(h_ZWNLow,'value',G.Filter.ZonalWaveNumberLowIdx) ;
Line = 4 ;
h_ZWNHighT = uicontrol('Style', 'text','Units','normalized','pos',[bl,1-bt-Line*LineHeight,Column,LineHeight], 'string','High Zonal Wave #');
uicontrol(h_ZWNHighT) ;  set(h_ZWNHighT,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',G.GUI.HeaderColor,'FontSize',G.Display.TextSize_L,'HorizontalAlignment','left','TooltipString','High range of Zonal Wave Number using FFT','FontName',G.GUI.Fonts.uiHeaderFont) ;
h_ZWNHigh = uicontrol('style','popup','Units','normalized','pos',[bl+Column,1-bt-Line*LineHeight,Column,LineHeight],...
    'string','None | 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9 | 10 | 11 | 12 | 13 | 14 | 15 | 16 | 17 | 18 | 19 | 20 ', ...
    'callback','set(gcbf,''userdata'',13); uiresume;');
uicontrol(h_ZWNHigh) ;  set(h_ZWNHigh,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'TooltipString','Low range of Zonal Wave Number using FFT','FontName',G.GUI.Fonts.uiButtonFont) ;
set(h_ZWNHigh,'value',G.Filter.ZonalWaveNumberHighIdx) ;
% Eddy Flag
Line = 5 ;
% h_ZET = uicontrol('Style', 'text','Units','normalized','pos',[bl,1-bt-Line*LineHeight,Column,LineHeight], 'string','Zonal Anomaly');
% uicontrol(h_ZET) ;  set(h_ZET,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',G.GUI.HeaderColor,'FontSize',16,'HorizontalAlignment','left','FontName',G.GUI.Fonts.uiHeaderFont) ;
h_ZE = uicontrol('style','checkbox','Units','normalized','pos',[bl,1-bt-Line*LineHeight ,Column,LineHeight], 'string','Zonal Anomaly',...
    'value',G.Filter.ZEFlag,  'callback','set(gcbf,''userdata'',14); uiresume;');
uicontrol(h_ZE) ;  set(h_ZE,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_L,'TooltipString','Zonal Eddy','FontName',G.GUI.Fonts.uiHeaderFont) ;

if any(G.ProjectionType == [3, 6, 7])
    set(h_ZWNLow,'Enable','off')
    set(h_ZWNHigh,'Enable','off')
    set(h_ZE,'Enable','off')
end
% Running Average
DaysRAstr = 'None |' ;
for i = 2:365
    DaysRAstr = [DaysRAstr,int2str(i),' |'] ;
end
Line = 6 ;
[YYY,X] = GOAT_ParseTimeResolution(G.TimeFieldVirtual) ; 
switch X
    case 1
    RAstr = 'Running Average (Days)' ; 
    LowPassStr = 'LowPass (Period=Days)' ;
    LowPassTipStr = 'Low Pass Filter. Select Period in Days' ;
    HighPassStr = 'HighPass (Period=Days)' ;
    HighPassTipStr = 'High-Pass Filter. Select Period in Days' ;
    case 0
    RAstr = 'Running Average (Months)' ;
    LowPassStr = 'LowPass (Period=Months)' ;
    LowPassTipStr = 'Low Pass Filter. Select Period in Months' ;
    HighPassStr = 'HighPass (Period=Months)' ;
    HighPassTipStr = 'High-Pass Filter. Select Period in Months' ;
    case 2
    RAstr = 'Running Average (12 H Intervals)' ;    
    LowPassStr = 'LowPass (Period=12H)' ;
    LowPassTipStr = 'Low Pass Filter. Select Period in 12 Hour Intervals' ;
    HighPassStr = 'HighPass (Period=12H)' ;
    HighPassTipStr = 'High-Pass Filter. Select Period in 12 Hour Intervals' ;
    case 4
    RAstr = 'Running Average (6 H Intervals)' ;    
    LowPassStr = 'LowPass (Period=6H)' ;
    LowPassTipStr = 'Low Pass Filter. Select Period in 6 Hour Intervals' ;
    HighPassStr = 'HighPass (Period=6H)' ;
    HighPassTipStr = 'High-Pass Filter. Select Period in 6 Hour Intervals' ;
    case 8
    RAstr = 'Running Average (3 H Intervals)' ;
    LowPassStr = 'LowPass (Period=3H)' ;
    LowPassTipStr = 'Low Pass Filter. Select Period in 3 Hour Intervals' ;
    HighPassStr = 'HighPass (Period=3H)' ;
    HighPassTipStr = 'High-Pass Filter. Select Period in 3 Hour Intervals' ;
    case 12
    RAstr = 'Running Average (2 H Intervals)' ;
    LowPassStr = 'LowPass (Period=2H)' ;
    LowPassTipStr = 'Low Pass Filter. Select Period in 2 Hour Intervals' ;
    HighPassStr = 'HighPass (Period=2H)' ;
    HighPassTipStr = 'High-Pass Filter. Select Period in 2 Hour Intervals' ;
    case 24
    RAstr = 'Running Average (1 H Intervals)' ;
    LowPassStr = 'LowPass (Period=1H)' ;
    LowPassTipStr = 'Low Pass Filter. Select Period in 1 Hour Intervals' ;
    HighPassStr = 'HighPass (Period=1H)' ;
    HighPassTipStr = 'High-Pass Filter. Select Period in 1 Hour Intervals' ;
    case -1
    RAstr = 'Running Average (data time intervlas)' ;
    LowPassStr = 'LowPass (Period = data time intervlas)' ;
    LowPassTipStr = 'Low Pass Filter. Select Period in data time intervlas' ;
    HighPassStr = 'HighPass (Period = data time intervlas)' ;
    HighPassTipStr = 'High-Pass Filter. Select Period in data time intervlas' ;
end
h_RAT = uicontrol('Style', 'text','Units','normalized','pos',[bl,1-bt-Line*LineHeight,Column,LineHeight], 'string',RAstr);
uicontrol(h_RAT) ;  set(h_RAT,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',G.GUI.HeaderColor,'FontSize',G.Display.TextSize_L,'HorizontalAlignment','left','FontName',G.GUI.Fonts.uiHeaderFont) ;
h_RA = uicontrol('style','popup','Units','normalized','pos',[bl+Column,1-bt-Line*LineHeight,Column,LineHeight],...
    'string',DaysRAstr,  'callback','set(gcbf,''userdata'',21); uiresume;');
uicontrol(h_RA) ;  set(h_RA,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'TooltipString',RAstr,'FontName',G.GUI.Fonts.uiButtonFont) ;
set(h_RA,'value',G.Filter.RAIdx) ;
% Running Eddy
Line = 7 ;
% h_RET = uicontrol('Style', 'text','Units','normalized','pos',[bl,1-bt-Line*LineHeight,Column,LineHeight], 'string','Running Eddy');
% uicontrol(h_RET) ;  set(h_RET,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',G.GUI.HeaderColor,'FontSize',G.Display.TextSize_L,'HorizontalAlignment','left','FontName',G.GUI.Fonts.uiHeaderFont,'FontName',G.GUI.Fonts.uiHeaderFont) ;
h_RE = uicontrol('style','checkbox','Units','normalized','pos',[bl,1-bt-Line*LineHeight,Column,LineHeight], 'string','Running Eddy',...
    'value',G.Filter.REFlag,  'callback','set(gcbf,''userdata'',22); uiresume;');
uicontrol(h_RE) ;  set(h_RE,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_L,'TooltipString','Running Eddy = Difference from Running Average (Set Above)','FontName',G.GUI.Fonts.uiHeaderFont) ;
% Low Pass
Freqstr = 'None |' ;
for i = 1:100
    Freqstr = [Freqstr,int2str(i),' |'] ;
end
Line = 8 ;
h_LowT = uicontrol('Style', 'text','Units','normalized','pos',[bl,1-bt-Line*LineHeight,Column,LineHeight], 'string',LowPassStr);
uicontrol(h_LowT) ;  set(h_LowT,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',G.GUI.HeaderColor,'FontSize',G.Display.TextSize_L,'HorizontalAlignment','left','TooltipString',LowPassTipStr,'FontName',G.GUI.Fonts.uiHeaderFont) ;
h_Low = uicontrol('style','popup','Units','normalized','pos',[bl+Column,1-bt-Line*LineHeight,Column,LineHeight],...
    'string',Freqstr,  'callback','set(gcbf,''userdata'',31); uiresume;');
uicontrol(h_Low) ;  set(h_Low,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'TooltipString',LowPassTipStr,'FontName',G.GUI.Fonts.uiButtonFont) ;
set(h_Low,'value',G.Filter.LowPassIdx+1) ;
% High Pass
Line = 9 ;
h_HighT = uicontrol('Style', 'text','Units','normalized','pos',[bl,1-bt-Line*LineHeight,Column,LineHeight], 'string',HighPassStr);
uicontrol(h_HighT) ;  set(h_HighT,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',G.GUI.HeaderColor,'FontSize',G.Display.TextSize_L,'HorizontalAlignment','left','TooltipString',HighPassTipStr,'FontName',G.GUI.Fonts.uiHeaderFont) ;
h_High = uicontrol('style','popup','Units','normalized','pos',[bl+Column,1-bt-Line*LineHeight,Column,LineHeight],...
    'string',Freqstr,  'callback','set(gcbf,''userdata'',32); uiresume;');
uicontrol(h_High) ;  set(h_High,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'TooltipString',HighPassTipStr,'FontName',G.GUI.Fonts.uiButtonFont) ;
set(h_High,'value',G.Filter.HighPassIdx+1) ;
% Indices
Line = 10 ;
h_IndicesT = uicontrol('Style', 'text','Units','normalized','pos',[bl,1-bt-Line*LineHeight,Column,LineHeight], 'string','Indices');
uicontrol(h_IndicesT) ;  set(h_IndicesT,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',G.GUI.HeaderColor,'FontSize',G.Display.TextSize_L,'HorizontalAlignment','left','TooltipString','Indices by dates found at the ''Indices'' folder','FontName',G.GUI.Fonts.uiHeaderFont) ;
h_Indices = uicontrol('style','popup','Units','normalized','pos',[bl+Column,1-bt-Line*LineHeight,Column,LineHeight],...
    'string',G.Filter.IndicesString,  'callback','set(gcbf,''userdata'',40); uiresume;');
uicontrol(h_Indices) ;  set(h_Indices,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'TooltipString','Indices by dates found at the ''Indices'' folder','FontName',G.GUI.Fonts.uiButtonFont) ;
set(h_Indices,'value',G.Filter.IndicesIdx) ;

% Clear
Line = NumLines  ;
h_Cancel = uicontrol('Units','normalized','pos',[bl*3+Column,1-bt-Line*LineHeight,Column - bl*2,LineHeight],...
    'string','CLEAR',  'callback','set(gcbf,''userdata'',99); uiresume;');
uicontrol(h_Cancel) ;  set(h_Cancel,'BackgroundColor',G.GUI.ButtonColor,'ForegroundColor',G.GUI.DropTextColor,'FontSize',G.Display.TextSize_L,'fontweight','bold','FontName',G.GUI.Fonts.uiButtonFont,'TooltipString','keypress = ''c''') ;

% OK
Line = NumLines  ;
h_OK = uicontrol('Units','normalized','pos',[bl/5,1-bt-Line*LineHeight,Column + bl*2,LineHeight],...
    'string','OK',  'callback','set(gcbf,''userdata'',100); uiresume;');
uicontrol(h_OK) ;  set(h_OK,'BackgroundColor',G.GUI.colors.celeste,'ForegroundColor',G.GUI.DropTextColor,'FontSize',G.Display.TextSize_L,'fontweight','bold','FontName',G.GUI.Fonts.uiButtonFont,'TooltipString','keypress = ''g''') ;

GUIcon = 1 ;
while GUIcon
    try
        set(0,'CurrentFigure',G.Filter_Fig) ;
        uiwait(gcf);
        select = get(gcf,'userdata');
    catch
        GUIcon = 0 ;
    end
    try
        switch select
            case 11 % Weights
                G.Filter.WeightIdx = get(h_Weights,'Value') ;
                if G.Filter.WeightIdx == 2
                    try % BM1 Settings BookMark
                    prompt = {'Factor               ' } ;
                    dlg_title = ''                   ;
                    num_lines = 1                                                   ;
                    def = {num2str(G.Filter.WeightFactor,5)}             ;  % Default values
                    answer = inputdlg(prompt,dlg_title,num_lines,def)               ;
                    G.Filter.WeightFactor        = eval(cell2mat(answer(1,1))) ; %
                catch
                    % Nevermind, Keep Defaults
                    end
                end
            case 12 % Low Range Zonal Wave Number
                G.Filter.ZonalWaveNumberLowIdx = get(h_ZWNLow,'value')  ;
             if G.Filter.ZonalWaveNumberLowIdx == 1 && G.Filter.ZonalWaveNumberHighIdx == 1
                G.Filter.ZonalWaveNumberIdx = - 1 ; 
             elseif G.Filter.ZonalWaveNumberLowIdx == 1 && G.Filter.ZonalWaveNumberHighIdx  > 1
                G.Filter.ZonalWaveNumberIdx = G.Filter.ZonalWaveNumberHighIdx - 1 ; % -1 = no Filter
             elseif G.Filter.ZonalWaveNumberLowIdx > 1 && G.Filter.ZonalWaveNumberHighIdx  == 1
                G.Filter.ZonalWaveNumberIdx = G.Filter.ZonalWaveNumberLowIdx - 2 ; 
             else 
                G.Filter.ZonalWaveNumberIdx = min([G.Filter.ZonalWaveNumberLowIdx-2,G.Filter.ZonalWaveNumberHighIdx-1]): max([G.Filter.ZonalWaveNumberLowIdx-2,G.Filter.ZonalWaveNumberHighIdx-1]) ; % 0 = no Filter
             end
            case 13 % High Range Zonal Wave Number
                G.Filter.ZonalWaveNumberHighIdx = get(h_ZWNHigh,'value')  ;
            if G.Filter.ZonalWaveNumberLowIdx == 1 && G.Filter.ZonalWaveNumberHighIdx == 1
                G.Filter.ZonalWaveNumberIdx = - 1 ; 
             elseif G.Filter.ZonalWaveNumberLowIdx == 1 && G.Filter.ZonalWaveNumberHighIdx  > 1
                G.Filter.ZonalWaveNumberIdx = G.Filter.ZonalWaveNumberHighIdx - 1 ; % -1 = no Filter
             elseif G.Filter.ZonalWaveNumberLowIdx > 1 && G.Filter.ZonalWaveNumberHighIdx  == 1
                G.Filter.ZonalWaveNumberIdx = G.Filter.ZonalWaveNumberLowIdx - 2 ; 
             else 
                G.Filter.ZonalWaveNumberIdx = min([G.Filter.ZonalWaveNumberLowIdx-2,G.Filter.ZonalWaveNumberHighIdx-1]): max([G.Filter.ZonalWaveNumberLowIdx-2,G.Filter.ZonalWaveNumberHighIdx-1]) ; % 0 = no Filter
             end
            case 14 % Zonal Wave Number
                G.Filter.ZEFlag = get(h_ZE,'value')  ;    
            case 21 % Running Average
                G.Filter.RAIdx       = get(h_RA,'Value')     ;
%                 if G.Filter.RAIdx > 1
%                     G.Filter.LowPassIdx        = 0 ; % 0 = no Filter
%                     set(h_Low,'value',1) ;
%                     G.Filter.HighPassIdx       = 0 ; % 0 = no Filter
%                     set(h_High,'value',1) ;
%                 end
            case 22 % Running Eddy
                G.Filter.REFlag       = get(h_RE,'Value')     ;
            case 31 % Low Pass Days
                G.Filter.LowPassIdx  = get(h_Low,'Value') - 1 ;
%                 if G.Filter.LowPassIdx > 0
%                     G.Filter.RAIdx        = 1 ; % 1 = no Filter
%                     set(h_RA,'value',1) ;
%                 end
            case 32 % High Pass Days
                G.Filter.HighPassIdx = get(h_High,'Value') - 1  ;
%                 if G.Filter.HighPassIdx > 0
%                     G.Filter.RAIdx        = 1 ; % 0 = no Filter
%                     set(h_RA,'value',1) ;
%                 end
            case 40
                G.Filter.IndicesIdx = get(h_Indices,'Value')   ;
                % Index
                if G.Filter.IndicesIdx == 1
                G.Filter.Index = ones(length(G.Addresses.Default),1) ;    
                elseif G.Filter.IndicesIdx < length(G.Filter.Indices) + 2
                load(fullfile(G.DataFolder,G.TimeField,'Indices',[cell2mat(G.Filter.Indices(G.Filter.IndicesIdx-1)),'.mat'])) ; 
                G.Filter.IndexAddress = fullfile(G.DataFolder,G.TimeField,'Indices',[cell2mat(G.Filter.Indices(G.Filter.IndicesIdx-1)),'.mat']) ; 
                G.Filter.Index = Index ; 
                else
                    try
                    [fid, fidf] = uigetfile('*.mat') ; 
                    load([fidf,fid]) ;
                    G.Filter.IndexAddress = [fidf,fid] ; 
                    G.Filter.Index = Index ;     
                    catch
                    G.Filter.IndicesIdx = 1 ; 
                    G.Filter.Index = ones(length(G.Addresses.Default),1) ;    
                    set(h_Indices,'Value',G.Filter.IndicesIdx) ;
                    end
                end
            case 99 % Clear
                G = GOAT_Filters_Clear(G) ; 
                GUIcon = 0 ;
            case 100 % OK
                GUIcon = 0 ;
        end
    catch
        %nevermind
    end
end

if G.Filter.HighPassIdx == G.Filter.LowPassIdx && G.Filter.LowPassIdx > 0
    ErrorMessage = 'Low and High Limits cannot be Equal' ;
    set(h_message,'string',ErrorMessage,'ForegroundColor',[.5 .1 .1]) ; % Update Message
    %soundsc(G.ErrorSound,3000)
    soundsc(G.ErrorSound,G.ErrorSoundFs) ;
    pause(3) ;
    try
        close(G.Filter_Fig) ;
    catch
        % Nevermind
    end
    G = GOAT_Filters(G) ; % Retry
    return ;
end
try
    close(G.Filter_Fig) ;
catch
    % Nevermind
end

return ;
% FILTER KeyPressFunction
function FILTERKeyPress(~,event,G)
try
switch event.Character
    case 'g'
        set(gcf,'userdata',100) ;
        uiresume ;
        return ; 
    case 'c'
        set(gcf,'userdata',99) ;
        uiresume ;
        return ; 
    
end
catch
    % Ignore
end