% GOAT_GetFieldSpan Get temporal span or Attributes of 'G.DataField' 
% syntax: assumes all information in contained in the GOAT structure G
% >> G = GOAT_GetFieldSpan(G) ; 
% Copyright (C) Ori Adam Feb.2012. 
%
% see also GOAT_GetDims, GOAT_NewFieldLimits

function G = GOAT_GetFieldSpan(G)
if ~G.EnvironmentFlag
    try
    set(G.h_Analyze,'Enable','off')  ; drawnow ; % Disable GO button
    catch
    % Nevermind, if this is the first call to GetFieldSpan before the GUI is set up    
    end
end
if ~G.DataFieldsType(find(strcmp(G.DataField,G.DataFields))) % In case a real field overrides a virtual field
    DataField = G.DataField ;
else
    switch G.DataField
        case G.VirtualFields.List
            G = GOAT_VirtualFields(G,'ReferenceField',G.DataField) ;
            DataField = G.VirtualFields.ReferenceField ;
        case G.VirtualFields.MyList
            Idx = find(strcmp(G.DataField,G.VirtualFields.MyList)) ;
            eval(['G = ',G.VirtualFields.MyFuncs{Idx},'(''ReferenceField'',G) ; ']) ;
            DataField = G.VirtualFields.ReferenceField ;
        otherwise
            DataField = G.DataField ;
    end
end
G.DataFieldReference = DataField ;

if ~strcmpi(G.TimeField,'list')
    % look for linked data
    switch exist(fullfile(G.DataFolder,G.TimeField,G.DataFieldReference,'link_field.mat'),'file') > 0
        case 1
            if isfield(G.ALP.Attributes,G.DataField)
                G.Addresses.Attributes = G.ALP.Attributes.(G.DataFieldReference) ;
                G.Addresses.Default    = G.ALP.Addresses.(G.DataFieldReference) ;
                G.Limits.Smin          = G.ALP.Smin.(G.DataFieldReference) ;
                G.Limits.Smax          = G.ALP.Smax.(G.DataFieldReference) ;
                G.Limits.years         = G.ALP.Years.(G.DataFieldReference) ;
            elseif isfield(G.ALP.Attributes,G.DataFieldReference)
                G.Addresses.Attributes = G.ALP.Attributes.(G.DataFieldReference) ;
                G.Addresses.Default    = G.ALP.Addresses.(G.DataFieldReference) ;
                G.Limits.Smin          = G.ALP.Smin.(G.DataFieldReference) ;
                G.Limits.Smax          = G.ALP.Smax.(G.DataFieldReference) ;
                G.Limits.years         = G.ALP.Years.(G.DataFieldReference) ;
                G.ALP.Attributes.(G.DataField) = G.ALP.Attributes.(G.DataFieldReference) ;
                G.ALP.Addresses.(G.DataField)  = G.ALP.Addresses.(G.DataFieldReference) ;
                G.ALP.Smin.(G.DataField)       = G.ALP.Smin.(G.DataFieldReference) ; 
                G.ALP.Smax.(G.DataField)       = G.ALP.Smax.(G.DataFieldReference) ;
                G.ALP.Years.(G.DataField)      = G.ALP.Years.(G.DataFieldReference) ;
            else
                if ~G.EnvironmentFlag
                    set(G.h_message,'string',['Linking field ',G.DataField,' . . .']) ; drawnow ; % Update Message
                end
                load(fullfile(G.DataFolder,G.TimeField,G.DataFieldReference,'link_field.mat')) ;
                try
                    if isfield(Attributes{end,3},'evalStr_load')
                        try
                            eval(Attributes{end,3}.evalStr_load) ; % evaluate eval-string in Defs. if it exists (used for OPeNDAP datasets with time varying indices)
                        catch
                            disp 'GetFIeldSpan Warning: failed to exectute evalStr_load. Data may not be up to date.'
                        end
                    end
                end
                G.Addresses.Attributes = GOAT_ALP_DecompressAttributes(Attributes) ;
                if ~strcmp(G.TimeFieldVirtual,G.TimeField)
                    [YYY , X ] = GOAT_ParseTimeResolution(G.TimeField) ;
                    [YYY , XV] = GOAT_ParseTimeResolution(G.TimeFieldVirtual) ;
                    G.Addresses.Attributes = [G.Addresses.Attributes(1:X/XV:end-1,:) ; G.Addresses.Attributes(end,:)] ;
                    G.Addresses.Attributes(end,4) = {G.TimeFieldVirtual} ;
                end
                try % Only if in GUI mode - Add option to download linked data
                    G = GOAT_setProjectionStr(G) ;
                    set(G.h_Projection,'string',G.GUI.ProjectionStr) ;
                end
                % create Address vector
                G = GOAT_Produce_DefaultAddress(G,G.Addresses.Attributes{end,2}(1),G.Addresses.Attributes{end,2}(2),G.TimeFieldVirtual,G.Addresses.Attributes{end,1}) ;
                G.Limits.years = G.Addresses.Attributes{end,2}(1):G.Addresses.Attributes{end,2}(2) ;
                
                FirstDateIdx = find(~cellfun(@isempty,G.Addresses.Attributes(:,4)),1,'first') ;
                if isempty(FirstDateIdx)
                    G.Limits.Smin = 1 ;
                    G.Limits.Smax = size(G.Addresses.Default,1)-1  ;
                else
                    G.Limits.Smin = FirstDateIdx ;
                end
                LastDateIdx = find(~cellfun(@isempty,G.Addresses.Attributes(1:end-1,4)),1,'last') ;
                if ~isempty(LastDateIdx)
                    G.Limits.Smax = LastDateIdx ;
                end
                % check to see if there are any '.mat' files.
                Yearsdir = dir(fullfile(G.DataFolder,G.TimeField,G.DataFieldReference,'')) ;
                NF = 0 ;
                YearsDirs = zeros(1,length(Yearsdir)) ;
                for j = 1:length(Yearsdir)
                    if Yearsdir(j,1).isdir && ...
                            ~strncmp(Yearsdir(j,1).name,'.',1)  && ... 
                            ~strcmpi(Yearsdir(j,1).name,'data')
                        NF = NF + 1 ;
                        YearsDirs(NF) = str2num(Yearsdir(j,1).name) ;
                    end
                end
                YearsDirs = YearsDirs(1:NF) ;
                if NF > 0
                    % look for overlapping years
                    if sum(ismember(YearsDirs(1:NF),G.Limits.years))
                        if ~G.EnvironmentFlag
                            set(G.h_message,'string',['Overriding linked ',G.DataFieldReference,' files with physical files . . .']) ; drawnow ; % Update Message
                        end
                        % look for mat files:
                        FieldAddress = fullfile(cellstr(repmat(fullfile(G.DataFolder,G.TimeField,G.DataFieldReference,''),[size(G.Addresses.Default,1) 1])),G.Addresses.Default) ;
                        ext0 = cell(size(G.Addresses.Default,1),1) ; ext0(:) = {'.mat'} ;
                        ext1 = cell(size(G.Addresses.Default,1),1) ; ext1(:) = {['.',G.Addresses.Attributes{end,1}]} ;
                        Kind = cell(size(G.Addresses.Default,1),1) ; Kind(:) = {'file'} ;
                        FieldAddress = cellfun(@regexprep,FieldAddress,ext1,ext0,'UniformOutput',false) ;
                        PhysicalFiles = find(cellfun(@exist,FieldAddress,Kind)) ;
                        if ~isempty(PhysicalFiles)
                            G.Addresses.Default(PhysicalFiles) = cellfun(@regexprep,G.Addresses.Default(PhysicalFiles),ext1(PhysicalFiles),ext0(PhysicalFiles),'UniformOutput',false) ;
                        end
                    end
                    % look for physical dates earlier than Attributes.
                    if min(YearsDirs) < min(G.Limits.years)
                        if ~G.EnvironmentFlag
                            set(G.h_message,'string',['Overriding linked ',G.DataFieldReference,' files with physical files . . .']) ; drawnow ; % Update Message
                        end
                        DefaultAddress = G.Addresses.Default ;
                        G = GOAT_Produce_DefaultAddress(G,min(YearsDirs),min(G.Limits.years)-1,G.TimeFieldVirtual) ;
                        FieldAddress = fullfile(cellstr(repmat(fullfile(G.DataFolder,G.TimeField,G.DataFieldReference,''),[size(G.Addresses.Default,1) 1])),G.Addresses.Default) ;
                        Kind = cell(size(G.Addresses.Default,1),1) ; Kind(:) = {'file'} ;
                        PhysicalFiles = find(cellfun(@exist,FieldAddress,Kind),1,'first') ;
                        if ~isempty(PhysicalFiles)
                            G.Limits.Smin = PhysicalFiles ;
                            G.Addresses.Attributes = [cell(size(G.Addresses.Default,1),6) ; G.Addresses.Attributes] ;
                            G.Addresses.Default    = [G.Addresses.Default ; DefaultAddress] ;
                            G.Limits.years = min(YearsDirs):max(G.Limits.years) ;
                        end
                    end
                    % look for physical dates later than Attributes.
                    if max(YearsDirs) > max(G.Limits.years)
                        if ~G.EnvironmentFlag
                            set(G.h_message,'string',['Overriding linked ',G.DataFieldReference,' files with physical files . . .']) ; drawnow ; % Update Message
                        end
                        DefaultAddress = G.Addresses.Default ;
                        G = GOAT_Produce_DefaultAddress(G,max(G.Limits.years)+1,max(YearsDirs),G.TimeFieldVirtual) ;
                        FieldAddress = fullfile(cellstr(repmat(fullfile(G.DataFolder,G.TimeField,G.DataFieldReference,''),[size(G.Addresses.Default,1) 1])),G.Addresses.Default) ;
                        Kind = cell(size(G.Addresses.Default,1),1) ; Kind(:) = {'file'} ;
                        PhysicalFiles = find(cellfun(@exist,FieldAddress,Kind),1,'last') ;
                        if ~isempty(PhysicalFiles)
                            G.Limits.Smax = PhysicalFiles + size(DefaultAddress,1) ;
                            G.Addresses.Attributes = [G.Addresses.Attributes ; cell(size(G.Addresses.Default,1),6) ] ;
                            G.Addresses.Default    = [DefaultAddress         ; G.Addresses.Default                 ] ;
                            G.Limits.years = min(G.Limits.years):max(YearsDirs) ;
                        end
                    end  
                end
                % save attributes for speedup in later operations
                G.ALP.Attributes.(G.DataFieldReference) = G.Addresses.Attributes ; 
                G.ALP.Addresses.(G.DataFieldReference)  = G.Addresses.Default    ; 
                G.ALP.Smin.(G.DataFieldReference)       = G.Limits.Smin          ; 
                G.ALP.Smax.(G.DataFieldReference)       = G.Limits.Smax          ; 
                G.ALP.Years.(G.DataFieldReference)      = G.Limits.years         ;
                G.ALP.Defs.(G.DataFieldReference)       = G.Addresses.Attributes{end,3}    ;
                if ~strcmp(G.DataFieldReference,G.DataField)
                G.ALP.Attributes.(G.DataField) = G.Addresses.Attributes ; 
                G.ALP.Addresses.(G.DataField)  = G.Addresses.Default    ; 
                G.ALP.Smin.(G.DataField)       = G.Limits.Smin          ; 
                G.ALP.Smax.(G.DataField)       = G.Limits.Smax          ; 
                G.ALP.Years.(G.DataField)      = G.Limits.years         ;
                G.ALP.Defs.(G.DataField)       = G.Addresses.Attributes{end,3}    ;
                end
            end
            G.Limits.YearsStr = '' ;
            for i = 1:length(G.Limits.years)
                G.Limits.YearsStr = [G.Limits.YearsStr,int2str(G.Limits.years(i)),' |'] ;
            end
            G.Limits.YearsStr = G.Limits.YearsStr(1:end-1);
            G.Limits.FirstYear = 1 ;
            G.Limits.LastYear  = length(G.Limits.years) ;
            [y1,m1,d1,h1] = GOAT_serial2date(G.Addresses.Default,G.Limits.Smin) ;
            [y2,m2,d2,h2] = GOAT_serial2date(G.Addresses.Default,G.Limits.Smax) ;
            G.Limits.FirstDay   = d1 ;
            G.Limits.FirstMonth = m1 ;
            G.Limits.LastDay   = d2 ;
            G.Limits.LastMonth = m2 ;
            try
            [YYY , X] = GOAT_ParseTimeResolution(G.TimeFieldVirtual) ;
            if X > 1
                G.Limits.FirstHour   = round(h1*X/24+1) ;
                G.Limits.LastHour    = round(h2*X/24+1) ;
            else
                G.Limits.FirstHour   = 1 ;
                G.Limits.LastHour    = 1 ;
            end
            catch
                G.Limits.FirstHour   = 1 ;
                G.Limits.LastHour    = 1 ;
            end
            if ~strcmp(G.TimeFieldVirtual,G.TimeField)
              if G.Limits.LastHour > X
                  G.Limits.LastHour = X ; 
              end
            end
            if ~G.EnvironmentFlag
                   set(G.h_message,'string',[G.DataField, ' good to GO.']) ; drawnow ; % Update Message
            end
        otherwise
            G.Addresses.Attributes = [] ;
            try % Only if in GUI mode - Add option to download linked data
                G = GOAT_setProjectionStr(G) ;
                set(G.h_Projection,'string',G.GUI.ProjectionStr) ;
            end
            % Get Years Span
            Yearsdir = dir(fullfile(G.DataFolder,G.TimeField,G.DataFieldReference,'')) ;
            NF = 0 ;
            G.Limits.years = zeros(1,length(Yearsdir)) ;
            for j = 1:length(Yearsdir)
                if Yearsdir(j,1).isdir && ...
                        ~strncmp(Yearsdir(j,1).name,'.',1)
                    NF = NF + 1 ;
                    G.Limits.years(NF) = str2num(Yearsdir(j,1).name) ;
                end
            end
            G.Limits.years = G.Limits.years(1:NF) ;
            years = sort(G.Limits.years) ;
            % create Address vector
            G = GOAT_Produce_DefaultAddress(G,min(years),max(years),G.TimeFieldVirtual) ;
            G.Limits.YearsStr = '' ;
            for i = 1:length(years)
                G.Limits.YearsStr = [G.Limits.YearsStr,int2str(years(i)),' |'] ;
            end
            G.Limits.YearsStr = G.Limits.YearsStr(1:end-1);
            G.Limits.FirstYear = 1 ;
            G.Limits.LastYear  = length(G.Limits.years) ;
            % Get days span
            Firstyeardir = dir(fullfile(G.DataFolder,G.TimeField,G.DataFieldReference,num2str(min(G.Limits.years),'%04.0f'),'')) ;
            NF = 0 ;
            Days = 10100 ; % Default
            for j = 1:length(Firstyeardir)
                [filepath, filename, ext] = fileparts(Firstyeardir(j,1).name) ;
                if ~strncmp(Firstyeardir(j,1).name,'.',1) && ...
                        strcmp(ext,'.mat') && ...
                        ~strcmp(filename,'info') && ...
                        ~strcmp(filename,'Topography')
                    NF = NF + 1 ;
                    Days(NF) = str2num(Firstyeardir(j,1).name(1:end-4)) ;
                end
            end
            FirstDayNum = min(round(Days)) ;
            y1 = min(G.Limits.years) ;
            if strncmpi(G.TimeFieldVirtual,'Julian',6) % Julian Days
                m1 = 0                 ;
                d1   = floor(FirstDayNum/100) ;
                h1   = FirstDayNum - d1*100 ;
            else % Gregorian
                m1 = floor(FirstDayNum/10000) ;
                d1 = floor((FirstDayNum - m1*10000)/100) ;
                h1 = FirstDayNum - m1*10000 - d1*100 ;
            end
            G.Limits.FirstDay   = d1 ;
            G.Limits.FirstMonth = m1 ;
            clear Days
            %Lastyeardir = dir([G.DataFolder,G.Dash,G.TimeField,G.Dash,G.DataFieldReference,G.Dash,int2str(G.Limits.years(end))]) ;
            Lastyeardir = dir(fullfile(G.DataFolder,G.TimeField,G.DataFieldReference,num2str(max(G.Limits.years),'%04.0f'),'')) ;
            NF = 0 ;
            Days = 10100 ; % Default
            for j = 1:length(Lastyeardir)
                [filepath, filename, ext] = fileparts(Lastyeardir(j,1).name) ;
                if ~strncmp(Lastyeardir(j,1).name,'.',1) && ...
                        strcmp(ext,'.mat') && ...
                        ~strcmp(filename,'info') && ...
                        ~strcmp(filename,'Topography')
                    NF = NF + 1 ;
                    Days(NF) = str2num(Lastyeardir(j,1).name(1:end-4)) ;
                end
            end
            LastDayNum = max(round(Days)) ;
            y2 = max(G.Limits.years) ;
            if strncmpi(G.TimeFieldVirtual,'Julian',6) % Julian Days
                m2 = 0                 ;
                d2   = floor(LastDayNum/100) ;
                h2   = LastDayNum - d2*100 ;
            else % Gregorian
                m2 = floor(LastDayNum/10000) ;
                d2 = floor((LastDayNum - m2*10000)/100) ;
                h2 = LastDayNum - m2*10000 - d2*100 ;
            end
            G.Limits.LastDay   = d2 ;
            G.Limits.LastMonth = m2 ;
            try
            [YYY , X] = GOAT_ParseTimeResolution(G.TimeFieldVirtual) ;
            if X > 1
                G.Limits.FirstHour   = round(h1*X/24+1) ;
                G.Limits.LastHour    = round(h2*X/24+1) ;
            else
                G.Limits.FirstHour   = 1 ;
                G.Limits.LastHour    = 1 ;
            end
            catch
                G.Limits.FirstHour   = 1 ;
                G.Limits.LastHour    = 1 ;
            end
            if ~strcmp(G.TimeFieldVirtual,G.TimeField)
              if G.Limits.LastHour > X
                  G.Limits.LastHour = X ; 
              end
            end
    end
    try
        G.Limits.SminOld  = G.Limits.Smin  ;
        G.Limits.SmaxOld  = G.Limits.Smax  ;
        G.Limits.Year0Old = G.Limits.Year0 ;
        G.Limits.Year0 = min(G.Limits.years) ;
        G.Limits.Smin  = GOAT_getSerial(y1,m1,d1,h1,G.TimeFieldVirtual,G.Limits.Year0) ;
        G.Limits.Smax  = GOAT_getSerial(y2,m2,d2,h2,G.TimeFieldVirtual,G.Limits.Year0) ;
        
    catch
        G.Limits.Year0 = min(G.Limits.years) ;
        G.Limits.Smin  = GOAT_getSerial(y1,m1,d1,h1,G.TimeFieldVirtual,G.Limits.Year0) ;
        G.Limits.Smax  = GOAT_getSerial(y2,m2,d2,h2,G.TimeFieldVirtual,G.Limits.Year0) ;
        G.Limits.SminOld  = G.Limits.Smin  ;
        G.Limits.SmaxOld  = G.Limits.Smax  ;
        G.Limits.Year0Old = G.Limits.Year0 ;
    end
    G = GOAT_Filters_Clear(G) ; 
    %G.Filter.Index = ones(length(G.Addresses.Default),1) ;
else % Linear Time
    % Create timestr vector
    G.Addresses.Attributes = [] ;
    Linears = dir(fullfile(G.DataFolder,G.TimeField,G.DataFieldReference)) ;
    nl = 0 ;
    G.Addresses.Linear = cell(1,1) ;
    G.Addresses.Linears = cell(1,1) ;
    G.Addresses.LinearAtributes = cell(1,1) ;
    for i = 1:length(Linears)
        if  Linears(i,1).isdir && ~strncmp(Linears(i,1).name,'.',1)
            nl = nl + 1 ;
            G.Addresses.Linears(nl,1) = {Linears(i,1).name} ;
            nf = 0 ;
            Timedir = dir(fullfile(G.DataFolder,G.TimeField,G.DataFieldReference,Linears(i,1).name)) ;
            TimeNum = 0 ;
            for j = 1:length(Timedir)
                [filepath, filename, ext] = fileparts(Timedir(j,1).name) ;
                if  ~Timedir(j,1).isdir && ~strncmp(Timedir(j,1).name,'.',1) && ...
                        strcmp(ext,'.mat') && ...
                        ~strcmp(filename,'info') && ...
                        ~strcmp(filename,'Topography') && ...
                        ~strcmp(filename,'link_field')
                    if strcmp(filename,'link_field')
                        disp 'Warning: GOAT_GetFieldSpan: link_field.mat file found along with other *.mat data files.' ;
                        disp 'Linked data in ''List'' time resolution cannot reside with other data and will be ignored.'
                    end
                    nf = nf + 1 ;
                    %TimeNum(nf) = str2num(Timedir(j,1).name(1:end-4)) ; % assuming *.mat
                    G.Addresses.Linear(nf,nl) = {fullfile(Linears(i,1).name,Timedir(j,1).name)} ;
                end
            end
            if nf > 0
                G.Addresses.LinearAtributes(1,nl) = {[]} ;
            else
                % try loading linked data
                try
                    load(fullfile(G.DataFolder,G.TimeField,G.DataFieldReference,Linears(i,1).name,'link_field.mat')) ;
                    try
                    eval(Attributes{end,3}.evalStr_load) ; % evaluate eval-string in Defs. if it exists (used for OPeNDAP datasets with time varying indices)
                    end
                    %G.Addresses.LinearAtributes(1,nl) = {Attributes} ;
                    G.Addresses.LinearAtributes(1,nl) = {GOAT_ALP_DecompressAttributes(Attributes)} ;
                catch
                    disp(['Warning: GOAT_GetFieldSpan: empty linear folder ',fullfile(G.DataFolder,G.TimeField,G.DataFieldReference,Linears(i,1).name)]) ;
                    nl = nl - 1 ;
                end
            end
        end
    end
    G.LinearsStr = '' ; % Cell array of available Time Resolution Folders
    for j = 1:nl
        linearname = G.Addresses.Linears{j} ;
        G.LinearsStr = [G.LinearsStr,linearname,' |'] ;
    end
    G.LinearsStr = G.LinearsStr(1:end-1) ;
    G.Addresses.LinearIdx = 1                      ; % set first folder as default folder
    G = GOAT_List_Initialize(G)                  ; % Initialize linear field (*.mat or linked)
    if ~G.EnvironmentFlag
       set(G.h_message,'string',[G.DataField, ' good to GO.']) ; drawnow ; % Update Message
    end
end
if ~G.EnvironmentFlag
    try
    set(G.h_Analyze,'Enable','on')  ; drawnow ; % Enable GO button
    catch
    % Nevermind, if this is the first call to GetFieldSpan before the GUI is set up    
    end
end
return