% GOAT_INFO_PLOT : list and describe current plot parameters
% syntax: assumes all information is contained in the GOAT structure G
% >>   GOAT_INFO_PLOT(G) ;
% Copyright (C) Ori Adam, May.2014
%
% see also, GOAT_INFO_LIMITS

function GOAT_INFO_PLOT(G)
%%
try
    Error  = cell(1,1) ; e = 2 ; Error(1,1) = {'Errors:'} ;
    Disp   = cell(1,1) ; d = 2 ; Disp(1,1)  = {'Status:'} ;
    Warn   = cell(1,1) ; w = 2 ; Warn(1,1)  = {'Warnings:'} ;
    headerline    = '=================================================================================' ;
    endline       = '(((((((((((((((((((((((((((((((((((((((()))))))))))))))))))))))))))))))))))))))))' ;
    % Projection Info
    G = GOAT_setProjectionStr(G) ;
    b =regexp(G.GUI.ProjectionStr,'[|]') ;
    b = [0 , b , length(G.GUI.ProjectionStr) + 1] ;
    bs = cell(length(b)-1,1) ;
    for i = 1:length(b)-1
        bs(i) = {G.GUI.ProjectionStr(b(i)+1:b(i+1)-1)} ;
    end
    % Begin Display
    disp '  ' ;
    cprintf('text' ,'((((((((((((    ');
    cprintf('*text','GOAT INFO PLOT STATUS ');
    cprintf('text' ,'    )))))))))))))\n');
    disp(headerline) ;
    % Projection
    dis(['Projection = ',bs{G.ProjectionType},'  [G.ProjectionType = ',int2str(G.ProjectionType),']']) ;
    disp(headerline) ;
    if ~isfield(G,'Limits')
        Error(e,1) = {'G.Limits indices are not defined'} ; e = e + 1 ;
    end
    try
        if ~isfield(G.Limits,'Default')
            Error(e,1) = {'G.Limits.Default indices are not defined'} ; e = e + 1 ;
        end
    end
    if ~isfield(G,'lat')
        Error(e,1) = {'G.lat (latitude ordinate) is not defined'} ; e = e + 1 ;
    end
    if ~isfield(G,'lon')
        Error(e,1) = {'G.lon (longitude ordinate) is not defined'} ; e  = e + 1 ;
    end
    if ~isfield(G,'P')
        Error(e,1) = {'G.P (vertical ordinate) is not defined'} ; e  = e + 1 ;
    end
    if ~isfield(G,'Data')
        Error(e,1) = {'G.Data (plotted field calculated by the function GOAT_Compile_data_Series) is not defined'} ; e  = e + 1 ;
    end
    if ~isfield(G,'X')
        Error(e,1) = {'G.X (meshed horizontal plot ordinate calculated by the function GOAT_Compile_data_Series) is not defined'} ; e  = e + 1 ;
    end
    if ~isfield(G,'Y')
        Error(e,1) = {'G.Y (meshed vertical plot ordinate calculated by the function GOAT_Compile_data_Series) is not defined'} ; e  = e + 1 ;
    end
    try
        if ~isfield(G.Limits.Default,'Lat1')
            Error(e,1) = {'G.Limits.Default.Lat1 index is not defined'} ; e  = e + 1 ;
        else
            Lat1 = min([G.lat(G.Limits.Default.Lat1),G.lat(G.Limits.Default.Lat2)]) ;
            Disp(d,1) = {['Lat1 = ',num2str(Lat1,5)]} ; d = d + 1 ;
        end
    end
    try
        if ~isfield(G.Limits.Default,'Lat2')
            Error(e,1) = {'G.Limits.Default.Lat2 index is not defined'} ; e  = e + 1 ;
        else
            Lat2 = max([G.lat(G.Limits.Default.Lat1),G.lat(G.Limits.Default.Lat2)]) ;
            Disp(d,1) = {['Lat1 = ',num2str(Lat2,5)]} ; d = d + 1 ;
        end
    end
    try
        try
            Lon1 = min(G.x) ; % min([G.lon(G.Limits.Default.Lon1),G.lon(G.Limits.Default.Lon2)]) ;
            Lon2 = max(G.x) ; % max([G.lon(G.Limits.Default.Lon1),G.lon(G.Limits.Default.Lon2)]) ;
        catch
            Lon1 = min([G.lon(G.Limits.Default.Lon1),G.lon(G.Limits.Default.Lon2)]) ;
            Lon2 = max([G.lon(G.Limits.Default.Lon1),G.lon(G.Limits.Default.Lon2)]) ;
        end
        Disp(d,1) = {['Lon1 = ',num2str(Lon1,5)]} ; d = d + 1 ;
        Disp(d,1) = {['Lon2 = ',num2str(Lon2,5)]} ; d = d + 1 ;
    catch
        Error(e,1) = {'Either G.x (truncated G.lon) or G.Limits.Default.Lon1 and G.Limits.Default.Lon2 indices are not defined'} ; e  = e + 1 ;
    end
    try
        P1   = min([G.lev(G.Limits.Default.Lev1),G.lev(G.Limits.Default.Lev2)]) ;
        P2   = max([G.lev(G.Limits.Default.Lev1),G.lev(G.Limits.Default.Lev2)]) ;
        Disp(d,1) = {['Level1 = ',num2str(P1,5)]} ; d = d + 1 ;
        Disp(d,1) = {['Level2 = ',num2str(P2,5)]} ; d = d + 1 ;
    catch
        Error(e,1) = {'Either G.P or G.Limits.Default.Lev1 and G.Limits.Default.Lev2 indices are not defined'} ; e  = e + 1 ;
    end
    
    try
        if G.Plotting.GUI_PlotFlag
            Disp(d,1) = {'G.Plotting.GUI_PlotFlag = 1     Plot is in GUI mode (subplot(1,10,[2 10]))'} ; d = d + 1 ;
        end
    end
    
    try
        ContourIdx = G.Settings.ContourIdx ;
        if ContourIdx > 1
            switch ContourIdx
                case 2
                    ContourType = 'contour(G.X,G.Y,G.Data'')' ;
                case 3
                    ContourType = 'surf(G.X,G.Y,G.Data'')' ;
                case 4
                    ContourType = 'surfc(G.X,G.Y,G.Data'')' ;
                case 5
                    ContourType = 'mesh(G.X,G.Y,G.Data'')' ;
                case 6
                    ContourType = 'meshc(G.X,G.Y,G.Data'')' ;
                case 7
                    ContourType = 'meshz(G.X,G.Y,G.Data'')' ;
                case 8
                    ContourType = 'imagesc([Lon1 Lon2],[Lat1 Lat2],double(G.Data'')' ;
            end
            Disp(d,1) = {['ContourIdx = ',int2str(ContourIdx),'  [',ContourType,' !! Experimental. Not supported in official GOAT versions !!] ']} ; d = d + 1 ;
        else
            Disp(d,1) = {'ContourIdx = 1 [default: contourf(G.X,G.Y,G.Data'')] '} ; d = d + 1 ;
        end
    catch
        ContourIdx = 1 ;
        Disp(d,1) = {'ContourIdx = 1 [default: contourf(G.X,G.Y,G.Data'')] '} ; d = d + 1 ;
    end
    
    try
        LineWidth      = G.Settings.LineWidth      ;
        CoastLineWidth = G.Settings.CoastLineWidth ;
        if LineWidth < 0
            Disp(d,1) = {['LineWidth = G.Settings.LineWidth = ',int2str(G.Settings.LineWidth),'   [use the function GOAT_darken_edge to darken edges]']} ; d = d + 1 ;
        else
            Disp(d,1) = {['LineWidth = G.Settings.LineWidth = ',int2str(G.Settings.LineWidth)]} ; d = d + 1 ;
        end
        if CoastLineWidth == 0
            Disp(d,1) = {['Coast-LineWidth = G.Settings.CoastLineWidth = ',int2str(G.Settings.CoastLineWidth),'   [No coastline]']} ; d = d + 1 ;
        else
            Disp(d,1) = {['Coast-LineWidth = G.Settings.CoastLineWidth = ',int2str(G.Settings.CoastLineWidth)]} ; d = d + 1 ;
        end
    catch
        LineWidth      = 2 ;
        CoastLineWidth = 1 ;
        Disp(d,1) = {['LineWidth = 2 [Default. G.Settings.LineWidth = undefined]']} ; d = d + 1 ;
        Disp(d,1) = {['Coast LineWidth = 1 [Default. G.Settings.CoastLineWidth = undefined]']} ; d = d + 1 ;
    end
    
    try
        Disp(d,1) = {['G.Settings.ColorMapIdx = ',int2str(G.Settings.ColorMapIdx)]} ; d = d + 1 ;
        ColorMapStr = cell2mat(G.Settings.ColorMap(G.Settings.ColorMapIdx)) ;
        Disp(d,1) = {['ColorMap = ''',ColorMapStr,'''']} ; d = d + 1 ;
    catch
        Disp(d,1) = {'ColorMapDef = ''jet'' ; [Default. G.Settings.ColorMapIdx = undefined]'} ; d = d + 1 ;
    end
    
    
    if ~isfield(G,'lonlabel')
        Error(e,1) = {'longitude label, ''G.lonlabel'' is not defined'} ; e  = e + 1 ;
    else
        Disp(d,1) = {['G.lonlabel = ''',G.lonlabel,'''']} ; d = d + 1 ;
    end
    if ~isfield(G,'latlabel')
        Error(e,1) = {'meridional label, ''G.latlabel'' is not defined'} ; e  = e + 1 ;
    else
        Disp(d,1) = {['G.latlabel = ''',G.latlabel,'''']} ; d = d + 1 ;
    end
    if ~isfield(G,'Plabel')
        Error(e,1) = {'vertical dimension label, ''G.Plabel'' is not defined'} ; e  = e + 1 ;
    else
        Disp(d,1) = {['G.Plabel = ''',G.Plabel,'''']} ; d = d + 1 ;
    end
    
    
    % Topo
    if G.ProjectionType == 2
        if ~isfield(G,'topo')
            Error(e,1) = {'G.topo structure is undefined. Cannot plot topography. See GOAT_GetTopography.'} ; e = e + 1 ;
        end
        try
            if ~isfield(G.topo,'TypeFlag')
                Error(e,1) = {'G.topo.TypeFlag is undefined. See GOAT_GetTopography.'} ; e = e + 1 ;
            end
            if ~isfield(G.topo,'lonG')
                Warn(w,1) = {'G.topo.lonG is undefined. See GOAT_GetTopography'} ; w = w + 1 ;
            end
            if ~isfield(G.topo,'latG')
                Warn(w,1) = {'G.topo.latG is undefined. See GOAT_GetTopography'} ; w = w + 1 ;
            end
            if ~isfield(G.topo,'topoA')
                Warn(w,1) = {'G.topo.topoA is undefined. See GOAT_GetTopography'} ; w = w + 1 ;
            end
            if ~isfield(G.topo,'topoO')
                Warn(w,1) = {'G.topo.topoO is undefined. See GOAT_GetTopography'} ; w = w + 1 ;
            end
            if ~isfield(G.topo,'topoG')
                Warn(w,1) = {'G.topo.topoG is undefined. See GOAT_GetTopography'} ; w = w + 1 ;
            end
            if ~isfield(G.topo,'topovec')
                Warn(w,1) = {'G.topo.topovec is undefined. See GOAT_GetTopography'} ; w = w + 1 ;
            end
            if ~isfield(G.topo,'TopotLineWidthIdx')
                Warn(w,1) = {'G.topo.TopotLineWidthIdx is undefined. See GOAT_GetTopography'} ; w = w + 1 ;
            end
        end
        try
            if ~isfield(G.topo,'CoastNans')
                Warn(w,1) = {'G.topo.CoastNans is undefined. See GOAT_GetTopography'} ; w = w + 1 ;
            end
            if ~isfield(G.topo,'AntLonAppendix')
                Warn(w,1) = {'G.topo.AntLonAppendix is undefined. See GOAT_GetTopography'} ; w = w + 1 ;
            end
            if ~isfield(G.topo,'AntLatAppendix')
                Warn(w,1) = {'G.topo.AntLatAppendix is undefined. See GOAT_GetTopography'} ; w = w + 1 ;
            end
            if ~isfield(G.topo,'landcoverlon')
                Warn(w,1) = {'G.topo.landcoverlon is undefined. See GOAT_GetTopography'} ; w = w + 1 ;
            end
            if ~isfield(G.topo,'landcoverlat')
                Warn(w,1) = {'G.topo.landcoverlat is undefined. See GOAT_GetTopography'} ; w = w + 1 ;
            end
            if ~isfield(G.topo,'ShadeColor')
                Warn(w,1) = {'G.topo.ShadeColor is undefined. See GOAT_GetTopography'} ; w = w + 1 ;
            end
            if ~isfield(G.topo,'alpha')
                Warn(w,1) = {'G.topo.alpha is undefined. See GOAT_GetTopography'} ; w = w + 1 ;
            end
            if ~isfield(G.topo,'Lakealpha')
                Warn(w,1) = {'G.topo.Lakealpha is undefined. See GOAT_GetTopography'} ; w = w + 1 ;
            end 
        end
    end
    % Flags
    Disp(d,1) = {'FLAGS: '} ; d = d + 1 ;
    try
        if G.Settings.ReverseColorMap
            Disp(d,1) = {'G.Settings.ReverseColorMap = 1   [invert colormap]'} ; d = d + 1 ;
        else
            Disp(d,1) = {'G.Settings.ReverseColorMap = 0 '} ; d = d + 1 ;
        end
    catch
        Error(e,1) = {'G.Settings.ReverseColorMap Flag (0/1) is not defined'} ; e  = e + 1 ;
    end
    try
        if G.Settings.LogLevelsFlag
            Disp(d,1) = {'G.Settings.LogLevelsFlag = 1   [use log levels for vertical dimension]'} ; d = d + 1 ;
        else
            Disp(d,1) = {'G.Settings.LogLevelsFlag = 0 '} ; d = d + 1 ;
        end
    catch
        Error(e,1) = {'G.Settings.LogLevelsFlag  (0/1) is not defined'} ; e  = e + 1 ;
    end
    try
        if G.Settings.SinLatFlag
            Disp(d,1) = {'G.Settings.SinLatFlag = 1   [plot sin(latitude)]'} ; d = d + 1 ;
        else
            Disp(d,1) = {'G.Settings.SinLatFlag = 0 '} ; d = d + 1 ;
        end
    catch
        Error(e,1) = {'G.Settings.SinLatFlag  (0/1) is not defined'} ; e  = e + 1 ;
    end
    try
        if G.Settings.FixZLimitsFlag
            Disp(d,1) = {'G.Settings.FixZLimitsFlag = 1   [fix colormap or vertical limits]'} ; d = d + 1 ;
        else
            Disp(d,1) = {'G.Settings.FixZLimitsFlag = 0 '} ; d = d + 1 ;
        end
    catch
        Error(e,1) = {'G.Settings.FixZLimitsFlag  (0/1) is not defined'} ; e  = e + 1 ;
    end
    try
        if G.Settings.SymZLimitsFlag
            Disp(d,1) = {'G.Settings.SymZLimitsFlag = 1   [symmetric Z limits with respect to Zero]'} ; d = d + 1 ;
        else
            Disp(d,1) = {'G.Settings.SymZLimitsFlag = 0 '} ; d = d + 1 ;
        end
    catch
        Error(e,1) = {'G.Settings.SymZLimitsFlag  (0/1) is not defined'} ; e  = e + 1 ;
    end
    try
        if G.Settings.XgridFlag
            Disp(d,1) = {'G.Settings.XgridFlag = 1   [show X-grid]'} ; d = d + 1 ;
        else
            Disp(d,1) = {'G.Settings.XgridFlag = 0 '} ; d = d + 1 ;
        end
    catch
        Error(e,1) = {'G.Settings.XgridFlag  (0/1) is not defined'} ; e  = e + 1 ;
    end
    try
        if G.Settings.YgridFlag
            Disp(d,1) = {'G.Settings.YgridFlag = 1   [show Y-grid]'} ; d = d + 1 ;
        else
            Disp(d,1) = {'G.Settings.YgridFlag = 0 '} ; d = d + 1 ;
        end
    catch
        Error(e,1) = {'G.Settings.YgridFlag  (0/1) is not defined'} ; e  = e + 1 ;
    end
    try
    if G.Settings.ContourLevelStep
        if  G.Settings.ContourLevelStep > (max(G.Data(~isnan(G.Data(:)))) - min(G.Data(~isnan(G.Data(:)))))/100 ;
            Disp(d,1) = {['G.Settings.ContourLevelStep = ',num2str(G.Settings.ContourLevelStep,5),'   [contour interval]']} ; d = d + 1 ;
        else
            Warn(w,1) = {'G.Settings.ContourLevelStep = Too Small   [Auto contour intervals will be used instead]'} ; w = w + 1 ;
        end
    else
        Disp(d,1) = {'G.Settings.ContourLevelStep = 0   [Auto contour intervals]'} ; d = d + 1 ;
    end
    catch
        Error(e,1) = {'G.Settings.ContourLevelStep  is not defined [contour interval]'} ; e  = e + 1 ;    
    end
    
    % Display messages
    if e > 2
        for i = 1:e-1
            err(Error{i,1}) ;
        end
    end
    if w > 2
        for i = 1:w-1
            war(Warn{i,1}) ;
        end
    end
    if d > 2
        for i = 1:d-1
            dis(Disp{i,1}) ;
        end
    end
    disp(endline) ;
    disp '  ' ;
    
catch
    if e > 2
        for i = 1:e-1
            err(Error{i,1}) ;
        end
    end
    if w > 2
        for i = 1:w-1
            war(Warn{i,1}) ;
        end
    end
    if d > 2
        for i = 1:d-1
            disp(Disp{i,1}) ;
        end
    end
    cprintf('Errors',['!!! ---','PLOT status report exited due to en error\n']) ;
    disp(endline) ;
    disp '  ' ;
end

function err(message)
cprintf('Errors' ,['|--->   ',message,'\n']) ;

function war(message)
cprintf('Blue' ,['|--->   ',message,'\n']) ;

function dis(message)
disp(['--- ',message]) ;





