% GOAT_Limits Set Spatial and Temporal Limits of the selected field
% Copyright (C) Ori Adam Feb.2012. compatible with version 2013i or later.
%
% see also GOAT_Compile_data_Series, GOAT_Compile_data_Climatology, GOAT_Compile_data

function G = GOAT_Limits(G)
pos        = G.GUI.pos.Limits ;
G.Limits_Fig = figure('name','GOAT: Determine Limits','position',pos,'KeyPressFcn',{@LIMITSKeyPress,G},'WindowKeyPressFcn',{@LIMITSKeyPress,G}) ;
G.continueFlag = 0 ;
NumLines = 17 ;
bl = 0.02              ; % Left Edge
bt = 0.05              ; % Top Edge
lh = (1-2*bt)/NumLines ; % Line Height
cl = (1-5*bl)/5        ; % column width (a total of 5 columns)
set(G.Limits_Fig,'Color',G.Display.FigBackgroudColor) ;
set(G.Limits_Fig,'MenuBar','none')
% Compile Top Message
switch G.ProjectionType
    case {1,2}
        ProjectionStr = 'Longitude-Latitude Projection' ;
    case 3
        ProjectionStr = 'Latitude-Pressure Projection' ;
    case 4
        ProjectionStr = 'Longitude-Pressure Projection' ;
    case 5
        G.Limits.Default.Day2same = 0 ;
        ProjectionStr = 'Longitude Hovmoller Diagram' ;
    case 6
        G.Limits.Default.Day2same = 0 ;
        ProjectionStr = 'Latitude Hovmoller Diagram' ;
    case 7
        G.Limits.Default.Day2same = 0 ;
        ProjectionStr = 'Pressure Hovmoller Diagram' ;
    case 8
        ProjectionStr = 'NH Polar Stereographic' ;
    case 9
        ProjectionStr = 'SH Polar Stereographic' ;
    case 10
        ProjectionStr = {'Install data as *.mat files. ';'Data will not be spatially truncated'} ;
end
messageStr = [ProjectionStr,G.message_Limits] ;

% Top Message
h_message = uicontrol('Style', 'text','Units','normalized','pos',[0,1-lh*.75,1,lh*.75], 'string',messageStr);
uicontrol(h_message) ;  set(h_message,'ForegroundColor',G.GUI.messageColor,'FontSize',13,'FontName',G.GUI.Fonts.MessageFont) ;
% Year Columm
Line = 1.3 ;
if ~strcmpi(G.TimeField,'list')
h_Text1 = uicontrol('Style', 'text','Units','normalized','pos',PosFunc(bl,bt,lh,cl,1,Line), 'string','Date 1');
uicontrol(h_Text1) ;  set(h_Text1,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',G.GUI.HeaderColor,'FontSize',G.Display.TextSize_M,'HorizontalAlignment','left','FontName',G.GUI.Fonts.uiHeaderFont) ;
end
Line = NumLines-2.5 ;
%h_mcode = uicontrol('Units','normalized','pos',PosFunc(bl,bt,lh,cl,5,Line), 'string','get m code');
h_mcode = uicontrol('Units','normalized','pos',[bl*4.5+cl*4,(1-bt-Line*lh),cl,lh*1], 'string','get m code');
uicontrol(h_mcode) ;  set(h_mcode,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',G.GUI.messageColor,'FontSize',G.Display.TextSize_M,'HorizontalAlignment','left','FontName',G.GUI.Fonts.uiHeaderFont, ...
    'fontweight','bold','TooltipString','Generate m code for selected data and Limits', 'callback','set(gcbf,''userdata'',77); uiresume;') ;
cObj = findjobj(h_mcode) ;
cObj.Border = [] ;

Line = 4.3 ;
if ~strcmpi(G.TimeField,'list')
h_Text1 = uicontrol('Style', 'text','Units','normalized','pos',PosFunc(bl,bt,lh,cl,1,Line), 'string','Date 2');
uicontrol(h_Text1) ;  set(h_Text1,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',G.GUI.HeaderColor,'FontSize',G.Display.TextSize_M,'HorizontalAlignment','left','FontName',G.GUI.Fonts.uiHeaderFont) ;
end
if ~strcmpi(G.TimeField,'list')
    
    Line = 2.2 ;
    h_year1T = uicontrol('style','text','Units','normalized','pos',PosFunc(bl,bt,lh,cl,1,Line), 'string','Year');
    uicontrol(h_year1T) ;  set(h_year1T,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'HorizontalAlignment','left','FontName',G.GUI.Fonts.uiHeaderFont) ;
    
    Line = 3.2 ;
    h_year1 = uicontrol('style','popup','Units','normalized','pos',PosFunc(bl,bt,lh,cl,1,Line),...
        'string',G.Limits.YearsStr,  'callback','set(gcbf,''userdata'',101); uiresume;');
    uicontrol(h_year1) ;  set(h_year1,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'HorizontalAlignment','left','FontName',G.GUI.Fonts.uiButtonFont) ;
    set(h_year1,'Value',G.Limits.Default.year1)
    
    Line = 5.2 ;
    h_year2T = uicontrol('style','text','Units','normalized','pos',PosFunc(bl,bt,lh,cl,1,Line), 'string','Year');
    uicontrol(h_year2T) ;  set(h_year2T,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'HorizontalAlignment','left','FontName',G.GUI.Fonts.uiHeaderFont) ;
    
    Line = 6.2 ;
    h_year2 = uicontrol('style','popup','Units','normalized','pos',PosFunc(bl,bt,lh,cl,1,Line),...
        'string',G.Limits.YearsStr,  'callback','set(gcbf,''userdata'',102); uiresume;');
    uicontrol(h_year2) ;  set(h_year2,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'HorizontalAlignment','left','FontName',G.GUI.Fonts.uiButtonFont) ;
    set(h_year2,'Value',G.Limits.Default.year2) ;
    if strcmpi(G.TimeField,'Monthly') || strncmpi(G.TimeFieldVirtual,'Daily',5)
    % 2nd Column: Months
    Line = 2.2 ;
    h_month1T = uicontrol('style','text','Units','normalized','pos',PosFunc(bl,bt,lh,cl,2,Line), 'string','Month');
    uicontrol(h_month1T) ;  set(h_month1T,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'HorizontalAlignment','left','FontName',G.GUI.Fonts.uiHeaderFont) ;
    Line = 3.2 ;
    h_month1 = uicontrol('style','popup','Units','normalized','pos',PosFunc(bl,bt,lh,cl,2,Line), ...
        'string','Jan | Feb | Mar | Apr | May | Jun | Jul | Aug | Sep | Oct | Nov | Dec ',  'callback','set(gcbf,''userdata'',201); uiresume;');
    uicontrol(h_month1) ;  set(h_month1,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'HorizontalAlignment','left','FontName',G.GUI.Fonts.uiButtonFont) ;
    set(h_month1,'Value',G.Limits.Default.month1) ;
    
    Line = 5.2 ;
    h_month2T = uicontrol('style','text','Units','normalized','pos',PosFunc(bl,bt,lh,cl,2,Line), 'string','Month');
    uicontrol(h_month2T) ;  set(h_month2T,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'HorizontalAlignment','left','FontName',G.GUI.Fonts.uiHeaderFont) ;
    Line = 6.2 ;
    h_month2 = uicontrol('style','popup','Units','normalized','pos',PosFunc(bl,bt,lh,cl,2,Line),...
        'string','Jan | Feb | Mar | Apr | May | Jun | Jul | Aug | Sep | Oct | Nov | Dec ',  'callback','set(gcbf,''userdata'',202); uiresume;');
    uicontrol(h_month2) ;  set(h_month2,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'HorizontalAlignment','left','FontName',G.GUI.Fonts.uiButtonFont) ;
    set(h_month2,'Value',G.Limits.Default.month2) ;
    % Day Column
    Daystr28 = '1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9 | 10 | 11 | 12 | 13 | 14 | 15 | 16 | 17 | 18 | 19 | 20 | 21 | 22 | 23 | 24 | 25 | 26 | 27 | 28  ' ;
    Daystr29 = '1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9 | 10 | 11 | 12 | 13 | 14 | 15 | 16 | 17 | 18 | 19 | 20 | 21 | 22 | 23 | 24 | 25 | 26 | 27 | 28 | 29  ' ;
    Daystr30 = '1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9 | 10 | 11 | 12 | 13 | 14 | 15 | 16 | 17 | 18 | 19 | 20 | 21 | 22 | 23 | 24 | 25 | 26 | 27 | 28 | 29 | 30  ' ;
    Daystr31 = '1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9 | 10 | 11 | 12 | 13 | 14 | 15 | 16 | 17 | 18 | 19 | 20 | 21 | 22 | 23 | 24 | 25 | 26 | 27 | 28 | 29 | 30 | 31 ' ;
    if strncmpi(G.TimeFieldVirtual,'Daily360',8)
    Daystr1 = Daystr30 ; Month1Days = 30 ;
    Daystr2 = Daystr30 ; Month2Days = 30 ;
    else
        switch G.Limits.Default.month1
        case {1,3,5,7,8,10,12}
            Daystr1 = Daystr31 ; Month1Days = 31 ;
        case {4,6,9,11}
            Daystr1 = Daystr30 ; Month1Days = 30 ;
        case 2
            if GOAT_leapyear(G.Limits.years(G.Limits.Default.year1))
                Daystr1 = Daystr29 ; Month1Days = 29 ;
            else
                Daystr1 = Daystr28 ; Month1Days = 28 ;
            end
    end
    switch G.Limits.Default.month2
        case {1,3,5,7,8,10,12}
            Daystr2 = Daystr31 ; Month2Days = 31 ;
        case {4,6,9,11}
            Daystr2 = Daystr30 ; Month2Days = 30 ;
        case 2
            if GOAT_leapyear(G.Limits.years(G.Limits.Default.year2))
                Daystr2 = Daystr29 ; Month2Days = 29 ;
            else
                Daystr2 = Daystr28 ; Month2Days = 28 ;
            end
    end
    end
    else
       if length(G.TimeFieldVirtual) == 6
        YYY = 365 ; 
        else
        Xidx = regexpi(G.TimeFieldVirtual,'X') ; 
        if isempty(Xidx)
        YYY = str2num(G.TimeFieldVirtual(7:end)) ; 
        else
        YYY = str2num(G.TimeFieldVirtual(7:Xidx-1)) ;     
        end
       end
       Daystr1 = [] ; 
       for i = 1:YYY
           Daystr1 = [Daystr1,int2str(i),' |'] ; 
       end
       Daystr1 = Daystr1(1:end-1) ;
       Month1Days = YYY ; 
       Daystr2 = Daystr1 ;
       Month2Days = YYY ; 
    end
    
    if ~strncmpi(G.TimeFieldVirtual,'Monthly',7)
        Line = 2.2 ;
        h_day1T = uicontrol('style','text','Units','normalized','pos',PosFunc(bl,bt,lh,cl,3,Line), 'string','Day');
        uicontrol(h_day1T) ;  set(h_day1T,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'HorizontalAlignment','left','FontName',G.GUI.Fonts.uiHeaderFont) ;
        Line = 3.2 ;
        h_day1 = uicontrol('style','popup','Units','normalized','pos',PosFunc(bl,bt,lh,cl,3,Line), ...
            'string',Daystr1,  'callback','set(gcbf,''userdata'',301); uiresume;');
        uicontrol(h_day1) ;  set(h_day1,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'HorizontalAlignment','left','FontName',G.GUI.Fonts.uiButtonFont) ;
        if G.Limits.Default.day1 > Month1Days
            G.Limits.Default.day1 = Month1Days ;
        end
        set(h_day1,'value',G.Limits.Default.day1)  ;
        Line = 5.2 ;
        h_day2T = uicontrol('style','text','Units','normalized','pos',PosFunc(bl,bt,lh,cl,3,Line), 'string','Day');
        uicontrol(h_day2T) ;  set(h_day2T,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'HorizontalAlignment','left','FontName',G.GUI.Fonts.uiHeaderFont) ;
        Line = 6.2 ;
        h_day2 = uicontrol('style','popup','Units','normalized','pos',PosFunc(bl,bt,lh,cl,3,Line),...
            'string',Daystr2,  'callback','set(gcbf,''userdata'',302); uiresume;');
        uicontrol(h_day2) ;  set(h_day2,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'HorizontalAlignment','left','FontName',G.GUI.Fonts.uiButtonFont) ;
        if G.Limits.Default.day2 > Month2Days
            G.Limits.Default.day2 = Month2Days ;
        end
        set(h_day2,'value',G.Limits.Default.day2)  ;
    else
        G.Limits.Default.day1 = 1 ;
        G.Limits.Default.day2 = 1 ;
    end
    
% HOURS
        Xidx = regexpi(G.TimeFieldVirtual,'X') ;
        if isempty(Xidx)
            X = 1 ;
        else
            X =   str2num(G.TimeFieldVirtual(Xidx+1:end)) ;
        end
    switch X
        case 2 ;
            HoursStr = '00:00 | 12:00' ;
        case 4
            HoursStr = '00:00 | 06:00 | 12:00 | 18:00 ' ;
        case 6
            HoursStr = '00:00 | 04:00 | 08:00 | 12:00 | 16:00 | 20:00 ' ;    
        case 8
            HoursStr = '00:00 | 03:00 | 06:00 | 09:00 | 12:00 | 15:00 | 18:00 | 21:00 ' ;
        case 12
            HoursStr = '00:00 | 02:00 | 04:00 | 06:00 | 08:00 | 10:00 | 12:00 | 14:00 | 16:00 | 18:00| 20:00 | 22:00 ' ;    
        case 24
            X = 24 ;
            HoursStr = '00:00 | 01:00 | 02:00 | 03:00 | 04:00 | 05:00 | 06:00 | 07:00 | 08:00 | 09:00| 10:00 | 11:00 | 12:00 | 13:00 | 14:00 | 15:00 | 16:00 | 17:00 | 18:00 | 19:00| 20:00 | 21:00 | 22:00 | 23:00 ' ;        
    end
    
    % Hours - Disabled
    if X>1
        % Hour Column
        Line = 2.2 ;
        h_H1T = uicontrol('style','text','Units','normalized','pos',PosFunc(bl,bt,lh,cl,4,Line), 'string','Hour');
        uicontrol(h_H1T) ;  set(h_H1T,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',[.2 .2 .2],'FontSize',12,'FontName',G.GUI.Fonts.uiHeaderFont) ;
        Line = 3.2 ;
        h_H1 = uicontrol('style','popup','Units','normalized','pos',PosFunc(bl,bt,lh,cl,4,Line), ...
            'string',HoursStr,  'callback','set(gcbf,''userdata'',401); uiresume;');
        uicontrol(h_H1) ;  set(h_H1,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',12,'FontName',G.GUI.Fonts.uiButtonFont) ;
        set(h_H1,'Value',G.Limits.Default.H1) ;
        
        Line = 5.2 ;
        h_H2T = uicontrol('style','text','Units','normalized','pos',PosFunc(bl,bt,lh,cl,4,Line), 'string','Hour');
        uicontrol(h_H2T) ;  set(h_H2T,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',[.2 .2 .2],'FontSize',12,'FontName',G.GUI.Fonts.uiHeaderFont) ;
        Line = 6.2 ;
        h_H2 = uicontrol('style','popup','Units','normalized','pos',PosFunc(bl,bt,lh,cl,4,Line),...
            'string',HoursStr,  'callback','set(gcbf,''userdata'',402); uiresume;');
        uicontrol(h_H2) ;  set(h_H2,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',12,'FontName',G.GUI.Fonts.uiButtonFont) ;
        set(h_H2,'Value',G.Limits.Default.H2) ;
    else
        G.Limits.Default.H1 = 1 ;
        G.Limits.Default.H2 = 1 ;
    end
else
    Line = 2.2 ;
    h_ListT = uicontrol('style','text','Units','normalized','pos',PosFunc(bl,bt,lh,cl,1,Line), 'string','List');
        uicontrol(h_ListT) ;  set(h_ListT,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_M,'HorizontalAlignment','left','FontName',G.GUI.Fonts.uiHeaderFont) ;
    h_ItemT = uicontrol('style','text','Units','normalized','pos',PosFunc(bl,bt,lh,cl,2,Line), 'string','Item');
        uicontrol(h_ItemT) ;  set(h_ItemT,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_M,'HorizontalAlignment','left','FontName',G.GUI.Fonts.uiHeaderFont) ;
    Line = 3.2 ;
    h_linears = uicontrol('style','popup','Units','normalized','pos',PosFunc(bl,bt,lh,cl,1,Line),...
        'string',G.LinearsStr,  'callback','set(gcbf,''userdata'',1110); uiresume;');
    uicontrol(h_linears) ;  set(h_linears,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'HorizontalAlignment','left','FontName',G.GUI.Fonts.uiButtonFont) ;
    set(h_linears,'Value',G.Addresses.LinearIdx) ; 
    
    h_time1 = uicontrol('style','popup','Units','normalized','pos',PosFunc(bl,bt,lh,cl,2,Line),...
        'string',G.LinearTimeStr,  'callback','set(gcbf,''userdata'',111); uiresume;');
    uicontrol(h_time1) ;  set(h_time1,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'HorizontalAlignment','left','FontName',G.GUI.Fonts.uiButtonFont) ;
    set(h_time1,'Value',G.Limits.Default.time1)
    
    Line = 6.2 ;
    h_time2 = uicontrol('style','popup','Units','normalized','pos',PosFunc(bl,bt,lh,cl,2,Line),...
        'string',G.LinearTimeStr,  'callback','set(gcbf,''userdata'',112); uiresume;');
    uicontrol(h_time2) ;  set(h_time2,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'HorizontalAlignment','left','FontName',G.GUI.Fonts.uiButtonFont) ;
    set(h_time2,'Value',G.Limits.Default.time2) ;
    
end

if ~any(G.ProjectionType == [5,6,7])
    % day2 same as Day1
    %     Line = 5.2 ;
    %     h_Day2sameT = uicontrol('style','text','Units','normalized','pos',PosFunc(bl,bt,lh,cl,5,Line), 'string','Same As Date 1');
    %     uicontrol(h_Day2sameT) ;  set(h_Day2sameT,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'HorizontalAlignment','left','FontName',G.GUI.Fonts.uiHeaderFont) ;
    if ~strcmpi(G.TimeField,'list')
    SameStr = 'Same as Date 1' ; 
    else
    SameStr = 'Same as Item 1' ;     
    end
    Line = 6.2 ;
    h_Day2same = uicontrol('style','checkbox','Units','normalized','pos',PosFunc(bl,bt,lh,cl,5,Line),'string',SameStr, ... [bl*5 + cl*(5-1)+cl/3,1-bt-Line*lh,cl,lh],...
        'Value',G.Limits.Default.Day2same,  'callback','set(gcbf,''userdata'',501); uiresume;');
    uicontrol(h_Day2same) ;  set(h_Day2same,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_M,'HorizontalAlignment','left','TooltipString','Make Date 2 = Date 1. keypress = ''d''') ;
end
if ~strcmpi(G.TimeField,'list')
    Line = 7.5 ;
    h_Sequential = uicontrol('style','checkbox','Units','normalized','pos',PosFunc(bl,bt,lh,cl,1,Line),'string','Sequential', ...
        'Value',G.Limits.Default.Sequential,  'callback','set(gcbf,''userdata'',601); uiresume;');
    uicontrol(h_Sequential) ;  set(h_Sequential,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_M,'HorizontalAlignment','left', ...
        'TooltipString','Average all data between chosen time limits','FontName',G.GUI.Fonts.uiHeaderFont) ;
    % Line = 7.5 ;
    % h_SequentialT = uicontrol('style','text','Units','normalized','pos', [[bl+cl/2,1-bt-Line*lh,cl/2+bl,lh]]); %,PosFunc(bl,bt,lh,cl,2,Line));
    % uicontrol(h_SequentialT) ;  set(h_SequentialT,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'HorizontalAlignment','left' , ...
    %     'TooltipString','Average all data between chosen time limits','FontName',G.GUI.Fonts.uiHeaderFont, 'string','Sequential') ;
    
    Line = 8.5 ;
    h_Seasonal = uicontrol('style','checkbox','Units','normalized','pos',PosFunc(bl,bt,lh,cl,1,Line),'string','Climatology',...
        'Value',abs(G.Limits.Default.Sequential-1),  'callback','set(gcbf,''userdata'',701); uiresume;','TooltipString','Average seasonal data between chosen time limits','FontName',G.GUI.Fonts.uiHeaderFont);
    uicontrol(h_Seasonal) ;  set(h_Seasonal,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_M,'HorizontalAlignment','left') ;
    % Line = 8.5 ;
    % h_SeasonalT = uicontrol('style','text','Units','normalized','pos',PosFunc(bl,bt,lh,cl,2,Line), 'string','Climatology');
    % uicontrol(h_SeasonalT) ;  set(h_SeasonalT,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'HorizontalAlignment','left' , ...
    %     'TooltipString','Average seasonal data between chosen time limits','FontName',G.GUI.Fonts.uiHeaderFont) ;
    if any(G.ProjectionType == [5,6,7])
        Line = 7.5 ;
        h_SeasonalAnom = uicontrol('style','checkbox','Units','normalized','pos',PosFunc(bl,bt,lh,cl,2,Line),'string','Anomaly',...
            'Value',G.Limits.Default.Anom,  'callback','set(gcbf,''userdata'',703); uiresume;','TooltipString','Show Anomaly from climatology','FontName',G.GUI.Fonts.uiHeaderFont);
        uicontrol(h_SeasonalAnom) ;  set(h_SeasonalAnom,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_M,'HorizontalAlignment','left') ;
        Line = 8.5 ;
        h_SeasonalCont = uicontrol('style','checkbox','Units','normalized','pos',PosFunc(bl,bt,lh,cl,2,Line),'string','Clim. Continuous',...
            'Value',G.Limits.Default.ClimatologyContinuous,  'callback','set(gcbf,''userdata'',702); uiresume;');
        uicontrol(h_SeasonalCont) ;  set(h_SeasonalCont,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_M,'HorizontalAlignment','left', ...
            'TooltipString','Present Seasonal data of chosen time limits in a continous manner','FontName',G.GUI.Fonts.uiHeaderFont) ;
        %     Line = 8.5 ;
        %     h_SeasonalContT = uicontrol('style','text','Units','normalized','pos',PosFunc(bl,bt,lh,cl,4,Line), 'string','Clim. Continuous');
        %     uicontrol(h_SeasonalContT) ;  set(h_SeasonalContT,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'HorizontalAlignment','left' , ...
        %         'TooltipString','Present Seasonal data of chosen time limits in a continous manner','FontName',G.GUI.Fonts.uiHeaderFont) ;
    end
else % Linear Time
    G.Limits.Default.Sequential = 1 ;
end

Line = 9.75 ;
h_Spatial1T = uicontrol('style','text','Units','normalized','pos',PosFunc(bl,bt,lh,cl,1,Line), 'string','Spatial 1');
uicontrol(h_Spatial1T) ;  set(h_Spatial1T,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_M,'HorizontalAlignment','left' , ...
    'TooltipString','Set Spatial Limits','FontName',G.GUI.Fonts.uiHeaderFont) ;

Line = 13 ;
h_Spatial2T = uicontrol('style','text','Units','normalized','pos',PosFunc(bl,bt,lh,cl,1,Line), 'string','Spatial 2');
uicontrol(h_Spatial2T) ;  set(h_Spatial2T,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_M,'HorizontalAlignment','left' , ...
    'TooltipString','Set Spatial Limits','FontName',G.GUI.Fonts.uiHeaderFont) ;

switch G.Settings.LevelsIdx
    case 1
        LevelStr = ['Pressure (N=',int2str(length(G.P)),')'] ;
    case 2
        LevelStr = ['Depth (N=',int2str(length(G.P)),')'] ;
    case 3
        LevelStr = ['Height (N=',int2str(length(G.P)),')'] ;
    case 4
        LevelStr = ['Pot. Temp. (N=',int2str(length(G.P)),')'] ;
    case 5
        LevelStr = ['Sigma (N=',int2str(length(G.P)),')'] ;
    case 6
        LevelStr = ['Geometric-Level (N=',int2str(length(G.P)),')'] ;    
    case 7
        LevelStr = ['Level (N=',int2str(length(G.P)),')'] ;        
end
Line = 11 ;
h_PressureT = uicontrol('style','text','Units','normalized','pos',PosFunc(bl,bt,lh,cl,1,Line), 'string',LevelStr);
uicontrol(h_PressureT) ;  set(h_PressureT,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'HorizontalAlignment','left' , ...
    'TooltipString','Choose Pressure/Depth Levels','FontName',G.GUI.Fonts.uiHeaderFont) ;

Line = 11 ;
h_LongT = uicontrol('style','text','Units','normalized','pos',PosFunc(bl,bt,lh,cl,2,Line), 'string',['Longitude (N=',int2str(length(G.lon)),')']);
uicontrol(h_LongT) ;  set(h_LongT,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'HorizontalAlignment','left' , ...
    'TooltipString','Choose Longitude Limits','FontName',G.GUI.Fonts.uiHeaderFont) ;

Line = 11 ;
h_LatT = uicontrol('style','text','Units','normalized','pos',PosFunc(bl,bt,lh,cl,3,Line), 'string',['Latitude (N=',int2str(length(G.lat)),')']);
uicontrol(h_LatT) ;  set(h_LatT,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'HorizontalAlignment','left' , ...
    'TooltipString','Choose Longitude Limits','FontName',G.GUI.Fonts.uiHeaderFont) ;
% Pressure 1
Line = 12 ;
h_P1 = uicontrol('style','popup','Units','normalized','pos',PosFunc(bl,bt,lh,cl,1,Line), ...
    'string',G.Limits.Levels,  'callback','set(gcbf,''userdata'',801); uiresume;');
uicontrol(h_P1) ;  set(h_P1,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'HorizontalAlignment','left','FontName',G.GUI.Fonts.uiButtonFont) ;
set(h_P1,'Value',G.Limits.Default.Lev1) ;
% Pressure 2
Line = 14.5 ;
h_P2 = uicontrol('style','popup','Units','normalized','pos',PosFunc(bl,bt,lh,cl,1,Line), ...
    'string',G.Limits.Levels,  'callback','set(gcbf,''userdata'',802); uiresume;');
uicontrol(h_P2) ;  set(h_P2,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'HorizontalAlignment','left','FontName',G.GUI.Fonts.uiButtonFont) ;
set(h_P2,'Value',G.Limits.Default.Lev2) ;

% Longitude 1
Line = 12 ;
h_Lon1 = uicontrol('style','popup','Units','normalized','pos',PosFunc(bl,bt,lh,cl,2,Line), ...
    'string',G.Limits.Longitudes,  'callback','set(gcbf,''userdata'',901); uiresume;');
uicontrol(h_Lon1) ;  set(h_Lon1,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'HorizontalAlignment','left','FontName',G.GUI.Fonts.uiButtonFont) ;
set(h_Lon1,'Value',G.Limits.Default.Lon1) ;
% Longitude 2
Line = 14.5 ;
h_Lon2 = uicontrol('style','popup','Units','normalized','pos',PosFunc(bl,bt,lh,cl,2,Line), ...
    'string',G.Limits.Longitudes,  'callback','set(gcbf,''userdata'',902); uiresume;');
uicontrol(h_Lon2) ;  set(h_Lon2,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'HorizontalAlignment','left','FontName',G.GUI.Fonts.uiButtonFont) ;
set(h_Lon2,'Value',G.Limits.Default.Lon2) ;


% Latitude 1
Line = 12 ;
h_Lat1 = uicontrol('style','popup','Units','normalized','pos',PosFunc(bl,bt,lh,cl,3,Line), ...
    'string',G.Limits.Latitudes,  'callback','set(gcbf,''userdata'',1001); uiresume;');
uicontrol(h_Lat1) ;  set(h_Lat1,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'HorizontalAlignment','left','FontName',G.GUI.Fonts.uiButtonFont) ;
set(h_Lat1,'Value',G.Limits.Default.Lat1) ;
% Latitude 2
Line = 14.5 ;
h_Lat2 = uicontrol('style','popup','Units','normalized','pos',PosFunc(bl,bt,lh,cl,3,Line), ...
    'string',G.Limits.Latitudes,  'callback','set(gcbf,''userdata'',1002); uiresume;');
uicontrol(h_Lat2) ;  set(h_Lat2,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'HorizontalAlignment','left','FontName',G.GUI.Fonts.uiButtonFont) ;
set(h_Lat2,'Value',G.Limits.Default.Lat2) ;


% Same pressure
switch G.ProjectionType
    case {1,2,8,9}
        Pstr = 'Level 1 Only' ;
        QtPstr = 'Use only selected Level 1. kepress = ''p''' ;
    case {3,4,7}
        Pstr = 'Composite levels' ;
        QtPstr = 'Composite levels According to ''Mean'' or ''Integral'' flags. keypress = ''p''' ;
    case 10
        Pstr = 'Composite levels' ;
        QtPstr = 'Composite levels According to ''Mean'' or ''Integral'' flags. keypress = ''p''' ;    
end
if ~any(G.ProjectionType == [5,6,10])
    Line = 12 ;
    h_sameP = uicontrol('style','checkbox','Units','normalized','pos',[bl*4 + cl*(4-1),1-bt-Line*lh,cl,lh],'string',Pstr,'TooltipString',QtPstr,...
        'Value',G.Limits.Default.sameP,  'callback','set(gcbf,''userdata'',1101); uiresume;');
    uicontrol(h_sameP) ;  set(h_sameP,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_M,'HorizontalAlignment','left','FontName',G.GUI.Fonts.uiButtonFont,'TooltipString',QtPstr) ;
end

Line = 11 ;
Tip = sprintf(['Type of vertical composite', ...
    '\nNote:', ...
    '\nfor Pressure (mb) and Sigma vertical ordinates integral is mass weighted by adding a factor of 1/g', ...
    '\nfor Depth (m) vertical ordinate integral is mass weighted by adding a factor of Water Density', ...
    '\nfor Height (m) and Potential Temperature (K) vertical ordinates the integral is not mass weighted']) ;
h_VerMeanIntT = uicontrol('style','text','Units','normalized','pos',PosFunc(bl,bt,lh,cl,5,Line), 'string','Ver. Composite');
uicontrol(h_VerMeanIntT) ;  set(h_VerMeanIntT,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'HorizontalAlignment','left' , ...
    'TooltipString',Tip,'FontName',G.GUI.Fonts.uiHeaderFont) ;
Line = 12 ;
switch G.VerticalOrdinate
    case {'Pressure (mb)','Depth (m)'}
        VerIntStr = 'Vertical Mean | Mass weighted Vertical Integral Boer | Mass Weighted Vertical Integral Nan ' ;
    otherwise
        VerIntStr = 'Vertical Mean | Vertical Integral Boer | Vertical Integral Nan ' ;
end
h_VerMeanInt = uicontrol('style','popup','Units','normalized','pos',PosFunc(bl,bt,lh,cl,5,Line),...
    'string',VerIntStr,  'callback','set(gcbf,''userdata'',1201); uiresume;','value',G.Limits.Default.VerMeanInt,'TooltipString',Tip);
uicontrol(h_VerMeanInt) ;  set(h_VerMeanInt,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'HorizontalAlignment','left','FontName',G.GUI.Fonts.uiButtonFont) ;


Line = NumLines-.5 ;
h_cancel = uicontrol('Units','normalized','pos',[bl*4.5+cl*4,(1-bt-Line*lh),cl,lh*1.5], 'string','CANCEL',  'callback','set(gcbf,''userdata'',666); uiresume;');
uicontrol(h_cancel) ;  set(h_cancel,'BackgroundColor',[.85 .85 .85],'ForegroundColor',G.GUI.DropTextColor,'FontSize',G.Display.TextSize_L,'fontweight','bold','FontName',G.GUI.Fonts.uiHeaderFont,'TooltipString','keypress = ''c''') ;

Line = NumLines-.5 ;
h_OK = uicontrol('Units','normalized','pos',[bl/2,1-bt-Line*lh,cl*4 + bl*3.5,lh*1.5], 'string','OK',  'callback','set(gcbf,''userdata'',999); uiresume;');
uicontrol(h_OK) ;  set(h_OK,'BackgroundColor',G.GUI.colors.celeste,'ForegroundColor',G.GUI.DropTextColor,'FontSize',G.Display.TextSize_L,'fontweight','bold','FontName',G.GUI.Fonts.uiHeaderFont,'TooltipString','keypress = ''g''') ;

if G.Settings.KeyPressFlag
    if ~any(G.ProjectionType == [5,6,7,10])
        set(G.Limits_Fig,'KeyPressFcn',{@LIMITSKeyPress,G,h_sameP,h_Day2same},'WindowKeyPressFcn',{@LIMITSKeyPress,G,h_sameP,h_Day2same} )
    elseif any(G.ProjectionType == [7])
        set(G.Limits_Fig,'KeyPressFcn',{@LIMITSKeyPress,G,h_sameP},'WindowKeyPressFcn',{@LIMITSKeyPress,G,h_sameP} )
    elseif any(G.ProjectionType == [5,6,10])
        set(G.Limits_Fig,'KeyPressFcn',{@LIMITSKeyPress,G},'WindowKeyPressFcn',{@LIMITSKeyPress,G} )
    end
end

GUIcon = 1 ;
while GUIcon
    if G.AutoPilot.Flag   && G.AutoPilot.ActionNumber <= G.AutoPilot.NumberOfActions
        Actions = G.AutoPilot.Actions(~strcmpi(G.AutoPilot.Actions(:,G.AutoPilot.ActionNumber),'None'),G.AutoPilot.ActionNumber) ;
        for j = 1: length(Actions)
            disp(['Limits Auto Code ',int2str(G.AutoPilot.ActionNumber),': ',cell2mat(Actions(j))]) ;
            eval(cell2mat(Actions(j))) ;
        end
    else
        try
            set(0,'CurrentFigure',G.Limits_Fig) ;
            uiwait(gcf);
            select = get(gcf,'userdata');
        catch
            GUIcon = 0 ;
        end
        try
            switch select
                case 77
                    GOAT2Code_Limits(G) ;
                    figure(G.GOAT_Fig) ;
                    %set(0,'CurrentFigure',G.Limits_Fig) ;
                    figure(G.Limits_Fig) ;
                    if G.GUI.Flags.CloseAfterGetmcode
                        close(G.GOAT_Fig) ;
                        G.continueFlag = 1 ;
                        GUIcon = 0 ;
                    end
                case 101 % year1
                    G.Limits.Default.year1 = get(h_year1,'Value') ;
                    if ~strncmpi(G.TimeFieldVirtual,'Daily360',8)
                    switch G.Limits.Default.month1
                        case {1,3,5,7,8,10,12}
                            Daystr1 = Daystr31 ; Month1Days = 31 ;
                        case {4,6,9,11}
                            Daystr1 = Daystr30 ; Month1Days = 30 ;
                        case 2
                            if GOAT_leapyear(G.Limits.years(G.Limits.Default.year1),G.TimeFieldVirtual)
                                Daystr1 = Daystr29 ; Month1Days = 29 ;
                            else
                                Daystr1 = Daystr28 ; Month1Days = 28 ;
                            end
                    end
                    G.Limits.Default.day1 = min([G.Limits.Default.day1, Month1Days]) ;
                    try
                    set(h_day1,'string',Daystr1,'value',G.Limits.Default.day1)  ;
                    end
                    end
                case 102 % year2
                    G.Limits.Default.year2 = get(h_year2,'Value') ;
                    if ~strncmpi(G.TimeFieldVirtual,'Daily360',8)
                    switch G.Limits.Default.month2
                        case {1,3,5,7,8,10,12}
                            Daystr2 = Daystr31 ; Month2Days = 31 ;
                        case {4,6,9,11}
                            Daystr2 = Daystr30 ; Month2Days = 30 ;
                        case 2
                            if GOAT_leapyear(G.Limits.years(G.Limits.Default.year2),G.TimeFieldVirtual)
                                Daystr2 = Daystr29 ; Month2Days = 29 ;
                            else
                                Daystr2 = Daystr28 ; Month2Days = 28 ;
                            end
                    end
                    G.Limits.Default.day2 = min([G.Limits.Default.day2, Month2Days]) ;
                    try
                    set(h_day2,'string',Daystr2,'value',G.Limits.Default.day2)  ;
                    end
                    end
                case 1110
                    G.Addresses.LinearIdx = get(h_linears,'Value') ;
                    G = GOAT_List_Initialize(G) ; 
                    set(h_time1,'string',G.LinearTimeStr,'Value',G.Limits.Default.time1) ; 
                    set(h_time2,'string',G.LinearTimeStr,'Value',G.Limits.Default.time2) ; 
                case 111
                    G.Limits.Default.time1 = get(h_time1,'Value') ;
                case 112
                    G.Limits.Default.time2 = get(h_time2,'Value') ;
                case 201 % Month1
                    G.Limits.Default.month1 = get(h_month1,'Value') ;
                    if ~strncmpi(G.TimeFieldVirtual,'Daily360',8)
                    switch G.Limits.Default.month1
                        case {1,3,5,7,8,10,12}
                            Daystr1 = Daystr31 ; Month1Days = 31 ;
                        case {4,6,9,11}
                            Daystr1 = Daystr30 ; Month1Days = 30 ;
                        case 2
                            if GOAT_leapyear(G.Limits.years(G.Limits.Default.year1),G.TimeFieldVirtual)
                                Daystr1 = Daystr29 ; Month1Days = 29 ;
                            else
                                Daystr1 = Daystr28 ; Month1Days = 28 ;
                            end
                    end
                    G.Limits.Default.day1 = min([G.Limits.Default.day1, Month1Days]) ;
                    end
                    try
                    set(h_day1,'string',Daystr1,'value',G.Limits.Default.day1) ;
                    end
                case 202 % Month2
                    G.Limits.Default.month2 = get(h_month2,'Value') ;
                    if ~strncmpi(G.TimeFieldVirtual,'Daily360',8)
                    switch G.Limits.Default.month2
                        case {1,3,5,7,8,10,12}
                            Daystr2 = Daystr31 ; Month2Days = 31 ;
                        case {4,6,9,11}
                            Daystr2 = Daystr30 ; Month2Days = 30 ;
                        case 2
                            if GOAT_leapyear(G.Limits.years(G.Limits.Default.year2),G.TimeFieldVirtual)
                                Daystr2 = Daystr29 ; Month2Days = 29 ;
                            else
                                Daystr2 = Daystr28 ; Month2Days = 28 ;
                            end
                    end
                    G.Limits.Default.day2 = min([G.Limits.Default.day2, Month2Days]) ;
                    end
                    try
                    set(h_day2,'string',Daystr2,'value',G.Limits.Default.day2) ;
                    end
                case 301 % Day1
                    G.Limits.Default.day1 = get(h_day1,'Value') ;
                case 302 % Day2
                    G.Limits.Default.day2 = get(h_day2,'Value') ;
                case 401 % Hour1
                    G.Limits.Default.H1 = get(h_H1,'Value') ;
                case 402 % Hour2
                    G.Limits.Default.H2 = get(h_H2,'Value') ;
                case 501 % Same as Date 1
                    G.Limits.Default.Day2same = get(h_Day2same,'Value') ;
                case 601 % Sequential
                    G.Limits.Default.Sequential = get(h_Sequential,'Value') ;
                    set(h_Seasonal,'Value',abs(G.Limits.Default.Sequential-1)) ;
                    if G.Limits.Default.Sequential
                        G.Limits.Default.ClimatologyContinuous = 0 ;
                        set(h_SeasonalCont,'Value',G.Limits.Default.ClimatologyContinuous) ;
                    end
                case 701 % Seasonal
                    G.Limits.Default.Sequential = abs(get(h_Seasonal,'Value')-1) ;
                    set(h_Sequential,'Value',G.Limits.Default.Sequential) ;
                    if G.Limits.Default.Sequential
                        G.Limits.Default.ClimatologyContinuous = 0 ;
                        set(h_SeasonalCont,'Value',G.Limits.Default.ClimatologyContinuous) ;
                    end
                case 702 % Seasonal Continuous
                    if  ~G.Limits.Default.Sequential
                        G.Limits.Default.ClimatologyContinuous = get(h_SeasonalCont,'Value') ;
                    else
                        G.Limits.Default.ClimatologyContinuous = 0 ;
                        set(h_SeasonalCont,'Value',G.Limits.Default.ClimatologyContinuous) ;
                    end
                case 703
                    G.Limits.Default.Anom = get(h_SeasonalAnom,'Value') ;
                    if ~G.Limits.Default.Sequential && ~G.Limits.Default.ClimatologyContinuous
                        G.Limits.Default.Anom = 0 ;
                        set(h_SeasonalAnom,'Value',G.Limits.Default.Anom) ; drawnow ;
                    end
                case 801 % pressure1
                    G.Limits.Default.Lev1 = get(h_P1,'Value') ;
                case 802 % pressure2
                    G.Limits.Default.Lev2 = get(h_P2,'Value') ;
                case 901 % long1
                    G.Limits.Default.Lon1 = get(h_Lon1,'Value') ;
                case 902 % long2
                    G.Limits.Default.Lon2 = get(h_Lon2,'Value') ;
                case 1001 % lat1
                    G.Limits.Default.Lat1 = get(h_Lat1,'Value') ;
                case 1002 % lat2
                    G.Limits.Default.Lat2 = get(h_Lat2,'Value') ;
                case 1101 % same Pressure
                    G.Limits.Default.sameP = get(h_sameP,'Value') ;
                case 1201 % same Pressure
                    G.Limits.Default.VerMeanInt = get(h_VerMeanInt,'Value') ;
                case 666 % CANCEL
                    G.continueFlag = 1 ;
                    GUIcon = 0 ;
                case 999 % OK
                    GUIcon = 0 ;
                otherwise
                    % do nothing
            end
        catch
            %nevermind
        end
    end
end
if ~any(G.ProjectionType == [5,6,10])
    G.Limits.Default.sameP    = get(h_sameP   ,'Value') ; % re-read value due to keypress
end
if ~any(G.ProjectionType == [5,6,7])
    G.Limits.Default.Day2same = get(h_Day2same,'Value') ; % re-read value due to keypress
end

if ~strcmpi(G.TimeField,'list')
    
    G.Limits.S1 = GOAT_getSerial(G.Limits.years(G.Limits.Default.year1),G.Limits.Default.month1,G.Limits.Default.day1,round((G.Limits.Default.H1-1)*24/X),G.TimeFieldVirtual,G.Limits.Year0) ;
    if G.Limits.Default.Day2same && G.Limits.Default.Sequential
        G.Limits.S2 = G.Limits.S1 ;
    else
        G.Limits.S2 = GOAT_getSerial(G.Limits.years(G.Limits.Default.year2),G.Limits.Default.month2,G.Limits.Default.day2,round((G.Limits.Default.H2-1)*24/X),G.TimeFieldVirtual,G.Limits.Year0) ;
    end
    
    % Warnings and Errors
    if ~G.continueFlag
        ErrorFlag = 0 ;
        if     (~G.Limits.Default.Sequential || any(G.ProjectionType == [5,6,7]))  && ...
                ~G.Limits.Default.Day2same && ...
                (G.Limits.Default.year1*1000000 + G.Limits.Default.month1*10000 + G.Limits.Default.day1*100 + (G.Limits.Default.H1-1)*24/X ) > ...
                (G.Limits.Default.year2*1000000 + G.Limits.Default.month2*10000 + G.Limits.Default.day2*100 + (G.Limits.Default.H2-1)*24/X )
            ErrorFlag = 1 ;
            ErrorMessage = 'Date 2 must be later than Date 1 For seasonal Averaging or Hovmoller Diagrams !' ;
        end
        
        if G.Filter.RAIdx > 1 || sum([G.Filter.LowPassIdx,G.Filter.HighPassIdx]) > 0
            Buf = max([ceil(G.Filter.RAIdx),G.Filter.LowPassIdx*2,G.Filter.HighPassIdx*2]) ;
            if G.Limits.S1 - Buf < G.Limits.Smin
                ErrorFlag = 1 ;
                ErrorMessage = 'Date 1 is too close to beginning of Data series for filter to be applied.' ;
            elseif G.Limits.S2 + Buf > G.Limits.Smax
                ErrorFlag = 1 ;
                ErrorMessage = 'Date 2 is too close to end of Data series for filter to be applied.' ;
            end
        end
        
        if ErrorFlag
            set(h_message,'string',ErrorMessage,'ForegroundColor',[.5 .1 .1]) ; % Update Message
            %soundsc(G.ErrorSound,3000)
            soundsc(G.ErrorSound,G.ErrorSoundFs) ;
            pause(3) ;
            try
                close(G.Limits_Fig) ;
            catch
                % Nevermind
            end
            G = GOAT_Limits(G) ; % Retry
            return ;
        end
        
    end
    
else % Linear Time
    G.Limits.S1 = G.Limits.Default.time1 ;
    if G.Limits.Default.Day2same
        G.Limits.S2 = G.Limits.Default.time1 ;
    else
        G.Limits.S2 = G.Limits.Default.time2 ;
    end
    
end
try
    close(G.Limits_Fig) ;
catch
    % Nevermind
end

return


% KeyPressFunction
function LIMITSKeyPress(~,event,G,h_sameP,h_Day2same)
try
    switch event.Character
        case 'g'
            set(gcf,'userdata',999) ;
            uiresume ;
            return ;
        case 'p'
            G.Limits.Default.sameP = abs(get(h_sameP,'Value')-1) ;
            set(h_sameP,'Value',G.Limits.Default.sameP) ;
            set(gcf,'userdata',0) ;
            uiresume ;
            return ;
        case 'd'
            G.Limits.Default.Day2same = abs(get(h_Day2same,'Value')-1) ;
            set(h_Day2same,'Value',G.Limits.Default.Day2same) ;
            set(gcf,'userdata',0) ;
            uiresume ;
            return ;
        case 'c'
            set(gcf,'userdata',666) ;
            uiresume ;
            return ;
            
    end
    
catch
    % Ignore
end

