% GOAT_List_Initialize Initialize a selected linear folder (linked or mat)
% Copyright (c) Ori Adam. March.24.2014
%
% see also, GOAT_GetFieldSpan, GOAT_Limits

function G = GOAT_List_Initialize(G)

if isempty(G.Addresses.LinearAtributes{1,G.Addresses.LinearIdx})
    % *.mat
    G.Addresses.Default = G.Addresses.Linear(~cellfun(@isempty,G.Addresses.Linear(:,G.Addresses.LinearIdx)),G.Addresses.LinearIdx) ;
    try
        Names = cell(length(G.Addresses.Default),1) ;
        for n = 1:length(G.Addresses.Default)
            [dummypath, Names{n}] = fileparts(G.Addresses.Default{n})  ;
        end
        nums = str2num(char(Names)) ;
        if ~isempty(nums)
            [newStr, Is] = sort(nums) ;
            G.Addresses.Default = G.Addresses.Default(Is) ;
        end
    end
else
    % Linked
    G.Addresses.Attributes = G.Addresses.LinearAtributes{1,G.Addresses.LinearIdx} ;
    G.Addresses.Default = G.Addresses.Attributes(1:end-1,1) ;
    for n = 1:length(G.Addresses.Default)
        if size(G.Addresses.Attributes,2) > 9
         if ~isempty(G.Addresses.Attributes{n,10})
             Header = G.Addresses.Attributes{n,10} ; 
             if isnumeric(Header)
                 Header = num2str(Header) ; 
             end
             [Path,Name,Ext] = fileparts(Header) ;
            if isempty(Ext)
             Ext = ['.',G.Addresses.Attributes{end,1}] ; 
            end
            Header = fullfile(G.Addresses.Linears{G.Addresses.LinearIdx},[Name,Ext]) ; 
         else
            [Path,Name,Ext] = fileparts(G.Addresses.Attributes{n,1}) ;
            if isempty(Ext)
             Ext = ['.',G.Addresses.Attributes{end,1}] ; 
            end
            Header = fullfile(G.Addresses.Linears{G.Addresses.LinearIdx},[Name,Ext]) ; 
         end
        else    
        [Path,Name,Ext] = fileparts(G.Addresses.Attributes{n,1}) ;
        if isempty(Ext)
            Ext = ['.',G.Addresses.Attributes{end,1}] ; 
        end
        Header = fullfile(G.Addresses.Linears{G.Addresses.LinearIdx},[Name,Ext]) ; 
        end
        G.Addresses.Default(n) = {Header} ;
    end
    % save attributes for speedup in later operations
    G.ALP.Attributes.(G.DataFieldReference) = G.Addresses.Attributes ; 
    G.ALP.Addresses.(G.DataFieldReference)  = G.Addresses.Default    ; 
    G.ALP.Smin.(G.DataFieldReference)       = 1                      ; 
    G.ALP.Smax.(G.DataFieldReference)       = length(G.Addresses.Default) ;
    G.ALP.Defs.(G.DataFieldReference)       = G.Addresses.Attributes{end,3} ;
end
try
    clear infostr
    %G.Limits.LinearInfo = '' ;
    if exist(fullfile(G.DataFolder,G.TimeField,G.DataFieldReference,G.Addresses.Linears{G.Addresses.LinearIdx},'info.mat'),'file')
        load(fullfile(G.DataFolder,G.TimeField,G.DataFieldReference,G.Addresses.Linears{G.Addresses.LinearIdx},'info.mat')) ;
        G.Limits.LinearInfo = infostr ;
    end
    if exist(fullfile(G.DataFolder,G.TimeField,G.DataFieldReference,G.Addresses.Linears{G.Addresses.LinearIdx},'Topography.mat'),'file')
        G = GOAT_GetTopography(G,fullfile(G.DataFolder,G.TimeField,G.DataFieldReference,G.Addresses.Linears{G.Addresses.LinearIdx},'Topography.mat')) ;
    end
end
if ~G.EnvironmentFlag
    try
        set(G.h_info,'TooltipString',G.Limits.LinearInfo) ;
        set(G.h_TempRes,'TooltipString',G.Addresses.Linears{G.Addresses.LinearIdx}) ;
        drawnow ;
    catch
        % Nevermind. No string, or in Environment mode
    end
end
G.LinearTimeStr = '' ; % Cell array of available Time Resolution Folders
%Lidx = length(G.Addresses.Linears{G.Addresses.LinearIdx}) + 2 ;

for j = 1:length(G.Addresses.Default)
    filename = cell2mat(G.Addresses.Default(j)) ;
    [Path,Name] = fileparts(filename) ;
    %G.LinearTimeStr = [G.LinearTimeStr,filename(Lidx:end-ExtLength),' |'] ;
    G.LinearTimeStr = [G.LinearTimeStr,Name,' |'] ;
end
G.LinearTimeStr = G.LinearTimeStr(1:end-1) ;
G.Limits.Smin    = 1                           ;
G.Limits.Smax    = length(G.Addresses.Default) ;
G.Limits.SminOld = 1                           ;
G.Limits.SmaxOld = length(G.Addresses.Default) ;
G.Limits.Default.time1 = 1                           ;
G.Limits.Default.time2 = length(G.Addresses.Default) ;
G = GOAT_Filters_Clear(G) ; 
G.Filter.Index = ones(length(G.Addresses.Default),1) ;
if ~G.EnvironmentFlag
    try % Only if in GUI mode, add option to install linked data
        G = GOAT_setProjectionStr(G) ;
        set(G.h_Projection,'string',G.GUI.ProjectionStr) ;
    catch
        % nevermind.
    end
end

