% LOAD load data and perform spatial truncation according to selected settings
% Copyright (C) Ori Adam Feb.2012. compatible with version 2013i or later
%
% see also VirtualFields, MyVirtualFields, GOAT_LoadData, GOAT_LoadDataAndTimeFilter

function [data,G] = GOAT_LoadDataAndProject(G,DataFolder,TimeField,DataField,datafile,Aidx)
try
    [data,G] = GOAT_LoadData(G,DataFolder,TimeField,DataField,datafile,Aidx) ; % Load the data File
catch
    [data,G] = GOAT_LoadData(G,DataFolder,TimeField,DataField,datafile) ; % Load the data File
end
if G.Filter.ZEFlag
    data = data - repmat(nanmean(data,1),[length(G.lon) 1 1]) ;
end
switch G.ProjectionType
    case {1,2,8,9}
        if G.Limits.Default.sameP
            G.DataLimits.Lev1 = G.Limits.Default.Lev1 ;
            G.DataLimits.Lev2 = G.DataLimits.Lev1 ;
        end
        if G.DataLimits.Lev1 == G.DataLimits.Lev2
            data = squeeze(data(:,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1)) ;
        else
          if ~any(G.Settings.LevelsIdx == [5,6,7]) ; 
            if G.Limits.Default.VerMeanInt == 2
                try
                    %load(fullfile(DataFolder,TimeField,'BoerBeta',datafile)) ;
                    [bata,G] = GOAT_load_bata(fullfile(DataFolder,TimeField,'BoerBeta',datafile),G,Aidx)  ; 
                    if size(bata,3) == size(data,3)
                    data = data.*bata ;
                    else % some datasets remove redundant upper atmosphere levels for fields such as specific humidity
                        if G.lev(end)-G.lev(1) < 0
                            data = data.*bata(:,:,1:size(data,3)) ;
                        else
                            data = data.*bata(:,:,end-size(data,3)+1:end) ;
                        end
                    end
                catch
                    GOAT_Display_warning(G,'Warning: BoerBeta could not be found or did not match variable dimensions. Integral proceeded without Boer Averaging') ;
                end
            elseif  G.Limits.Default.VerMeanInt == 3
                data(isnan(data)) = 0 ;
            end
          end
            data = trapz(G.DataLimits.p,data(:,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2),3)  ; % weighted mean
        end
    case 3
        if G.Limits.Default.sameP % This Flag Means Vertical Averaging In this Projection
            if G.DataLimits.Lev1 == G.DataLimits.Lev2
                if G.DataLimits.Lon2 >= G.DataLimits.Lon1
                    if G.Settings.NanMeanFlag
                        data = squeeze(nanmean(data(G.DataLimits.Lon1:G.DataLimits.Lon2,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1),1)) ;
                    else
                        data = squeeze(mean(data(G.DataLimits.Lon1:G.DataLimits.Lon2,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1),1)) ;
                    end
                else
                    if G.Settings.NanMeanFlag
                        data = squeeze(nanmean(data([G.DataLimits.Lon1:end ,1:G.DataLimits.Lon2],G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1),1)) ;
                    else
                        data = squeeze(mean(data([G.DataLimits.Lon1:end ,1:G.DataLimits.Lon2],G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1),1)) ;
                    end
                end
            else
                if G.DataLimits.Lon2 >= G.DataLimits.Lon1
                    if G.Settings.NanMeanFlag
                        data = squeeze(trapz(G.DataLimits.p,squeeze(nanmean(data(G.DataLimits.Lon1:G.DataLimits.Lon2,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2),1)),2))' ;
                    else
                        data = squeeze(trapz(G.DataLimits.p,squeeze(mean(data(G.DataLimits.Lon1:G.DataLimits.Lon2,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2),1)),2))' ;
                    end
                else
                    if G.Settings.NanMeanFlag
                        data = squeeze(trapz(G.DataLimits.p,squeeze(nanmean(data([G.DataLimits.Lon1:end ,1:G.DataLimits.Lon2],G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2),1)),2))' ;
                    else
                        data = squeeze(trapz(G.DataLimits.p,squeeze(mean(data([G.DataLimits.Lon1:end ,1:G.DataLimits.Lon2],G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2),1)),2))' ;
                    end
                end
            end
        else
            if G.DataLimits.Lon2 >= G.DataLimits.Lon1
                if G.Settings.NanMeanFlag
                    data = squeeze(nanmean(data(G.DataLimits.Lon1:G.DataLimits.Lon2,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2),1)) ;
                else
                    data = squeeze(mean(data(G.DataLimits.Lon1:G.DataLimits.Lon2,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2),1)) ;
                end
            else
                if G.Settings.NanMeanFlag
                    data = squeeze(nanmean(data([G.DataLimits.Lon1:end ,1:G.DataLimits.Lon2],G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2),1)) ;
                else
                    data = squeeze(mean(data([G.DataLimits.Lon1:end ,1:G.DataLimits.Lon2],G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2),1)) ;
                end
            end
        end
    case 4
        if G.Limits.Default.sameP
            if G.DataLimits.Lev1 == G.DataLimits.Lev2
                try
                    if G.Settings.NanMeanFlag
                        cosmat = G.Vectors.cosmat(:,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1) ; 
                        cosmat(isnan(data(:,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1))) = nan ; 
                        data = squeeze(GOAT_nanmeanw(data(:,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1),cosmat,2)) ;
                    else
                        data = squeeze(GOAT_meanw(data(:,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1),G.Vectors.cosmat(:,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1),2)) ;
                    end
                catch
                    disp 'GOAT_LoadDataAndProject: cos(lat) weighted average failed. proceeded without area weighted averaging' ;
                    if G.Settings.NanMeanFlag
                        data = squeeze(nanmean(data(:,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1),2)) ;
                    else
                        data = squeeze(mean(data(:,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1),2)) ;
                    end
                end
            else
                try
                    if G.Settings.NanMeanFlag
                        
                        data = squeeze(trapz(G.DataLimits.p,squeeze(GOAT_nanmeanw(data(:,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2),G.Vectors.cosmat(:,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2),2)),2)) ;
                    else
                        data = squeeze(trapz(G.DataLimits.p,squeeze(GOAT_meanw(data(:,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2),G.Vectors.cosmat(:,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2),2)),2)) ;
                    end
                catch
                    disp 'GOAT_LoadDataAndProject: cos(lat) weighted average failed. proceeded without area weighted averaging' ;
                    if G.Settings.NanMeanFlag
                        data = squeeze(trapz(G.DataLimits.p,squeeze(nanmean(data(:,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2),2)),2)) ;
                    else
                        data = squeeze(trapz(G.DataLimits.p,squeeze(mean(data(:,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2),2)),2)) ;
                    end
                end
            end
        else
            try
                if G.Settings.NanMeanFlag
                    cosmat = G.Vectors.cosmat(:,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2) ; 
                    cosmat(isnan(data(:,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2))) = nan ; 
                    data = squeeze(GOAT_nanmeanw(data(:,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2),cosmat,2)) ;
                else
                    data = squeeze(GOAT_meanw(data(:,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2),G.Vectors.cosmat(:,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2),2)) ;
                end
            catch
                disp 'GOAT_LoadDataAndProject: cos(lat) weighted average failed. proceeded without area weighted averaging' ;
                if G.Settings.NanMeanFlag
                    data = squeeze(nanmean(data(:,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2),2)) ;
                else
                    data = squeeze(mean(data(:,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2),2)) ;
                end
            end
        end
    case 5
        if G.DataLimits.Lev1 == G.DataLimits.Lev2
            try
                if G.Settings.NanMeanFlag
                    cosmat = squeeze(G.Vectors.cosmat(:,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1)) ; 
                    cosmat(isnan(squeeze(data(:,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1)))) = nan ; 
                    data = GOAT_nanmeanw(squeeze(data(:,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1)),cosmat,2) ;
                else
                    data = GOAT_meanw(squeeze(data(:,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1)),squeeze(G.Vectors.cosmat(:,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1)),2) ;
                end
            catch
                disp 'GOAT_LoadDataAndProject: cos(lat) weighted average failed. proceeded without area weighted averaging' ;
                if G.Settings.NanMeanFlag
                    data = nanmean(squeeze(data(:,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1)),2) ;
                else
                    data = mean(squeeze(data(:,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1)),2) ;
                end
            end
        else
            try
                if G.Settings.NanMeanFlag
                    cosmat = G.Vectors.cosmat(:,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2) ; 
                    cosmat(isnan(data(:,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2))) = nan ; 
                    data = trapz(G.DataLimits.p,squeeze(GOAT_nanmeanw(data(:,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2),cosmat,2)),2)  ; % weighted mean
                else
                    data = trapz(G.DataLimits.p,squeeze(GOAT_meanw(data(:,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2),G.Vectors.cosmat(:,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2),2)),2)  ; % weighted mean
                end
            catch
                if G.Settings.NanMeanFlag
                    data = trapz(G.DataLimits.p,squeeze(nanmean(data(:,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2),2)),2)  ; % weighted mean
                else
                    data = trapz(G.DataLimits.p,squeeze(mean(data(:,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2),2)),2)  ; % weighted mean
                end
            end
        end
    case 6
        if G.DataLimits.Lev1 == G.DataLimits.Lev2
            if G.DataLimits.Lon2 >= G.DataLimits.Lon1
                if G.Settings.NanMeanFlag
                    data = nanmean(squeeze(data(G.DataLimits.Lon1:G.DataLimits.Lon2,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1)),1) ;
                else
                    data = mean(squeeze(data(G.DataLimits.Lon1:G.DataLimits.Lon2,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1)),1) ;
                end
            else
                if G.Settings.NanMeanFlag
                    data = nanmean(squeeze(data([G.DataLimits.Lon1:end ,1:G.DataLimits.Lon2],G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1)),1) ;
                else
                    data = mean(squeeze(data([G.DataLimits.Lon1:end ,1:G.DataLimits.Lon2],G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1)),1) ;
                end
            end
        else
            if G.DataLimits.Lon2 >= G.DataLimits.Lon1
                if G.Settings.NanMeanFlag
                    data = trapz(G.DataLimits.p,squeeze(nanmean(data(G.DataLimits.Lon1:G.DataLimits.Lon2,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2),1)),2)  ; % weighted mean
                else
                    data = trapz(G.DataLimits.p,squeeze(mean(data(G.DataLimits.Lon1:G.DataLimits.Lon2,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2),1)),2)  ; % weighted mean
                end
            else
                if G.Settings.NanMeanFlag
                    data = trapz(G.DataLimits.p,squeeze(nanmean(data([G.DataLimits.Lon1:end ,1:G.DataLimits.Lon2],G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2),1)),2)  ; % weighted mean
                else
                    data = trapz(G.DataLimits.p,squeeze(mean(data([G.DataLimits.Lon1:end ,1:G.DataLimits.Lon2],G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2),1)),2)  ; % weighted mean
                end
            end
        end
    case 7
        if G.DataLimits.Lev2 == G.DataLimits.Lev1
            try
                if G.DataLimits.Lon2 >= G.DataLimits.Lon1
                    if G.Settings.NanMeanFlag
                        data = squeeze(GOAT_nanmeanw(nanmean(data(G.DataLimits.Lon1:G.DataLimits.Lon2,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1),1),G.Vectors.cosmat(1,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1),2)) ;
                    else
                        data = squeeze(GOAT_meanw(mean(data(G.DataLimits.Lon1:G.DataLimits.Lon2,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1),1),G.Vectors.cosmat(1,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1),2)) ;
                    end
                else
                    if G.Settings.NanMeanFlag
                        data = squeeze(GOAT_nanmeanw(nanmean(data([G.DataLimits.Lon1:end ,1:G.DataLimits.Lon2],G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1),1),G.Vectors.cosmat(1,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1),2)) ;
                    else
                        data = squeeze(GOAT_meanw(mean(data([G.DataLimits.Lon1:end ,1:G.DataLimits.Lon2],G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1),1),G.Vectors.cosmat(1,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1),2)) ;
                    end
                end
            catch
                disp 'GOAT_LoadDataAndProject: cos(lat) weighted average failed. proceeded without area weighted averaging' ;
                if G.DataLimits.Lon2 >= G.DataLimits.Lon1
                    if G.Settings.NanMeanFlag
                        data = squeeze(nanmean(nanmean(data(G.DataLimits.Lon1:G.DataLimits.Lon2,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1),1))) ;
                    else
                        data = squeeze(mean(mean(data(G.DataLimits.Lon1:G.DataLimits.Lon2,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1),1))) ;
                    end
                else
                    if G.Settings.NanMeanFlag
                        data = squeeze(nanmean(nanmean(data([G.DataLimits.Lon1:end ,1:G.DataLimits.Lon2],G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1),1))) ;
                    else
                        data = squeeze(mean(mean(data([G.DataLimits.Lon1:end ,1:G.DataLimits.Lon2],G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1),1))) ;
                    end
                end
            end
        elseif G.Limits.Default.sameP
            try
                if G.DataLimits.Lon2 >= G.DataLimits.Lon1
                    if G.Settings.NanMeanFlag
                        data = trapz(G.DataLimits.p,squeeze(GOAT_nanmeanw(squeeze(nanmean(data(G.DataLimits.Lon1:G.DataLimits.Lon2,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2),1)),squeeze(G.Vectors.cosmat(1,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2)),1))',1) ;
                    else
                        data = trapz(G.DataLimits.p,squeeze(GOAT_meanw(   squeeze(mean(   data(G.DataLimits.Lon1:G.DataLimits.Lon2,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2),1)),squeeze(G.Vectors.cosmat(1,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2)),1))',1) ;
                    end
                else
                    if G.Settings.NanMeanFlag
                        data = trapz(G.DataLimits.p,squeeze(GOAT_nanmeanw(squeeze(nanmean(data([G.DataLimits.Lon1:end ,1:G.DataLimits.Lon2],G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2),1)),squeeze(G.Vectors.cosmat(1,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2)),1))',1) ;
                    else
                        data = trapz(G.DataLimits.p,squeeze(GOAT_meanw(squeeze(mean(data([G.DataLimits.Lon1:end ,1:G.DataLimits.Lon2],G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2),1)),squeeze(G.Vectors.cosmat(1,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2)),1))',1) ;
                    end
                end
            catch
                disp 'GOAT_LoadDataAndProject: cos(lat) weighted average failed. proceeded without area weighted averaging' ;
                if G.DataLimits.Lon2 >= G.DataLimits.Lon1
                    if G.Settings.NanMeanFlag
                        data = trapz(G.DataLimits.p,squeeze(nanmean(squeeze(nanmean(data(G.DataLimits.Lon1:G.DataLimits.Lon2,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2),1)),1))',1) ;
                    else
                        data = trapz(G.DataLimits.p,squeeze(mean(squeeze(mean(data(G.DataLimits.Lon1:G.DataLimits.Lon2,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2),1)),1))',1) ;
                    end
                else
                    if G.Settings.NanMeanFlag
                        data = trapz(G.DataLimits.p,squeeze(nanmean(squeeze(nanmean(data([G.DataLimits.Lon1:end ,1:G.DataLimits.Lon2],G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2),1)),1))',1) ;
                    else
                        data = trapz(G.DataLimits.p,squeeze(mean(squeeze(mean(data([G.DataLimits.Lon1:end ,1:G.DataLimits.Lon2],G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2),1)),1))',1) ;
                    end
                end
            end
        else
            try
                if G.DataLimits.Lon2 >= G.DataLimits.Lon1
                    if G.Settings.NanMeanFlag
                        data = squeeze(GOAT_nanmeanw(squeeze(nanmean(data(G.DataLimits.Lon1:G.DataLimits.Lon2,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2),1)),squeeze(G.Vectors.cosmat(1,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2)),1))' ;
                    else
                        data = squeeze(GOAT_meanw(squeeze(mean(data(G.DataLimits.Lon1:G.DataLimits.Lon2,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2),1)),squeeze(G.Vectors.cosmat(1,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2)),1))' ;
                    end
                else
                    if G.Settings.NanMeanFlag
                        data = squeeze(GOAT_nanmeanw(squeeze(nanmean(data([G.DataLimits.Lon1:end ,1:G.DataLimits.Lon2],G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2),1)),squeeze(G.Vectors.cosmat(1,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2)),1))' ;
                    else
                        data = squeeze(GOAT_meanw(squeeze(mean(data([G.DataLimits.Lon1:end ,1:G.DataLimits.Lon2],G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2),1)),squeeze(G.Vectors.cosmat(1,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2)),1))' ;
                    end
                end
            catch
                disp 'GOAT_LoadDataAndProject: cos(lat) weighted average failed. proceeded without area weighted averaging' ;
                if G.DataLimits.Lon2 >= G.DataLimits.Lon1
                    if G.Settings.NanMeanFlag
                        data = squeeze(nanmean(squeeze(nanmean(data(G.DataLimits.Lon1:G.DataLimits.Lon2,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2),1)),1))' ;
                    else
                        data = squeeze(mean(squeeze(mean(data(G.DataLimits.Lon1:G.DataLimits.Lon2,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2),1)),1))' ;
                    end
                else
                    if G.Settings.NanMeanFlag
                        data = squeeze(nanmean(squeeze(nanmean(data([G.DataLimits.Lon1:end ,1:G.DataLimits.Lon2],G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2),1)),1))' ;
                    else
                        data = squeeze(mean(squeeze(mean(data([G.DataLimits.Lon1:end ,1:G.DataLimits.Lon2],G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2),1)),1))' ;
                    end
                end
            end
        end
end