% GOAT_darken_edge  darken the edge of contourf plots to improve surface boundaries visibility
% Copyright (C) Ori Adam Sep.2013. compatible with version v2013j or later
% based on plotting tips found at: http://www.mathworks.ch/ch/help/matlab/ref/caxis.html
% Input arguments:
% GOAT_darken_edge(h,cmap,'clim',[clim1 clim2])
% or
% GOAT_darken_edge(h,cmap,'cvec',cvev)
% where
% h = contour handle
% cmap = colormap
% clim (optional) = limit values of map (i.e., equivalent to caxis([clim1 clim2])
% cvec (optional) = enter contour step values
%
%
% see also, GOAT_plotUpdate

function GOAT_darken_edge(h,cmap,varargin)


%parse varargin
climFlag = 0 ;
cvecFlag = 0 ;
if nargin > 2
    for v = 1:floor(length(varargin)/2)
        switch lower(varargin{v*2-1})
            case 'clim'
                climFlag = 1 ;
                clim = varargin{v*2} ;
            case 'cvec'
                cvecFlag = 1 ;
                cvec = varargin{v*2} ;
            otherwise
                disp(['Unrecognized parameter entered to GOAT_darken_edge ',varargin{v*2-1}])
        end
    end
end
dark_cmap  = brighten(cmap, -.3);

if verLessThan('matlab','8.4.0') % No Children in versions >=2014b

hc         = get(h, 'Children');
hcval      = cell2mat(get(hc, 'CData')) ;
cvalues    = sort(unique(hcval));
if climFlag
    cmin = min(clim) ;
    cmax = max(clim) ;
else
    cmin = min(cvalues) ;
    cmax = max(cvalues) ;
end
m = size(cmap,1) ;
if cvecFlag
index = 1:m ;
else
index = fix((cvalues-cmin)/(cmax-cmin)*m)+1;
%Clamp values outside the range [1 m]
index(index<1) = 1;
index(index>m) = m;
end
for k = 1:length(cvalues)
    idx = [] ;
    if ~isnan(cvalues(k))
        idx   = find(hcval == cvalues(k));
        if ~isempty(idx)
            edge_color = dark_cmap(index(k), :);
            set(hc(idx), 'LineStyle', '-', ...
                'EdgeColor', edge_color, ...
                'LineWidth', .3)
        end
    end
end


else % version < v2014b
    %disp 'GOAT_darken_edge warning: cannot locate contour lines on versions >=2014b' ; 
    % darkening edges by recontouring each contour with dakened color
    LevelList = get(h,'LevelList') ; % get contour levels 
    set(h,'LineStyle','none')
    if climFlag
    cmin = min(clim) ;
    cmax = max(clim) ;
     else
    cmin = min(LevelList) ;
    cmax = max(LevelList) ;
    end
    Xdata = get(h,'XData') ;
    Ydata = get(h,'YData') ;
    Zdata = get(h,'ZData') ; 
    for j = 2:length(LevelList)-1
        ColorIdx = floor((LevelList(j-1) - cmin)/(cmax-cmin)*(size(cmap,1)-1))+1 ;
        LineColor = dark_cmap(ColorIdx,:) ;
    %[Cd hd] = contour(Xdata,Ydata,Zdata,[LevelList(j) LevelList(j)],'linewidth',.5,'linecolor',LineColor,'LineStyle','-') ; 
    contour(Xdata,Ydata,Zdata,[LevelList(j) LevelList(j)],'linewidth',.5,'linecolor',LineColor,'LineStyle','-') ; 
    end
end
