% GOAT_holdUpdate GOAT main GUI plot function for contour overlay in Hold mode
% Copyright (C) Ori Adam Jan.2014.
% Compatible with version 2014A or later
%
% see also GOAT_Defaults, GOAT_SignificancePlot, GOAT_darken_edge, GOAT_plotUpdate

function [G, Cout, hout, Clout] = GOAT_holdUpdate(G,Defs)
if nargin < 2
    Defs = [] ;
end


if verLessThan('matlab','8.4.0') % work around Children tagging by controling contours levels
    if ~G.ExportFlag
        if (isfield(Defs,'delete') && ~isempty(Defs.delete) && ~strcmpi(Defs.delete,'off'))
            try
                cl0 = findobj('Tag','Hold') ;
                for i = 1:length(cl0)
                    %try
                    %    delete(get(cl0(i),'parent')) ;
                    %end
                    delete(cl0(i)) ;
                end
            end
        end
    end
    hold all ;
    Lat1 = min([G.lat(G.Limits.Default.Lat1),G.lat(G.Limits.Default.Lat2)]) ;
    Lat2 = max([G.lat(G.Limits.Default.Lat1),G.lat(G.Limits.Default.Lat2)]) ;
    if G.Settings.SinLatFlag && any(G.ProjectionType == [1,2,3,6])
        Lat1 = sind(Lat1) ;
        Lat2 = sind(Lat2) ;
    end
    try
        Lon1 = min(G.x) ; % min([G.lon(G.Limits.Default.Lon1),G.lon(G.Limits.Default.Lon2)]) ;
        Lon2 = max(G.x) ; % max([G.lon(G.Limits.Default.Lon1),G.lon(G.Limits.Default.Lon2)]) ;
    catch
        Lon1 = min([G.lon(G.Limits.Default.Lon1),G.lon(G.Limits.Default.Lon2)]) ;
        Lon2 = max([G.lon(G.Limits.Default.Lon1),G.lon(G.Limits.Default.Lon2)]) ;
    end
    P1   = min([G.lev(G.Limits.Default.Lev1),G.lev(G.Limits.Default.Lev2)]) ;
    P2   = max([G.lev(G.Limits.Default.Lev1),G.lev(G.Limits.Default.Lev2)]) ;
    
    try
        LineWidth      = G.Settings.LineWidth      ;
    catch
        LineWidth      = 2 ;
    end
    
    if LineWidth < 1  ; LineWidth = 2 ; end
    
    
    switch G.ProjectionType
        case {1,2} % Long-Lat
            try
                if G.Settings.SinLatFlag
                    if isfield(Defs,'contour_levels') && ~isempty(Defs.contour_levels)
                        [Ch,hh] =  contour(G.X,sind(G.Y),G.Data',Defs.contour_levels,'linewidth',LineWidth,'linecolor',G.GUI.colors.HoldGrey,'Tag','Hold') ; hold on
                    else
                        [Ch,hh] =  contour(G.X,sind(G.Y),G.Data','linewidth',LineWidth,'linecolor',G.GUI.colors.HoldGrey,'Tag','Hold') ; hold on
                    end
                else
                    if isfield(Defs,'contour_levels') && ~isempty(Defs.contour_levels)
                        [Ch,hh] =  contour(G.X,G.Y,G.Data',Defs.contour_levels,'linewidth',LineWidth,'linecolor',G.GUI.colors.HoldGrey,'Tag','Hold') ; hold on
                    else
                        [Ch,hh] =  contour(G.X,G.Y,G.Data','linewidth',LineWidth,'linecolor',G.GUI.colors.HoldGrey,'Tag','Hold') ; hold on
                    end
                end
                %             try
                %                 if G.Settings.ContourLevelStep
                %                     if  G.Settings.ContourLevelStep > (nanmax(G.Data(:)) - nanmin(G.Data(:)))/100 ;
                %                         set(hh,'levelstep',G.Settings.ContourLevelStep) ;
                %                     end
                %                 end
                %             catch
                %                 % Nevermind
                %             end
                %set(hh,'linewidth',LineWidth,'linecolor',G.GUI.colors.HoldGrey,'Tag','Hold') ;
                if isfield(Defs,'clabels') && ~isequal(Defs.clabels,'off')
                    clh = clabelgoat(Ch,hh,'fontsize',12,'textformat','%2.5g') ;
                    for i = 1:length(clh)
                        set(clh(i),'Tag','Hold') ;
                    end
                    chandle = get(hh,'children') ;
                    location = 1 ;
                    a = 1 ;
                    levels = zeros(1,1) ;
                    while 1
                        levels(a) = Ch(1,location) ;
                        steps = Ch(2,location) ;
                        location = location + steps + 1 ;
                        if location > size(Ch,2)
                            break
                        else
                            a = a + 1 ;
                        end
                    end
                    levels = fliplr(levels) ;
                    %set(chandle(levels<0) ,'LineStyle',':') ;
                    %set(chandle(levels==0),'LineStyle',':','edgecolor',[.2 .2 .2] ) ;
                    set(chandle(levels<0) ,'LineStyle','--') ;
                    set(chandle(levels==0),'LineStyle','--','edgecolor',[.2 .2 .2] ) ;
                    set(chandle(levels>0) ,'LineStyle','-' ) ;
                else
                    clh     = [] ;
                end
                caxis(G.Hold.Clim) ;
                set(G.Display.colorbarHandle,'Ylim',G.Hold.colorbarLimits)
            catch
                disp 'error while attempting to create a contour plot in Hold mode'
            end
            hold off
            if Lon1~=Lon2
                xlim([Lon1 Lon2])   ;
            else
                xlim([Lon1 Lon2+.1]) ;
            end
            ylim([Lat1 Lat2])   ;
            if nargout > 1
                Cout  = Ch ;
                hout  = hh ;
                Clout = clh ;
            end
        case 3 % Lat-Pressure
            try
                if G.Limits.Default.sameP || P2 == P1
                    if G.Settings.SinLatFlag
                        h = plot(sind(G.X),G.Data,'--k','linewidth',LineWidth) ;
                    else
                        h = plot(G.X,G.Data,'--k','linewidth',LineWidth) ;
                    end
                    set(h,'Tag','Hold') ;
                    xlim([Lat1 Lat2])   ;
                elseif Lat1 == Lat2
                    h = plot(G.Y,G.Data,'--k','linewidth',LineWidth) ;
                    set(h,'Tag','Hold') ;
                    xlim([P1 P2])   ;
                else
                    if G.Settings.SinLatFlag
                        if G.Settings.LogLevelsFlag
                            if isfield(Defs,'contour_levels') && ~isempty(Defs.contour_levels)
                                [Ch,hh] =  contour(sind(G.X),log(single(G.Y)),G.Data',Defs.contour_levels) ;
                            else
                                [Ch,hh] =  contour(sind(G.X),log(single(G.Y)),G.Data') ;
                            end
                        else
                            if isfield(Defs,'contour_levels') && ~isempty(Defs.contour_levels)
                                [Ch,hh] =  contour(sind(G.X),G.Y,G.Data',Defs.contour_levels) ;
                            else
                                [Ch,hh] =  contour(sind(G.X),G.Y,G.Data') ;
                            end
                        end
                    else
                        if G.Settings.LogLevelsFlag
                            if isfield(Defs,'contour_levels') && ~isempty(Defs.contour_levels)
                                [Ch,hh] =  contour(G.X,log(single(G.Y)),G.Data',Defs.contour_levels) ;
                            else
                                [Ch,hh] =  contour(G.X,log(single(G.Y)),G.Data') ;
                            end
                        else
                            if isfield(Defs,'contour_levels') && ~isempty(Defs.contour_levels)
                                [Ch,hh] =  contour(G.X,G.Y,G.Data',Defs.contour_levels) ;
                            else
                                [Ch,hh] =  contour(G.X,G.Y,G.Data') ;
                            end
                        end
                    end
                    try
                        if any(G.Settings.LevelsIdx == [1,2,5,7])
                            axis ij
                        end
                    catch
                        axis ij
                    end
                    try
                        if G.Settings.ContourLevelStep
                            if  G.Settings.ContourLevelStep > (max(max(G.Data)) - min(min(G.Data)))/100 ;
                                set(hh,'levelstep',G.Settings.ContourLevelStep) ;
                            end
                        end
                    catch
                        % Nevermind
                    end
                    set(hh,'linewidth',LineWidth,'linecolor',G.GUI.colors.HoldGrey,'Tag','Hold') ;
                    cl = clabelgoat(Ch,hh,'fontsize',12,'textformat','%2.5g') ;
                    chandle = get(hh,'children') ;
                    location = 1 ;
                    a = 1 ;
                    levels = zeros(1,1) ;
                    while 1
                        levels(a) = Ch(1,location) ;
                        steps = Ch(2,location) ;
                        location = location + steps + 1 ;
                        if location > size(Ch,2)
                            break
                        else
                            a = a + 1 ;
                        end
                    end
                    levels = fliplr(levels) ;
                    set(chandle(levels<0),'LineStyle',':') ;
                    set(chandle(levels==0),'LineStyle',':','edgecolor',[.2 .2 .2]) ;
                    set(chandle(levels>0),'LineStyle','-') ;
                    for i = 1:length(cl)
                        set(cl(i),'Tag','Hold') ;
                    end
                    if G.Settings.LogLevelsFlag
                        ylim([log(single(P1)) log(single(P2))])
                    else
                        ylim([P1 P2]) ;
                    end
                    xlim([Lat1 Lat2])   ;
                    caxis(G.Hold.Clim) ;
                    set(G.Display.colorbarHandle,'Ylim',G.Hold.colorbarLimits)
                end
            catch
                disp 'error while attempting to create a contour plot in Hold mode'
            end
        case 4 % Lon-Pressure
            try
                if G.Limits.Default.sameP
                    h = plot(G.X,G.Data,'--k','linewidth',LineWidth) ;
                    set(h,'Tag','Hold') ;
                    xlim([Lon1 Lon2])   ;
                elseif Lon1 == Lon2
                    h = plot(G.Y,G.Data,'--k','linewidth',LineWidth) ;
                    set(h,'Tag','Hold') ;
                    xlim([P1 P2])   ;
                else
                    if G.Settings.LogLevelsFlag
                        [Ch,hh] =  contour(G.X,log(single(G.Y)),G.Data') ;
                    else
                        [Ch,hh] =  contour(G.X,G.Y,G.Data') ;
                    end
                    
                    try
                        if any(G.Settings.LevelsIdx == [1,2,5,7])
                            axis ij
                        end
                    catch
                        axis ij
                    end
                    try
                        if G.Settings.ContourLevelStep
                            if  G.Settings.ContourLevelStep > (max(max(G.Data)) - min(min(G.Data)))/100 ;
                                set(hh,'levelstep',G.Settings.ContourLevelStep) ;
                            end
                        end
                    catch
                        % Nevermind
                    end
                    set(hh,'linewidth',LineWidth,'linecolor',G.GUI.colors.HoldGrey,'Tag','Hold') ;
                    cl = clabelgoat(Ch,hh,'fontsize',12,'textformat','%2.5g') ;
                    chandle = get(hh,'children') ;
                    location = 1 ;
                    a = 1 ;
                    levels = zeros(1,1) ;
                    while 1
                        levels(a) = Ch(1,location) ;
                        steps = Ch(2,location) ;
                        location = location + steps + 1 ;
                        if location > size(Ch,2)
                            break
                        else
                            a = a + 1 ;
                        end
                    end
                    levels = fliplr(levels) ;
                    set(chandle(levels<0),'LineStyle',':') ;
                    set(chandle(levels==0),'LineStyle',':','edgecolor',[.2 .2 .2]) ;
                    set(chandle(levels>0),'LineStyle','-') ;
                    for i = 1:length(cl)
                        set(cl(i),'Tag','Hold') ;
                    end
                    if G.Settings.LogLevelsFlag
                        ylim([log(single(P1)) log(single(P2))])
                    else
                        ylim([P1 P2])
                    end
                    xlim([Lon1 Lon2])   ;
                    caxis(G.Hold.Clim) ;
                    set(G.Display.colorbarHandle,'Ylim',G.Hold.colorbarLimits)
                end
            catch
                disp 'error while attempting to create a contour plot in Hold mode'
            end
        case 5 % Hovmoller Lon
            try
                if Lon1 == Lon2
                    h = plot(G.y,G.Data,'--k','linewidth',LineWidth) ;
                    set(h,'Tag','Hold') ;
                else
                    [Ch,hh] =  contour(G.X,G.Y,G.Data') ;
                    try
                        if G.Settings.ContourLevelStep
                            if  G.Settings.ContourLevelStep > (max(max(G.Data)) - min(min(G.Data)))/100 ;
                                set(hh,'levelstep',G.Settings.ContourLevelStep) ;
                            end
                        end
                    catch
                        % Nevermind
                    end
                    set(hh,'linewidth',LineWidth,'linecolor',G.GUI.colors.HoldGrey,'Tag','Hold') ;
                    cl = clabelgoat(Ch,hh,'fontsize',12,'textformat','%2.5g') ;
                    chandle = get(hh,'children') ;
                    location = 1 ;
                    a = 1 ;
                    levels = zeros(1,1) ;
                    while 1
                        levels(a) = Ch(1,location) ;
                        steps = Ch(2,location) ;
                        location = location + steps + 1 ;
                        if location > size(Ch,2)
                            break
                        else
                            a = a + 1 ;
                        end
                    end
                    levels = fliplr(levels) ;
                    set(chandle(levels<0),'LineStyle',':') ;
                    set(chandle(levels==0),'LineStyle',':','edgecolor',[.2 .2 .2]) ;
                    set(chandle(levels>0),'LineStyle','-') ;
                    for i = 1:length(cl)
                        set(cl(i),'Tag','Hold') ;
                    end
                    caxis(G.Hold.Clim) ;
                    set(G.Display.colorbarHandle,'Ylim',G.Hold.colorbarLimits)
                end
            catch
                disp 'error while attempting to create a contour plot in Hold mode'
            end
        case 6 % Hovmoller Lat
            try
                if Lat1 == Lat2
                    h = plot(G.y,G.Data,'--k','linewidth',LineWidth) ;
                    set(h,'Tag','Hold') ;
                else
                    if G.Settings.SinLatFlag
                        [Ch,hh] =  contour(sind(G.X),G.Y,G.Data') ;
                    else
                        [Ch,hh] =  contour(G.X,G.Y,G.Data') ;
                    end
                    try
                        if G.Settings.ContourLevelStep
                            if  G.Settings.ContourLevelStep > (max(max(G.Data)) - min(min(G.Data)))/100 ;
                                set(hh,'levelstep',G.Settings.ContourLevelStep) ;
                            end
                        end
                    catch
                        % Nevermind
                    end
                    set(hh,'linewidth',LineWidth,'linecolor',G.GUI.colors.HoldGrey,'Tag','Hold') ;
                    cl = clabelgoat(Ch,hh,'fontsize',12,'textformat','%2.5g') ;
                    chandle = get(hh,'children') ;
                    location = 1 ;
                    a = 1 ;
                    levels = zeros(1,1) ;
                    while 1
                        levels(a) = Ch(1,location) ;
                        steps = Ch(2,location) ;
                        location = location + steps + 1 ;
                        if location > size(Ch,2)
                            break
                        else
                            a = a + 1 ;
                        end
                    end
                    levels = fliplr(levels) ;
                    set(chandle(levels<0),'LineStyle',':') ;
                    set(chandle(levels==0),'LineStyle',':','edgecolor',[.2 .2 .2]) ;
                    set(chandle(levels>0),'LineStyle','-') ;
                    for i = 1:length(cl)
                        set(cl(i),'Tag','Hold') ;
                    end
                    caxis(G.Hold.Clim) ;
                    set(G.Display.colorbarHandle,'Ylim',G.Hold.colorbarLimits)
                end
            catch
                disp 'error while attempting to create a contour plot in Hold mode'
            end
        case 7 % Hovmoller P
            try
                if P2 == P1 || G.Limits.Default.sameP ;
                    h = plot(G.y,G.Data,'--k','linewidth',LineWidth) ;
                    set(h,'Tag','Hold') ;
                else
                    [Ch,hh] =  contour(G.X,G.Y,G.Data') ;
                    try
                        if G.Settings.ContourLevelStep
                            if  G.Settings.ContourLevelStep > (max(max(G.Data)) - min(min(G.Data)))/100 ;
                                set(hh,'levelstep',G.Settings.ContourLevelStep) ;
                            end
                        end
                    catch
                        % Nevermind
                    end
                    set(hh,'linewidth',LineWidth,'linecolor',G.GUI.colors.HoldGrey,'Tag','Hold') ;
                    cl = clabelgoat(Ch,hh,'fontsize',12,'textformat','%2.5g') ;
                    chandle = get(hh,'children') ;
                    location = 1 ;
                    a = 1 ;
                    levels = zeros(1,1) ;
                    while 1
                        levels(a) = Ch(1,location) ;
                        steps = Ch(2,location) ;
                        location = location + steps + 1 ;
                        if location > size(Ch,2)
                            break
                        else
                            a = a + 1 ;
                        end
                    end
                    levels = fliplr(levels) ;
                    set(chandle(levels<0),'LineStyle',':') ;
                    set(chandle(levels==0),'LineStyle',':','edgecolor',[.2 .2 .2]) ;
                    set(chandle(levels>0),'LineStyle','-') ;
                    for i = 1:length(cl)
                        set(cl(i),'Tag','Hold') ;
                    end
                    caxis(G.Hold.Clim) ;
                    set(G.Display.colorbarHandle,'Ylim',G.Hold.colorbarLimits)
                end
                
            catch
                disp 'error while attempting to create a contour plot in Hold mode'
            end
        case {8,9}
            Yz = [G.Y,G.Y(:,1)] ;
            Dataz = [G.Data ; G.Data(1,:)] ;
            if G.x(1) == 0
                Xz = [G.X,(G.X(:,1)+360-0.001)] ;
            else
                Xz = [G.X,G.X(:,1)-0.001] ;
            end
            hold on
            MaxFlag = 0 ;
            MinFlag = 0 ;
            if nanmax(Dataz(:)) >=0
                MaxFlag = 1 ;
                DataZPlus = Dataz  ; DataZPlus(DataZPlus<0) = nan ;
            end
            if nanmin(Dataz(:)) <0
                MinFlag = 1 ;
                DataZMinus = Dataz  ; DataZMinus(DataZMinus>=0) = nan ;
            end
            if MaxFlag
                [Cp , hp] = contourm(double(Yz),double(Xz),double(DataZPlus'),'color',[.45 .45 .45],'linewidth',LineWidth,'Tag','Hold') ; hold on % alpha(.8) ;
                try
                    [Ch , hh0] = contourm(double(Yz),double(Xz),double(Dataz'),[0 0],'color',[.45 .45 .45],'linewidth',LineWidth,'Tag','Hold') ; hold on % alpha(.8) ;
                    [Ch , hh] = contourm(double(Yz),double(Xz),double(Dataz'),[0 0],'color',[.25 .25 .25],'linewidth',LineWidth,'lineStyle',':','Tag','Hold') ; hold on % alpha(.8) ;
                end
            end
            if MinFlag
                [Cm , hm] = contourm(double(Yz),double(Xz),double(DataZMinus'),'color',[.3 .3 .3],'linewidth',LineWidth,'Tag','Hold') ; hold on % alpha(.8) ;
            end
            if MaxFlag
                clp = clabelm(Cp,hp) ;
                clh = clabelm(Ch,hh) ;
                %set(hp ,'Tag','Hold') ;
                %set(hh ,'Tag','Hold') ;
                for i = 1:length(clp)
                    set(clp(i),'Tag','Hold') ;
                end
                for i = 1:length(clh)
                    set(clh(i),'Tag','Hold') ;
                end
            end
            if MinFlag
                clm = clabelm(Cm,hm) ;
                %set(hm ,'Tag','Hold') ;
                for i = 1:length(clm)
                    set(clm(i),'Tag','Hold') ;
                end
            end
            caxis(G.Hold.Clim) ;
            set(G.Display.colorbarHandle,'Ylim',G.Hold.colorbarLimits)
    end
    
    % From MATLAB 2014b Onwards
else
    
    
    if ~isfield(Defs,'contour_levels') || ...
            isfield(Defs,'contour_levels') && isempty(Defs.contour_levels)
        try
            DataMax = nanmax(G.Data(:)) ;
            DataMin = nanmin(G.Data(:)) ;
        catch
            Data = G.Data(~isnan(G.Data)) ;
            DataMax = max(Data(:)) ;
            DataMin = min(Data(:)) ;
        end
        if DataMin == DataMax
            Defs.contour_levels = DataMin ;
        elseif DataMin >= 0 ||  DataMax <= 0
            Defs.contour_levels = linspace(DataMin,DataMax,8) ;
        else
            Defs.contour_levels = linspace(DataMin,DataMax,9) ;
        end
    end
    
    if ~G.ExportFlag
        if (isfield(Defs,'delete') && ~isempty(Defs.delete) && ~strcmpi(Defs.delete,'off'))
            try
                cl0 = findobj('Tag','Hold') ;
                for i = 1:length(cl0)
                    %try
                    %    delete(get(cl0(i),'parent')) ;
                    %end
                    delete(cl0(i)) ;
                end
            end
        end
    end
    hold all ;
    Lat1 = min([G.lat(G.Limits.Default.Lat1),G.lat(G.Limits.Default.Lat2)]) ;
    Lat2 = max([G.lat(G.Limits.Default.Lat1),G.lat(G.Limits.Default.Lat2)]) ;
    if G.Settings.SinLatFlag && any(G.ProjectionType == [1,2,3,6])
        Lat1 = sind(Lat1) ;
        Lat2 = sind(Lat2) ;
    end
    try
        Lon1 = min(G.x) ; % min([G.lon(G.Limits.Default.Lon1),G.lon(G.Limits.Default.Lon2)]) ;
        Lon2 = max(G.x) ; % max([G.lon(G.Limits.Default.Lon1),G.lon(G.Limits.Default.Lon2)]) ;
    catch
        Lon1 = min([G.lon(G.Limits.Default.Lon1),G.lon(G.Limits.Default.Lon2)]) ;
        Lon2 = max([G.lon(G.Limits.Default.Lon1),G.lon(G.Limits.Default.Lon2)]) ;
    end
    P1   = min([G.lev(G.Limits.Default.Lev1),G.lev(G.Limits.Default.Lev2)]) ;
    P2   = max([G.lev(G.Limits.Default.Lev1),G.lev(G.Limits.Default.Lev2)]) ;
    
    try
        LineWidth      = G.Settings.LineWidth      ;
    catch
        LineWidth      = 2 ;
    end
    
    if LineWidth < 1  ; LineWidth = 2 ; end
    
    
    switch G.ProjectionType
        case {1,2} % Long-Lat
            try
                if G.Settings.SinLatFlag
                    if isfield(Defs,'contour_levels') && ~isempty(Defs.contour_levels)
                        if ~isempty(Defs.contour_levels(Defs.contour_levels>0))
                            [Cplus ,hplus ] =  contour(G.X,sind(G.Y),G.Data',Defs.contour_levels(Defs.contour_levels>0),'linewidth',LineWidth,'linecolor',G.GUI.colors.HoldGrey,'Tag','Hold','LineStyle','-') ; hold on
                        else
                            Cplus = [] ; hplus = [] ;
                        end
                        if ~isempty(Defs.contour_levels(Defs.contour_levels<0))
                            [Cminus,hminus] =  contour(G.X,sind(G.Y),G.Data',Defs.contour_levels(Defs.contour_levels<0),'linewidth',LineWidth,'linecolor',G.GUI.colors.HoldGrey,'Tag','Hold','LineStyle','--') ; hold on
                        else
                            Cminus = [] ; hminus = [] ;
                        end
                        if ~isempty(Defs.contour_levels(Defs.contour_levels==0))
                            [Czero,hzero] =  contour(G.X,sind(G.Y),G.Data',[0 0],'linewidth',LineWidth,'linecolor',[.2 .2 .2],'Tag','Hold') ; hold on
                        else
                            Czero = [] ; hzero = [] ;
                        end
                    else
                        % Should not occur for version > 2014b
                    end
                else
                    if isfield(Defs,'contour_levels') && ~isempty(Defs.contour_levels)
                        if ~isempty(Defs.contour_levels(Defs.contour_levels>0))
                            [Cplus ,hplus ] =  contour(G.X,G.Y,G.Data',Defs.contour_levels(Defs.contour_levels>0),'linewidth',LineWidth,'linecolor',G.GUI.colors.HoldGrey,'Tag','Hold','LineStyle','-') ; hold on
                        else
                            Cplus = [] ; hplus = [] ;
                        end
                        if ~isempty(Defs.contour_levels(Defs.contour_levels<0))
                            [Cminus,hminus] =  contour(G.X,G.Y,G.Data',Defs.contour_levels(Defs.contour_levels<0),'linewidth',LineWidth,'linecolor',G.GUI.colors.HoldGrey,'Tag','Hold','LineStyle','--') ; hold on
                        else
                            Cminus = [] ; hminus = [] ;
                        end
                        if ~isempty(Defs.contour_levels(Defs.contour_levels==0))
                            [Czero,hzero] =  contour(G.X,G.Y,G.Data',[0 0],'linewidth',LineWidth,'linecolor',[.2 .2 .2],'Tag','Hold') ; hold on
                        else
                            Czero = [] ; hzero = [] ;
                        end
                    else
                        % should not occur
                    end
                end
                %             try
                %                 if G.Settings.ContourLevelStep
                %                     if  G.Settings.ContourLevelStep > (nanmax(G.Data(:)) - nanmin(G.Data(:)))/100 ;
                %                         set(hh,'levelstep',G.Settings.ContourLevelStep) ;
                %                     end
                %                 end
                %             catch
                %                 % Nevermind
                %             end
                %set(hh,'linewidth',LineWidth,'linecolor',G.GUI.colors.HoldGrey,'Tag','Hold') ;
                if isfield(Defs,'clabels') && ~isequal(Defs.clabels,'off')
                    if ~isempty(Cplus)
                        clplus = clabel(Cplus,'fontsize',12) ;
                        for i = 1:length(clplus)
                            set(clplus(i),'Tag','Hold') ;
                        end
                    end
                    if ~isempty(Cminus)
                        clminus = clabel(Cminus,'fontsize',12) ;
                        for i = 1:length(clminus)
                            set(clminus(i),'Tag','Hold') ;
                        end
                    end
                    if ~isempty(Czero)
                        clzero = clabel(Czero,'fontsize',12) ;
                        for i = 1:length(clzero)
                            set(clzero(i),'Tag','Hold') ;
                        end
                    end
                else
                    clzero  = [] ;
                    clplus  = [] ;
                    clminus = [] ;
                end
                try
                    caxis(G.Hold.Clim) ;
                catch
                    disp('GOAT_holdPlot warning: attempting to reset caxis to G.Hold.Clim failed.')
                end
                try
                set(G.Display.colorbarHandle,'Ylim',G.Hold.colorbarLimits)
                catch
                    disp('GOAT_holdPlot warning: attempting to reset colorbar Ylim to G.Hold.colorbarlimits failed.')
                end
            catch
                disp 'error while attempting to create a contour plot in Hold mode'
            end
            hold off
            if Lon1~=Lon2
                xlim([Lon1 Lon2])   ;
            else
                xlim([Lon1 Lon2+.1]) ;
            end
            ylim([Lat1 Lat2])   ;
            if nargout > 1
                Cout.plus  = Cplus  ; Cout.minus  = Cminus  ; Cout.zero  = Czero ;
                hout.plus  = hplus  ; hout.minus  = hminus  ; hout.zero  = hzero ;
                Clout.plus = clplus ; Clout.minus = clminus ; Clout.zero = clzero ;
            end
        case 3 % Lat-Pressure
            try
                if G.Limits.Default.sameP || P2 == P1
                    if G.Settings.SinLatFlag
                        h = plot(sind(G.X),G.Data,'--k','linewidth',LineWidth) ;
                    else
                        h = plot(G.X,G.Data,'--k','linewidth',LineWidth) ;
                    end
                    set(h,'Tag','Hold') ;
                    xlim([Lat1 Lat2])   ;
                elseif Lat1 == Lat2
                    h = plot(G.Y,G.Data,'--k','linewidth',LineWidth) ;
                    set(h,'Tag','Hold') ;
                    xlim([P1 P2])   ;
                else
                    if G.Settings.LogLevelsFlag
                        LY = log(single(G.Y)) ;
                    else
                        LY = G.Y ;
                    end
                    if G.Settings.SinLatFlag
                        SX = sind(G.X) ;
                    else
                        SX = G.X ;
                    end
                    if ~isempty(Defs.contour_levels(Defs.contour_levels>0))
                        [Cplus ,hplus ] =  contour(SX,LY,G.Data',Defs.contour_levels(Defs.contour_levels>0),'linewidth',LineWidth,'linecolor',G.GUI.colors.HoldGrey,'Tag','Hold','LineStyle','-') ; hold on
                    else
                        Cplus = [] ; hplus = [] ;
                    end
                    if ~isempty(Defs.contour_levels(Defs.contour_levels<0))
                        [Cminus,hminus] =  contour(SX,LY,G.Data',Defs.contour_levels(Defs.contour_levels<0),'linewidth',LineWidth,'linecolor',G.GUI.colors.HoldGrey,'Tag','Hold','LineStyle','--') ; hold on
                    else
                        Cminus = [] ; hminus = [] ;
                    end
                    if ~isempty(Defs.contour_levels(Defs.contour_levels==0))
                        [Czero,hzero] =  contour(SX,LY,G.Data',[0 0],'linewidth',LineWidth,'linecolor',[.2 .2 .2],'Tag','Hold') ; hold on
                    else
                        Czero = [] ; hzero = [] ;
                    end
                    
                    try
                        if any(G.Settings.LevelsIdx == [1,2,5,7])
                            axis ij
                        end
                    catch
                        axis ij
                    end
                    try
                        if G.Settings.ContourLevelStep
                            if  G.Settings.ContourLevelStep > (max(max(G.Data)) - min(min(G.Data)))/100 ;
                                if ~isempty(hplus)
                                    set(hplus,'levelstep',G.Settings.ContourLevelStep) ;
                                end
                                if ~isempty(hminus)
                                    set(hminus,'levelstep',G.Settings.ContourLevelStep) ;
                                end
                            end
                        end
                    catch
                        % Nevermind
                    end
                    %  set(hh,'linewidth',LineWidth,'linecolor',G.GUI.colors.HoldGrey,'Tag','Hold') ;
                    if isfield(Defs,'clabels') && ~isequal(Defs.clabels,'off')
                        if ~isempty(Cplus)
                            clplus = clabel(Cplus,'fontsize',12) ;
                            for i = 1:length(clplus)
                                set(clplus(i),'Tag','Hold') ;
                            end
                        end
                        if ~isempty(Cminus)
                            clminus = clabel(Cminus,'fontsize',12) ;
                            for i = 1:length(clminus)
                                set(clminus(i),'Tag','Hold') ;
                            end
                        end
                        if ~isempty(Czero)
                            clzero = clabel(Czero,'fontsize',12) ;
                            for i = 1:length(clzero)
                                set(clzero(i),'Tag','Hold') ;
                            end
                        end
                    else
                        clzero  = [] ;
                        clplus  = [] ;
                        clminus = [] ;
                    end
                    
                    
                    if G.Settings.LogLevelsFlag
                        ylim([log(single(P1)) log(single(P2))])
                    else
                        ylim([P1 P2]) ;
                    end
                    xlim([Lat1 Lat2])   ;
                    try
                        caxis(G.Hold.Clim) ;
                    catch
                        disp('GOAT_holdPlot warning: attempting to reset caxis to G.Hold.Clim failed.')
                    end
                    try
                        set(G.Display.colorbarHandle,'Ylim',G.Hold.colorbarLimits)
                    catch
                    disp('GOAT_holdPlot warning: attempting to reset colorbar Ylim to G.Hold.colorbarlimits failed.')
                end
                end
            catch
                disp 'error while attempting to create a contour plot in Hold mode'
            end
            if nargout > 1
                Cout.plus  = Cplus  ; Cout.minus  = Cminus  ; Cout.zero  = Czero ;
                hout.plus  = hplus  ; hout.minus  = hminus  ; hout.zero  = hzero ;
                Clout.plus = clplus ; Clout.minus = clminus ; Clout.zero = clzero ;
            end
        case 4 % Lon-Pressure
            try
                if G.Limits.Default.sameP
                    h = plot(G.X,G.Data,'--k','linewidth',LineWidth) ;
                    set(h,'Tag','Hold') ;
                    xlim([Lon1 Lon2])   ;
                elseif Lon1 == Lon2
                    h = plot(G.Y,G.Data,'--k','linewidth',LineWidth) ;
                    set(h,'Tag','Hold') ;
                    xlim([P1 P2])   ;
                else
                    if G.Settings.LogLevelsFlag
                        [Ch,hh] =  contour(G.X,log(single(G.Y)),G.Data') ;
                    else
                        [Ch,hh] =  contour(G.X,G.Y,G.Data') ;
                    end
                    
                    try
                        if any(G.Settings.LevelsIdx == [1,2,5,7])
                            axis ij
                        end
                    catch
                        axis ij
                    end
                    try
                        if G.Settings.ContourLevelStep
                            if  G.Settings.ContourLevelStep > (max(max(G.Data)) - min(min(G.Data)))/100 ;
                                set(hh,'levelstep',G.Settings.ContourLevelStep) ;
                            end
                        end
                    catch
                        % Nevermind
                    end
                    set(hh,'linewidth',LineWidth,'linecolor',G.GUI.colors.HoldGrey,'Tag','Hold') ;
                    cl = clabelgoat(Ch,hh,'fontsize',12,'textformat','%2.5g') ;
                    chandle = get(hh,'children') ;
                    location = 1 ;
                    a = 1 ;
                    levels = zeros(1,1) ;
                    while 1
                        levels(a) = Ch(1,location) ;
                        steps = Ch(2,location) ;
                        location = location + steps + 1 ;
                        if location > size(Ch,2)
                            break
                        else
                            a = a + 1 ;
                        end
                    end
                    levels = fliplr(levels) ;
                    set(chandle(levels<0),'LineStyle',':') ;
                    set(chandle(levels==0),'LineStyle',':','edgecolor',[.2 .2 .2]) ;
                    set(chandle(levels>0),'LineStyle','-') ;
                    for i = 1:length(cl)
                        set(cl(i),'Tag','Hold') ;
                    end
                    if G.Settings.LogLevelsFlag
                        ylim([log(single(P1)) log(single(P2))])
                    else
                        ylim([P1 P2])
                    end
                    xlim([Lon1 Lon2])   ;
                    caxis(G.Hold.Clim) ;
                    set(G.Display.colorbarHandle,'Ylim',G.Hold.colorbarLimits)
                end
            catch
                disp 'error while attempting to create a contour plot in Hold mode'
            end
        case 5 % Hovmoller Lon
            try
                if Lon1 == Lon2
                    h = plot(G.y,G.Data,'--k','linewidth',LineWidth) ;
                    set(h,'Tag','Hold') ;
                else
                    [Ch,hh] =  contour(G.X,G.Y,G.Data') ;
                    try
                        if G.Settings.ContourLevelStep
                            if  G.Settings.ContourLevelStep > (max(max(G.Data)) - min(min(G.Data)))/100 ;
                                set(hh,'levelstep',G.Settings.ContourLevelStep) ;
                            end
                        end
                    catch
                        % Nevermind
                    end
                    set(hh,'linewidth',LineWidth,'linecolor',G.GUI.colors.HoldGrey,'Tag','Hold') ;
                    cl = clabelgoat(Ch,hh,'fontsize',12,'textformat','%2.5g') ;
                    chandle = get(hh,'children') ;
                    location = 1 ;
                    a = 1 ;
                    levels = zeros(1,1) ;
                    while 1
                        levels(a) = Ch(1,location) ;
                        steps = Ch(2,location) ;
                        location = location + steps + 1 ;
                        if location > size(Ch,2)
                            break
                        else
                            a = a + 1 ;
                        end
                    end
                    levels = fliplr(levels) ;
                    set(chandle(levels<0),'LineStyle',':') ;
                    set(chandle(levels==0),'LineStyle',':','edgecolor',[.2 .2 .2]) ;
                    set(chandle(levels>0),'LineStyle','-') ;
                    for i = 1:length(cl)
                        set(cl(i),'Tag','Hold') ;
                    end
                    caxis(G.Hold.Clim) ;
                    set(G.Display.colorbarHandle,'Ylim',G.Hold.colorbarLimits)
                end
            catch
                disp 'error while attempting to create a contour plot in Hold mode'
            end
        case 6 % Hovmoller Lat
            try
                if Lat1 == Lat2
                    h = plot(G.y,G.Data,'--k','linewidth',LineWidth) ;
                    set(h,'Tag','Hold') ;
                else
                    if G.Settings.SinLatFlag
                        [Ch,hh] =  contour(sind(G.X),G.Y,G.Data') ;
                    else
                        [Ch,hh] =  contour(G.X,G.Y,G.Data') ;
                    end
                    try
                        if G.Settings.ContourLevelStep
                            if  G.Settings.ContourLevelStep > (max(max(G.Data)) - min(min(G.Data)))/100 ;
                                set(hh,'levelstep',G.Settings.ContourLevelStep) ;
                            end
                        end
                    catch
                        % Nevermind
                    end
                    set(hh,'linewidth',LineWidth,'linecolor',G.GUI.colors.HoldGrey,'Tag','Hold') ;
                    cl = clabelgoat(Ch,hh,'fontsize',12,'textformat','%2.5g') ;
                    chandle = get(hh,'children') ;
                    location = 1 ;
                    a = 1 ;
                    levels = zeros(1,1) ;
                    while 1
                        levels(a) = Ch(1,location) ;
                        steps = Ch(2,location) ;
                        location = location + steps + 1 ;
                        if location > size(Ch,2)
                            break
                        else
                            a = a + 1 ;
                        end
                    end
                    levels = fliplr(levels) ;
                    set(chandle(levels<0),'LineStyle',':') ;
                    set(chandle(levels==0),'LineStyle',':','edgecolor',[.2 .2 .2]) ;
                    set(chandle(levels>0),'LineStyle','-') ;
                    for i = 1:length(cl)
                        set(cl(i),'Tag','Hold') ;
                    end
                    caxis(G.Hold.Clim) ;
                    set(G.Display.colorbarHandle,'Ylim',G.Hold.colorbarLimits)
                end
            catch
                disp 'error while attempting to create a contour plot in Hold mode'
            end
        case 7 % Hovmoller P
            try
                if P2 == P1 || G.Limits.Default.sameP ;
                    h = plot(G.y,G.Data,'--k','linewidth',LineWidth) ;
                    set(h,'Tag','Hold') ;
                else
                    [Ch,hh] =  contour(G.X,G.Y,G.Data') ;
                    try
                        if G.Settings.ContourLevelStep
                            if  G.Settings.ContourLevelStep > (max(max(G.Data)) - min(min(G.Data)))/100 ;
                                set(hh,'levelstep',G.Settings.ContourLevelStep) ;
                            end
                        end
                    catch
                        % Nevermind
                    end
                    set(hh,'linewidth',LineWidth,'linecolor',G.GUI.colors.HoldGrey,'Tag','Hold') ;
                    cl = clabelgoat(Ch,hh,'fontsize',12,'textformat','%2.5g') ;
                    chandle = get(hh,'children') ;
                    location = 1 ;
                    a = 1 ;
                    levels = zeros(1,1) ;
                    while 1
                        levels(a) = Ch(1,location) ;
                        steps = Ch(2,location) ;
                        location = location + steps + 1 ;
                        if location > size(Ch,2)
                            break
                        else
                            a = a + 1 ;
                        end
                    end
                    levels = fliplr(levels) ;
                    set(chandle(levels<0),'LineStyle',':') ;
                    set(chandle(levels==0),'LineStyle',':','edgecolor',[.2 .2 .2]) ;
                    set(chandle(levels>0),'LineStyle','-') ;
                    for i = 1:length(cl)
                        set(cl(i),'Tag','Hold') ;
                    end
                    caxis(G.Hold.Clim) ;
                    set(G.Display.colorbarHandle,'Ylim',G.Hold.colorbarLimits)
                end
                
            catch
                disp 'error while attempting to create a contour plot in Hold mode'
            end
        case {8,9}
            Yz = [G.Y,G.Y(:,1)] ;
            Dataz = [G.Data ; G.Data(1,:)] ;
            if G.x(1) == 0
                Xz = [G.X,(G.X(:,1)+360-0.001)] ;
            else
                Xz = [G.X,G.X(:,1)-0.001] ;
            end
            hold on
            MaxFlag = 0 ;
            MinFlag = 0 ;
            if nanmax(Dataz(:)) >=0
                MaxFlag = 1 ;
                DataZPlus = Dataz  ; DataZPlus(DataZPlus<0) = nan ;
            end
            if nanmin(Dataz(:)) <0
                MinFlag = 1 ;
                DataZMinus = Dataz  ; DataZMinus(DataZMinus>=0) = nan ;
            end
            if MaxFlag
                [Cp , hp] = contourm(double(Yz),double(Xz),double(DataZPlus'),'color',[.45 .45 .45],'linewidth',LineWidth,'Tag','Hold') ; hold on % alpha(.8) ;
                try
                    [Ch , hh0] = contourm(double(Yz),double(Xz),double(Dataz'),[0 0],'color',[.45 .45 .45],'linewidth',LineWidth,'Tag','Hold') ; hold on % alpha(.8) ;
                    [Ch , hh] = contourm(double(Yz),double(Xz),double(Dataz'),[0 0],'color',[.25 .25 .25],'linewidth',LineWidth,'lineStyle',':','Tag','Hold') ; hold on % alpha(.8) ;
                end
            end
            if MinFlag
                [Cm , hm] = contourm(double(Yz),double(Xz),double(DataZMinus'),'color',[.3 .3 .3],'linewidth',LineWidth,'Tag','Hold') ; hold on % alpha(.8) ;
            end
            if MaxFlag
                clp = clabelm(Cp,hp) ;
                clh = clabelm(Ch,hh) ;
                %set(hp ,'Tag','Hold') ;
                %set(hh ,'Tag','Hold') ;
                for i = 1:length(clp)
                    set(clp(i),'Tag','Hold') ;
                end
                for i = 1:length(clh)
                    set(clh(i),'Tag','Hold') ;
                end
            end
            if MinFlag
                clm = clabelm(Cm,hm) ;
                %set(hm ,'Tag','Hold') ;
                for i = 1:length(clm)
                    set(clm(i),'Tag','Hold') ;
                end
            end
            caxis(G.Hold.Clim) ;
            set(G.Display.colorbarHandle,'Ylim',G.Hold.colorbarLimits)
    end
    
    
end

hold off ;

