% GOAT_plotUpdate_StreamLines plots quiver or streamlines
% suntax:
% >> GOAT_plotUpdate_StreamLines(G)
% Input parameters
% G.StreamLines.DecimationFactor_X = integer denoting decimation of x dimension
% G.StreamLines.DecimationFactor_Y = integer denoting decimation of y dimension
% G.StreamLines.TypeIdx = 1) plot using streamslice 2) plot using quiver
% G.StreamLines.quiverscale = set quiver scale for controling arrow proportions
% G.Data_u = zonal velocity 
% G.Data_v = meridional velocity 
% G.Data_w = vertical velocity 
%
% see also, GOAT_plotUpdate

function GOAT_plotUpdate_StreamLines(G)
Lat1 = min([G.lat(G.Limits.Default.Lat1),G.lat(G.Limits.Default.Lat2)]) ;
Lat2 = max([G.lat(G.Limits.Default.Lat1),G.lat(G.Limits.Default.Lat2)]) ;
if G.Settings.SinLatFlag && any(G.ProjectionType == [1,2,3,6])
    Lat1 = sind(Lat1) ;
    Lat2 = sind(Lat2) ;
end
try
    Lon1 = min(G.x) ; % min([G.lon(G.Limits.Default.Lon1),G.lon(G.Limits.Default.Lon2)]) ;
    Lon2 = max(G.x) ; % max([G.lon(G.Limits.Default.Lon1),G.lon(G.Limits.Default.Lon2)]) ;
catch
    Lon1 = min([G.lon(G.Limits.Default.Lon1),G.lon(G.Limits.Default.Lon2)]) ;
    Lon2 = max([G.lon(G.Limits.Default.Lon1),G.lon(G.Limits.Default.Lon2)]) ;
end
P1   = min([G.lev(G.Limits.Default.Lev1),G.lev(G.Limits.Default.Lev2)]) ;
P2   = max([G.lev(G.Limits.Default.Lev1),G.lev(G.Limits.Default.Lev2)]) ;

try
    if G.Plotting.GUI_PlotFlag
        subplot(1,10,[2 10])
    end
catch
    % Nevermind, PlotUpdate is used outside GUI
end

try
    ContourIdx = G.Settings.ContourIdx ;
catch
    ContourIdx = 1 ;
end
try
    LineWidth      = G.Settings.LineWidth      ;
    CoastLineWidth = G.Settings.CoastLineWidth ;
catch
    LineWidth      = 2 ;
    CoastLineWidth = 1 ;
end
if ContourIdx == 2 && LineWidth <=0
    LineWidth = 2 ;
end
try
    ColorMapStr = cell2mat(G.Settings.ColorMap(G.Settings.ColorMapIdx)) ;
    switch ColorMapStr
        case {'jet','hsv','Lines','Pink','Copper','Bone','Gray','Winter','Autumn','Spring','summer','Cool','Hot'}
            ColorMapDef = ColorMapStr ;
        otherwise
            eval(['ColorMapDef = G.Settings.ColorMaps.',ColorMapStr,' ; '])
    end
catch
    ColorMapDef = 'jet' ;
end

dc1 = G.StreamLines.DecimationFactor_X ;
dc2 = G.StreamLines.DecimationFactor_Y ;


switch G.ProjectionType
    case {1,2} % Long-Lat
        hold on ;
        if ~G.Settings.SinLatFlag
            switch G.StreamLines.TypeIdx
                case 1
                    h =  streamslice(double(G.X(1:dc2:size(G.X,1),1:dc1:size(G.X,2))),double(G.Y(1:dc2:size(G.X,1),1:dc1:size(G.X,2))),double(G.Data_u(1:dc1:size(G.X,2),1:dc2:size(G.X,1))'),double(G.Data_v(1:dc1:size(G.X,2),1:dc2:size(G.X,1))')) ;
                case 2
                    h =       quiver(double(G.X(1:dc2:size(G.X,1),1:dc1:size(G.X,2))),double(G.Y(1:dc2:size(G.X,1),1:dc1:size(G.X,2))),double(G.Data_u(1:dc1:size(G.X,2),1:dc2:size(G.X,1))'),double(G.Data_v(1:dc1:size(G.X,2),1:dc2:size(G.X,1))'),G.StreamLines.quiverscale) ;
            end
        else
            switch G.StreamLines.TypeIdx
                case 1
                    h =  streamslice(double(G.X(1:dc2:size(G.X,1),1:dc1:size(G.X,2))),sind(double(G.Y(1:dc2:size(G.X,1),1:dc1:size(G.X,2)))),double(G.Data_u(1:dc1:size(G.X,2),1:dc2:size(G.X,1))'),double(G.Data_v(1:dc1:size(G.X,2),1:dc2:size(G.X,1))')) ;
                case 2
                    h =       quiver(double(G.X(1:dc2:size(G.X,1),1:dc1:size(G.X,2))),sind(double(G.Y(1:dc2:size(G.X,1),1:dc1:size(G.X,2)))),double(G.Data_u(1:dc1:size(G.X,2),1:dc2:size(G.X,1))'),double(G.Data_v(1:dc1:size(G.X,2),1:dc2:size(G.X,1))'),G.StreamLines.quiverscale) ;
            end
        end
        set(h,'color',[.15 .15 .4]) ;
        %colormap(ColorMapDef) ; colorbar ;hold on
        %clabel(C,h,'fontsize',13) ;
%       set(h,'linewidth',LineWidth) ;
        if G.ProjectionType == 1
            if G.Settings.SinLatFlag
                if strcmpi(G.topo.CoastView,'CoastLine') ;
                    plot(G.topo.landcoverlon,sind(G.topo.landcoverlat),'k',G.topo.landcoverlon+360,sind(G.topo.landcoverlat),'k','Linewidth',CoastLineWidth) ;
                    %                     elseif strcmpi(G.topo.CoastView,'Mask') ;
                    %                         hold all ;
                    %                         G.topo.maskRI.YWorldLimits = sind(G.topo.maskRI.YWorldLimits) ;
                    %                         hm = imshow(G.topo.ocean_mask',G.topo.maskRI) ; alpha(.6) ;
                else
                    hold all
                    l = 1:G.topo.CoastNans(1)-1 ;
                    if ~isempty(G.topo.AntLonAppendix)
                        fh = fill([G.topo.landcoverlon(l) ; G.topo.AntLonAppendix],[sind(G.topo.landcoverlat(l)) ; sind(G.topo.AntLatAppendix)],G.topo.ShadeColor,[G.topo.landcoverlon(l) ; G.topo.AntLonAppendix]+360,[sind(G.topo.landcoverlat(l)) ; sind(G.topo.AntLatAppendix)],G.topo.ShadeColor,'edgecolor','none') ; alpha(fh,G.topo.alpha) ; hold on
                    else
                        fh = fill(G.topo.landcoverlon(l)-360,sind(G.topo.landcoverlat(l)),G.topo.ShadeColor,G.topo.landcoverlon(l),sind(G.topo.landcoverlat(l)),G.topo.ShadeColor,G.topo.landcoverlon(l)+360,sind(G.topo.landcoverlat(l)),G.topo.ShadeColor,'edgecolor','none') ; alpha(fh,G.topo.alpha) ; hold on
                    end
                    if length(G.topo.CoastNans) > 1
                        for i = 1:length(G.topo.CoastNans) -1
                            l = G.topo.CoastNans(i)+1:G.topo.CoastNans(i+1)-1 ;
                            if any(i == G.topo.LakesIdx)
                                fh = fill(G.topo.landcoverlon(l)-360,sind(G.topo.landcoverlat(l)),'w',G.topo.landcoverlon(l),sind(G.topo.landcoverlat(l)),'w',G.topo.landcoverlon(l)+360,sind(G.topo.landcoverlat(l)),'w','edgecolor','none') ; alpha(fh,G.topo.Lakealpha) ; hold on
                            else
                                fh = fill(G.topo.landcoverlon(l)-360,sind(G.topo.landcoverlat(l)),G.topo.ShadeColor,G.topo.landcoverlon(l),sind(G.topo.landcoverlat(l)),G.topo.ShadeColor,G.topo.landcoverlon(l)+360,sind(G.topo.landcoverlat(l)),G.topo.ShadeColor,'edgecolor','none') ; alpha(fh,G.topo.alpha) ; hold on
                            end
                        end
                        l = G.topo.CoastNans(end)+1:length(G.topo.landcoverlat) ;
                        fh = fill(G.topo.landcoverlon(l)-360,sind(G.topo.landcoverlat(l)),G.topo.ShadeColor,G.topo.landcoverlon(l),sind(G.topo.landcoverlat(l)),G.topo.ShadeColor,G.topo.landcoverlon(l)+360,sind(G.topo.landcoverlat(l)),G.topo.ShadeColor,'edgecolor','none') ; alpha(fh,G.topo.alpha) ; hold on
                    end
                end
                if CoastLineWidth > 0
                    plot(G.topo.landcoverlon-360,sind(G.topo.landcoverlat),'k',G.topo.landcoverlon,sind(G.topo.landcoverlat),'k',G.topo.landcoverlon+360,sind(G.topo.landcoverlat),'k','Linewidth',CoastLineWidth) ;
                end
                if G.topo.TypeFlag > 1
                    try
                        switch G.topo.TypeFlag
                            case 2
                                if Lon1 < 0
                                    [Ctopo, htopo] = contour([G.topo.lonG'-360 ; G.topo.lonG'],sind([G.topo.latG]) ,[G.topo.topoA , G.topo.topoA],G.topo.topovec) ; hold on
                                else
                                    [Ctopo, htopo] = contour(G.topo.lonG,sind(G.topo.latG),G.topo.topoA,G.topo.topovec) ; hold on
                                end
                            case 3
                                if Lon1 < 0
                                    [Ctopo, htopo] = contour([G.topo.lonG'-360 ; G.topo.lonG'],sind([G.topo.latG]) ,[G.topo.topoO , G.topo.topoO],G.topo.topovec) ; hold on
                                else
                                    [Ctopo, htopo] = contour(G.topo.lonG,sind(G.topo.latG),G.topo.topoO,G.topo.topovec) ; hold on
                                end
                            case 4
                                if Lon1 < 0
                                    [Ctopo, htopo] = contour([G.topo.lonG'-360 ; G.topo.lonG'],sind([G.topo.latG]) ,[G.topo.topoG , G.topo.topoG],G.topo.topovec) ; hold on
                                else
                                    [Ctopo, htopo] = contour(G.topo.lonG,sind(G.topo.latG),G.topo.topoG,G.topo.topovec) ; hold on
                                end
                        end
                        set(htopo,'linewidth',G.topo.TopotLineWidthIdx/2,'color',[.25 .25 .25]) ;
                    catch
                        % Null topography
                    end
                end
            else
                if strcmpi(G.topo.CoastView,'CoastLine') ;
                    plot(G.topo.landcoverlon,G.topo.landcoverlat,'k',G.topo.landcoverlon+360,G.topo.landcoverlat,'k','Linewidth',CoastLineWidth) ;
                    %                     elseif strcmpi(G.topo.CoastView,'Mask') ;
                    %                         hold all ;
                    %                         hm = imshow(G.topo.ocean_mask',G.topo.maskRI) ; alpha(.6) ;
                else
                    hold all
                    l = 1:G.topo.CoastNans(1)-1 ;
                    if ~isempty(G.topo.AntLonAppendix)
                        fh = fill([G.topo.landcoverlon(l) ; G.topo.AntLonAppendix],[G.topo.landcoverlat(l) ; G.topo.AntLatAppendix],G.topo.ShadeColor,[G.topo.landcoverlon(l) ; G.topo.AntLonAppendix]+360,[G.topo.landcoverlat(l) ; G.topo.AntLatAppendix],G.topo.ShadeColor,'edgecolor','none') ; alpha(fh,G.topo.alpha) ; hold on
                    else
                        fh = fill(G.topo.landcoverlon(l)-360,G.topo.landcoverlat(l),G.topo.ShadeColor,G.topo.landcoverlon(l),G.topo.landcoverlat(l),G.topo.ShadeColor,G.topo.landcoverlon(l)+360,G.topo.landcoverlat(l),G.topo.ShadeColor,'edgecolor','none') ; alpha(fh,G.topo.alpha) ; hold on
                    end
                    if length(G.topo.CoastNans) > 1
                        for i = 1:length(G.topo.CoastNans) -1
                            l = G.topo.CoastNans(i)+1:G.topo.CoastNans(i+1)-1 ;
                            if any(i == G.topo.LakesIdx)
                                fh = fill(G.topo.landcoverlon(l)-360,G.topo.landcoverlat(l),'w',G.topo.landcoverlon(l),G.topo.landcoverlat(l),'w',G.topo.landcoverlon(l)+360,G.topo.landcoverlat(l),'w','edgecolor','none') ; alpha(fh,G.topo.Lakealpha) ; hold on
                            else
                                fh = fill(G.topo.landcoverlon(l)-360,G.topo.landcoverlat(l),G.topo.ShadeColor,G.topo.landcoverlon(l),G.topo.landcoverlat(l),G.topo.ShadeColor,G.topo.landcoverlon(l)+360,G.topo.landcoverlat(l),G.topo.ShadeColor,'edgecolor','none') ; alpha(fh,G.topo.alpha) ; hold on
                            end
                        end
                        l = G.topo.CoastNans(end)+1:length(G.topo.landcoverlat) ;
                        fh = fill(G.topo.landcoverlon(l)-360,G.topo.landcoverlat(l),G.topo.ShadeColor,G.topo.landcoverlon(l),G.topo.landcoverlat(l),G.topo.ShadeColor,G.topo.landcoverlon(l)+360,G.topo.landcoverlat(l),G.topo.ShadeColor,'edgecolor','none') ; alpha(fh,G.topo.alpha) ; hold on
                    end
                end
                if CoastLineWidth > 0
                    plot(G.topo.landcoverlon-360,G.topo.landcoverlat,'k',G.topo.landcoverlon,G.topo.landcoverlat,'k',G.topo.landcoverlon+360,G.topo.landcoverlat,'k','Linewidth',CoastLineWidth) ;
                end
                if G.topo.TypeFlag > 1
                    try
                        switch G.topo.TypeFlag
                            case 2
                                if Lon1 < 0
                                    [Ctopo, htopo] = contour([G.topo.lonG'-360 ; G.topo.lonG'],[G.topo.latG] ,[G.topo.topoA , G.topo.topoA],G.topo.topovec) ; hold on
                                else
                                    [Ctopo, htopo] = contour(G.topo.lonG,G.topo.latG,G.topo.topoA,G.topo.topovec) ; hold on
                                end
                            case 3
                                if Lon1 < 0
                                    [Ctopo, htopo] = contour([G.topo.lonG'-360 ; G.topo.lonG'],[G.topo.latG] ,[G.topo.topoO , G.topo.topoO],G.topo.topovec) ; hold on
                                else
                                    [Ctopo, htopo] = contour(G.topo.lonG,G.topo.latG,G.topo.topoO,G.topo.topovec) ; hold on
                                end
                            case 4
                                if Lon1 < 0
                                    [Ctopo, htopo] = contour([G.topo.lonG'-360 ; G.topo.lonG'],[G.topo.latG] ,[G.topo.topoG , G.topo.topoG],G.topo.topovec) ; hold on
                                else
                                    [Ctopo, htopo] = contour(G.topo.lonG,G.topo.latG,G.topo.topoG,G.topo.topovec) ; hold on
                                end
                        end
                        set(htopo,'linewidth',G.topo.TopotLineWidthIdx/2,'color',[.25 .25 .25]) ;
                    catch
                        % Null topography
                    end
                end
            end
        else
            if G.Settings.SinLatFlag
                if CoastLineWidth > 0
                    plot(G.topo.landcoverlon-360,sind(G.topo.landcoverlat),'k',G.topo.landcoverlon,sind(G.topo.landcoverlat),'k',G.topo.landcoverlon+360,sind(G.topo.landcoverlat),'k','Linewidth',CoastLineWidth) ;
                end
                if G.topo.TypeFlag > 1 && CoastLineWidth > 0
                    try
                        switch G.topo.TypeFlag
                            case 2
                                if Lon1 < 0
                                    [Ctopo, htopo] = contour([G.topo.lonG'-360 ; G.topo.lonG'],sind([G.topo.latG]) ,[G.topo.topoA , G.topo.topoA],G.topo.topovec) ; hold on
                                else
                                    [Ctopo, htopo] = contour(G.topo.lonG,sind(G.topo.latG),G.topo.topoA,G.topo.topovec) ; hold on
                                end
                            case 3
                                if Lon1 < 0
                                    [Ctopo, htopo] = contour([G.topo.lonG'-360 ; G.topo.lonG'],sind([G.topo.latG]) ,[G.topo.topoO , G.topo.topoO],G.topo.topovec) ; hold on
                                else
                                    [Ctopo, htopo] = contour(G.topo.lonG,sind(G.topo.latG),G.topo.topoO,G.topo.topovec) ; hold on
                                end
                            case 4
                                if Lon1 < 0
                                    [Ctopo, htopo] = contour([G.topo.lonG'-360 ; G.topo.lonG'],sind([G.topo.latG]) ,[G.topo.topoG , G.topo.topoG],G.topo.topovec) ; hold on
                                else
                                    [Ctopo, htopo] = contour(G.topo.lonG,sind(G.topo.latG),G.topo.topoG,G.topo.topovec) ; hold on
                                end
                        end
                        set(htopo,'linewidth',G.topo.TopotLineWidthIdx/2,'color',[.25 .25 .25]) ;
                    catch
                        % Null topography
                    end
                end
            else
                if CoastLineWidth > 0
                    plot(G.topo.landcoverlon,G.topo.landcoverlat,'k',G.topo.landcoverlon+360,G.topo.landcoverlat,'k','Linewidth',CoastLineWidth) ;
                end
                if G.topo.TypeFlag > 1 && CoastLineWidth > 0
                    try
                        switch G.topo.TypeFlag
                            case 2
                                if Lon1 < 0
                                    [Ctopo, htopo] = contour([G.topo.lonG'-360 ; G.topo.lonG'],[G.topo.latG] ,[G.topo.topoA , G.topo.topoA],G.topo.topovec) ; hold on
                                else
                                    [Ctopo, htopo] = contour(G.topo.lonG,G.topo.latG,G.topo.topoA,G.topo.topovec) ; hold on
                                end
                            case 3
                                if Lon1 < 0
                                    [Ctopo, htopo] = contour([G.topo.lonG'-360 ; G.topo.lonG'],[G.topo.latG] ,[G.topo.topoO , G.topo.topoO],G.topo.topovec) ; hold on
                                else
                                    [Ctopo, htopo] = contour(G.topo.lonG,G.topo.latG,G.topo.topoO,G.topo.topovec) ; hold on
                                end
                            case 4
                                if Lon1 < 0
                                    [Ctopo, htopo] = contour([G.topo.lonG'-360 ; G.topo.lonG'],[G.topo.latG] ,[G.topo.topoG , G.topo.topoG],G.topo.topovec) ; hold on
                                else
                                    [Ctopo, htopo] = contour(G.topo.lonG,G.topo.latG,G.topo.topoG,G.topo.topovec) ; hold on
                                end
                        end
                        set(htopo,'linewidth',G.topo.TopotLineWidthIdx/2,'color',[.25 .25 .25]) ;
                    catch
                        % Null topography
                    end
                end
            end
        end
        xlim([Lon1 Lon2])   ;
        ylim([Lat1 Lat2])   ;
        
    case 3 % Lat-Pressure
        try
            if ~G.Settings.SinLatFlag
                switch G.StreamLines.TypeIdx
                    case 1
                        h =  streamslice(double(G.X(1:dc2:size(G.X,1),1:dc1:size(G.X,2))),double(G.Y(1:dc2:size(G.X,1),1:dc1:size(G.X,2))),double(G.Data_v(1:dc1:size(G.X,2),1:dc2:size(G.X,1))'),double(G.Data_w(1:dc1:size(G.X,2),1:dc2:size(G.X,1))')) ;
                    case 2
                        h =       quiver(double(G.X(1:dc2:size(G.X,1),1:dc1:size(G.X,2))),double(G.Y(1:dc2:size(G.X,1),1:dc1:size(G.X,2))),double(G.Data_v(1:dc1:size(G.X,2),1:dc2:size(G.X,1))'),double(G.Data_w(1:dc1:size(G.X,2),1:dc2:size(G.X,1))'),G.StreamLines.quiverscale) ;
                end
            else
                switch G.StreamLines.TypeIdx
                    case 1
                        h =  streamslice(sind(double(G.X(1:dc2:size(G.X,1),1:dc1:size(G.X,2)))),double(G.Y(1:dc2:size(G.X,1),1:dc1:size(G.X,2))),double(G.Data_v(1:dc1:size(G.X,2),1:dc2:size(G.X,1))'),double(G.Data_w(1:dc1:size(G.X,2),1:dc2:size(G.X,1))')) ;
                    case 2
                        h =       quiver(sind(double(G.X(1:dc2:size(G.X,1),1:dc1:size(G.X,2)))),double(G.Y(1:dc2:size(G.X,1),1:dc1:size(G.X,2))),double(G.Data_v(1:dc1:size(G.X,2),1:dc2:size(G.X,1))'),double(G.Data_w(1:dc1:size(G.X,2),1:dc2:size(G.X,1))'),G.StreamLines.quiverscale) ;
                end
            end
            set(h,'color',[.1 .1 .5]) ;
            %colormap(ColorMapDef) ; colorbar ;
            set(h,'linewidth',G.Settings.LineWidth) ;
            axis ij
            xlabel('$$\textrm{Latitude}$$','Fontsize',13,'interpreter','latex') ;
            ylabel('$$\textrm{Level}$$','Fontsize',13,'interpreter','latex') ;
            ylim([P1 P2])
            xlim([Lat1 Lat2])   ;
            
        catch
            % Nevermind, no Data
        end
    case 4 % Lon-Pressure
        try
            switch G.StreamLines.TypeIdx
                case 1
                    h =  streamslice(double(G.X(1:dc2:size(G.X,1),1:dc1:size(G.X,2))),double(G.Y(1:dc2:size(G.X,1),1:dc1:size(G.X,2))),double(G.Data_u(1:dc1:size(G.X,2),1:dc2:size(G.X,1))'),double(G.Data_w(1:dc1:size(G.X,2),1:dc2:size(G.X,1))')) ;
                case 2
                    h =       quiver(double(G.X(1:dc2:size(G.X,1),1:dc1:size(G.X,2))),double(G.Y(1:dc2:size(G.X,1),1:dc1:size(G.X,2))),double(G.Data_u(1:dc1:size(G.X,2),1:dc2:size(G.X,1))'),double(G.Data_w(1:dc1:size(G.X,2),1:dc2:size(G.X,1))'),G.StreamLines.quiverscale) ;
            end
            set(h,'color',[.1 .1 .5]) ;
            %colormap(ColorMapDef) ; colorbar ;
            set(h,'linewidth',LineWidth) ;
            axis ij
            xlabel('$$\textrm{Longitude}$$','Fontsize',13,'interpreter','latex') ;
            ylabel('$$\textrm{Level}$$','Fontsize',13,'interpreter','latex') ;
            ylim([P1 P2])
            xlim([Lon1 Lon2])   ;
        catch
            % Nevermind, no Data
        end
end
% Axis Configurations
try
    if G.Settings.XgridFlag
        set(gca,'XGrid','on') ;
    end
    if G.Settings.YgridFlag
        set(gca,'YGrid','on') ;
    end
end
try
    if G.Settings.FixZLimitsFlag
        caxis([G.Settings.ZlimitMin G.Settings.ZlimitMax]) ;
        colorbar('fontname',G.GUI.Fonts.gcaFont) ;
    end
end
set(gca,'fontsize',12,'fontname',G.GUI.Fonts.gcaFont) ;
hold off ;
