% convert data string vec to data string.
% Copyright (C) Ori Adam Aug.2013. compatible with version 2013i or later.
% suntax: Datestr = datastring2date(datastring,dateformat)
% available formats:
% 'linear'
% 'mmm'
% 'mmm.dd'
% 'yy.mmm'
% 'yy.mmm.dd'
% 'yy.mm.dd'
% 'yyyy.mmm.dd'
% 'yyyy.mm.dd'
% 'yyyy.ddd'  - Julian
%
% see also datastringvec2date, GOAT_serial2date

function Datestr = datastring2date(datastring,dateformat)
try
    [path,name] = fileparts(cell2mat(datastring)) ;
catch
    [path,name] = fileparts(datastring) ;
end
if strcmp(dateformat,'linear')
    Datestr = name ;
    return ;
elseif strncmpi(dateformat,'Julian',6)
    dateformat = dateformat(8:end)             ;
    DayNum = round(str2double(name))           ;
    year   = round(str2double(path))           ;
    month = 0                 ;
    day   = floor(DayNum/100) ;
    hour   = DayNum - day*100 ;
else
    DayNum = round(str2double(name))           ;
    year   = round(str2double(path))           ;
    month  = floor(DayNum/10000)               ;
    day    = floor((DayNum - month*10000)/100) ;
    hour   = DayNum - month*10000 - day*100    ;
end
    switch dateformat
        case 'mmm'
            switch month
                case 1
                    Datestr = 'Jan' ;
                case 2
                    Datestr = 'Feb' ;
                case 3
                    Datestr = 'Mar' ;
                case 4
                    Datestr = 'Apr' ;
                case 5
                    Datestr = 'May' ;
                case 6
                    Datestr = 'Jun' ;
                case 7
                    Datestr = 'Jul' ;
                case 8
                    Datestr = 'Aug' ;
                case 9
                    Datestr = 'Sep' ;
                case 10
                    Datestr = 'Oct' ;
                case 11
                    Datestr = 'Nov' ;
                case 12
                    Datestr = 'Dec' ;
            end
        case 'mmm.dd'
            switch month
                case 1
                    Datestr = 'Jan' ;
                case 2
                    Datestr = 'Feb' ;
                case 3
                    Datestr = 'Mar' ;
                case 4
                    Datestr = 'Apr' ;
                case 5
                    Datestr = 'May' ;
                case 6
                    Datestr = 'Jun' ;
                case 7
                    Datestr = 'Jul' ;
                case 8
                    Datestr = 'Aug' ;
                case 9
                    Datestr = 'Sep' ;
                case 10
                    Datestr = 'Oct' ;
                case 11
                    Datestr = 'Nov' ;
                case 12
                    Datestr = 'Dec' ;
            end
            if day ~= 0
                Datestr = [Datestr,'.',num2str(day,'%02.0f')] ;
            end
        case 'yy.mmm'
            switch month
                case 1
                    Datestr = 'Jan' ;
                case 2
                    Datestr = 'Feb' ;
                case 3
                    Datestr = 'Mar' ;
                case 4
                    Datestr = 'Apr' ;
                case 5
                    Datestr = 'May' ;
                case 6
                    Datestr = 'Jun' ;
                case 7
                    Datestr = 'Jul' ;
                case 8
                    Datestr = 'Aug' ;
                case 9
                    Datestr = 'Sep' ;
                case 10
                    Datestr = 'Oct' ;
                case 11
                    Datestr = 'Nov' ;
                case 12
                    Datestr = 'Dec' ;
            end
            Datestr = [num2str(year-round(year/100)*100,'%02.0f'),'.',Datestr] ;
        case 'yy.mmm.dd'
            switch month
                case 1
                    Datestr = 'Jan' ;
                case 2
                    Datestr = 'Feb' ;
                case 3
                    Datestr = 'Mar' ;
                case 4
                    Datestr = 'Apr' ;
                case 5
                    Datestr = 'May' ;
                case 6
                    Datestr = 'Jun' ;
                case 7
                    Datestr = 'Jul' ;
                case 8
                    Datestr = 'Aug' ;
                case 9
                    Datestr = 'Sep' ;
                case 10
                    Datestr = 'Oct' ;
                case 11
                    Datestr = 'Nov' ;
                case 12
                    Datestr = 'Dec' ;
            end
            if day>0
                Datestr = [num2str(year-round(year/100)*100,'%02.0f'),'.',Datestr,'.',num2str(day,'%02.0f')] ;
            else
                Datestr = [num2str(year-round(year/100)*100,'%02.0f'),'.',Datestr] ;
            end
        case 'yyyy.mmm.dd'
            switch month
                case 1
                    Datestr = 'Jan' ;
                case 2
                    Datestr = 'Feb' ;
                case 3
                    Datestr = 'Mar' ;
                case 4
                    Datestr = 'Apr' ;
                case 5
                    Datestr = 'May' ;
                case 6
                    Datestr = 'Jun' ;
                case 7
                    Datestr = 'Jul' ;
                case 8
                    Datestr = 'Aug' ;
                case 9
                    Datestr = 'Sep' ;
                case 10
                    Datestr = 'Oct' ;
                case 11
                    Datestr = 'Nov' ;
                case 12
                    Datestr = 'Dec' ;
            end
            if day>0
                Datestr = [num2str(year,'%04.0f'),'.',Datestr,'.',num2str(day,'%02.0f')] ;
            else
                Datestr = [num2str(year,'%04.0f'),'.',Datestr] ;
            end
        case 'yy.mm.dd'
            if day>0
                Datestr = [num2str(year-round(year/100)*100,'%02.0f'),'.',num2str(month,'%02.0f'),'.',num2str(day,'%02.0f')] ;
            else
                Datestr = [num2str(year-round(year/100)*100,'%02.0f'),'.',num2str(month,'%02.0f')] ;
            end
        case 'yyyy.mm.dd'
            if day>0
                Datestr = [num2str(year,'%04.0f'),'.',num2str(month,'%02.0f'),'.',num2str(day,'%02.0f')] ;
            else
                Datestr = [num2str(year,'%04.0f'),'.',num2str(month,'%02.0f')] ;
            end
        case 'yyyy.ddd'    
            Datestr = [num2str(year,'%04.0f'),'.',num2str(day,'%03.0f')] ;
    end
end