% GOAT_Install  GOAT installation code. can also be used as a GOAT 'rehash' function.
% copyright (C) Ori Adam, Oct.2013
% This script adds appropriate paths and some shortcuts to at the shortcut tab, under the category 'GOAT'
% the shortcuts are:
% GOAT  : launch GOAT
% nc2mat: a general funcrion for importing *.nc/netcdf/cdf data to a structure 'Data'. It uses the GOAT function Data = GOAT_nc2mat(filename)
% rgb   : a simple gui for visualizing [r g b] combinations.
% goat-geo.org: a link to the official GOAT website, temporarily hosted at oriadam.info
% DataDownload: an auto data download tool that installs data in GOAT format
% GOAT_Update: retreive latest version from server and update version without affecting personal virtual fields, Scripts and MyDefaults

% Add Paths
cdPath = fileparts(cd) ;
[GOATpath,GOATname] = fileparts(cdPath) ;
if ~strcmp(GOATname,'GOAT')
    disp 'Could not install GOAT. Run this fucntion from the ... GOAT/Installation folder.'
else
    disp 'adding paths:'
    disp(['addpath(genpath(',fullfile(cdPath,'Code'),'))  '])    ;
    disp(['addpath(genpath(',fullfile(cdPath,'Scripts'),'))'])  ;
    disp(['addpath(genpath(',fullfile(cdPath,'MyScripts'),'))'])  ;
    addpath(genpath(fullfile(cdPath,'Code')))     ;
    addpath(genpath(fullfile(cdPath,'Scripts')))  ;
    addpath(genpath(fullfile(cdPath,'MyScripts')))  ;
    disp '  '
    disp 'saving paths.'
    savepath ;
    disp '  '
    % Remove existing Shortcuts
    %RemoveShortcuts('','GOAT') ;
    %RemoveShortcuts('','nc2mat') ;
    %RemoveShortcuts('','rgb') ;
    %RemoveShortcuts('','goat-geo.org') ;
    %RemoveShortcuts('','DataDownload') ;
    %RemoveShortcuts('GOAT','GOAT') ;
    %RemoveShortcuts('GOAT','nc2mat') ;
    %RemoveShortcuts('GOAT','rgb') ;
    %RemoveShortcuts('GOAT','goat-geo.org') ;
    %RemoveShortcuts('GOAT','DataDownload') ;
    
    % Add new Shortcuts
    
    try
        callback = ['cd ''',fullfile(cdPath,'Code'),''' ;  GOAT ;' ] ;
        AddShortcut('GOAT',callback,fullfile(cdPath,'Code','Art','GoatBackground.jpg'),'GOAT') ;
        disp 'GOAT shortcut added to category: GOAT'
        disp 'Pressing the ''GOAT'' shortcut runs GOAT'
        disp '  '
    end
%     try
%         callback = '[fid, fidf] = uigetfile({''*.*'',''All Files''}) ; Data = GOAT_nc2mat([fidf,fid]) ; ' ;
%         AddShortcut('nc2mat',callback,'','GOAT') ;
%         disp 'nc2mat shortcut added to category: GOAT'
%         disp 'Pressing the ''nc2mat'' shortcut allows you to browse for a file and convert it to Matlab format using the callback'
%         disp(callback) ; 
%         disp '  '
%     end
    try
        AddShortcut('goat-geo.org','web(''http://www.goat-geo.org'',''-browser'')','','GOAT') ;
        disp '''www.goat-geo.org'' link added to category: GOAT'
        disp '  '
    end
    try
        AddShortcut('DataDownload',['cd ''',fullfile(cdPath,'Code'),''' ; GOAT_AutoDownload'],'','GOAT') ;
        disp 'DataDownload shortcut added to category: GOAT'
        disp 'Pressing the ''DataDownload'' shortcut runs the automated data download utility GOAT_AutoDownload'
        disp '  '
    end
    try
        callback = ['cd ''',fullfile(cdPath,'Code'),''' ;  GOAT_VersionUpdateCode ; ' ] ;
        AddShortcut('GOAT_Update',callback,'','GOAT') ;
        disp 'GOAT_Update shortcut added to category: GOAT'
        disp 'Pressing the ''GOAT_Update'' shortcut runs an auto update code.'
        disp 'Press this shortcut to automatically update to the latest version instead of reinstalling.' ; 
    end
    CurrentMatlabVersion = version('-release') ;
    VersionYear = str2num(CurrentMatlabVersion(1:4)) ;
    if VersionYear < 2013
        Quest = sprintf('Would you like to add quick access shortcuts?') ;
        button = questdlg(Quest,'Auto Update','Yes','No Thanks','Yes') ;
        if button
            try
            RemoveShortcuts('','GOAT') ;
            RemoveShortcuts('','nc2mat') ;
            RemoveShortcuts('','DataDownload') ;
            RemoveShortcuts('','GOAT_Update') ;
            end
            try
                callback = ['cd ''',fullfile(cdPath,'Code'),''' ;  GOAT ;' ] ;
                AddShortcut('GOAT',callback,fullfile(cdPath,'Code','Art','GoatBackground.jpg'),'') ;
                disp 'GOAT shortcut added to quick access'
            catch
                disp  'failed to add a ''GOAT'' shortcut. To add the shortcut highlight the callback text below and drag it to the shortcut toolbar'
                disp(callback)
                disp '  '
            end
%             try
%                 callback = '[fid, fidf] = uigetfile({''*.*'',''All Files''}) ; Data = GOAT_nc2mat([fidf,fid]) ; ' ;
%                 AddShortcut('nc2mat',callback,'','') ;
%                 disp 'nc2mat shortcut added to quick access'
%             catch
%                 disp  'failed to add a ''nc2mat'' shortcut. To add the shortcut highlight the callback text below and drag it to the shortcut toolbar'
%                 disp(callback)
%                 disp '  '
%             end
            try
                AddShortcut('DataDownload',['cd ''',fullfile(cdPath,'Code'),''' ; GOAT_AutoDownload'],'','') ;
                disp 'DataDownload shortcut added to quick access'
            catch
                disp  'failed to add a ''DataDownload'' shortcut. To add the shortcut highlight the callback text below and drag it to the shortcut toolbar'
                disp(['cd ''',fullfile(cdPath,'Code'),''' ; GOAT_AutoDownload'])
                disp '  '
            end
            try
                callback = ['cd ''',fullfile(cdPath,'Code'),''' ;  GOAT_VersionUpdateCode ; ' ] ;
                AddShortcut('GOAT_Update',callback,'','') ;
                disp 'GOAT_Update shortcut added to quick access'
            catch
                disp  'failed to add a ''GOAT_Update'' shortcut. To add the shortcut highlight the callback text below and drag it to the shortcut toolbar'
                disp(callback)
                disp '  '
            end
        end
    end
    


disp 'installation complete.'
disp '  '
disp 'to launch GOAT, press the GOAT shortcut at the shortcuts tab, or change directory to'
disp(fullfile(cdPath,'Code'))     ;
disp 'and type GOAT at the command-line' 
disp '  '
disp 'If GOAT does not launch properly try the following:'
disp '1. make sure you are installing from the . . . GOAT/Install  directory'
disp '2. refresh paths by typing rehash at the command-line'
disp '3. restart MATLAB'
disp '4. manually add and save the following paths with all subfolders: '
disp(['     ',fullfile(cdPath,'Code')])       ;
disp(['     ',fullfile(cdPath,'Scripts')])    ;
disp(['     ',fullfile(cdPath,'MyScripts')])  ;
disp '  '
disp 'Thank you for using GOAT'
disp '  '

clear callback cdPath CurrentMatlabVersion VersionYear button
end