% GOAT_Examples_TimeSeriesSurvey 
% This example shows how to survey a timeseries/climatology of a given field for various datasets 
G = GOAT_Environment('../WORK/Project_WorkFolder') ; 
%%
DataSet = 'NCEP-I' ;
% Specify dataset folder and Time Resolution 
switch DataSet
    case 'NCEP-I'
      DataFolder = '/Users/OriAdam/GOAT/Data/NCEP-I' ; 
      TimeResolution = 'Daily' ;    
    case  'ERA40'
      DataFolder = '/Volumes/data1/OriAdam/GOAT/Data/ERA40' ;
      TimeResolution = 'DailyX4' ; 
    case  'HadGEM2-ES_historical_1'
      DataFolder = '/Volumes/data1/OriAdam/GOAT/CMIP5Data/historical/HadGEM2-ES/r1i1p1' ;
      TimeResolution = 'Monthly' ;    
end
% Initialize variable for the given spatial and temporal resolution
G = GOAT_Initialize_Dataset(G,DataFolder,TimeResolution) ; % Go through contents of Data folder. identify existing fields, temporal span, etc.
G = GOAT_Initialize_Field(G,'pr')        ; % get spatial and temporal dimensions and display settings                                              
%
ExportDir = ['../WORK/Project_WorkFolder/',DataSet] ; mkdir(ExportDir) ; % Pre-Define a folder to which results are exported

% Projection type and definitions
G.ProjectionType = 1 ; % 1) lat-lon with coast 2) lat-lon 3) lat-height 4)lon-height 5) hovmoller-lon 6) hovmoller-lat 7) hovmoller height         
G.Limits.Default.sameP = 1      ; % Projection 1,2) 1/0 set level2 = level1 | Projection 3,4,7) 1/0 composite levels. Irrelevant for surface of top-of-atmosphere fields                               
G.Limits.Default.VerMeanInt = 1 ; % 1) Vertical Mean 2) Integral using Boer Beta 3) set Nan values to zero and vertically integrate                
G.Settings.NanMeanFlag = 0  ; % use mean(x) when 0 and nanmean(x) when 1 
% Spatial Extent
Lat1    = -30  ; G.Limits.Default.Lat1 = dsearchn(G.lat,Lat1)  ;
Lat2    = 30   ; G.Limits.Default.Lat2 = dsearchn(G.lat,Lat2)  ;
Lon1    = 70   ; G.Limits.Default.Lon1 = dsearchn(G.lon,Lon1)  ;
Lon2    = 220  ; G.Limits.Default.Lon2 = dsearchn(G.lon,Lon2)  ;
% Lev1   = 100  ; G.Limits.Default.Lev1   = dsearchn(G.lev,Lev1)   ; % if commented all available span is used
% Lev2   = 1000 ; G.Limits.Default.Lev2   = dsearchn(G.lev,Lev2)   ; % if commented all available span is used
G.DataLimits.Lon1 = min([G.Limits.Default.Lon1,G.Limits.Default.Lon2]) ;
G.DataLimits.Lon2 = max([G.Limits.Default.Lon1,G.Limits.Default.Lon2]) ;
G.DataLimits.Lat1 = min([G.Limits.Default.Lat1,G.Limits.Default.Lat2]) ;
G.DataLimits.Lat2 = max([G.Limits.Default.Lat1,G.Limits.Default.Lat2]) ;
G.DataLimits.Lev1 = min([G.Limits.Default.Lev1,G.Limits.Default.Lev2]) ;
G.DataLimits.Lev2 = max([G.Limits.Default.Lev1,G.Limits.Default.Lev2]) ;
G.DataLimits.p    = double(G.P(G.DataLimits.Lev1:G.DataLimits.Lev2))/double(G.P(G.DataLimits.Lev2) - G.P(G.DataLimits.Lev1))       ; % Weighted Pressure In Case of vertical Averaging
% Filters
% Create Time Series from spatial and temporal definitions
G.Filter.Index = ones(length(G.Addresses.Default),1) ; % Default Index (all ones)
G.Filter.WeightIdx     = 1 ; % 1: None | 2: -1 | 3: cos | 4: -cos | 5: 1/(a*cos) | 6: -1/(a*cos)
G.Filter.weight = WeightFunc(G)  ;
% Temporal definitions
switch DataSet
    case 'NCEP-I'
        year1  = 1980 ;  month1 = 1   ;   day1 = 1   ;  hour1  = 0   ;
        year2  = 2000 ;  month2 = 12  ;   day2 = 31  ;  hour2  = 0   ;
    case 'HadGEM2-ES_historical_1'
        year1  = 1980 ;  month1 = 1   ;   day1 = 1   ;  hour1  = 0   ;
        year2  = 2000 ;  month2 = 12  ;   day2 = 31  ;  hour2  = 0   ;
    case 'ERA40'
        year1  = 1957 ;  month1 = 9   ;   day1 = 1   ;  hour1  = 0   ;
        year2  = 2001 ;  month2 = 8   ;   day2 = 31  ;  hour2  = 18  ;
end
G.Limits.Default.Sequential = 0        ; % 1/0 1) Simple Mean  ;   0) Seasonal Average
G.Limits.Default.Day2same   = 0        ; % A flag for setting Date 1 equal to date 2
G.Limits.S1 = GOAT_getSerial(year1,month1,day1,hour1,G.TimeField) ;
G.Limits.S2 = GOAT_getSerial(year2,month2,day2,hour2,G.TimeField) ;
G.DataStringVec   = GOAT_Data_string_vec(G) ; % A vector of the Time Series File Indices
% Calculate Time Series and climatology
G = GOAT_Compile_data_Climatology(G) ; % calculated climatology (daily or monthly) of the truncated data
pr_Clim = G.DataClim ; 
G.Limits.Default.DataSeriesAnomFlag = 1 ; % if zero anomaly is not calculated. Flag can only be 1 if 'GOAT_Compile_data_Climatology' has been run first.
G = GOAT_Compile_data_Series(G) ; % produce a time series of the truncated data 
pr      = G.DataSeries     ; %
pr_Anom = G.DataSeriesAnom ; % 'DataSeriesAnom' field exists only if 'G.Limits.Default.DataSeriesAnomFlag' is set to 1 and 'GOAT_Compile_data_Climatology' has be run  
%% produce temporal vectors that can be used for indexing
YearsVec  = zeros(length(G.DataStringVec),1) ;
MonthsVec = zeros(length(G.DataStringVec),1) ;
DaysVec   = zeros(length(G.DataStringVec),1) ;
HoursVec  = zeros(length(G.DataStringVec),1) ;
NF   = length(G.DataStringVec) ;
for i = 1:NF
    [YearsVec(i), MonthsVec(i), DaysVec(i), HoursVec(i)] = GOAT_serial2date(G.Addresses.Default,G.DataStringVec(i)) ;
end
t    = linspace(year1,year2+1,length(G.DataStringVec)+1) ; t = t(1:end-1) ;
IDX_DJFM = find(MonthsVec == 12 | MonthsVec == 1 | MonthsVec == 2 | MonthsVec == 3) ; 
IDX_JJAS = find(MonthsVec == 6  | MonthsVec == 7 | MonthsVec == 8 | MonthsVec == 9) ; 
%% plot and save
% Annual mean
figMean = figure('name',['Mean ',G.DataField],'position',[50 50 G.scrsz(3)*.45 G.scrsz(4)*.55]) ;  
[C,h] =  contourf(G.X,G.Y,squeeze(mean(pr,1))') ; colorbar ; 
if any(G.ProjectionType == [3,4]) 
axis ij ; % for lat-pressure we would like to present along decreasing pressure 
end
ylim([Lat1 Lat2]) ;
xlim([Lon1 Lon2])   ;
xlabel('$$ \textrm{longitude}  $$','fontsize',14,'interpreter','latex') ; 
ylabel('$$ \textrm{latitude}   $$','fontsize',14,'interpreter','latex') ; 
set(gca,'fontsize',12,'fontname','sans-serif')
title(['$$ \textrm{Mean ',G.DataField,'} \  \ [',G.DataFieldUnits,'] $$'],'fontsize',16,'interpreter','latex')
set(gcf,'PaperPositionMode','auto','papersize',G.Saving.PapersizeSquare)          ;
print('-dpdf','-r70',fullfile(ExportDir,'Mean_pr')) ;
% Or use GOAT's plot definitions
figMean_GUI = figure('name',['Mean ',G.DataField],'position',G.pos) ;  
GOAT_plotUpdate(G) ; 
title(['$$ \textrm{Mean ',G.DataField,'} \  \ [',G.DataFieldUnits,'] $$'],'fontsize',16,'interpreter','latex')
set(gcf,'PaperPositionMode','auto','papersize',G.Saving.PapersizeGUI)          ;
print('-dpdf','-r70',fullfile(ExportDir,'Mean_pr_GUI')) ;


%% DJFM 
figDJFM = figure('name',['DJFM ',G.DataField],'position',[50 50 G.scrsz(3)*.45 G.scrsz(4)*.55]) ;  
[C,h] =  contourf(G.X,G.Y,squeeze(mean(pr(IDX_DJFM,:,:),1))') ; colorbar ; 
if any(G.ProjectionType == [3,4]) 
axis ij ; % for lat-pressure we would like to present along decreasing pressure 
end
ylim([Lat1 Lat2]) ;
xlim([Lon1 Lon2])   ;
xlabel('$$ \textrm{longitude}  $$','fontsize',14,'interpreter','latex') ; 
ylabel('$$ \textrm{latitude}   $$','fontsize',14,'interpreter','latex') ; 
set(gca,'fontsize',12,'fontname','sans-serif')
title(['$$ \textrm{DJFM ',G.DataField,'} \  \ [',G.DataFieldUnits,'] $$'],'fontsize',16,'interpreter','latex')
set(gcf,'PaperPositionMode','auto','papersize',G.Saving.PapersizeSquare)          ;
print('-dpdf','-r70',fullfile(ExportDir,'DJFM_pr')) ;
% Or use GOAT's plot definitions
figDJFM_GUI = figure('name',['Mean ',G.DataField],'position',G.pos) ;  
GOAT_plotUpdate(G) ; 
title(['$$ \textrm{DJFM ',G.DataField,'} \  \ [',G.DataFieldUnits,'] $$'],'fontsize',16,'interpreter','latex')
set(gcf,'PaperPositionMode','auto','papersize',G.Saving.PapersizeGUI)          ;
print('-dpdf','-r70',fullfile(ExportDir,'DJFM_pr_GUI')) ;


%% plot anomaly during Jan.1980
figJanAnom = figure('name',['Jan Anom ',G.DataField],'position',[50 50 G.scrsz(3)*.45 G.scrsz(4)*.55]) ;  
[C,h] =  contourf(G.X,G.Y,squeeze(mean(pr_Anom((YearsVec == 1980 & MonthsVec == 1),:,:),1))') ; colorbar ; 
if any(G.ProjectionType == [3,4]) 
axis ij ; % for lat-pressure we would like to present along decreasing pressure 
end
ylim([Lat1 Lat2]) ;
xlim([Lon1 Lon2])   ;
xlabel('$$ \textrm{longitude}  $$','fontsize',14,'interpreter','latex') ; 
ylabel('$$ \textrm{latitude}   $$','fontsize',14,'interpreter','latex') ; 
set(gca,'fontsize',12,'fontname','sans-serif')
title(['$$ \textrm{Jan.1980 Anomaly ',G.DataField,'} \   \  \ \ [',G.DataFieldUnits,'] $$'],'fontsize',16,'interpreter','latex')
set(gcf,'PaperPositionMode','auto','papersize',G.Saving.PapersizeSquare)          ;
print('-dpdf','-r70',fullfile(ExportDir,'Anom_pr')) ;

%% save data
save(fullfile(ExportDir,'pr.mat'),'pr','pr_Clim','pr_Anom','t','YearsVec','MonthsVec','DaysVec') ; 
