% GOAT_MySets : Quick customized parameter sets for GOAT structure 'G'
% Copyright (C) Ori Adam Sep.2014.
% Syntax:
% >> G = GOAT_MySets(G,setname)           ; % General customized sets
% >> G = GOAT_MySets(G,setname,category)  ; % Customized sets for a given category: = 'Field' | 'Dataset'  Customized definitions for Fields and Datasets
%
% see also GOAT_MyDefaults

function G = GOAT_MySets(G,setname,category)
WarningOffFlag = 1 ; % do not display warning and error messages
switch nargin
    case 0
        help GOAT_MySets ;
        return
    case 1
        disp 'Set name not specified: G = GOAT_MySets(G,setname)'
        return
    case 3 % particular sets for Fields and Datasets
        switch lower(category)
            case 'field'
                switch setname
                    case 'Dummy'
                        
                    otherwise
                        if ~WarningOffFlag
                            disp(['GOAT_MySets: Unrecognized Field set name ',setname])
                        end
                end
            case 'dataset'
                switch setname
                    case {'ERSST','MERRA','ORAS4','GODAS','GISTEMP'}
                        G.Settings.NanMeanFlag = 1 ;
                    otherwise
                        if ~WarningOffFlag
                            disp(['GOAT_MySets: Unrecognized Dataset set name ',setname])
                        end
                end
        end
    case 2  % General sets
        switch setname
            case 'C1_0'  % example
                G.ProjectionType = 1            ; % 1) lat-lon with coast 2) lat-lon 3) lat-height 4)lon-height 5) hovmoller-lon 6) hovmoller-lat 7) hovmoller height 8/9) NH/SH polar stereographic
                G.Limits.Default.Day2same   = 0 ; % 1/0 Set Date 2 same as Date 1
                G.Limits.Default.Sequential = 1 ; % 1/0 Sequential or Seasonal averaging Flag
                G.Limits.Default.ClimatologyContinuous = 0 ; % present seasonal data sequentially in Hovmoller plots
                G.Limits.Default.Anom  = 0      ; % display anomalies in Hovmoller plots
                G.Limits.Default.sameP = 0      ; % Projection 1,2,8,9) 1/0 set level2 = level1 | Projection 3,4,7) 1/0 composite levels
                G.Limits.Default.VerMeanInt = 1 ; % 1) Vertical Mean 2) Integral using Boer Beta 3) set Nan values to zero and vertically integrate
                G.Settings.NanMeanFlag = 1  ; % use mean(x) when 0 and nanmean(x) when 1
                G.Filter.Index = ones(length(G.Addresses.Default),1) ; % default index
            case 'P1_1' % example
                G.ProjectionType = 1            ; % 1) lat-lon with coast 2) lat-lon 3) lat-height 4)lon-height 5) hovmoller-lon 6) hovmoller-lat 7) hovmoller height 8/9) NH/SH polar stereographic
                G.Limits.Default.Day2same   = 1 ; % 1/0 Set Date 2 same as Date 1
                G.Limits.Default.Sequential = 1 ; % 1/0 Sequential or Seasonal averaging Flag
                G.Limits.Default.ClimatologyContinuous = 0 ; % present seasonal data sequentially in Hovmoller plots
                G.Limits.Default.Anom  = 0      ; % display anomalies in Hovmoller plots
                G.Limits.Default.sameP = 0      ; % Projection 1,2,8,9) 1/0 set level2 = level1 | Projection 3,4,7) 1/0 composite levels
                G.Limits.Default.VerMeanInt = 1 ; % 1) Vertical Mean 2) Integral using Boer Beta 3) set Nan values to zero and vertically integrate
                G.Settings.NanMeanFlag = 1  ; % use mean(x) when 0 and nanmean(x) when 1
                G.Filter.Index = ones(length(G.Addresses.Default),1) ; % default index
                G.Limits.S1 = GOAT_getSerial(G.Limits.Year0,1,1,0,G.TimeFieldVirtual,G.Limits.Year0) ; 
                G.Limits.S2 = G.Limits.S1 ; 
                G = GOAT_Compile_data_Series(G) ; 
            otherwise
                if ~WarningOffFlag
                    disp(['GOAT_MySets: Unrecognized set name ',setname])
                end
        end
end



