% GOAT_VirtualField_DivMoistStaticEnergy
% Copyright (C) Ori Adam Dec.2012. Revised Dec.2013. Compatible with version 2013l or later.
% Input:
% Mode = 'Active': returns 1/0 (yes/no) whether to include the field in the list of fields appearing in the GUI Fields dropdown menu.
%                  when 0 the field will not appear in the GUI Fields list but will be available via GOAT_Environment coding
% Mode = 'VirtualFieldName': returns the name of the virtual field
% Mode = 'RequiredFieldsList': returns the actual fields that need to exist for the calculation of the virtual field
% Mode = 'ReferenceField': returns a field name with the same spatial and temporal dimensions as the virtual field
% Mode = 'Dims': returns G.VirtualFields.DataFieldUnits = Virtual Field Units and G.VirtualFields.FieldDescription = Virtual Field Description 
% Mode = 'Load': the actual virtual field calculation code.
% G = GOAT main structure
% datafile = GOAT file frame address. e.g., '1979/10101.mat'
% DataFolder = GOAT data folder, e.g. 'user/GOAT/Data/NCEP-I'
% TimeField = GOAT Calendar. e.g., 'Monthly','Daily','DailyX4', etc.
% Aidx: datafile = G.DataStringVec(Aidx) ; this index can be used for calculating time derivatives.
%
% see also GOAT_VirtualFields

function [G , data] = GOAT_VirtualField_DivMoistStaticEnergy(Mode,G,datafile,DataFolder,TimeField,Aidx)

switch Mode
    case 'Active'
        G.VirtualFields.ActiveFlag =  0 ; % | 1 for 'yes' and 0 for 'no'
    case 'VirtualFieldName'
        G.VirtualFields.MyName = 'DivMSEflux' ;
    case 'RequiredFieldsList'  % The existing fields required for the calculation of the virtual field
        G.VirtualFields.RequiredFieldsList = {'va','ua','ta','zg','hus'} ;
    case 'ReferenceField' % a field with similar Dims
        G.VirtualFields.ReferenceField = 'ta' ;
    case 'Dims'
        G.VirtualFields.DataFieldUnits = 'W/m^2' ;
        G.VirtualFields.FieldDescription = 'Moist Static Energy flux divergence' ;
    case 'Load'
        % calculate v dot Grad MSE
        G.DataField = 'ua' ; [ua,G] = GOAT_load_data(fullfile(DataFolder,TimeField,G.DataField,datafile),G,Aidx) ; 
        G.DataField = 'va' ; [va,G] = GOAT_load_data(fullfile(DataFolder,TimeField,G.DataField,datafile),G,Aidx) ; 
        G.DataField = 'ta' ; [T,G] = GOAT_load_data(fullfile(DataFolder,TimeField,G.DataField,datafile),G,Aidx) ; 
        L = GOAT_Calculate_L(T,G,'era') ;
        G.DataField = 'zg' ; [zg,G] = GOAT_load_data(fullfile(DataFolder,TimeField,G.DataField,datafile),G,Aidx) ; 
        G.DataField = 'hus' ; [hus,G] = GOAT_load_data(fullfile(DataFolder,TimeField,G.DataField,datafile),G,Aidx) ; 
        if isequal(size(T,3),size(hus,3)) %~strcmp(DataFolder(end-3:end),'NCEP')
         MSE = G.Consts.SpHeatConstPr*T + zg*G.Consts.GravitationalConst + L.*hus ;
        else % NCEP: hus is only 1000 to 300 mb P levels
         MSE = G.Consts.SpHeatConstPr.*T + zg*G.Consts.GravitationalConst ;
         MSE(:,:,1:size(hus,3)) = MSE(:,:,1:size(hus,3)) + L(:,:,1:size(hus,3)).*hus ;
        end
        % Zonal Derivative of u Component
        mu = MSE*0 ; 
        MSEu = MSE.*ua ; 
        mu(2:end-1,:,:) = (MSEu(3:end,:,:) - MSEu(1:end-2,:,:))/(G.lon(3)-G.lon(1))*180/pi ;
        mu(1      ,:,:) = (MSEu(2    ,:,:) - MSEu(end    ,:,:))/(G.lon(3)-G.lon(1))*180/pi ;
        mu(end    ,:,:) = (MSEu(1    ,:,:) - MSEu(end-1  ,:,:))/(G.lon(3)-G.lon(1))*180/pi ;
        mu = mu./G.Consts.EarthRadius./G.Vectors.cosmat ;
        % enforcing zero at poles
        mu(:,[1 end],:) = 0 ;
        % V component
        mv = MSE.*va.*G.Vectors.cosmat ;
        mv (:,2:end-1,:) = (mv(:,3:end,:) - mv(:,1:end-2,:))/(G.lat(3)-G.lat(1))*180/pi ;
        mv = mv./G.Consts.EarthRadius./G.Vectors.cosmat ;
        % enforcing zero at poles
        mv(:,[1 end],:) = 0 ;
        data = mv + mu ; 
        G.DataField = 'DivMSEflux' ;
end
