% GOAT_VirtualField_VorticityHorizontal
% Copyright (C) Ori Adam Dec.2012. Revised Dec.2013. Compatible with version 2013l or later.
% Input:
% Mode = 'Active': returns 1/0 (yes/no) whether to include the field in the list of fields appearing in the GUI Fields dropdown menu.
%                  when 0 the field will not appear in the GUI Fields list but will be available via GOAT_Environment coding
% Mode = 'VirtualFieldName': returns the name of the virtual field
% Mode = 'RequiredFieldsList': returns the actual fields that need to exist for the calculation of the virtual field
% Mode = 'ReferenceField': returns a field name with the same spatial and temporal dimensions as the virtual field
% Mode = 'Dims': returns G.VirtualFields.DataFieldUnits = Virtual Field Units and G.VirtualFields.FieldDescription = Virtual Field Description 
% Mode = 'Load': the actual virtual field calculation code.
% G = GOAT main structure
% datafile = GOAT file frame address. e.g., '1979/10101.mat'
% DataFolder = GOAT data folder, e.g. 'user/GOAT/Data/NCEP-I'
% TimeField = GOAT Calendar. e.g., 'Monthly','Daily','DailyX4', etc.
% Aidx: datafile = G.DataStringVec(Aidx) ; this index can be used for calculating time derivatives.
%
% see also GOAT_VirtualFields

function [G , data] = GOAT_VirtualField_VorticityHorizontal(Mode,G,datafile,DataFolder,TimeField,Aidx)

switch Mode
    case 'Active'
        G.VirtualFields.ActiveFlag =  0 ; % | 1 for 'yes' and 0 for 'no'
    case 'VirtualFieldName'
        G.VirtualFields.MyName = 'vor_h' ;
    case 'RequiredFieldsList'  % The existing fields required for the calculation of the virtual field
        G.VirtualFields.RequiredFieldsList = {'va','ua'} ;
    case 'ReferenceField' % a field with similar Dims
        G.VirtualFields.ReferenceField = 'va' ;
    case 'Dims'
        G.VirtualFields.DataFieldUnits = '1/s' ;
        G.VirtualFields.FieldDescription = 'Horizontal Vorticity' ;
    case 'Load'
        % V component
        G.DataField = 'va' ; [data,G] = GOAT_load_data(fullfile(DataFolder,TimeField,G.DataField,datafile),G,Aidx) ;         
        vctag = data*0 ;
        % Zonal Derivative of u
        vctag(2:end-1,:,:) = (data(3:end,:,:) - data(1:end-2,:,:))/(G.lon(3)-G.lon(1))*180/pi ;
        vctag(1      ,:,:) = (data(2    ,:,:) - data(end    ,:,:))/(G.lon(3)-G.lon(1))*180/pi ;
        vctag(end    ,:,:) = (data(1    ,:,:) - data(end-1  ,:,:))/(G.lon(3)-G.lon(1))*180/pi ;
        Vcomponent = vctag./G.Consts.EarthRadius./G.Vectors.cosmat ;
        % enforcing zero at poles
        Vcomponent(:,[1,end],:) = 0 ;
        % U component
        G.DataField = 'ua' ; [data,G] = GOAT_load_data(fullfile(DataFolder,TimeField,G.DataField,datafile),G,Aidx) ;         
        u2 = data.*G.Vectors.cosmat ;
        utag = data*0 ;
        utag (:,2:end-1,:) = (u2(:,3:end,:) - u2(:,1:end-2,:))/(G.lat(3)-G.lat(1))*180/pi ;
        Ucomponent = -utag./G.Consts.EarthRadius./G.Vectors.cosmat ;
        % enforcing zero at poles
        Ucomponent(:,[1,0],:) = 0 ;
        data = Vcomponent + Ucomponent ;
        G.DataField = 'vor_h' ; 
end
